package modules;

import com.google.inject.AbstractModule;
import com.google.inject.multibindings.Multibinder;
import se.su.dsv.scipro.activityplan.*;
import se.su.dsv.scipro.checklist.*;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.date.DateServiceImpl;
import se.su.dsv.scipro.economy.TransactionService;
import se.su.dsv.scipro.economy.TransactionServiceImpl;
import se.su.dsv.scipro.events.EventModule;
import se.su.dsv.scipro.file.PublicFileDescriptionService;
import se.su.dsv.scipro.file.PublicFileDescriptionServiceImpl;
import se.su.dsv.scipro.finalseminar.*;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.finalthesis.FinalThesisServiceImpl;
import se.su.dsv.scipro.forum.ForumModule;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsServiceImpl;
import se.su.dsv.scipro.group.GroupFacade;
import se.su.dsv.scipro.group.GroupFacadeImpl;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.group.GroupServiceImpl;
import se.su.dsv.scipro.integration.activityfinalseminar.ActivityFinalSeminarRepository;
import se.su.dsv.scipro.integration.activityfinalseminar.ActivityFinalSeminarRepositoryImpl;
import se.su.dsv.scipro.integration.activityforum.ActivityThreadRepository;
import se.su.dsv.scipro.integration.activityforum.ActivityThreadRepositoryImpl;
import se.su.dsv.scipro.mail.MailModule;
import se.su.dsv.scipro.match.ApplicationPeriodFacade;
import se.su.dsv.scipro.match.ApplicationPeriodFacadeImpl;
import se.su.dsv.scipro.match.MatchModule;
import se.su.dsv.scipro.milestones.service.*;
import se.su.dsv.scipro.milestones.service.impl.MilestoneActivityTemplateServiceImpl;
import se.su.dsv.scipro.milestones.service.impl.MilestonePhaseTemplateServiceImpl;
import se.su.dsv.scipro.milestones.service.impl.MilestoneServiceImpl;
import se.su.dsv.scipro.milestones.service.impl.MilestoneStatisticsServiceImpl;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.misc.DaysServiceImpl;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodServiceImpl;
import se.su.dsv.scipro.notes.NoteService;
import se.su.dsv.scipro.notes.NoteServiceImpl;
import se.su.dsv.scipro.notifications.GradingNotifications;
import se.su.dsv.scipro.notifications.NotificationModule;
import se.su.dsv.scipro.notifications.settings.service.DeliveryConfigurationService;
import se.su.dsv.scipro.notifications.settings.service.DeliveryConfigurationServiceImpl;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationService;
import se.su.dsv.scipro.notifications.settings.service.ReceiverConfigurationServiceImpl;
import se.su.dsv.scipro.peer.*;
import se.su.dsv.scipro.phd.*;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.plagiarism.TurnItInCheckServiceImpl;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsService;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsServiceImpl;
import se.su.dsv.scipro.project.ProjectPeopleStatisticsService;
import se.su.dsv.scipro.project.ProjectPeopleStatisticsServiceImpl;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectServiceImpl;
import se.su.dsv.scipro.projectpartner.ProjectPartnerService;
import se.su.dsv.scipro.projectpartner.ProjectPartnerServiceImpl;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.reviewing.ProjectFinalSeminarStatisticsService;
import se.su.dsv.scipro.reviewing.ProjectFinalSeminarStatisticsServiceImpl;
import se.su.dsv.scipro.springdata.serviceimpls.SupervisorServiceImpl;
import se.su.dsv.scipro.springdata.serviceimpls.UnitServiceImpl;
import se.su.dsv.scipro.springdata.serviceimpls.UserProfileServiceImpl;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.thesislink.ExternalLinkService;
import se.su.dsv.scipro.thesislink.ExternalLinkServiceImpl;

public class CoreModule extends AbstractModule {
    @Override
    protected void configure() {
        install(new RepositoryModule());
        bind(FooterLinkService.class).to(FooterLinkServiceImpl.class);
        bind(PublicFileDescriptionService.class).to(PublicFileDescriptionServiceImpl.class);
        bind(ActivityThreadRepository.class).to(ActivityThreadRepositoryImpl.class);
        bind(ActivityFinalSeminarRepository.class).to(ActivityFinalSeminarRepositoryImpl.class);
        bind(UserService.class).to(UserServiceImpl.class);
        bind(PasswordService.class).to(PasswordServiceImpl.class);
        bind(GeneralSystemSettingsService.class).to(GeneralSystemSettingsServiceImpl.class);
        bind(ProjectTypeService.class).to(ProjectTypeServiceImpl.class);
        bind(UnitService.class).to(UnitServiceImpl.class);
        bind(ResearchAreaService.class).to(ResearchAreaServiceImpl.class);
        bind(DateService.class).to(DateServiceImpl.class);
        bind(GradingNotifications.class).asEagerSingleton();
        bind(ActivityPlanFacade.class).to(ActivityPlanFacadeImpl.class);
        bind(ProjectService.class).to(ProjectServiceImpl.class);
        bind(ProjectFinalSeminarStatisticsService.class).to(ProjectFinalSeminarStatisticsServiceImpl.class);
        bind(ProjectPeopleStatisticsService.class).to(ProjectPeopleStatisticsServiceImpl.class);
        bind(DeliveryConfigurationService.class).to(DeliveryConfigurationServiceImpl.class);
        bind(ReceiverConfigurationService.class).to(ReceiverConfigurationServiceImpl.class);
        bind(ActivityService.class).to(ActivityServiceImpl.class);
        bind(ActivityPlanService.class).to(ActivityPlanServiceImpl.class);
        bind(ActivityPlanTemplateService.class).to(ActivityPlanTemplateServiceImpl.class);
        bind(ChecklistService.class).to(ChecklistServiceImpl.class);
        bind(UserProfileService.class).to(UserProfileServiceImpl.class);
        bind(FinalSeminarService.class).to(ExtendedFinalSeminarServiceImpl.class);
        bind(FinalSeminarCreationService.class).to(FinalSeminarCreationServiceImpl.class);
        bind(FinalSeminarSettingsService.class).to(FinalSeminarSettingsServiceImpl.class);
        bind(SupervisorService.class).to(SupervisorServiceImpl.class);
        bind(DaysService.class).to(DaysServiceImpl.class);
        bind(NonWorkDayPeriodService.class).to(NonWorkDayPeriodServiceImpl.class);
        bind(FinalSeminarOppositionService.class).to(FinalSeminarOppositionServiceImpl.class);
        bind(OppositionReportService.class).to(OppositionReportServiceImpl.class);
        bind(ApplicationPeriodFacade.class).to(ApplicationPeriodFacadeImpl.class);
        bind(GroupFacade.class).to(GroupFacadeImpl.class);
        bind(ExternalLinkService.class).to(ExternalLinkServiceImpl.class);
        bind(PeerRequestService.class).to(PeerRequestServiceImpl.class);
        bind(PeerReviewService.class).to(PeerReviewServiceImpl.class);
        bind(MilestoneActivityTemplateService.class).to(MilestoneActivityTemplateServiceImpl.class);
        bind(FinalThesisService.class).to(FinalThesisServiceImpl.class);
        bind(TransactionService.class).to(TransactionServiceImpl.class);
        bind(ChecklistTemplateService.class).to(ChecklistTemplateServiceImpl.class);
        bind(ActivityTemplateService.class).to(ActivityTemplateServiceImpl.class);
        bind(PeerPortal.class).to(PeerPortalImpl.class);
        bind(TurnItInCheckService.class).to(TurnItInCheckServiceImpl.class);
        bind(TurnitinSettingsService.class).to(TurnitinSettingsServiceImpl.class);
        bind(FinalSeminarRespondentService.class).to(FinalSeminarRespondentServiceImpl.class);
        bind(ProjectPartnerService.class).to(ProjectPartnerServiceImpl.class);
        bind(GradingReportService.class).to(GradingReportServiceImpl.class);
        bind(UserNameService.class).to(UserNameServiceImpl.class);
        bind(MileStoneService.class).to(MilestoneServiceImpl.class);
        bind(MilestoneStatisticsService.class).to(MilestoneStatisticsServiceImpl.class);
        bind(MilestonePhaseTemplateService.class).to(MilestonePhaseTemplateServiceImpl.class);
        bind(ReportService.class).to(ReportServiceImpl.class);
        bind(CommentThreadService.class).to(CommentThreadServiceImpl.class);
        bind(CommentService.class).to(CommentServiceImpl.class);
        bind(PerformReviewService.class).to(PeerPortalImpl.class);
        bind(EventService.class).to(EventServiceImpl.class);
        bind(ChecklistAnswerService.class).to(ChecklistAnswerServiceImpl.class);
        bind(StudyPlanService.class).to(StudyPlanServiceImpl.class);
        bind(FinalSeminarUploadController.class).to(FinalSeminarUploadControllerImpl.class);
        bind(FinalSeminarActiveParticipationService.class).to(FinalSeminarActiveParticipationServiceImpl.class);
        bind(DepartmentServiceService.class).to(DepartmentServiceServiceImpl.class);
        bind(ExternalResourceService.class).to(ExternalResourceServiceImpl.class);
        bind(LanguageService.class).to(LanguageServiceImpl.class);
        bind(GroupService.class).to(GroupServiceImpl.class);
        bind(HashService.class).to(HashServiceImpl.class);
        bind(NoteService.class).to(NoteServiceImpl.class);
        bind(PasswordFacade.class).to(PasswordFacadeImpl.class);
        bind(UserFacade.class).to(UserFacadeImpl.class);
        bind(MilestoneActivator.class).asEagerSingleton();

        install(new PlagiarismModule());
        install(new NotificationModule());
        install(new ProfileModule());
        install(new EventModule());
        install(new MatchModule());
        install(new MailModule());
        install(new ForumModule());

        Multibinder.newSetBinder(binder(), UserSearchService.class);
    }
}
