package se.su.dsv.scipro.activityplan;

import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

@Entity
@Cacheable(true)
@Data
@ToString(exclude = {"activityTemplates"})
@EqualsAndHashCode(callSuper = true)
@SuppressWarnings("PMD.UnusedPrivateField")
public class ActivityPlanTemplate extends DomainObject {

    @Id
	@GeneratedValue
	private Long id;

    @OrderColumn(name = "numberInOrder")
	@OneToMany(mappedBy="activityPlanTemplate", orphanRemoval=true, cascade=CascadeType.ALL)
	private List<ActivityTemplate> activityTemplates = new ArrayList<>();

    public List<ActivityTemplate> getActivityTemplates(){
        return Collections.unmodifiableList(activityTemplates);
    }

    public void setActivityTemplates(List<ActivityTemplate> activityTemplates){
        this.activityTemplates = new ArrayList<>(activityTemplates);
    }
	
	@ManyToOne(optional=false)
	private User creator;

	@Column(nullable=false)
	private String title;
	
	@Lob
	private String description;
	
	@Column(nullable=false)
	private Boolean isSysAdminTemplate = false;

	public void addActivity(ActivityTemplate activity){
		activity.setActivityPlanTemplate(this);
        activity.setNumberInOrder(activityTemplates.size());
		activityTemplates.add(activity);
	}

    public void clearActivities(){
        activityTemplates.clear();
    }

    public void addActivities(final Collection<ActivityTemplate> activities){
        activityTemplates.addAll(activities);
    }
}
