package se.su.dsv.scipro.checklist;

import com.google.inject.persist.Transactional;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

import static com.mysema.query.support.Expressions.allOf;

public class ChecklistTemplateServiceImpl extends AbstractServiceImpl<ChecklistTemplate,Long> implements ChecklistTemplateService {

    public static final String PEER = "Peer";

    @Inject
    public ChecklistTemplateServiceImpl(Provider<EntityManager> em) {
        super(em, ChecklistTemplate.class, QChecklistTemplate.checklistTemplate);
    }

    @Override
    public List<ChecklistTemplate> findAll(String filterString, Pageable pageable) {
        String filter = filterString == null ? "" : filterString;
        return findAll(filterStringIsCreatorName(filter).or(filterStringIsChecklistTemplateName(filter)), pageable);
    }

    @Override
    public long count(String filterString) {
        String filter = filterString == null ? "" : filterString;
        return count(filterStringIsCreatorName(filter).or(filterStringIsChecklistTemplateName(filter)));
    }

    @Transactional
    @Override
    public void upChecklistTemplate(ChecklistTemplate checklistTemplate) {
        if (findByTemplateNumber(checklistTemplate.getTemplateNumber() + 1) != null) {
            ChecklistTemplate downChecklistTemplate = findByTemplateNumber(checklistTemplate.getTemplateNumber() + 1);
            downChecklistTemplate.setTemplateNumber(checklistTemplate.getTemplateNumber());
            checklistTemplate.setTemplateNumber(checklistTemplate.getTemplateNumber() + 1);
            save(downChecklistTemplate);
            save(checklistTemplate);
        }
    }

    private ChecklistTemplate findByTemplateNumber(int templateNumber) {
        return findOne(QChecklistTemplate.checklistTemplate.templateNumber.eq(templateNumber));
    }

    @Transactional
    @Override
    public void downChecklistTemplate(ChecklistTemplate checklistTemplate) {
        if (findByTemplateNumber(checklistTemplate.getTemplateNumber() - 1) != null) {
            ChecklistTemplate upChecklistTemplate = findByTemplateNumber(checklistTemplate.getTemplateNumber() - 1);
            upChecklistTemplate.setTemplateNumber(checklistTemplate.getTemplateNumber());
            checklistTemplate.setTemplateNumber(checklistTemplate.getTemplateNumber() - 1);
            save(upChecklistTemplate);
            save(checklistTemplate);
        }
    }

    @Transactional
    @Override
    public void safeDeleteChecklistTemplate(ChecklistTemplate checklistTemplate) {
        for (ChecklistTemplate clt : findAll(QChecklistTemplate.checklistTemplate.templateNumber.gt(checklistTemplate.getTemplateNumber()))) {
            clt.setTemplateNumber(clt.getTemplateNumber() - 1);
        }
        delete(checklistTemplate);
    }

    @Override
    public List<ChecklistTemplate> findPeerRequestChecklists(final Project project) {
        return findAll(allOf(
                QChecklistTemplate.checklistTemplate.projectTypes.contains(project.getProjectType()),
                QChecklistTemplate.checklistTemplate.categories.any().categoryName.eq(PEER)
        ));
    }

    @Override
    public List<ChecklistTemplate> findByProject(Project project) {
        return findAll(QChecklistTemplate.checklistTemplate.projectTypes.contains(project.getProjectType()));
    }

    @Override
    public ChecklistTemplate findByName(final String name) {
        return findOne(QChecklistTemplate.checklistTemplate.name.eq(name));
    }

    public static BooleanExpression filterStringIsCreatorName(String filterString) {
        return QChecklistTemplate.checklistTemplate.creator.firstName.contains(filterString).or(QChecklistTemplate.checklistTemplate.creator.lastName.contains(filterString));
    }

    public static BooleanExpression filterStringIsChecklistTemplateName(String filterString) {
        return QChecklistTemplate.checklistTemplate.name.contains(filterString);
    }
}
