package se.su.dsv.scipro.crosscutting;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.reviewing.*;

import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
class ReviewingNotifications {
    private final NotificationController notificationController;

    @Inject
    ReviewingNotifications(final EventBus eventBus, final NotificationController notificationController) {
        this.notificationController = notificationController;
        eventBus.register(this);
    }

    @Subscribe
    public void onFinalSeminarApprovalRequest(final FinalSeminarApprovalRequestEvent finalSeminarApprovalRequestEvent) {
        notificationController.notifyProject(finalSeminarApprovalRequestEvent.getProject(), ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED, new NotificationSource());
    }

    @Subscribe
    public void onFinalSeminarApprovalRejected(final FinalSeminarApprovalRejectedEvent finalSeminarApprovalRejectedEvent) {
        notificationController.notifyProject(finalSeminarApprovalRejectedEvent.getProject(), ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REJECTED, new NotificationSource());
    }

    @Subscribe
    public void onFinalSeminarApprovalApproved(final FinalSeminarApprovalApprovedEvent finalSeminarApprovalApprovedEvent) {
        notificationController.notifyProject(finalSeminarApprovalApprovedEvent.getProject(), ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_APPROVED, new NotificationSource());
    }

    @Subscribe
    public void onRoughDraftApprovalRequest(final RoughDraftApprovalRequestedEvent roughDraftApprovalRequestedEvent) {
        notificationController.notifyProject(roughDraftApprovalRequestedEvent.getProject(), ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED, new NotificationSource());
    }

    @Subscribe
    public void onRoughDraftApprovalRejected(final RoughDraftApprovalRejectedEvent roughDraftApprovalRejectedEvent) {
        notificationController.notifyProject(roughDraftApprovalRejectedEvent.getProject(), ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REJECTED, new NotificationSource());
    }

    @Subscribe
    public void onRoughDraftApprovalApproved(final RoughDraftApprovalApprovedEvent roughDraftApprovalApprovedEvent) {
        notificationController.notifyProject(roughDraftApprovalApprovedEvent.getProject(), ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_APPROVED, new NotificationSource());
    }
}
