
package se.su.dsv.scipro.finalseminar;

import lombok.Data;
import org.springframework.data.domain.Pageable;
import scala.util.Either;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.FilteredService;
import se.su.dsv.scipro.system.GenericService;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

public interface FinalSeminarService extends GenericService<FinalSeminar, Long>, FilteredService<FinalSeminar, Long, FinalSeminarService.Filter> {
    Either<SchedulingError, FinalSeminar> move(FinalSeminar finalSeminar, Date to);
    Either<OpposeError, FinalSeminarOpposition> oppose(User student, FinalSeminar finalSeminar, Project project);
    void participate(User student, FinalSeminar finalSeminar, Project project);
    OppositionRegistrationStatus canOppose(User user, FinalSeminar finalSeminar, Project project);
    boolean canParticipate(User user, FinalSeminar finalSeminar);
    Iterable<FinalSeminar> findAll(Filter params);
    Date thesisUploadDeadline(FinalSeminar finalSeminar);

    boolean hasDeadlinePassed(FinalSeminar finalSeminar);
    boolean hasThesis(FinalSeminar finalSeminar);
    FinalSeminar findByProject(Project project);
    Iterable<FinalSeminar> findByStartDateAfter(Date date);


    Iterable<FinalSeminar> findProjectOpposing(Project project, Pageable pageable);
    Iterable<FinalSeminar> findProjectParticipating(Project project, Pageable pageable);

    FinalSeminar findByFileDescription(FileDescription fileDescription);
    FinalSeminar deleteThesis(FinalSeminar seminar);
    void delete(FinalSeminar finalSeminar);

    boolean isOpponent(FinalSeminar seminar, User opponent);
    boolean isActiveParticipant(FinalSeminar seminar, User activeParticipant);

    boolean hasHadFinalSeminar(Project project);

    List<FinalSeminar> findUnfinishedSeminars(Date after, Date before, Pageable pageable);

    long countUnfinishedSeminars(Date after, Date before);

    FinalSeminar cancel(FinalSeminar finalSeminar);

    List<FinalSeminar> findUserOpposing(Project project, User user);
    List<FinalSeminar> findUserParticipating(Project project, User user);

    @Data
    @SuppressWarnings({"PMD.UnusedPrivateField", "UnusedDeclaration"})
    class Filter implements Serializable {
        private Date fromDate;
        private Date toDate;
        private ProjectType projectType;
        private User reviewer;
        private Boolean lazyDeleted;
        private Boolean exempted;
        private Boolean onlyActiveProjects;
        private Boolean onlyActiveOrCompletedProjects;
        private User headSupervisor;
        private Boolean hasDocument;
    }
}
