package se.su.dsv.scipro.finalseminar;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.exceptions.InvalidFileFormatException;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.plagiarism.TurnItInCheck;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Date;

public class FinalSeminarUploadControllerImpl implements FinalSeminarUploadController {

    private static final Logger LOGGER = LoggerFactory.getLogger(FinalSeminarUploadControllerImpl.class);

    private final TurnItInCheckService turnItInCheckService;
    private final FileDescriptionService fileDescriptionService;
    private final FinalSeminarService finalSeminarService;
    private final ProjectService projectService;
    private final EventBus eventBus;

    private final FinalSeminarSettingsService finalSeminarSettingsService;

	@Inject
    public FinalSeminarUploadControllerImpl(ProjectService projectService, FileDescriptionService fileDescriptionService,
                                            FinalSeminarSettingsService finalSeminarSettingsService, FinalSeminarService finalSeminarService,
                                            TurnItInCheckService turnItInCheckService,
                                            EventBus eventBus) {
        this.projectService = projectService;
        this.fileDescriptionService = fileDescriptionService;
        this.finalSeminarSettingsService = finalSeminarSettingsService;
        this.finalSeminarService = finalSeminarService;
        this.turnItInCheckService = turnItInCheckService;
        this.eventBus = eventBus;
	}

    @Override
    public FileDescription storeSeminarDocument(FileUpload fileUpload, FinalSeminar seminar) {
        assertParticipant(fileUpload.getUploader(), seminar);
        assertMimeType(fileUpload.getContentType());

        FileDescription fd = fileDescriptionService.storeFile(fileUpload);
        setSeminarDocument(fd, seminar);
        return fd;

    }

    @Transactional
    private void setSeminarDocument(FileDescription fileDescription, FinalSeminar seminar) {
        FinalSeminar reloadedSeminar = finalSeminarService.findOne(seminar.getId());
        fileDescription.setExposed(true);
        reloadedSeminar.setDocument(fileDescription);
        reloadedSeminar.setDocumentUploadDate(new Date());

        reloadedSeminar = finalSeminarService.save(reloadedSeminar);
        TurnItInCheck turnItInCheck = new TurnItInCheck(fileDescription);
        turnItInCheckService.save(turnItInCheck);

	    eventBus.post(new FinalSeminarThesisUploadedEvent(reloadedSeminar));
    }

    private void assertParticipant(final User uploader, final FinalSeminar seminar) {
        final Project p = projectService.findOne(seminar.getProject().getId());
        boolean isParticipant = false;
        for (User s : p.getProjectParticipants()) {
            if (s.equals(uploader)) {
                isParticipant = true;
                break;
            }
        }
        if (!isParticipant) {
            throw new IllegalStateException("Users not a participant of the seminar project cannot upload the seminar document");
        }
    }

    private void assertMimeType(final String type) {
        final boolean requirePdf = finalSeminarSettingsService.getInstance().getThesisMustBePDF();
        if (type == null || (requirePdf && !type.equals("application/pdf"))) {
            LOGGER.debug("Faulty file type uploaded. Expected " + (requirePdf ? "\"application/pdf\"" : "non null") + ", was: \"" + type + "\"");
            throw new InvalidFileFormatException("The file must be " + (requirePdf ? "a PDF" : "non null"));
        }
    }

}
