package se.su.dsv.scipro.forum;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import lombok.experimental.Delegate;
import se.su.dsv.scipro.file.*;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

class ProjectForumServiceImpl implements ProjectForumService {
    private final ProjectThreadRepository projectThreadRepository;
    private final ForumPostRepository postRepository;
    private final FileDescriptionService fileDescriptionService;
    private final BasicForumService basicForumService;
    private final EventBus eventBus;

    @Inject
    ProjectForumServiceImpl(
            final ProjectThreadRepository projectThreadRepository,
            final ForumPostRepository postRepository,
            final FileDescriptionService fileDescriptionService,
            final BasicForumService basicForumService,
            final EventBus eventBus) {
        this.projectThreadRepository = projectThreadRepository;
        this.postRepository = postRepository;
        this.fileDescriptionService = fileDescriptionService;
        this.basicForumService = basicForumService;
        this.eventBus = eventBus;
    }

    @Override
    @Transactional
    public ProjectThread createThread(
            final Project project,
            final User user,
            final String subject,
            final String content,
            final Set<FileUpload> fileUploads) {
        ProjectThread thread = createThread(project, subject);

        createThreadReply(thread, user, content, fileUploads, project);

        return thread;
    }

    @Override
    public ProjectThread createThread(final Project project, final String subject) {
        ForumThread forumThread1 = basicForumService.createThread(subject);
        ProjectThread thread = new ProjectThread();
        thread.setProject(project);
        thread.setForumThread(forumThread1);

        thread = projectThreadRepository.save(thread);
        return thread;
    }

    private ForumPost createThreadReply(ProjectThread projectThread, final User user, final String content, final Set<FileUpload> fileUploads, Project project) {
        return createReply(projectThread, user, content, storeForumFiles(project, fileUploads));
    }

    @Override
    @Transactional
    public ForumPost createReply(final ProjectThread projectThread, final User poster, final String content, final Set<FileDescription> attachments) {
        ForumPost reply = basicForumService.createReply(projectThread.getForumThread(), poster, content, attachments);
        eventBus.post(new NewProjectForumReplyEvent(projectThread, reply));
        return reply;
    }

    private Set<FileDescription> storeForumFiles(final Project project, final Set<FileUpload> fileUploads) {
        Set<FileDescription> storedFiles = new HashSet<>();
        for (FileUpload fileUpload : fileUploads){
            storedFiles.add(fileDescriptionService.storeFile(new ForumFile(project, fileUpload), FileSource.FORUM));
        }
        return storedFiles;
    }

    @Override
    public List<ForumPost> getPosts(final ProjectThread projectThread) {
        return basicForumService.getPosts(projectThread.getForumThread());
    }

    @Override
    public void markRead(final User user, final ProjectThread thread) {
        basicForumService.setThreadRead(user, thread.getForumThread(), true);
    }

    @Override
    public List<ProjectThread> getThreads(Project project) {
        return projectThreadRepository.findByProject(project);
    }

    @Override
    public List<ForumPost> latestPost(Project project, int amount) {
        return postRepository.latestPost(project, amount);
    }

    @Override
    public ProjectThread findOne(long threadId) {
        return projectThreadRepository.findOne(threadId);
    }

    private static class ForumFile implements ProjectFileUpload {
        private final Project project;
        @SuppressWarnings("UnusedDeclaration")
        @Delegate
        private final FileUpload fileUpload;

        public ForumFile(Project project, FileUpload fileUpload) {
            this.project = project;
            this.fileUpload = fileUpload;
        }

        @Override
        public Project getProject() {
            return project;
        }

        @Override
        public boolean isDeletable() {
            return false;
        }
    }

    // Down here is the impl for the shitty un-threaded forum

}
