package se.su.dsv.scipro.forum.dataobjects;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.system.LazyDeletableDomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.HashSet;
import java.util.Set;

@EqualsAndHashCode(callSuper = false)
@Entity
@Table(name = "forum_post")
@Data
@SuppressWarnings("PMD.UnusedPrivateField")
public class ForumPost extends LazyDeletableDomainObject {

    @Id
    @GeneratedValue
    private Long id;

    @Lob
    @Column(name = "content", nullable = false)
    private String content;

    @ManyToOne
    @JoinColumn(name = "user", nullable = false)
    private User postedBy;

    @ManyToOne
    @JoinColumn(name = "thread", nullable = false)
    private ForumThread forumThread;

    @OneToMany(orphanRemoval = true)
    @JoinTable(name = "forum_post_file_description",
            joinColumns = {@JoinColumn(name = "forum_post_id")},
            inverseJoinColumns = {@JoinColumn(name = "file_description_id")})
    private Set<FileDescription> attachments = new HashSet<>();

    public String getSubject() {
        return forumThread.getSubject();
    }

    public Set<FileDescription> getAttachments() {
        return attachments;
    }

    public User getPostedBy() {
        return postedBy;
    }

    public String getContent() {
        return content;
    }

    public ForumThread getForumThread() {
        return forumThread;
    }
}
