package se.su.dsv.scipro.forum.dataobjects;

import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import se.su.dsv.scipro.system.LazyDeletableDomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@EqualsAndHashCode(callSuper = false, of = "id")
@Data
@Entity
@Table(name = "thread")
@Inheritance(strategy = InheritanceType.JOINED)
@ToString(exclude = "posts")
@SuppressWarnings("PMD.UnusedPrivateField")
public class ForumThread extends LazyDeletableDomainObject {

    @Id
    @GeneratedValue
    private Long id;

    @OneToMany(mappedBy = "forumThread", cascade = CascadeType.ALL)
    private List<ForumPost> posts = new ArrayList<>();

    @Basic
    @Column(name = "subject", nullable = false)
    private String subject;

    public void addPost(ForumPost post) {
        posts.add(post);
    }

    public int getPostCount() {
        return posts.size();
    }

    @PostLoad
    void lazyDeletion() {
        Iterator<ForumPost> iterator = posts.iterator();
        while (iterator.hasNext()) {
            if (iterator.next().isDeleted()) {
                iterator.remove();
            }
        }
    }

    public String getSubject() {
        return subject;
    }

    public User getCreatedBy(){
        return getPosts().get(0).getPostedBy();
    }

    public boolean hasAttachments() {
        for (ForumPost post : posts) {
            if (!post.getAttachments().isEmpty()) {
                return true;
            }
        }
        return false;
    }
}
