package se.su.dsv.scipro.forummail;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;

import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import java.io.IOException;
import java.util.ListIterator;

public class ForumMailFormatterImpl implements MailContentParser, ForumMailFormatter {

    private static final Logger LOGGER = LoggerFactory.getLogger(ForumMailFormatter.class);
    public static final String DELIMITER = "---------------- REPLY ABOVE THIS LINE ----------------";

    @Override
    public String format(Part message) {
        try {
            String messageBody = checkMimeType(message);
            return messageBody != null ?
                    substringOnDelimiter(messageBody) : "";
        } catch (MessagingException | IOException e) {
            LOGGER.error(e.getMessage(), e);
            return "";
        }
    }

    private String substringOnDelimiter(String str) {
        int delimiter = str.indexOf(DELIMITER);
        if (delimiter >= 0) {
            return str.substring(0, delimiter);
        }
        return str;
    }

    protected String checkMimeType(Part message) throws
            MessagingException, IOException {
        if (message.isMimeType("text/*")) {
            return (String) message.getContent();
        }
        if (message.isMimeType("multipart/*")) {
            Multipart mp = (Multipart) message.getContent();
            for (int i = 0; i < mp.getCount(); i++) {
                Part bp = mp.getBodyPart(i);
                if (bp.isMimeType("text/plain")) {
                    return checkMimeType(bp);
                }
            }
        }
        return null;
    }

    @Override
    public String formatBody(ForumThread forumThread) {
        StringBuilder content = new StringBuilder(DELIMITER);
        content.append("\n\n");
        int nesting = 0;
        ListIterator<ForumPost> posts = forumThread.getPosts().listIterator(forumThread.getPostCount());
        while (posts.hasPrevious()) {
            ForumPost forumPost = posts.previous();
            String before = StringUtils.repeat('>', nesting) + (nesting > 0 ? ' ' : "");
            content.append(before).append(forumPost.getPostedBy().getFullName()).append(":\n");
            content.append(before).append(forumPost.getContent()).append('\n');
            content.append(before).append('\n');
            nesting++;
        }
        content.append("\n\n");
        return content.toString();
    }
}
