package se.su.dsv.scipro.group;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Data
@EqualsAndHashCode(of = "id", callSuper = false)
@Entity
@Table(name = "project_group")
@Cacheable(true)
@SuppressWarnings("PMD.UnusedPrivateField")
public class Group extends DomainObject {

    public static final int STRING_MAX_LENGTH = 255;

    @Id
    @GeneratedValue
    private Long id;

    @Column(length = STRING_MAX_LENGTH)
    @Basic(optional = false)
    private String title;

    @Basic(optional = true)
    private String description;

    //Creator, should be a supervisor
    @ManyToOne(optional = false)
    @JoinColumn(name = "user_id")
    private User user;

    @ManyToMany
    @JoinTable(name = "project_group_project", inverseJoinColumns = @JoinColumn(name = "project_id"))
    private Set<Project> projects = new HashSet<>();

    private boolean active = true;

    public boolean isAuthor(final User user) {
        for (Project project : projects) {
            if (project.isParticipant(user)) {
                return true;
            }
        }
        return false;
    }

    public List<Member> getMembers() {
        List<Member> members = new ArrayList<>();
        for (Project project : projects) {
            members.addAll(project.getMembers());
        }
        return members;
    }

    public Set<Project> getProjects() {
        return projects;
    }

    public String getTitle() {
        return title;
    }
}