package se.su.dsv.scipro.mail;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.Predicate;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class MailEventServiceImpl extends AbstractServiceImpl<MailEvent,Long> implements MailEventService {
    @Inject
    public MailEventServiceImpl(Provider<EntityManager> em) {
        super(em, MailEvent.class, QMailEvent.mailEvent);
    }

    @Override
    public Iterable<MailEvent> findUnsent() {
        return findAll(QMailEvent.mailEvent.sent.isFalse());
    }

    @Override
    public Iterable<MailEvent> findAll(Filter filter, Pageable pageable) {
        return findAll(toPredicate(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(toPredicate(filter));
    }

    private Predicate toPredicate(Filter filter) {
        BooleanBuilder predicate = new BooleanBuilder();
        if (filter.getUser() != null) {
            predicate.and(QMailEvent.mailEvent.recipients.contains(filter.getUser()));
        }
        if (filter.getSubject() != null && !filter.getSubject().isEmpty()) {
            predicate.and(QMailEvent.mailEvent.subject.contains(filter.getSubject()));
        }
        if (filter.getSentAfter() != null) {
            predicate.and(QMailEvent.mailEvent.dateCreated.after(filter.getSentAfter()));
        }
        if (filter.getSentBefore() != null) {
            predicate.and(QMailEvent.mailEvent.dateCreated.before(filter.getSentBefore()));
        }
        return predicate;
    }
}
