package se.su.dsv.scipro.match;

import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.ProjectType;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.*;

@Entity
@Cacheable(true)
@Table(name="ApplicationPeriod")
public class ApplicationPeriod extends DomainObject {

    @Id
    @GeneratedValue
    private Long id;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "applicationPeriod", cascade=CascadeType.ALL, orphanRemoval=true)
    private Set<ApplicationPeriodProjectType> projectTypes = new HashSet<>();

    @NotNull
    private String name;

    @NotNull
    private Date startDate = new Date();

    @NotNull
    private Date endDate = new Date();
    
    @NotNull
    private Date courseStartDate = new Date();

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "applicationPeriod", cascade=CascadeType.ALL, orphanRemoval=true)
    private Set<ApplicationPeriodExemption> applicationPeriodExemptions = new HashSet<>();

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "applicationPeriod", cascade=CascadeType.ALL, orphanRemoval=true)
    private Set<Target> targets = new HashSet<>();

    public ApplicationPeriod() {
    }

    public ApplicationPeriod(String name) {
        this.name = name;
    }

    public Set<ApplicationPeriodExemption> getApplicationPeriodExemptions() {
        return Collections.unmodifiableSet(applicationPeriodExemptions);
    }

    public void setApplicationPeriodExemptions(final Set<ApplicationPeriodExemption> applicationPeriodExemptions) {
        this.applicationPeriodExemptions = new HashSet<>(applicationPeriodExemptions);
    }

    public void addApplicationPeriodExemption(final ApplicationPeriodExemption exemption){
        applicationPeriodExemptions.add(exemption);
    }

    public void removeApplicationPeriodExemption(final ApplicationPeriodExemption exemption){
        applicationPeriodExemptions.remove(exemption);
    }

    public int noOfApplicationPeriodExemptions(){
        return applicationPeriodExemptions.size();
    }

    public boolean hasApplicationPeriodExemptions(){
        return !applicationPeriodExemptions.isEmpty();
    }

    public Set<Target> getTargets() {
        return Collections.unmodifiableSet(targets);
    }

    @Override
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Date getStartDate() {
        return new Date(startDate.getTime());
    }

    public void setStartDate(Date startDate) {
        this.startDate = new Date(startDate.getTime());
    }

    public Date getEndDate() {
        return new Date(endDate.getTime());
    }

    public void setEndDate(Date endDate) {
        this.endDate = new Date(endDate.getTime());
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

	public void setCourseStartDate(Date courseStartDate) {
		this.courseStartDate = new Date(courseStartDate.getTime());
	}

	public Date getCourseStartDate() {
		return new Date(courseStartDate.getTime());
	}
	
	@Override
	public String toString() {
		return name;
	}

    public boolean isOpen() {
        Date now = new Date();
        return startDate.before(now) && endDate.after(now);
    }

    public Set<ApplicationPeriodProjectType> getApplicationPeriodProjectTypes() {
        return projectTypes;
    }

    public Set<ProjectType> getProjectTypes() {
        Set<ProjectType> answerSet = new HashSet<>();
        for (ApplicationPeriodProjectType appt : getApplicationPeriodProjectTypes()){
            answerSet.add(appt.getProjectType());
        }
        return Collections.unmodifiableSet(answerSet);
    }

    public void setProjectTypes(Iterable<ProjectType> projectTypes) {
        this.projectTypes.clear();
        for (ProjectType pt : projectTypes){
            this.projectTypes.add(new ApplicationPeriodProjectType(this, pt));
        }
    }

    public void addProjectType(ProjectType projectType) {
        this.projectTypes.add(new ApplicationPeriodProjectType(this, projectType));
    }

    public void removeProjectType(ProjectType projectType) {
        Iterator<ApplicationPeriodProjectType> iterator = this.projectTypes.iterator();
        while (iterator.hasNext()) {
            ApplicationPeriodProjectType next = iterator.next();
            if (next.getProjectType().equals(projectType)) {
                iterator.remove();
            }
        }
    }
}