package se.su.dsv.scipro.match;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.IdeaEvent;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;

import javax.inject.Inject;

class ProjectStartNotifier {
    private final NotificationController notificationController;

    @Inject
    ProjectStartNotifier(final EventBus eventBus,
                         final NotificationController notificationController) {
        eventBus.register(this);
        this.notificationController = notificationController;
    }

    @Subscribe
    public void projectStarted(final ProjectStartedEvent projectStartedEvent) {
        Idea idea = projectStartedEvent.getIdea();
        NotificationSource source = source(idea.getExport().getReason());
        notificationController.notifyProject(idea.getProject(), ProjectEvent.Event.EXPORTED_SUCCESS, source);
    }

    @Subscribe
    public void projectStartDenied(final ProjectStartDeniedEvent projectStartDeniedEvent) {
        Idea idea = projectStartDeniedEvent.getIdea();
        NotificationSource source = source(idea.getExport().getReason());
        notificationController.notifyIdea(idea, IdeaEvent.Event.EXPORTED_FAIL, source);
    }

    private NotificationSource source(final String reason) {
        NotificationSource notificationSource = new NotificationSource();
        notificationSource.setMessage(reason);
        return notificationSource;
    }
}
