package se.su.dsv.scipro.milestones.service.impl;

import se.su.dsv.scipro.milestones.MilestoneActivityTemplateRepository;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.dataobjects.QMilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.Collection;
import java.util.Date;
import java.util.List;

public class MilestoneActivityTemplateServiceImpl implements MilestoneActivityTemplateService {

    private final MilestoneActivityTemplateRepository milestoneActivityTemplateRepository;

    @Inject
    public MilestoneActivityTemplateServiceImpl(final MilestoneActivityTemplateRepository milestoneActivityTemplateRepository) {
        this.milestoneActivityTemplateRepository = milestoneActivityTemplateRepository;
    }

    @Override
    public List<MilestoneActivityTemplate> findAll() {
        return milestoneActivityTemplateRepository.findAll();
    }

    @Override
    public List<MilestoneActivityTemplate> getActivities(MilestoneActivityTemplate.Type type, ProjectType projectType, Date projectStartDate) {
        return milestoneActivityTemplateRepository.findActivities(type, projectType, projectStartDate);
    }

    @Override
    public List<MilestoneActivityTemplate> getAllActivities(MilestoneActivityTemplate.Type type, Collection<ProjectType> projectTypes) {
        return milestoneActivityTemplateRepository.findAllActivitiesByProjectType(type, projectTypes);
    }

    @Override
    public List<MilestoneActivityTemplate> findByMileStonePhase(MilestonePhaseTemplate milestonePhaseTemplate) {
        return milestoneActivityTemplateRepository.findByMilestonePhase(milestonePhaseTemplate);
    }

    @Override
    public List<MilestoneActivityTemplate> findActiveByPhase(MilestonePhaseTemplate milestonePhaseTemplate, Project project) {
        return milestoneActivityTemplateRepository.findActiveByPhase(milestonePhaseTemplate, project.getProjectType(), project.getDateCreated());
    }

    @Override
    public List<MilestoneActivityTemplate> getActivitiesInOrder(MilestoneActivityTemplate.Type type, Project project) {
        return milestoneActivityTemplateRepository.findAllActivitiesInOrder(type, project.getProjectType(), project.getDateCreated());
    }

    @Override
    public Long countActivities(MilestoneActivityTemplate.Type type, ProjectType projectType, Date projectStartDate) {
        return milestoneActivityTemplateRepository.countActivities(type, projectType, projectStartDate);
    }

    @Override
    public MilestoneActivityTemplate save(MilestoneActivityTemplate activity, MilestonePhaseTemplate milestonePhaseTemplate) {
        if ((activity.getId() == null || activity.getId() == 0) || !activity.getMilestonePhaseTemplate().equals(milestonePhaseTemplate)) {
            activity.setSortOrder(SciProUtilities.safeLongToInt(milestoneActivityTemplateRepository.countByMilestonePhase(activity.getMilestonePhaseTemplate())));
        }
        return milestoneActivityTemplateRepository.save(activity);
    }

    @Override
    public long countActivities(ProjectType projectType, Date projectStartDate) {
        return milestoneActivityTemplateRepository.countActivities(projectType, projectStartDate);
    }

    @Override
    public MilestoneActivityTemplate findByEvent(String eventName) {
        return milestoneActivityTemplateRepository.findOne(QMilestoneActivityTemplate.milestoneActivityTemplate.activatedBy.name.eq(eventName));
    }

    @Override
    public void moveUp(MilestoneActivityTemplate activity) {
        int current = activity.getSortOrder();
        MilestoneActivityTemplate downActivity = milestoneActivityTemplateRepository.findBySortOrderAndMilestonePhaseTemplate(current - 1, activity.getMilestonePhaseTemplate());
        if (downActivity != null) {
            downActivity.setSortOrder(current);
            activity.setSortOrder(current - 1);
            milestoneActivityTemplateRepository.save(activity);
            milestoneActivityTemplateRepository.save(downActivity);
        }
    }

    @Override
    public void moveDown(MilestoneActivityTemplate activity) {
        int current = activity.getSortOrder();
        MilestoneActivityTemplate upActivity = milestoneActivityTemplateRepository.findBySortOrderAndMilestonePhaseTemplate(current + 1, activity.getMilestonePhaseTemplate());
        if (upActivity != null) {
            upActivity.setSortOrder(current);
            activity.setSortOrder(current + 1);
            milestoneActivityTemplateRepository.save(activity);
            milestoneActivityTemplateRepository.save(upActivity);
        }
    }

}
