package se.su.dsv.scipro.nonworkperiod;

import com.google.inject.persist.Transactional;
import com.mysema.query.types.expr.BooleanExpression;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;
import java.util.Iterator;

public class NonWorkDayPeriodServiceImpl extends AbstractServiceImpl<NonWorkDayPeriod,Long> implements NonWorkDayPeriodService {
    @Inject
    public NonWorkDayPeriodServiceImpl(Provider<EntityManager> em) {
        super(em, NonWorkDayPeriod.class, QNonWorkDayPeriod.nonWorkDayPeriod);
    }

    private BooleanExpression startsBefore(final Date currentDate) {
        return QNonWorkDayPeriod.nonWorkDayPeriod.startDate.before(currentDate);
    }

    private BooleanExpression endsAfter(final Date currentDate) {
        return QNonWorkDayPeriod.nonWorkDayPeriod.endDate.after(currentDate);
    }

    private BooleanExpression sameStartDate(final Date date) {
        return QNonWorkDayPeriod.nonWorkDayPeriod.startDate.eq(date);
    }

    private BooleanExpression sameEndDate(final Date date) {
        return QNonWorkDayPeriod.nonWorkDayPeriod.endDate.eq(date);
    }

    @Override
    @Transactional
    public <S extends NonWorkDayPeriod> S save(S nwdp) {
        nwdp.setStartDate(SciProUtilities.setTimeOfDayTo0000(nwdp.getStartDate()));
        nwdp.setEndDate(SciProUtilities.setTimeOfDayTo2359(nwdp.getEndDate()));

        Iterator<NonWorkDayPeriod> iterator = findAll(
                startsBefore(nwdp.getStartDate()).and(endsAfter(nwdp.getStartDate())).or(
                        startsBefore(nwdp.getEndDate()).and(endsAfter(nwdp.getEndDate())).or(
                                sameStartDate(nwdp.getStartDate()).or(sameEndDate(nwdp.getEndDate()))
                        )
                )
        ).iterator();

        if (iterator.hasNext() && !iterator.next().getId().equals(nwdp.getId())) {
            throw new IllegalStateException("Non work day periods can not have overlapping start and end dates");
        } else {
            return super.save(nwdp);
        }
    }

    @Override
    public boolean isNonWorkDay(Date date) {
        QNonWorkDayPeriod period = QNonWorkDayPeriod.nonWorkDayPeriod;

        NonWorkDayPeriod p = findOne(period.startDate.before(date).and(period.endDate.after(date)));
        return p != null; // If a period exists covering the given date it is a non work day
    }
}
