package se.su.dsv.scipro.notifications.dataobject;

import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;

@Entity
public class Notification extends DomainObject {


    public enum Type {
        PROJECT("Project related"), IDEA("Idea related"), FINAL_SEMINAR("Final seminar related"), PEER("Peer related"), MILESTONE("Milestone related"),
        GROUP("Group related"), FORUM("Forum related");

        private String description;

        Type(String description) {
            this.description = description;
        }

        public String getDescription() {
            return description;
        }
    }

    @Id
    @GeneratedValue
    private Long id;

    @Basic(optional = false)
    private boolean unread = true;

    @Basic(optional = false)
    private boolean mailed = false;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "notificationData_id")
    private NotificationEvent notificationEvent;

    @ManyToOne
    private User user;

    public Notification() {

    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public boolean isUnread() {
        return unread;
    }

    public void setUnread(boolean unread) {
        this.unread = unread;
    }

    public NotificationEvent getNotificationEvent() {
        return notificationEvent;
    }

    public void setNotificationEvent(NotificationEvent notificationEvent) {
        this.notificationEvent = notificationEvent;
    }

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public boolean isMailed() {
        return mailed;
    }

    public void setMailed(boolean mailed) {
        this.mailed = mailed;
    }

    public String getTitle() {
        return getNotificationEvent().getTitle();
    }

    public String getSource() {
        return getNotificationEvent().getSource();
    }

    public String getAdditionalSource() {
        return getNotificationEvent().getAdditionalSource();
    }

    public Type getType() {
        return getNotificationEvent().getType();
    }

    public Enum getEvent() {
        return getNotificationEvent().getEvent();
    }

    /**
     * @return the project that is associated with this notification if there
     *         is one, otherwise {@code null} is returned
     */
    public Project getProject() {
        return getNotificationEvent().getProject();
    }

    public User getCausedBy() {
        return getNotificationEvent().getCausedBy();
    }
}
