package se.su.dsv.scipro.notifications.interfaces.impl;

import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarActiveParticipation;
import se.su.dsv.scipro.finalseminar.FinalSeminarOpposition;
import se.su.dsv.scipro.notifications.dataobject.SeminarEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import java.text.MessageFormat;

public class SeminarEventFormatter extends AbstractEventFormatter<SeminarEvent> {

    SeminarEventFormatter(NotificationMailFormatterImpl formatter) {
        super(formatter);
    }

    @Override
    public String format(SeminarEvent event) {
        FinalSeminar seminar = event.getSeminar();
        Project project = seminar.getProject();

        String date = formatter.getDateString(seminar.getStartDate(), DateStyle.DATETIME);
        String room = seminar.getRoom();
        String title = project.getTitle();
        String level = project.getProjectType().getName();
        String authors = formatUserList(project.getProjectParticipants(), new UserConverter<User>() {
            @Override
            public User toUser(User object) {
                return object;
            }
        });
        String supervisor = formatUser(project.getHeadSupervisor());
        String coSupervisors = formatUserList(project.getCoSupervisors(), new UserConverter<User>() {
            @Override
            public User toUser(User object) {
                return object;
            }
        });
        String reviewers = formatUserList(project.getReviewers(), new UserConverter<User>() {
            @Override
            public User toUser(User object) {
                return object;
            }
        });
        String participants = formatUserList(seminar.getActiveParticipations(), new UserConverter<FinalSeminarActiveParticipation>() {
            @Override
            public User toUser(FinalSeminarActiveParticipation object) {
                return object.getUser();
            }
        });
        String oppositions = formatUserList(seminar.getOppositions(), new UserConverter<FinalSeminarOpposition>() {
            @Override
            public User toUser(FinalSeminarOpposition object) {
                return object.getUser();
            }
        });

        String format = getProperty(event.getType().name());
        return MessageFormat.format(format,
                date,
                room,
                title,
                level,
                authors,
                supervisor,
                coSupervisors,
                reviewers,
                participants,
                oppositions,
                formatter.getFinalSeminarDeadline(seminar),
                formatter.getFinalSeminarUploadedOnString(seminar));
    }
}
