package se.su.dsv.scipro.notifications.settings.service;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryConfiguration;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryMethod;
import se.su.dsv.scipro.notifications.settings.entities.QDeliveryConfiguration;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

import static com.mysema.query.support.Expressions.allOf;

public class DeliveryConfigurationServiceImpl extends AbstractServiceImpl<DeliveryConfiguration, Long> implements DeliveryConfigurationService {

    private final UserProfileService profileService;

    @Inject
    public DeliveryConfigurationServiceImpl(Provider<EntityManager> em, UserProfileService profileService) {
        super(em, DeliveryConfiguration.class, QDeliveryConfiguration.deliveryConfiguration);
        this.profileService = profileService;
    }

    @Override
    @Transactional
    public boolean setDelivery(final Notification.Type type, final Enum event, final DeliveryMethod method, final boolean enabled, final User user) {
        DeliveryConfiguration configuration = getConfiguration(type, event, method, user);
        configuration.setEnabled(enabled);
        configuration = save(configuration);
        return configuration.isEnabled();
    }

    @Override
    public boolean isDelivery(Notification.Type type, Enum event, DeliveryMethod method) {
        return isDelivery(type, event, method, null);
    }

    @Override
    public boolean isDelivery(final Notification.Type type, final Enum event, final DeliveryMethod method, final User user) {
        return getConfiguration(type, event, method, user).isEnabled();
    }

    @Override
    public boolean isMailCompilation(final User user) {
        return profileService.findByUser(user).isMailCompilation();
    }

    private DeliveryConfiguration getConfiguration(final Notification.Type type, final Enum event, final DeliveryMethod method, final User user) {
        QDeliveryConfiguration q = QDeliveryConfiguration.deliveryConfiguration;
        DeliveryConfiguration configuration = null;

        if (user != null) {
            configuration = findOne(allOf(q.type.eq(type), q.event.eq(event.name()), q.method.eq(method), q.user.eq(user)));
        }

        if (configuration == null) {
            configuration = findOne(allOf(q.type.eq(type), q.event.eq(event.name()), q.method.eq(method), q.user.isNull())); // Try system setting
        }

        if (configuration == null) {
            configuration = new DeliveryConfiguration(type, event.name(), method, user); // Still null, create new setting
        }

        return configuration;
    }
}
