package se.su.dsv.scipro.phd;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.util.Collections;
import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

@Entity
@Getter
@EqualsAndHashCode(of = {"project"}, callSuper = false)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class StudyPlan extends DomainObject {
    private static final Comparator<AnnualReview> BY_UPLOAD_DATE_DESC = Collections.reverseOrder(new Comparator<AnnualReview>() {
        @Override
        public int compare(AnnualReview o1, AnnualReview o2) {
            return o1.getDateCreated().compareTo(o2.getDateCreated());
        }
    });

    @Id
    @GeneratedValue
    private Long id;

    @OneToOne(optional = false)
    private Project project;

    @OneToOne(optional = true)
    @Setter
    private FileDescription basicContract;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "studyPlan", orphanRemoval = true)
    @Setter
    private Set<AnnualReview> annualReviews = new TreeSet<>(BY_UPLOAD_DATE_DESC);

    protected StudyPlan() {}

    public StudyPlan(Project project) {
        this.project = project;
    }

    public void addAnnualReview(FileDescription annualReview) {
        annualReviews.add(new AnnualReview(annualReview, this));
    }

    public void removeAnnualReview(AnnualReview annualReview){
        annualReviews.remove(annualReview);
    }
}