package se.su.dsv.scipro.phd;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class StudyPlanServiceImpl extends AbstractServiceImpl<StudyPlan, Long> implements StudyPlanService {
    private final FileDescriptionService fileDescriptionService;

    @Inject
    public StudyPlanServiceImpl(Provider<EntityManager> em, FileDescriptionService fileDescriptionService) {
        super(em, StudyPlan.class, QStudyPlan.studyPlan);
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    @Transactional
    public void addBasicContract(FileUpload fileUpload, StudyPlan studyPlan) {
        FileDescription fileDescription = fileDescriptionService.storeFile(fileUpload);
        studyPlan.setBasicContract(fileDescription);
        save(studyPlan);
    }

    @Override
    @Transactional
    public void addAnnualReview(FileUpload fileUpload, StudyPlan studyPlan) {
        FileDescription fileDescription = fileDescriptionService.storeFile(fileUpload);
        studyPlan.addAnnualReview(fileDescription);
        save(studyPlan);
    }

    @Override
    @Transactional
    public void removeAnnualReview(StudyPlan phd, AnnualReview annualReview) {
        fileDescriptionService.delete(annualReview.getAnnualReview().getId());
        phd.removeAnnualReview(annualReview);
        save(phd);
    }

    @Override
    public StudyPlan findByProject(Project project) {
        return findOne(QStudyPlan.studyPlan.project.eq(project));
    }
}
