package se.su.dsv.scipro.project;

import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.QUser;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class ProjectPeopleStatisticsServiceImpl extends AbstractServiceImpl<Project, Long> implements ProjectPeopleStatisticsService {

    @Inject
    protected ProjectPeopleStatisticsServiceImpl(Provider<EntityManager> em) {
        super(em, Project.class, QProject.project);
    }

    @Override
    public Long peopleStudents(ProjectService.Filter filter) {
        return from(QProject.project)
                .join(QProject.project.projectParticipants, QUser.user)
                .where(QProject.project.projectType.in(filter.getProjectTypes()), QProject.project.projectStatus.eq(ProjectStatus.ACTIVE))
                .singleResult(QUser.user.countDistinct());
    }

    @Override
    public Long peopleHeadSupervisors(ProjectService.Filter filter) {
        return from(QProject.project)
                .join(QProject.project.headSupervisor, QUser.user)
                .where(QProject.project.projectType.in(filter.getProjectTypes()), QProject.project.projectStatus.eq(ProjectStatus.ACTIVE))
                .singleResult(QUser.user.countDistinct());
    }

    @Override
    public Long peopleCoSupervisors(ProjectService.Filter filter) {
        return from(QProject.project)
                .join(QProject.project.coSupervisors, QUser.user)
                .where(QProject.project.projectType.in(filter.getProjectTypes()), QProject.project.projectStatus.eq(ProjectStatus.ACTIVE))
                .singleResult(QUser.user.countDistinct());
    }

    @Override
    public Long peopleReviewers(ProjectService.Filter filter) {
        return from(QProject.project)
                .join(QProject.project.reviewers, QUser.user)
                .where(QProject.project.projectType.in(filter.getProjectTypes()), QProject.project.projectStatus.eq(ProjectStatus.ACTIVE))
                .singleResult(QUser.user.countDistinct());
    }
}
