package se.su.dsv.scipro.project;

import lombok.Data;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.system.*;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.List;

public interface ProjectService extends GenericService<Project, Long>, FilteredService<Project, Long, ProjectService.Filter> {
    Long countAuthors(Filter params);

    List<Project> getProjectsByUser(User user);

    Project findByExternalIdentifier(Long identifier);

    boolean isPartOfProject(Project project, User user);

    boolean isSupervisorOrFollower(Project project, User user);

    Member.Type getMemberType(Project project, User user);

    List<Project> getActiveProjects(User author, ProjectType pc);

    List<Project> findAll(Filter params);

    Project toggleActiveStatus(Project project, boolean state);

    Long countMultiProjectAuthors(Filter params);

    List<User> findMultipleAuthors(Filter filter);

    List<Project> findByUserAndStatus(User user, ProjectStatus status);

    Project complete(Project project);

    @Data
    @SuppressWarnings({"PMD.UnusedPrivateField", "UnusedDeclaration"})
    class Filter implements Serializable {
        private Collection<ProjectType> projectTypes;
        private String titleContains;
        private User supervisor;
        private boolean filterSupervisor = true;
        private Collection<ProjectTeamMemberRoles> roles;
        private User authorUser;
        private Date createdAfter;
        private Date createdBefore;
        private Collection<ProjectStatus> statuses;
        private boolean filterExternal = false;
        private boolean noReviewer = false;
        private Unit unit;
        private Integer redStateTime;
    }
}
