package se.su.dsv.scipro.report;

public abstract class AbstractMaster30GradeCalculator implements GradeCalculator {

    public static final int NUMBER_OF_PROJECT_CRITERIA_REQUIRED_FOR_FX = 12;
    public static final int NUMBER_OF_INDIVIDUAL_CRITERIA_REQUIRED_FOR_FX = 2;
    public static final int POINTS_REQUIRED_FOR_A = 29;
    public static final int POINTS_REQUIRED_FOR_B = 27;
    public static final int POINTS_REQUIRED_FOR_C = 25;
    public static final int POINTS_REQUIRED_FOR_D = 23;

    @Override
    public long getPoints(GradingReport gradingReport) {
        return new Master30Calculation(getGradingCriteria(gradingReport)).calculateTotalPoints();
    }

    @Override
    public GradingReport.Grade getGrade(GradingReport gradingReport) {
        return new Master30Calculation(getGradingCriteria(gradingReport)).getGrade();
    }

    public abstract Iterable<? extends GradingCriterion> getGradingCriteria(GradingReport gradingReport);

    private static class Master30Calculation extends Calculation {

        private final Iterable<? extends GradingCriterion> gradingCriteria;

        public Master30Calculation(Iterable<? extends GradingCriterion> gradingCriteria) {
            this.gradingCriteria = gradingCriteria;
        }

        @Override
        protected GradingReport.Grade convertPointsToGrade() {
            int totalPoints = calculateTotalPoints();

            if (meetsRequirementsForA(totalPoints)) {
                return GradingReport.Grade.A;
            } else if (totalPoints > POINTS_REQUIRED_FOR_B) {
                return GradingReport.Grade.B;
            } else if (totalPoints > POINTS_REQUIRED_FOR_C) {
                return GradingReport.Grade.C;
            } else if (totalPoints > POINTS_REQUIRED_FOR_D) {
                return GradingReport.Grade.D;
            } else {
                return GradingReport.Grade.E;
            }
        }

        private boolean meetsRequirementsForA(int totalPoints) {
            return totalPoints > POINTS_REQUIRED_FOR_A && hasPointsForDeadlineCriterion();
        }

        @Override
        protected int getNumberOfProjectCriteriaRequiredToGetFx() {
            return NUMBER_OF_PROJECT_CRITERIA_REQUIRED_FOR_FX;
        }

        @Override
        protected int getNumberOfIndividualCriteriaRequiredToGetFx() {
            return NUMBER_OF_INDIVIDUAL_CRITERIA_REQUIRED_FOR_FX;
        }

        @Override
        protected Iterable<? extends GradingCriterion> getGradingCriteria() {
            return gradingCriteria;
        }

        private boolean hasPointsForDeadlineCriterion() {
            for (GradingCriterion gradingCriterion : gradingCriteria) {
                if (gradingCriterion.getTitle().equals("Ö4 Deadlines") && gradingCriterion.getPoints() == 1) {
                    return true;
                }
            }
            return false;
        }
    }
}
