package se.su.dsv.scipro.report;

public abstract class Calculation {

    public GradingReport.Grade getGrade() {
        if (!allCriteriaHavePoints()) {
            return null;
        }

        if (passedAllCriteria()) {
            return convertPointsToGrade();
        } else if (fulfilledFxRequirements()) {
            return GradingReport.Grade.FX;
        } else {
            return GradingReport.Grade.F;
        }
    }

    protected boolean fulfilledFxRequirements() {
        return countFxFlaggedProjectCriteriaWithAtLeastOnePoint() >= getNumberOfProjectCriteriaRequiredToGetFx()
                && countFxFlaggedIndividualCriteriaWithAtLeastOnePoint() >= getNumberOfIndividualCriteriaRequiredToGetFx();
    }

    protected abstract GradingReport.Grade convertPointsToGrade();

    protected abstract int getNumberOfProjectCriteriaRequiredToGetFx();

    protected abstract int getNumberOfIndividualCriteriaRequiredToGetFx();

    protected boolean passedAllCriteria() {
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            if (!gradingCriterion.meetsMinimumPointRequirement()) {
                return false;
            }
        }
        return true;
    }

    protected boolean allCriteriaHavePoints() {
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            if (!gradingCriterion.hasPointsAssigned()) {
                return false;
            }
        }
        return true;
    }

    public int calculateTotalPoints() {
        int total = 0;
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            total += gradingCriterion.getPoints() != null ? gradingCriterion.getPoints() : 0;
        }
        return total;
    }

    protected abstract Iterable<? extends GradingCriterion> getGradingCriteria();

    protected int countFxFlaggedProjectCriteriaWithAtLeastOnePoint() {
        int completed = 0;
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            if (gradingCriterion.isFx() && gradingCriterion.getPoints() > 0 && gradingCriterion.isProjectCriterion()) {
                completed++;
            }
        }
        return completed;
    }

    protected int countFxFlaggedIndividualCriteriaWithAtLeastOnePoint() {
        int completed = 0;
        for (GradingCriterion gradingCriterion : getGradingCriteria()) {
            if (gradingCriterion.isFx() && gradingCriterion.getPoints() > 0 && gradingCriterion.isIndividualCriterion()) {
                completed++;
            }
        }
        return completed;
    }
}
