package se.su.dsv.scipro.report;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Getter
@ToString(exclude = {"gradingCriterionPoints"})
@EqualsAndHashCode(callSuper = true, exclude = {"gradingCriterionPoints"})
@Entity
@DiscriminatorColumn(name = "type")
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public abstract class GradingCriterion extends AbstractGradingCriterion {
    public static final int FEEDBACK_LENGTH = 2000;

    @ManyToOne(optional = false)
    private GradingReport gradingReport;

    @OneToMany(mappedBy = "gradingCriterion", orphanRemoval = true, cascade = CascadeType.PERSIST)
    private List<GradingCriterionPoint> gradingCriterionPoints = new ArrayList<>();

    @Basic
    @Setter
    private Integer points;

    @Basic
    @Column(length = FEEDBACK_LENGTH)
    @Setter
    private String feedback;

    protected GradingCriterion() {
        // JPA
    }

    GradingCriterion(GradingReport gradingReport, GradingCriterionTemplate gradingCriterionTemplate) {
        super(gradingCriterionTemplate.getTitle(), gradingCriterionTemplate.getSortOrder(), gradingCriterionTemplate.getPointsRequiredToPass());
        this.gradingReport = gradingReport;
        for (GradingCriterionPointTemplate pointTemplate : gradingCriterionTemplate.getGradingCriterionPointTemplates()) {
            gradingCriterionPoints.add(new GradingCriterionPoint.Builder()
                    .point(pointTemplate.getPoint())
                    .description(pointTemplate.getDescription())
                    .gradingCriterion(this).build());
        }
    }

    public boolean meetsMinimumPointRequirement() {
        return getPoints() >= getPointsRequiredToPass();
    }

    public boolean isAssessed() {
        return points != null && feedback != null && !feedback.isEmpty();
    }

    public boolean hasPointsAssigned() {
        return points != null;
    }

    public int getMaxPoints() {
        return Collections.max(gradingCriterionPoints).getPoint();
    }

    @Override
    public String getDescription() {
        Collections.sort(gradingCriterionPoints);
        String separator = System.getProperty("line.separator");
        StringBuilder desc = new StringBuilder();
        for (GradingCriterionPoint criterionPoint : gradingCriterionPoints) {
            if (criterionPoint.getDescription() != null && !criterionPoint.getDescription().equals("")) {
                desc.append("For ").append(criterionPoint.getPoint().toString()).append(" ")
                        .append((criterionPoint.getPoint() != 1 ? "points" : "point")).append(" - ")
                        .append(criterionPoint.getDescription()).append(separator);
            }
        }
        return desc.toString();
    }

    public GradingReport getGradingReport() {
        return this.gradingReport;
    }

    public Integer getPoints() {
        return this.points;
    }

    public String getFeedback() {
        return this.feedback;
    }
}
