package se.su.dsv.scipro.report;

import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Data
@ToString(exclude = {"gradingCriterionPointTemplates"})
@EqualsAndHashCode(callSuper = true, exclude = {"gradingCriterionPointTemplates"})
@Entity
@DiscriminatorColumn(name = "type", length = GradingCriterionTemplate.LENGTH)
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@Table(name = "grading_criterion_template")
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public abstract class GradingCriterionTemplate extends AbstractGradingCriterion {
    public static final int LENGTH = 64;
    @ManyToOne(optional = false)
    private GradingReportTemplate gradingReportTemplate;

    @OneToMany(mappedBy = "gradingCriterionTemplate", orphanRemoval = true, cascade = CascadeType.ALL)
    private List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();

    protected GradingCriterionTemplate() {

    }

    GradingCriterionTemplate(GradingReportTemplate gradingReportTemplate, int sortOrder, String title, int pointsRequiredToPass, List<GradingCriterionPointTemplate> gradingCriterionPointTemplates) {
        super(title, sortOrder, pointsRequiredToPass);
        this.gradingReportTemplate = gradingReportTemplate;
        this.gradingCriterionPointTemplates = gradingCriterionPointTemplates;
        for (GradingCriterionPointTemplate gradingCriterionPointTemplate : gradingCriterionPointTemplates) {
            gradingCriterionPointTemplate.setGradingCriterionTemplate(this);
        }
    }

    public abstract void accept(GradingCriterionTemplateVisitor visitor);

    public abstract GradingCriterion createFor(GradingReport gradingReport);

    public int getMaxPoints() {
        return Collections.max(gradingCriterionPointTemplates).getPoint();
    }

    @Override
    public String getDescription() {
        Collections.sort(gradingCriterionPointTemplates);
        StringBuilder desc = new StringBuilder();
        String separator = System.getProperty("line.separator");
        for (GradingCriterionPointTemplate criterionPointTemplate : gradingCriterionPointTemplates) {
            if (criterionPointTemplate.getDescription() != null && !criterionPointTemplate.getDescription().equals("")) {
                desc.append(criterionPointTemplate.getPoint().toString()).append(" p - ").append(criterionPointTemplate.getDescription()).append(separator);
            }
        }
        return desc.toString();
    }
}
