package se.su.dsv.scipro.report;

import se.su.dsv.scipro.project.Project;

import java.util.ArrayList;
import java.util.List;

public class ReviewerGradingReportFactory implements GradingCriterionTemplateVisitor {

    private final List<GradingCriterionTemplate> gradingCriterionTemplates = new ArrayList<>();

    public ReviewerGradingReportFactory using(Iterable<GradingCriterionTemplate> criteria) {
        gradingCriterionTemplates.clear();
        for (GradingCriterionTemplate gradingCriterionTemplate : criteria) {
            gradingCriterionTemplate.accept(this);
        }
        return this;
    }

    @Override
    public void visit(ProjectGradingCriterionTemplate criterionTemplate) {
        gradingCriterionTemplates.add(criterionTemplate);
    }

    @Override
    public void visit(IndividualGradingCriterionTemplate criterionTemplate) {
    }

    public ReviewerGradingReport create(Project project) {
        ReviewerGradingReport reviewerGradingReport = new ReviewerGradingReport(project);
        for (GradingCriterionTemplate gradingCriterionTemplate : gradingCriterionTemplates) {
            reviewerGradingReport.addCriterion(gradingCriterionTemplate.createFor(reviewerGradingReport));
        }
        return reviewerGradingReport;
    }
}
