package se.su.dsv.scipro.report;

import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.List;

public class SupervisorGradingReportFactory implements GradingCriterionTemplateVisitor {

    private final List<GradingCriterionTemplate> gradingCriterionTemplates = new ArrayList<>();

    public SupervisorGradingReportFactory using(Iterable<GradingCriterionTemplate> criteria) {
        gradingCriterionTemplates.clear();
        for (GradingCriterionTemplate gradingCriterionTemplate : criteria) {
            gradingCriterionTemplate.accept(this);
        }
        return this;
    }

    @Override
    public void visit(ProjectGradingCriterionTemplate criterionTemplate) {
        gradingCriterionTemplates.add(criterionTemplate);
    }

    @Override
    public void visit(IndividualGradingCriterionTemplate criterionTemplate) {
        gradingCriterionTemplates.add(criterionTemplate);
    }

    public SupervisorGradingReport create(Project project, User student) {
        SupervisorGradingReport supervisorGradingReport = new SupervisorGradingReport(project, student);
        for (GradingCriterionTemplate gradingCriterionTemplate : gradingCriterionTemplates) {
            supervisorGradingReport.addCriterion(gradingCriterionTemplate.createFor(supervisorGradingReport));
        }
        return supervisorGradingReport;
    }
}

