package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import scala.Option;
import scala.runtime.AbstractFunction0;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.dataobjects.*;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.List;
import java.util.Set;

public class ReviewerInteractionServiceImpl implements ReviewerInteractionService {

    private final ReviewerThreadRepository reviewerThreadRepository;
    private final BasicForumService forumService;
    private final EventBus eventBus;

    @Inject
    public ReviewerInteractionServiceImpl(ReviewerThreadRepository reviewerThreadRepository, BasicForumService forumService, final EventBus eventBus) {
        this.reviewerThreadRepository = reviewerThreadRepository;
        this.forumService = forumService;
        this.eventBus = eventBus;
    }

    @Override
    public ReviewerThread getReviewerThread(final Project project) {
        Option<ReviewerThread> optReviewerThread = reviewerThreadRepository.findByProject(project);
        return optReviewerThread.getOrElse(new AbstractFunction0<ReviewerThread>() {
            @Override
            public ReviewerThread apply() {
                return createNew(project);
            }
        });
    }

    private ReviewerThread createNew(final Project project) {
        ForumThread forumThread = forumService.createThread("");
        ReviewerThread reviewerThread = new ReviewerThread();
        reviewerThread.setProject(project);
        reviewerThread.setForumThread(forumThread);
        return reviewerThreadRepository.save(reviewerThread);
    }

    @Override
    public List<ForumPost> getPosts(final Project project) {
        ReviewerThread reviewerThread = getReviewerThread(project);
        return forumService.getPosts(reviewerThread.getForumThread());
    }

    @Override
    public ForumPost reply(final Project project, final User user, final String content, final Set<FileDescription> attachments) {
        ReviewerThread reviewerThread = getReviewerThread(project);
        ForumPost reply = forumService.createReply(reviewerThread.getForumThread(), user, content, attachments);
        eventBus.post(new NewReviewerInteractionMessage(project, reply));
        return reply;
    }

    @Override
    public void markAllRead(User user, Project project) {
        for (ForumPost post : getPosts(project)) {
            forumService.setRead(user, post, true);
        }
    }

}
