package se.su.dsv.scipro.security.auth.roles;

import java.util.Collection;
import java.util.EnumSet;

public enum Roles implements IRole {
	AUTHOR {
        @Override
        Collection<Roles> authorizedFor() { return EnumSet.of(AUTHOR); }
    },
    SUPERVISOR {
        @Override
        Collection<Roles> authorizedFor() {
            return EnumSet.of(AUTHOR, SUPERVISOR);
        }
    },
    REVIEWER {
        @Override
        Collection<Roles> authorizedFor() {
            return EnumSet.of(REVIEWER);
        }
    },
    EXAMINER {
        @Override
        Collection<Roles> authorizedFor() {
            return EnumSet.of(AUTHOR, SUPERVISOR, EXAMINER);
        }
    },
    ADMIN {
        @Override
        Collection<Roles> authorizedFor() {
            return EnumSet.of(AUTHOR, SUPERVISOR, ADMIN);
        }
    },
    SYSADMIN {
        @Override
        Collection<Roles> authorizedFor() {
            return EnumSet.allOf(Roles.class);
        }
    },
    BETA {
        @Override
        Collection<Roles> authorizedFor() { return EnumSet.of(BETA); }
    };

    @Override
    public boolean isActualRole(final Roles role) {
        return role == this;
    }

    @Override
    public boolean authorizedForRole(final Roles role) {
        return authorizedFor().contains(role);
    }

    abstract Collection<Roles> authorizedFor();
}
