package se.su.dsv.scipro.settings.dataobjects;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;

@Data
@EqualsAndHashCode(of = "user", callSuper = false)
@Entity
@Table(name = "user_profile")
@SuppressWarnings("PMD.UnusedPrivateField")
public class UserProfile extends DomainObject {

    public enum Picture { NONE, FILE, GRAVATAR }

    @Id
    @GeneratedValue
    private Long id;

    @OneToOne(optional = false)
    private User user;

    @Basic(optional = true)
    private String skypeId;

    @Basic(optional = true)
    private String phoneNumber;

    @Basic(optional = true)
    private String otherInfo;

    @Basic(optional = false)
    private boolean mailCompilation = false;

    @Basic(optional = false)
    private boolean expandChecklistOverview = true;

    @ElementCollection
    @Enumerated(EnumType.STRING)
    private Collection<ProjectStatus> defaultProjectStatusFilter = EnumSet.of(ProjectStatus.ACTIVE);

    @ElementCollection
    @Enumerated(EnumType.STRING)
    private Collection<ProjectTeamMemberRoles> defaultProjectTeamMemberRolesFilter = EnumSet.allOf(ProjectTeamMemberRoles.class);

    @Basic(optional = false)
    private boolean defaultSupervisorFilter = true;

    @ManyToMany
    private Collection<ProjectType> defaultProjectTypeFilter = new ArrayList<>();

    @Basic(optional = false)
    @Enumerated(EnumType.STRING)
    private Picture picture = Picture.NONE;

    @ManyToOne
    private FileDescription profilePicture;

    @Basic
    @Enumerated(EnumType.STRING)
    private Roles selectedRole;

    @Basic(optional = false)
    private boolean receiveForumMail = false;
}
