package se.su.dsv.scipro.system;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

public class UserFacadeImpl implements UserFacade {

    private static final Logger LOGGER = LoggerFactory.getLogger(UserFacadeImpl.class);
    private final UserService userService;
    private final UserNameService userNameService;
    private final HashService hashService;

    @Inject
    public UserFacadeImpl(final UserService userService, final UserNameService userNameService, final HashService hashService ) {
        this.userService = userService;
        this.userNameService = userNameService;
        this.hashService = hashService;
    }

    @Override
    public User createUser(User user) {
        Username username = new Username();
        username.setUsername(user.getEmailAddress());
        username.setUser(user);
        userService.save(user);
        userNameService.save(username);
        return user;
    }

    @Override
    public Pair<User, String> attemptCreateUser(String firstName, String lastName, String email) {
        if (StringUtils.isBlank(firstName) || StringUtils.isBlank(lastName) || StringUtils.isBlank(email)){
            return new Pair<>(null, "values missing");
        } else if (userNameService.findByUsername(email)!=null || userService.findByEmailAddress(email) !=null){
            return new Pair<>(null, "user with same username/email already exists");
        } else {
            try {
                InternetAddress internetAddress = new InternetAddress(email, true);
                internetAddress.validate();
            } catch (AddressException e) {
                String message = "invalid email";
                LOGGER.warn(message + ": " + email, e);
                return new Pair<>(null, message);
            }
            User user = new User();
            user.setFirstName(firstName);
            user.setLastName(lastName);
            user.setEmailAddress(email);
            user = createUser(user);
            Hash hash = hashService.createForUser(user);
            return new Pair<>(user, hash.getHash());
        }
    }
}
