package se.su.dsv.scipro.system;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.Predicate;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

import static com.mysema.query.support.Expressions.anyOf;

public class UserServiceImpl extends AbstractServiceImpl<User,Long> implements UserService {

    @Inject
    public UserServiceImpl(Provider<EntityManager> em) {
        super(em, User.class, QUser.user);
    }

    @Override
    public User findByEmailAddress(String emailAddress) {
        return findOne(QUser.user.emailAddress.eq(emailAddress));
    }

    @Override
    public Iterable<User> findAll(Filter filter, Pageable pageable) {
        return findAll(fromParams(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(fromParams(filter));
    }

    @Override
    public User findByUsername(String username) {
        return findOne(QUser.user.usernames.any().username.eq(username));
    }

    @Override
    public User findByExternalIdentifier(Long externalIdentifier) {
        return findOne(QUser.user.identifier.eq(externalIdentifier));
    }

    @Override
    public List<User> getAutoCompleteUsers(String term, int maxResults) {
        QUser user = QUser.user;
        return findAll(anyOf(
                user.emailAddress.contains(term),
                user.firstName.concat(" ").concat(user.lastName).contains(term),
                user.in(new JPASubQuery().from(QUsername.username1)
                        .where(QUsername.username1.username.contains(term))
                        .list(QUsername.username1.user)
                )
        ), new PageRequest(0, maxResults));
    }

    private static Predicate fromParams(Filter filter) {
        BooleanBuilder booleanBuilder = new BooleanBuilder();
        if (filter.isInternalUsersOnly()) {
            booleanBuilder.and(QUser.user.identifier.isNull());
        }
        if (!filter.getRoles().isEmpty()) {
            booleanBuilder.and(QUser.user.roles.any().in(filter.getRoles()));
        }
        else {
            booleanBuilder.and(QUser.user.roles.size().eq(0));
        }
        if (filter.getName() != null && !filter.getName().isEmpty()) {
            booleanBuilder.and(anyOf(
                    QUser.user.firstName.concat(" ").concat(QUser.user.lastName).containsIgnoreCase(filter.getName()),
                    QUser.user.emailAddress.containsIgnoreCase(filter.getName())
            ));
        }
        return booleanBuilder;
    }
}
