package se.su.dsv.scipro.workerthreads;

import com.google.common.annotations.VisibleForTesting;

import java.util.Calendar;
import java.util.concurrent.TimeUnit;

class DailySchedule extends Schedule {
    private static final long MINUTES_PER_HOUR = TimeUnit.HOURS.toMinutes(1);
    private static final long MINUTES_PER_DAY = TimeUnit.DAYS.toMinutes(1);

    private final int hour;
    private final int minute;

    DailySchedule(final int hour, final int minute) {
        this.hour = hour;
        this.minute = minute;
    }

    @Override
    public long getDelay() {
        return minutesUntil(hour, minute);
    }

    @Override
    public long getPeriod() {
        return MINUTES_PER_DAY;
    }

    @Override
    public TimeUnit getTimeUnit() {
        return TimeUnit.MINUTES;
    }

    private static long minutesUntil(int targetHour, int targetMinute) {
        Calendar now = Calendar.getInstance();
        int currentHour = now.get(Calendar.HOUR_OF_DAY);
        int currentMinute = now.get(Calendar.MINUTE);

        return minutesUntil(currentHour, currentMinute, targetHour, targetMinute);
    }

    @VisibleForTesting
    static long minutesUntil(int currentHour, int currentMinute, int targetHour, int targetMinute) {
        if (currentHour > targetHour || (currentHour == targetHour && currentMinute > targetMinute)) {
            return MINUTES_PER_DAY - minutesUntil(targetHour, targetMinute, currentHour, currentMinute);
        }

        int hx = targetHour - currentHour;
        int mx = targetMinute - currentMinute;
        return hx * MINUTES_PER_HOUR + mx;
    }
}
