package se.su.dsv.scipro.workerthreads;

import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodService;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.Collections;
import java.util.HashSet;

@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ManualMatchRemindWorker extends AbstractWorker {

    private final ApplicationPeriodService applicationPeriodService;
    private final MailEventService mailService;
    private final GeneralSystemSettingsService generalSystemSettingsService;
    private final IdeaService ideaService;

    @Inject
    public ManualMatchRemindWorker(final ApplicationPeriodService applicationPeriodService,
                                   final MailEventService mailService,
                                   final GeneralSystemSettingsService generalSystemSettingsService,
                                   final IdeaService ideaService) {
        this.applicationPeriodService = applicationPeriodService;
        this.mailService = mailService;
        this.generalSystemSettingsService = generalSystemSettingsService;
        this.ideaService = ideaService;
    }

    @Override
    protected void doWork() {

        String matchResponsibleMail = generalSystemSettingsService.getGeneralSystemSettingsInstance().getMatchResponsibleMail();

        if (matchResponsibleMail==null || matchResponsibleMail.isEmpty()){
            return;
        }

        for (ApplicationPeriod applicationPeriod : applicationPeriodService.findAll()){
            if (SciProUtilities.isToday(applicationPeriod.getEndDate())){
                MailEvent mailEvent = new MailEvent();
                mailEvent.setNonUserRecipients(new HashSet<>(Collections.singletonList(matchResponsibleMail)));
                mailEvent.setSubject("Manual matching required for application period " + applicationPeriod.getName());

                StringBuilder sb = new StringBuilder();

                sb.append("Application period: ").append(applicationPeriod.getName()).append("\n");
                sb.append("Start date: ").append(applicationPeriod.getStartDate()).append("\n");
                sb.append("End date: ").append(applicationPeriod.getEndDate()).append("\n");
                sb.append("Course start date: ").append(applicationPeriod.getCourseStartDate()).append("\n");

                for (ProjectType projectType : applicationPeriod.getProjectTypes()){

                    IdeaService.Filter params = new IdeaService.Filter();
                    params.setLevels(Collections.singletonList(projectType));
                    params.setStatuses(Collections.singletonList(Idea.Status.UNMATCHED));
                    params.setTypes(Collections.singletonList(Idea.Type.STUDENT));

                    sb.append(projectType.getName()).append(" ideas to match: ").append(ideaService.countByApplicationPeriod(applicationPeriod, params)).append("\n");
                }

                sb.append("Total no of ideas in application period: ").append(ideaService.countByApplicationPeriod(applicationPeriod, new IdeaService.Filter())).append("\n");

                mailEvent.setMessageBody(sb.toString());
                mailService.save(mailEvent);

            }
        }
    }
}