package se.su.dsv.scipro.workerthreads;

import org.joda.time.DateTime;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;

import javax.inject.Inject;
import java.util.Date;

public class ThesisUploadDeadlineWorker extends AbstractWorker {

    @Inject
    FinalSeminarService finalSeminarService;

    @Override
    protected void doWork() {
        this.beginTransaction();

        Date now = new Date();

        /**
         * Feature should not look at seminars held before this date
         */
        DateTime validFrom = new DateTime();
        validFrom = validFrom.withDate(2013, 8, 8);

        try {
            FinalSeminarService.Filter params = new FinalSeminarService.Filter();
            if (now.after(validFrom.toDate())) {
                params.setFromDate(now);
            } else {
                params.setFromDate(validFrom.toDate());
            }
            params.setExempted(false);
            params.setLazyDeleted(false);

            Iterable<FinalSeminar> it = finalSeminarService.findAll(params);

            for (FinalSeminar fs : it) {
                Date seminarUploadDeadline = finalSeminarService.thesisUploadDeadline(fs);
                if (isLate(fs, now, seminarUploadDeadline)) {
                    notify(fs);
                }
            }
            commitTransaction();
        } catch (Exception e) {
            this.rollbackTransaction();
            this.setSuccessfulWorker(false);
            throw new RuntimeException(e);
        }
    }

    private boolean isLate(final FinalSeminar finalSeminar, final Date now, final Date deadline) {
        return deadline.before(now) && (finalSeminar.getDocument() == null || !finalSeminar.getDocument().getDateCreated().before(deadline));
    }

    private void notify(FinalSeminar fs) {
        finalSeminarService.cancel(fs);
    }
}
