package se.su.dsv.scipro.finalseminar

import java.util.Date
import javax.inject.Inject

import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.reviewing.{FinalSeminarApproval, FinalSeminarApprovalService, Status}

sealed trait FinalSeminarCreationReviewerApproval
case object NoReviewer extends FinalSeminarCreationReviewerApproval
case class AwaitingDecision(finalSeminarApproval: FinalSeminarApproval) extends FinalSeminarCreationReviewerApproval
case object RequestNeeded extends FinalSeminarCreationReviewerApproval
case class Denied(finalSeminarApproval: FinalSeminarApproval) extends FinalSeminarCreationReviewerApproval
case object Granted extends FinalSeminarCreationReviewerApproval
case class GrantedByTimeout(finalSeminarApproval: FinalSeminarApproval) extends FinalSeminarCreationReviewerApproval

trait FinalSeminarCreationService {
  def isCreationAllowed(project: Project): FinalSeminarCreationReviewerApproval
}

class FinalSeminarCreationServiceImpl @Inject()(val finalSeminarApprovalService: FinalSeminarApprovalService) extends FinalSeminarCreationService {

  override def isCreationAllowed(project: Project): FinalSeminarCreationReviewerApproval = {
    if (project.getReviewers.isEmpty) NoReviewer
    else {
      finalSeminarApprovalService.findBy(project)
        .map(checkApprovalStatus)
        .getOrElse(RequestNeeded)
    }
  }

  private def checkApprovalStatus(finalSeminarApproval: FinalSeminarApproval): FinalSeminarCreationReviewerApproval = {
    finalSeminarApproval.getCurrentStatus match {
      case Status.APPROVED => Granted
      case Status.REJECTED => Denied(finalSeminarApproval)
      case Status.UNDECIDED => checkDate(finalSeminarApproval)
    }
  }

  private def checkDate(approval: FinalSeminarApproval): FinalSeminarCreationReviewerApproval = {
    val deadline: Date = approval.getCurrentDeadline
    if (deadline.after(new Date)) AwaitingDecision(approval)
    else GrantedByTimeout(approval)
  }
}

