package se.su.dsv.scipro.system

import java.util.{List ⇒ JList}
import javax.inject.Inject

import com.google.inject.persist.Transactional

class FooterLinkServiceImpl @Inject()(repository: FooterLinkRepo) extends FooterLinkService {

  @Transactional
  override def moveUp(link: FooterLink) = {
    swapToOrder(link, link.getOrder - 1)
  }

  @Transactional
  override def moveDown(link: FooterLink) = {
    swapToOrder(link, link.getOrder + 1)
  }

  private def swapToOrder(link: FooterLink, targetOrder: Int) = {
    def swap(other: FooterLink) = {
      val originalOrder = link.getOrder
      link.setOrder(other.getOrder)
      other.setOrder(originalOrder)
      repository.save(link)
      repository.save(other)
    }

    repository.findByColumnAndOrder(link.getFooterColumn, targetOrder) foreach swap
  }

  override def getLinks(column: FooterColumn): JList[FooterLink] = repository.getLinks(column)

  @Transactional
  override def delete(link: FooterLink) = repository.delete(link)

  def applyOrder(link: FooterLink): FooterLink = {
    if (Option(link.getOrder).isEmpty) {
      link.setOrder(repository.getLinks(link.getFooterColumn).size())
    }
    link
  }

  @Transactional
  override def save(link: FooterLink) = {
    repository.save(applyOrder(link))
  }
}