package se.su.dsv.scipro.finalseminar;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Date;
import java.util.TreeSet;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class FinalSeminarRespondentServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private FinalSeminarRespondentServiceImpl finalSeminarRespondentService;

    private FinalSeminar finalSeminar;
    private User user;
    private User user2;

    @Before
    public void setUp() throws Exception {
        user = createUser();
        user2 = createUser();
        finalSeminar = createFinalSeminar(createProject(createProjectType()));
    }

    @Test
    public void creates_new_respondents_for_seminar_if_none_exist() throws Exception {
        assertEquals(0, finalSeminarRespondentService.count());
        finalSeminarRespondentService.findOrCreate(finalSeminar);
        assertEquals(2, finalSeminarRespondentService.count());
    }

    @Test
    public void saves_respondent() throws Exception {
        FinalSeminarRespondent finalSeminarRespondent = new FinalSeminarRespondent(user, finalSeminar);
        FinalSeminarRespondent save = save(finalSeminarRespondent);
        assertNotNull(save.getId());
    }

    private ProjectType createProjectType() {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);
        return save(projectType);
    }

    private Project createProject(ProjectType projectType) {
        TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
        projectParticipants.addAll(Arrays.asList(user, user2));
        Project project = Project.builder()
                .title("title")
                .projectType(projectType)
                .projectParticipants(projectParticipants)
                .headSupervisor(createUser())
                .build();
        return save(project);
    }

    private User createUser() {
        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        return save(user);
    }

    private FinalSeminar createFinalSeminar(Project project) {
        Language english = new Language();
        english.setName("English");
        save(english);
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setRoom("101");
        finalSeminar.setStartDate(new Date());
        finalSeminar.setProject(project);
        finalSeminar.setPresentationLanguage(english);
        finalSeminar.setReportLanguage(english);
        return save(finalSeminar);
    }
}
