package se.su.dsv.scipro.forummail;

import org.junit.Test;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

import static org.junit.Assert.assertEquals;

public class ForumMailServiceImplTest extends ForumMailModuleTest {

    @Inject
    ForumMailService mailService;

    @Test
    public void find_by_message_id() throws Exception {
        String messageID = "abc123";
        ForumMail forumMail = createForumMail(messageID);
        assertEquals(forumMail, mailService.findByMessageID(messageID));
    }

    private ForumMail createForumMail(String messageID) {
        return save(new ForumMailProject(
                createMailEvent(messageID),
                createUser(),
                createForumThread())
        );
    }

    private User createUser() {
        return save(User.builder().firstName("first").lastName("last")
        .emailAddress("first@last.com").build());
    }

    private ProjectThread createForumThread() {
        ForumThread forumThread = new ForumThread();
        forumThread.setSubject("thread subject");
        save(forumThread);
        ProjectType bachelor = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        User user = save(User.builder().firstName("Bob").lastName("Example").emailAddress("bob@example.com").build());
        Project project = save(Project.builder().title("Project").projectType(bachelor).headSupervisor(user).build());
        ProjectThread projectThread = new ProjectThread();
        projectThread.setForumThread(forumThread);
        projectThread.setProject(project);
        return save(projectThread);
    }

    private MailEvent createMailEvent(String messageID) {
        MailEvent mailEvent = new MailEvent();
        mailEvent.setMessageID(messageID);
        mailEvent.setMessageBody("body");
        mailEvent.setSubject("subject");
        return save(mailEvent);
    }
}
