package se.su.dsv.scipro.integration.activityfinalseminar;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.activityplan.Activity;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.finalseminar.FinalSeminar;
import se.su.dsv.scipro.finalseminar.FinalSeminarCancelledEvent;
import se.su.dsv.scipro.finalseminar.FinalSeminarCreatedEvent;
import se.su.dsv.scipro.finalseminar.FinalSeminarDeletedEvent;
import se.su.dsv.scipro.project.Project;

import java.util.Date;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class FinalSeminarActivityHandlerTest {

    @Mock
    private ActivityPlanFacade activityPlanFacade;
    @Mock
    private ActivityFinalSeminarRepository activityFinalSeminarRepository;
    @Mock
    private EventBus eventBus;
    @InjectMocks
    private FinalSeminarActivityHandler  finalSeminarActivityHandler;

    @Test
    public void on_create() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(new Project());
        finalSeminar.setStartDate(new Date());
        FinalSeminarCreatedEvent finalSeminarCreatedEvent = new FinalSeminarCreatedEvent(finalSeminar);

        when(activityFinalSeminarRepository.findByFinalSeminar(finalSeminar))
                .thenReturn(Option.<ActivityFinalSeminar>empty());

        finalSeminarActivityHandler.finalSeminarCreated(finalSeminarCreatedEvent);

        verify(activityPlanFacade).saveActivity(isA(Activity.class));
        verify(activityFinalSeminarRepository).save(isA(ActivityFinalSeminar.class));
    }

    @Test
    public void on_delete() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(new Project());
        FinalSeminarDeletedEvent finalSeminarDeletedEvent = new FinalSeminarDeletedEvent(finalSeminar);

        mockExistingActivity(finalSeminar);

        finalSeminarActivityHandler.finalSeminarDeleted(finalSeminarDeletedEvent);

        verify(activityFinalSeminarRepository).delete(isA(ActivityFinalSeminar.class));
        verify(activityPlanFacade).deleteActivity(isA(Activity.class));
    }

    @Test
    public void on_cancel() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(new Project());
        FinalSeminarCancelledEvent finalSeminarCancelledEvent = new FinalSeminarCancelledEvent(finalSeminar);

        mockExistingActivity(finalSeminar);

        finalSeminarActivityHandler.finalSeminarCancelled(finalSeminarCancelledEvent);

        verify(activityFinalSeminarRepository).delete(isA(ActivityFinalSeminar.class));
        verify(activityPlanFacade).deleteActivity(isA(Activity.class));
    }

    private void mockExistingActivity(final FinalSeminar finalSeminar) {
        Activity activity = new Activity();
        ActivityFinalSeminar finalSeminarActivity = new ActivityFinalSeminar(activity, finalSeminar);
        when(activityFinalSeminarRepository.findByFinalSeminar(finalSeminar))
                .thenReturn(Option.apply(finalSeminarActivity));
    }
}