package se.su.dsv.scipro.mail;

import com.sun.mail.smtp.SMTPMessage;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.slf4j.Logger;

import javax.mail.Address;
import javax.mail.SendFailedException;
import javax.mail.internet.InternetAddress;
import java.util.Arrays;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class FailureTest {
    @Mock
    private Logger logger;
    @Mock
    private SMTPMessage message;
    @Mock
    private SendFailedException exception;
    @InjectMocks
    private Failure failure;

    @Before
    public void setUp() throws Exception {
        final InternetAddress delivered = new InternetAddress("bill.gates@microsoft.com");
        final InternetAddress failed = new InternetAddress("steve.jobs@apple.com");
        final InternetAddress invalid = new InternetAddress("mangledemail@e");
        final Address[] recipients = new Address[]{delivered, failed, invalid};
        when(message.getAllRecipients()).thenReturn(recipients);
        when(exception.getInvalidAddresses()).thenReturn(new Address[]{invalid});
        when(exception.getValidSentAddresses()).thenReturn(new Address[]{delivered});
        when(exception.getValidUnsentAddresses()).thenReturn(new Address[]{failed});
    }

    @Test
    public void logs_all_recipients() throws Exception {
        failure.log(logger);
        verify(logger).info("Failed to deliver the mail to all recipients: " + Arrays.toString(message.getAllRecipients()));
    }

    @Test
    public void logs_delivered_mails() throws Exception {
        failure.log(logger);
        verify(logger).info("Delivered: " + Arrays.toString(exception.getInvalidAddresses()));
    }

    @Test
    public void logs_non_delivered_mails() throws Exception {
        failure.log(logger);
        verify(logger).warn("Could not deliver to: " + Arrays.toString(exception.getValidUnsentAddresses()));
    }

    @Test
    public void logs_invalid_emails() throws Exception {
        failure.log(logger);
        verify(logger).error("Invalid email found: " + Arrays.toString(exception.getInvalidAddresses()));
    }
}