package se.su.dsv.scipro.match;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import static org.junit.Assert.assertEquals;

public class IdeaTest {

    private Match supervisor;
    private Match noSupervisor;
    private Idea idea;

    @Before
    public void setUp() {
        idea = new Idea();
        idea.setProjectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        supervisor = new Match();
        supervisor.setSupervisor(new User());
        noSupervisor = new Match();
    }

    @Test
    public void new_idea_is_unmatched() throws Exception {
        assertEquals(Idea.Status.UNMATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_with_project_is_completed() throws Exception {
        idea.setProject(new Project());

        assertEquals(Idea.Status.COMPLETED, idea.getMatchStatus());
    }

    @Test
    public void idea_with_supervisor_and_confirmed_students_and_application_period_is_matched() throws Exception {
        addSupervisor();
        addStudent();
        addApplicationPeriod();
        assertEquals(Idea.Status.MATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_with_match_without_supervisor_is_unmatched() throws Exception {
        idea.setMatch(noSupervisor);

        assertEquals(Idea.Status.UNMATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_with_partner_that_has_confirmed_is_unmatched() throws Exception {
        addStudent();
        assertEquals(Idea.Status.UNMATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_with_supervisor_without_authors_is_unmatched() throws Exception {
        addSupervisor();
        assertEquals(Idea.Status.UNMATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_no_application_period_is_unmatched() throws Exception {
        addStudent();
        addSupervisor();
        assertEquals(Idea.Status.UNMATCHED, idea.getMatchStatus());
    }

    @Test
    public void idea_without_project_that_is_inactive_has_status_inactive() throws Exception {
        idea.setInactive(true);
        assertEquals(Idea.Status.INACTIVE, idea.getMatchStatus());
    }

    private void addSupervisor() {
        idea.setMatch(supervisor);
    }

    private void addApplicationPeriod() {
        idea.setApplicationPeriod(new ApplicationPeriod());
    }

    private void addStudent() {
        IdeaParticipation ideaParticipation = new IdeaParticipation();
        idea.addIdeaParticipation(ideaParticipation);
    }
}
