package se.su.dsv.scipro.misc;

import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Matchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;

import java.util.Date;

@RunWith(MockitoJUnitRunner.class)
public class DaysServiceImplTest {

    @Mock
    NonWorkDayPeriodService nonWorkDays;
    @InjectMocks
    DaysServiceImpl service;
    private DateTime now;
    private DateTime then;

    @Test
    public void testAfterWeekends() throws Exception {
        // given
        int dayOfMonth = 15;

        now(2013, 5, dayOfMonth);
        then(2013, 5, dayOfMonth + 9);

        // then
        Date ahead = service.workDaysAfter(now(), 7);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    @Test
    public void testAheadWeekends() throws Exception {
        // given
        int dayOfMonth = 26;
        assert dayOfMonth > 7 : "Day of month must be > 7 to prevent rollover to previous month";

        now(2013, 2, dayOfMonth);
        then(2013, 2, dayOfMonth - 7);

        // when
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.isA(Date.class))).thenReturn(false);

        // then
        Date ahead = service.workDaysAhead(now(), 5);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    @Test
    public void testAfterNonWorkDays() throws Exception {
        // given
        now(2013, 5, 15);
        then(2013, 5, 16);

        // then
        Date ahead = service.workDaysAfter(now(), 1);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    @Test
    public void testAheadNonWorkDays() throws Exception {
        // given
        now(2013, 4, 11);
        then(2013, 4, 9);

        // when
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.isA(Date.class))).thenReturn(true, false);

        // then
        Date ahead = service.workDaysAhead(now(), 1);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    @Test
    public void testAheadNonWorkDayWeekend() throws Exception {
        // given
        now(2013, 4, 30);
        then(2013, 4, 26);

        // when
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.isA(Date.class))).thenReturn(false);
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.eq(now.minusDays(1).toDate()))).thenReturn(true);

        // then
        Date ahead = service.workDaysAhead(now(), 1);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    @Test
    public void testAheadOverlappingNonWorkDayWeekend() throws Exception {
        // given
        now(2013, 4, 30);
        then(2013, 4, 26);

        // when
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.isA(Date.class))).thenReturn(true);
        Mockito.when(nonWorkDays.isNonWorkDay(Matchers.eq(now.minusDays(4).toDate()))).thenReturn(false);

        // then
        Date ahead = service.workDaysAhead(now(), 1);

        Assert.assertEquals("Wrong date", then(), ahead);
    }

    private void then(int year, int month, int day) {
        then = new DateTime(year, month, day, 13, 14, 15, 0);
    }

    private Date then() {
        return then.toDate();
    }

    private void now(int year, int month, int day) {
        now = new DateTime(year, month, day, 13, 14, 15, 0);
    }

    private Date now() {
        return now.toDate();
    }
}
