package se.su.dsv.scipro.notifications;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReport;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.report.ReviewerGradingReport;
import se.su.dsv.scipro.report.ReviewerGradingReportSubmittedEvent;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class GradingNotificationsTest {
    @Mock
    private EventBus eventBus;
    @Mock
    private NotificationController notificationController;
    @InjectMocks
    private GradingNotifications gradingNotifications;

    @Test
    public void registers_itself() throws Exception {
        verify(eventBus).register(gradingNotifications);
    }

    @Test
    public void sends_out_notification_when_reviewer_submits_grading_report_to_examiner() throws Exception {
        final Project project = newProject();
        final GradingReport reviewerReport = newReviewerReport(project);
        gradingNotifications.reviewerSubmittedGradingReport(new ReviewerGradingReportSubmittedEvent(reviewerReport));
        verify(notificationController).notifyProject(eq(project), eq(ProjectEvent.Event.REVIEWER_GRADING_REPORT_SUBMITTED), any(NotificationSource.class));
    }

    private Project newProject() {
        return Project.builder().title("Some project").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
    }

    private ReviewerGradingReport newReviewerReport(final Project project) {
        GradingReportTemplate gradingReportTemplate = new GradingReportTemplate(project.getProjectType(), 15);
        return gradingReportTemplate.createReviewerReport(project);
    }
}