package se.su.dsv.scipro.notifications.settings.service;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.PeerEvent;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryMethod;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class DeliveryConfigurationServiceImplTest extends IntegrationTest {

    @Inject
    private DeliveryConfigurationServiceImpl service;

    private Notification.Type type;
    private PeerEvent.Event event;
    private DeliveryMethod method;
    private User user;

    @Before
    public void setUp() throws Exception {
        type = Notification.Type.PEER;
        event = PeerEvent.Event.REQUEST_DELETED;
        method = DeliveryMethod.WEB;
        user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    @Test
    public void testIsDelivery() throws Exception {
        assertTrue(service.isDelivery(type, event, method));
    }

    @Test
    public void testUserFallsBackOnDefault() throws Exception {
        service.setDelivery(type, event, method, false, null);
        assertFalse(service.isDelivery(type, event, method, user));
    }

    @Test
    public void testSetUserDelivery() throws Exception {
        service.setDelivery(type, event, method, false, null);
        service.setDelivery(type, event, method, true, user);
        assertTrue(service.isDelivery(type, event, method, user));
    }
}
