package se.su.dsv.scipro.peer;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeSettings;

import java.util.Date;

import static org.junit.Assert.assertEquals;

public class PeerReviewTest {
    @Test
    public void submitting_review_late_expires_the_review() throws Exception {
        PeerReview peerReview = createLateReview();
        peerReview.submit();

        assertEquals(PeerReview.ReviewStatus.EXPIRED, peerReview.getStatus());
    }

    @Test
    public void submitting_in_time_review_marks_it_as_completed() throws Exception {
        PeerReview inTimeReview = createInTimeReview();
        inTimeReview.submit();

        assertEquals(PeerReview.ReviewStatus.COMPLETED, inTimeReview.getStatus());
    }

    @Test
    public void expiring_a_peer_review_marks_it_as_expired() throws Exception {
        PeerReview peerReview = createPeerReview();
        peerReview.expire();

        assertEquals(PeerReview.ReviewStatus.EXPIRED, peerReview.getStatus());
    }

    private PeerReview createInTimeReview() {
        PeerReview peerReview = createPeerReview();
        peerReview.setComment(StringUtils.repeat("a;", PeerReview.MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS));
        peerReview.setDateCreated(inTimeDate());
        return peerReview;
    }

    private PeerReview createLateReview() {
        PeerReview peerReview = createPeerReview();
        peerReview.setComment(StringUtils.repeat("a;", PeerReview.MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS));
        peerReview.setDateCreated(lateDate());
        return peerReview;
    }

    private PeerReview createPeerReview() {
        PeerReview peerReview = new PeerReview();
        peerReview.setProject(defaultProject());
        peerReview.setPeerRequest(new PeerRequest());
        return peerReview;
    }

    private Project defaultProject() {
        ProjectTypeSettings projectTypeSettings = new ProjectTypeSettings();
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        projectType.setProjectTypeSettings(projectTypeSettings);
        return Project.builder().title("Peer project").projectType(projectType).build();
    }

    private Date inTimeDate() {
        return new Date();
    }

    private Date lateDate() {
        return new DateTime().minusDays(ProjectTypeSettings.DEFAULT_NUM_DAYS_TO_SUBMIT_PEER_REVIEW * 2).toDate();
    }
}
