package se.su.dsv.scipro.phd;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

import static org.hamcrest.CoreMatchers.everyItem;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.junit.Assert.assertEquals;

public class DepartmentServiceServiceImplTest extends IntegrationTest {

    @Inject
    DepartmentServiceServiceImpl departmentServiceService;
    private User anotherUser;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        anotherUser = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    @Test
    public void get_current_returns_all_currently_active_and_future() throws Exception {
        createDepartmentService(user, new Date(), new DateTime().plusDays(2).toDate());
        createDepartmentService(user, new Date(), new DateTime().plusDays(5).toDate());
        createDepartmentService(user, new Date(), new DateTime().minusDays(1).toDate());
        createDepartmentService(anotherUser, new Date(), new DateTime().plusDays(5).toDate());

        List<DepartmentService> serviceList = departmentServiceService.getCurrent(user);
        assertEquals(serviceList.size(), 2);
        assertThat(serviceList, everyItem(hasNotEnded()));
    }

    @Test
    public void get_previous_returns_all_ended() throws Exception {
        createDepartmentService(user, new Date(), new DateTime().minusDays(2).toDate());
        createDepartmentService(anotherUser, new Date(), new DateTime().minusDays(2).toDate());
        createDepartmentService(anotherUser, new Date(), new DateTime().plusDays(2).toDate());

        List<DepartmentService> serviceList = departmentServiceService.getPrevious(anotherUser);
        assertEquals(serviceList.size(), 1);
        assertThat(serviceList, everyItem(hasEnded()));
    }

    private DepartmentService createDepartmentService(User user, Date started, Date ended) {
        DepartmentService departmentService = new DepartmentService(user);
        departmentService.setDateStarted(started);
        departmentService.setDateEnded(ended);
        departmentService.setHours(10);
        return save(departmentService);
    }

    private Matcher<DepartmentService> hasNotEnded() {
        return new TypeSafeMatcher<DepartmentService>() {
            @Override
            protected boolean matchesSafely(DepartmentService item) {
                return item.getDateEnded().after(new Date());
            }

            @Override
            public void describeTo(Description description) {       }
        };
    }

    private Matcher<DepartmentService> hasEnded() {
        return new TypeSafeMatcher<DepartmentService>() {
            @Override
            protected boolean matchesSafely(DepartmentService item) {
                return item.getDateEnded().before(new Date());
            }

            @Override
            public void describeTo(Description description) {       }
        };
    }
}
