package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;
import scala.Option;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.dataobjects.*;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.Collections;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;
import static org.mockito.Matchers.anySetOf;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ReviewerInteractionServiceImplTest {
    @Mock
    private EventBus eventBus;
    @Mock
    private ReviewerThreadRepository reviewerThreadRepository;
    @Mock
    private BasicForumService basicForumService;
    @InjectMocks
    private ReviewerInteractionServiceImpl reviewerInteractionService;

    @Before
    public void setUp() throws Exception {
        when(reviewerThreadRepository.findByProject(isA(Project.class))).thenReturn(Option.<ReviewerThread>empty());
        when(basicForumService.createThread(anyString())).thenReturn(new ForumThread());
        when(basicForumService.createReply(isA(ForumThread.class), isA(User.class), anyString(), anySetOf(FileDescription.class)))
                .thenAnswer(new Answer<ForumPost>() {
                    @Override
                    public ForumPost answer(final InvocationOnMock invocation) throws Throwable {
                        ForumPost forumPost = new ForumPost();
                        forumPost.setPostedBy(invocation.getArgumentAt(1, User.class));
                        return forumPost;
                    }
                });
        when(reviewerThreadRepository.save(isA(ReviewerThread.class))).thenAnswer(new Answer<ReviewerThread>() {
            @Override
            public ReviewerThread answer(final InvocationOnMock invocation) throws Throwable {
                return invocation.getArgumentAt(0, ReviewerThread.class);
            }
        });
    }

    @Test
    public void sends_out_event_on_new_reply() throws Exception {
        final Project project = createProject();
        final User reviewer = createUser();

        reviewerInteractionService.reply(project, reviewer, "Rough draft approvad", Collections.<FileDescription>emptySet());

        ArgumentCaptor<NewReviewerInteractionMessage> captor = ArgumentCaptor.forClass(NewReviewerInteractionMessage.class);
        verify(eventBus).post(captor.capture());

        assertThat(captor.getValue().getProject(), is(project));
    }

    private User createUser() {
        return User.builder().firstName("Bob").lastName("Reviewer").emailAddress("bob@example.com").build();
    }

    private Project createProject() {
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        return Project.builder().title("Project").projectType(bachelor).build();
    }
}