package se.su.dsv.scipro.security.auth;

import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.Password;
import se.su.dsv.scipro.system.PasswordHandler;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.Username;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class LocalAuthenticationTest extends IntegrationTest {
    public static final char[] TEST_PASSWORD = "s3cre7".toCharArray();
    public static final String TEST_USERNAME = "kalle@dsv.su.se";

    @Inject
    private LocalAuthentication localAuthentication;

    @Before
    public void setUp() throws Exception {
        User user = save(User.builder().firstName("Kalle").lastName("Tester").emailAddress("kalle@dsv.su.se").build());
        createUsername(user);
        createPassword(user);
    }

    @Test
    public void authenticate_with_correct_credentials() throws Exception {
        assertTrue(localAuthentication.authenticate(new Credentials(TEST_USERNAME, TEST_PASSWORD)));
    }

    @Test
    public void authenticate_with_wrong_credentials() throws Exception {
        assertFalse(localAuthentication.authenticate(new Credentials(TEST_USERNAME, "wrong".toCharArray())));
    }

    @Test
    public void if_no_user_is_found_then_authentication_fails() throws Exception {
        assertFalse(localAuthentication.authenticate(new Credentials("foobar", "password".toCharArray())));
    }

    private void createPassword(final User user) {
        byte[] salt = PasswordHandler.generateSalt();
        byte[] hash = PasswordHandler.generateHash(TEST_PASSWORD, salt);
        Password password = new Password();
        password.setHash(hash);
        password.setSalt(salt);
        password.setUser(user);
        save(password);
    }

    private void createUsername(final User user) {
        Username username = new Username();
        username.setUser(user);
        username.setUsername(TEST_USERNAME);
        save(username);
    }
}
