package se.su.dsv.scipro.system;

import org.junit.Test;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.List;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

public class ResearchAreaServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private ResearchAreaService researchAreaService;

    @Test
    public void creates_new_research_area() throws Exception {
        Long identifier = 1L;
        ResearchArea researchArea = createResearchArea(identifier, "Some title");

        ResearchArea created = researchAreaService.updateExternalResearchArea(researchArea.getIdentifier(), researchArea.getTitle());
        assertEquals(1, researchAreaService.count());
        assertNotNull(created);
        assertNotNull(created.getId());
        assertEquals(identifier, created.getIdentifier());
    }

    @Test
    public void updates_external_research_area() throws Exception {
        ResearchArea old = createResearchArea(2L, "Some title");
        save(old);

        String title = "New title";

        ResearchArea updated = researchAreaService.updateExternalResearchArea(old.getIdentifier(), title);

        assertEquals(title, updated.getTitle());
        assertEquals(1, researchAreaService.count());
    }

    @Test
    public void find_not_deleted_ordered_by_title() throws Exception {
        ResearchArea area1 = createAndSave(1L, "GHI", false);
        createAndSave(2L, "DEF", true);
        ResearchArea area3 = createAndSave(3L, "ABC", false);

        List<ResearchArea> notDeleted = researchAreaService.findNotDeleted();
        assertThat(notDeleted.size(), is(2));
        assertThat(notDeleted, hasItem(area1));
        assertThat(notDeleted, hasItem(area3));
    }

    private ResearchArea createAndSave(Long identifier, String title, boolean deleted) {
        ResearchArea area = createResearchArea(identifier, title);
        area.setDeleted(deleted);
        return save(area);
    }

    private ResearchArea createResearchArea(Long identifier, String title) {
        ResearchArea researchArea = new ResearchArea();
        researchArea.setIdentifier(identifier);
        researchArea.setTitle(title);
        return researchArea;
    }
}
