package se.su.dsv.scipro.test;

import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.persist.PersistService;
import com.google.inject.persist.jpa.JpaPersistModule;
import org.junit.After;
import org.junit.Before;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

public abstract class GuiceTest {
    @Inject
    private PersistService persistService;
    @Inject
    private Provider<EntityManager> entityManager;

    @Before
    public final void prepareGuice() throws Exception {
        Injector injector = Guice.createInjector(new JpaPersistModule("testPersistenceUnit"), moduleUnderTest());
        injector.injectMembers(this);

        persistService.start();
        EntityTransaction transaction = entityManager.get().getTransaction();
        transaction.begin();
        transaction.setRollbackOnly();
    }

    protected abstract Module moduleUnderTest();

    @After
    public final void shutDownPersistence() throws Exception {
        entityManager.get().getTransaction().rollback();
        persistService.stop();
    }

    protected <T> T save(T entity) {
        EntityManager em = entityManager.get();
        if (em.contains(entity)) {
            return em.merge(entity);
        }
        else {
            em.persist(entity);
            return entity;
        }
    }
}
