package se.su.dsv.scipro.firstmeeting

import java.util.Date

import org.hamcrest.CoreMatchers._
import org.junit.Assert._
import org.junit.Test
import se.su.dsv.scipro.`match`.{ApplicationPeriod, Idea, Match}
import se.su.dsv.scipro.firstmeeting.{FirstMeetingReminderWorker => Worker}
import se.su.dsv.scipro.system.User

import scala.collection.JavaConverters._

class FirstMeetingReminderWorkerTest{

  @Test def should_remind_if_is_same_date() = {
    val today = new Date
    val idea = newIdea(today)

    val needReminding = Worker.findNeedRemindingMethod(_ => today, _ => Set(today))(today)(Set(idea))

    assertThat(needReminding.asJava, hasItem((idea,today)))
  }

  @Test def should_not_remind_if_is_different_date() = {
    val date = new Date(987452359L)
    val idea = newIdea(date)

    val needReminding = Worker.findNeedRemindingMethod(_ => new Date, _ => Set(date))(new Date)(Set(idea))

    assertThat(needReminding.asJava, not(hasItem((idea,new Date(0)))))
  }

  @Test def reminder_should_include_title_of_every_idea() = {
    val idea = newIdea(new Date)

    val reminder = Worker.generateReminder("test-baseURL")(idea.getMatch.getSupervisor, Set((idea,new Date)))
    assertThat(reminder.getMessageBody, containsString(idea.getTitle))
  }

  @Test def generated_reminder_should_be_sent_to_the_supervisor() = {
    val idea = newIdea(new Date)
    val supervisor = idea.getMatch.getSupervisor

    val reminder = Worker.generateReminder("test-baseURL")(supervisor, Set((idea,new Date)))
    assertThat(reminder.getRecipients, hasItem(supervisor))
  }

  def newUser: User = {
    val user = User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build()
    user.setId(234L)
    user
  }

  def newApplicationPeriod(startDate: Date): ApplicationPeriod = {
    val period = new ApplicationPeriod()
    period.setCourseStartDate(startDate)
    period
  }

  def newIdea(startDate: Date): Idea = {
    val idea = new Idea
    idea.setTitle("I hate writing all this stupid manual test data")
    idea.setId(234L)
    idea.setMatch(newMatch(idea))
    idea.setApplicationPeriod(newApplicationPeriod(startDate))
    idea
  }

  def newMatch(idea: Idea): Match = {
    val supervisorMatch = new Match()
    supervisorMatch.setSupervisor(newUser)
    supervisorMatch.setIdea(idea)
    supervisorMatch
  }
}