package se.su.dsv.scipro.notifications

import org.junit.runner.RunWith
import org.junit.{Before, Test}
import org.mockito.Mockito.verify
import org.mockito.runners.MockitoJUnitRunner
import org.mockito.{Matchers, Mock}
import se.su.dsv.scipro.notifications.dataobject.{NotificationSource, ProjectEvent}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.report.{ReviewerGradingReportInitialAssessmentDoneEvent, GradingReportTemplate, SupervisorGradingReportInitialAssessmentDoneEvent}
import se.su.dsv.scipro.system.{DegreeLevel, ProjectType, User}

@RunWith(classOf[MockitoJUnitRunner])
class GradingReportNotificationsTest {
  test =>
  @Mock var notificationController: NotificationController = _

  private var gradingNotifications: GradingReportNotifications = _

  val bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")
  val project = Project.builder().title("My project").projectType(bachelor).build()
  val author = User.builder().firstName("First").lastName("Author").emailAddress("first@example.com").build()
  val gradingReportTemplate = new GradingReportTemplate(bachelor, 15)
  val supervisorGradingReport = gradingReportTemplate.createSupervisorReport(project, author)
  val reviewerGradingReport = gradingReportTemplate.createReviewerReport(project)

  @Before
  def setUp(): Unit = {
    gradingNotifications = new GradingReportNotifications {
      override def notificationController: NotificationController = test.notificationController
    }
  }

  @Test
  def sends_out_notification_when_supervisor_is_done_with_initial_assessment(): Unit = {
    gradingNotifications.supervisorDoneWithInitialAssessment(SupervisorGradingReportInitialAssessmentDoneEvent(supervisorGradingReport))
    verify(notificationController).notifyProject(is(project), is(ProjectEvent.Event.SUPERVISOR_GRADING_INITIAL_ASSESSMENT_DONE), isA[NotificationSource])
  }

  @Test
  def sends_out_notification_when_reviewer_is_done_with_initial_assessment(): Unit = {
    gradingNotifications.reviewerDoneWithInitialAssessment(ReviewerGradingReportInitialAssessmentDoneEvent(reviewerGradingReport))
    verify(notificationController).notifyProject(is(project), is(ProjectEvent.Event.REVIEWER_GRADING_INITIAL_ASSESSMENT_DONE), isA[NotificationSource])
  }

  private def is[A](a: A) = Matchers.eq(a)
  private def isA[A : Manifest]: A = Matchers.isA(implicitly[Manifest[A]].runtimeClass.asInstanceOf[Class[A]])
}
