package se.su.dsv.scipro.integration.daisy;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.io.ExternalExporter;
import se.su.dsv.scipro.io.exceptions.ExternalExportException;
import se.su.dsv.scipro.io.facade.ExporterFacade;
import se.su.dsv.scipro.match.Decision;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaCreationJudge;
import se.su.dsv.scipro.match.IdeaParticipation;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.SortedSet;
import java.util.TreeSet;

class Daisy implements IdeaCreationJudge {

    private static final Logger LOGGER = LoggerFactory.getLogger(Daisy.class);
    private final ExporterFacade exporterFacade;
    private final ExternalExporter externalExporter;

    @Inject
    Daisy(final ExporterFacade exporterFacade, final ExternalExporter externalExporter) {
        this.exporterFacade = exporterFacade;
        this.externalExporter = externalExporter;
    }

    @Override
    public Decision ruling(final Idea idea) {
        Project project = Project.builder()
                .title(idea.getTitle())
                .projectType(idea.getProjectType())
                .headSupervisor(idea.getMatch().getSupervisor())
                .projectParticipants(getAuthors(idea))
                .build();

        try {
            exporterFacade.exportProject(project, project.getHeadSupervisor().getUnit());
            externalExporter.deleteProject(project);
            return Decision.allowed();
        } catch (ExternalExportException e) {
            LOGGER.error(e.getMessage(), e);
            return Decision.denied(e.getMessage());
        }
    }

    private SortedSet<User> getAuthors(final Idea idea) {
        SortedSet<User> authors = new TreeSet<>(new User.ByNameComparator());
        for (IdeaParticipation ideaParticipation : idea.getIdeaParticipations()) {
            authors.add(ideaParticipation.getUser());
        }
        return authors;
    }
}
