package se.su.dsv.scipro.integration.daisy;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.io.ExternalImporter;
import se.su.dsv.scipro.io.exceptions.ExternalImportException;
import se.su.dsv.scipro.system.CurrentUser;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

@Singleton
public class SSOFilter implements Filter {

    private static final Logger LOGGER = LoggerFactory.getLogger(SSOFilter.class);

    private final UserService userService;
    private final ExternalImporter externalImporter;
    private final Provider<CurrentUser> currentUserProvider;

    @Inject
    public SSOFilter(final UserService userService,
                     final ExternalImporter externalImporter,
                     final Provider<CurrentUser> currentUserProvider) {
        this.userService = userService;
        this.externalImporter = externalImporter;
        this.currentUserProvider = currentUserProvider;
    }

    @Override
    public void init(final FilterConfig filterConfig) {
    }

    @Override
    public void destroy() {
    }

    @Override
    public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException {
        if (request instanceof HttpServletRequest) {
            HttpServletRequest httpServletRequest = (HttpServletRequest) request;
            String remoteUser = httpServletRequest.getRemoteUser();

            if (remoteUser != null) {
                User user = getUser(remoteUser);
                CurrentUser currentUser = currentUserProvider.get();
                if (!currentUser.exists()) {
                    currentUser.set(user);
                }
            }
        }
        chain.doFilter(request, response);
    }

    private User getUser(String username) {
        User local = userService.findByUsername(username);
        return local != null ? local : getUserFromDaisy(username);
    }

    private User getUserFromDaisy(final String username) {
        LOGGER.debug("Attempting external import of user: " + username);
        try {
            return externalImporter.importUser(username);
        } catch (final ExternalImportException eie) {
            LOGGER.info("User is authenticated via Shibboleth but does not exist in Daisy: {}", username, eie);
            return null;
        }
    }
}
