package se.su.dsv.scipro.io.impl;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.io.dto.Person;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.match.ProgramService;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.*;

import javax.ws.rs.core.Response;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ImporterTransactionsImplTest {

    public static final String SOME_FIRST_NAME = "Kalle";
    public static final String SOME_LAST_NAME = "Testare";
    public static final String SOME_EMAIL = "kalle@dsv.su.se";
    public static final long SOME_ID = 1L;

    @Mock
    private DaisyAPI daisyAPI;
    @Mock
    private UserService userService;
    @Mock
    private ResearchAreaService researchAreaService;
    @Mock
    private ProjectService projectService;
    @Mock
    private ProjectTypeService projectTypeService;
    @Mock
    private UserNameService userNameService;
    @Mock
    private ProgramService programService;
    @Mock
    private LanguageService languageService;

    private ImporterTransactionsImpl importerTransactions;

    @Before
    public void createInstance() throws Exception {
        importerTransactions = new ImporterTransactionsImpl(userService, researchAreaService, projectService,
                userNameService, daisyAPI, programService, languageService);

        when(daisyAPI.getStudent(anyLong())).thenReturn(Response.status(Response.Status.NOT_FOUND).build());
    }

    @Test
    public void updates_person_data() throws Exception {
        Person person = createPerson();

        importerTransactions.importPerson(person);

        ArgumentCaptor<User> captor = ArgumentCaptor.forClass(User.class);
        verify(userService).save(captor.capture());

        assertUserPersonData(captor.getValue());
    }

    private static void assertUserPersonData(User user) {
        assertEquals(SOME_FIRST_NAME, user.getFirstName());
        assertEquals(SOME_LAST_NAME, user.getLastName());
        assertEquals(SOME_EMAIL, user.getEmailAddress());
    }

    private Person createPerson() {
        return createPerson(SOME_ID);
    }

    private Person createPerson(Long id) {
        Person person = new Person();
        person.setFirstName(SOME_FIRST_NAME);
        person.setLastName(SOME_LAST_NAME);
        person.setEmail(SOME_EMAIL);
        person.setId(id);
        return person;
    }
}
