import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.persist.PersistFilter;
import com.google.inject.servlet.GuiceServletContextListener;
import com.google.inject.servlet.ServletModule;
import org.apache.wicket.guice.GuiceWebApplicationFactory;
import org.apache.wicket.protocol.http.WicketFilter;

import java.util.HashMap;
import java.util.Map;
import java.util.ServiceLoader;

public class ApplicationBootstrap extends GuiceServletContextListener {
    @Override
    protected Injector getInjector() {
        return Guice.createInjector(
                new ServletModule() {
                    @Override
                    protected void configureServlets() {
                        filter("/*").through(PersistFilter.class);
                    }
                },
                new AbstractModule() {
                    @Override
                    protected void configure() {
                        ServiceLoader<Module> modules = ServiceLoader.load(Module.class);
                        for (Module module : modules) {
                            install(module);
                        }
                    }
                },
                new ServletModule() {
                    @Override
                    protected void configureServlets() {
                        Map<String,String> params = new HashMap<>();
                        params.put(WicketFilter.FILTER_MAPPING_PARAM, "/*");
                        params.put(WicketFilter.APP_FACT_PARAM, GuiceWebApplicationFactory.class.getName());
                        params.put("injectorContextAttribute", Injector.class.getName());
                        filter("/*").through(new WicketFilter(), params);
                    }
                }
        );
    }
}
