package se.su.dsv.scipro;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.apache.wicket.protocol.http.servlet.ResponseIOException;
import org.apache.wicket.protocol.http.servlet.ServletWebRequest;
import org.apache.wicket.protocol.http.servlet.ServletWebResponse;

import javax.servlet.http.HttpServletResponse;
import java.net.SocketException;

/**
 * Workaround filter for a Wicket bug
 *
 * https://issues.apache.org/jira/browse/WICKET-3869?focusedCommentId=13546809&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-13546809
 */
public class FilteredWebResponse extends ServletWebResponse {

    public FilteredWebResponse(ServletWebRequest webRequest, HttpServletResponse httpServletResponse) {
        super(webRequest, httpServletResponse);
    }

    @Override
    public void write(final CharSequence sequence) {
        attempt(new Attempt() {
            @Override
            public void tryExecute() {
                FilteredWebResponse.super.write(sequence);
            }
        });
    }

    @Override
    public void write(final byte[] array) {
        attempt(new Attempt() {
            @Override
            public void tryExecute() {
                FilteredWebResponse.super.write(array);
            }
        });
    }

    @Override
    public void write(final byte[] array, final int offset, final int length) {
        attempt(new Attempt() {
            @Override
            public void tryExecute() {
                FilteredWebResponse.super.write(array, offset, length);
            }
        });
    }

    @Override
    public void flush() {
        attempt(new Attempt() {
            @Override
            public void tryExecute() {
                FilteredWebResponse.super.flush();
            }
        });
    }

    private void attempt(Attempt attempt) {
        try {
            attempt.tryExecute();
        } catch (ResponseIOException e) {
            if (!(ExceptionUtils.getRootCause(e) instanceof SocketException)) {
                throw e;
            }
        }
    }

    private interface Attempt {
        void tryExecute();
    }
}