package se.su.dsv.scipro;

import com.googlecode.wickedcharts.wicket6.JavaScriptResourceRegistry;
import org.apache.wicket.Page;
import org.apache.wicket.RuntimeConfigurationType;
import org.apache.wicket.Session;
import org.apache.wicket.authorization.strategies.CompoundAuthorizationStrategy;
import org.apache.wicket.protocol.http.servlet.ServletWebRequest;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.Response;
import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.resource.DynamicJQueryResourceReference;
import org.apache.wicket.settings.IExceptionSettings;
import se.su.dsv.scipro.activityplan.*;
import se.su.dsv.scipro.admin.pages.*;
import se.su.dsv.scipro.admin.pages.settings.*;
import se.su.dsv.scipro.applicationperiod.AdminEditApplicationPeriodPage;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.basepages.errorpages.InternalErrorPage;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.basepages.errorpages.PageExpiredPage;
import se.su.dsv.scipro.checklists.AdminChecklistPage;
import se.su.dsv.scipro.checklists.AdminEditChecklistTemplatePage;
import se.su.dsv.scipro.checklists.ProjectViewChecklistPage;
import se.su.dsv.scipro.checklists.SupervisorViewChecklistPage;
import se.su.dsv.scipro.examiner.pages.ExaminerStartPage;
import se.su.dsv.scipro.finalseminar.*;
import se.su.dsv.scipro.finalthesis.SupervisorFinalThesisListingPage;
import se.su.dsv.scipro.forum.mail.ForumMailGroupPage;
import se.su.dsv.scipro.forum.mail.ForumMailProjectPage;
import se.su.dsv.scipro.forum.pages.ProjectForumBasePage;
import se.su.dsv.scipro.forum.pages.SupervisorForumBasePage;
import se.su.dsv.scipro.forum.pages.threaded.*;
import se.su.dsv.scipro.generalsystemsettings.AdminGeneralSettingsPage;
import se.su.dsv.scipro.group.AuthorGroupPage;
import se.su.dsv.scipro.group.AuthorMyGroupsPage;
import se.su.dsv.scipro.group.CreateThreadPage;
import se.su.dsv.scipro.group.ViewThreadPage;
import se.su.dsv.scipro.loginlogout.pages.*;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.milestones.AdminMileStoneSettingsPage;
import se.su.dsv.scipro.milestones.pages.ProjectMileStonePage;
import se.su.dsv.scipro.milestones.pages.SupervisorMileStonePage;
import se.su.dsv.scipro.nonworkdays.NonWorkDaysPage;
import se.su.dsv.scipro.notes.NotesPage;
import se.su.dsv.scipro.notifications.NotificationLoader;
import se.su.dsv.scipro.notifications.NotificationModule;
import se.su.dsv.scipro.notifications.pages.NotificationLandingPage;
import se.su.dsv.scipro.notifications.pages.NotificationsPage;
import se.su.dsv.scipro.notifications.pages.SupervisorNotificationSettingsPage;
import se.su.dsv.scipro.notifications.settings.pages.AdminNotificationSettingsPage;
import se.su.dsv.scipro.peer.*;
import se.su.dsv.scipro.profiles.CurrentProfile;
import se.su.dsv.scipro.profiles.Profiles;
import se.su.dsv.scipro.project.pages.ProjectDetailsPage;
import se.su.dsv.scipro.project.pages.ProjectFilePage;
import se.su.dsv.scipro.project.pages.ProjectStartPage;
import se.su.dsv.scipro.project.pages.PublicActiveThesisProjectListingPage;
import se.su.dsv.scipro.projectpartner.AdminProjectPartnerPage;
import se.su.dsv.scipro.projectpartner.ProjectPartnerPage;
import se.su.dsv.scipro.repository.LifecycleManagedWebApplication;
import se.su.dsv.scipro.repository.components.FileReference;
import se.su.dsv.scipro.reviewer.*;
import se.su.dsv.scipro.security.auth.ProjectModuleBasedAuthorizationStrategy;
import se.su.dsv.scipro.security.auth.RoleBasedAuthorizationStrategy;
import se.su.dsv.scipro.security.auth.SystemModuleBasedAuthorizationStrategy;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.pages.BasicProfilePage;
import se.su.dsv.scipro.statistics.*;
import se.su.dsv.scipro.supervisor.pages.*;
import se.su.dsv.scipro.user.*;
import se.su.dsv.scipro.util.AdditionalExceptionLogger;

import javax.inject.Inject;
import javax.servlet.http.HttpServletResponse;

public class SciProApplication extends LifecycleManagedWebApplication {

    @Inject
    public SciProApplication(CurrentProfile currentProfile) {
        setConfigurationType(currentProfile.getCurrentProfile() == Profiles.PROD
                ? RuntimeConfigurationType.DEPLOYMENT
                : RuntimeConfigurationType.DEVELOPMENT);
    }

    /**
     * Workaround filter for a Wicket bug
     * <p/>
     * https://issues.apache.org/jira/browse/WICKET-3869?focusedCommentId=13546809&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-13546809
     */
    @Override
    protected WebResponse newWebResponse(final WebRequest webRequest, final HttpServletResponse httpServletResponse) {
        return new FilteredWebResponse((ServletWebRequest) webRequest, httpServletResponse);
    }

    @Override
    protected void init() {
        super.init();

        getMarkupSettings().setCompressWhitespace(true);
        getMarkupSettings().setStripComments(true);
        getMarkupSettings().setStripWicketTags(true);
        getMarkupSettings().setDefaultMarkupEncoding("UTF-8");
        getRequestCycleSettings().setResponseRequestEncoding("UTF-8");
        getRequestCycleListeners().add(new AdditionalExceptionLogger());
        getJavaScriptLibrarySettings().setJQueryReference(new DynamicJQueryResourceReference());

        mountRootPages();
        mountNotificationAndSettingsPages();
        mountAdminPages();
        mountProjectPages();
        mountSupervisorPages();
        mountPeerPages();
        mountErrorPages();
        mountPublicPages();
        mountForumPage();

        //Set up authorization strategies for local authentication.
        CompoundAuthorizationStrategy cas = new CompoundAuthorizationStrategy();
        cas.add(new RoleBasedAuthorizationStrategy());
        cas.add(new ProjectModuleBasedAuthorizationStrategy());
        cas.add(new SystemModuleBasedAuthorizationStrategy());
        getSecuritySettings().setAuthorizationStrategy(cas);

        getResourceSettings().getStringResourceLoaders().add(new NotificationLoader());

        JavaScriptResourceRegistry.getInstance().setHighchartsReference("js/highcharts.js");
    }

    private void mountForumPage() {
        mountPage("forum/mail/project", ForumMailProjectPage.class);
        mountPage("forum/mail/group", ForumMailGroupPage.class);
    }

    private void mountPublicPages() {
        mountPage("public/seminars", PublicFinalSeminarListingPage.class);
        mountPage("public/projects", PublicActiveThesisProjectListingPage.class);
        mountResource("file", new FileReference());
    }

    private void mountErrorPages() {
        mountPage("404", NotFoundPage.class);
        mountPage("403", AccessDeniedPage.class);
        mountPage("500", InternalErrorPage.class);
        mountPage("page_expired_error", PageExpiredPage.class);
        getApplicationSettings().setAccessDeniedPage(AccessDeniedPage.class);
        getApplicationSettings().setInternalErrorPage(InternalErrorPage.class);
        getExceptionSettings().setUnexpectedExceptionDisplay(IExceptionSettings.SHOW_INTERNAL_ERROR_PAGE);
        getApplicationSettings().setPageExpiredErrorPage(PageExpiredPage.class);
    }

    private void mountPeerPages() {
        //project
        mountPage("project/peer", PeerPortalPage.class);
        //supervisor
        mountPage("supervisor/peer/allpeers", SupervisorPeersPage.class);
    }

    private void mountSupervisorPages() {
        mountPage("supervisor", SupervisorStartPage.class);
        mountPage("supervisor/all", SupervisorAllProjectsPage.class);
        mountPage("supervisor/allfinalseminars", SupervisorFinalSeminarListingPage.class);
        mountPage("supervisor/all/finalseminar", SupervisorFinalSeminarDetailsPage.class);
        mountPage("supervisor/templates", SupervisorActivityPlanTemplatesPage.class);
        mountPage("supervisor/project/details", SupervisorProjectDetailsPage.class);

        mountPage("supervisor/project/forum", SupervisorForumBasePage.class);
        mountPage("supervisor/project/finalseminar", SupervisorFinalSeminarPage.class);

        mountPage("supervisor/project/forum2", SupervisorThreadedForumPage.class);
        mountPage("supervisor/project/forum/thread", SupervisorViewForumThreadPage.class);
        mountPage("supervisor/project/forum/create", SupervisorCreateForumThreadPage.class);

        mountPage("supervisor/checklist/viewchecklist", SupervisorViewChecklistPage.class);
        mountPage("supervisor/project/activityplan", SupervisorActivityPlanPage.class);
        mountPage("supervisor/project/activityplan/editactivity", SupervisorEditActivityPage.class);
        mountPage("supervisor/project/activityplan/edit", SupervisorEditActivityTemplatePage.class);

        mountPage("supervisor/project/files", SupervisorFilePage.class);
        mountPage("supervisor/projectideas/newidea", SupervisorIdeaSubmissionPage.class);
        mountPage("supervisor/projectideas/allselestudentideas", SupervisorSelectableStudentIdeasPage.class);
        mountPage("supervisor/projectideas/allsupervisorideas", SupervisorNewAllSupervisorIdeasPage.class);
        mountPage("supervisor/projectideas", SupervisorIdeaStartPage.class);
        mountPage("supervisor/projectideas/studentideas", SupervisorAllStudentIdeasPage.class);

        mountPage("supervisor/finalversions", SupervisorFinalThesisListingPage.class);

        mountPage("supervisor/examining", ExaminerStartPage.class);
        mountPage("supervisor/project/milestone", SupervisorMileStonePage.class);
        mountPage("supervisor/project/admin", SupervisorAdminPage.class);

        mountPage("supervisor/groups", SupervisorMyGroupsPage.class);
        mountPage("supervisor/groups/edit", SupervisorEditGroupPage.class);
        mountPage("supervisor/group", SupervisorGroupPage.class);
        mountPage("supervisor/group/createthread", SupervisorGroupCreateThreadPage.class);
        mountPage("supervisor/group/viewthread", SupervisorViewGroupThreadPage.class);

        mountPage("supervisor/interactwithreviewer", SupervisorInteractWithReviewerPage.class);
        mountPage("supervisor/milestonestatistics", SupervisorMilestoneStatisticsPage.class);

        mountPage("reviewer/interactwithsupervisor", ReviewerInteractionPage.class);
        mountPage("reviewer/reviewing", ReviewerStartPage.class);
        mountPage("reviewer/grading", ReviewerGradingPage.class);
        mountPage("reviewer/decision", ReviewerApprovalDecisionPage.class);
        mountPage("reviewer/approval/roughdraft", RoughDraftApprovalDecisionPage.class);
        mountPage("reviewer/approval/finalseminar", FinalSeminarApprovalDecisionPage.class);

    }

    private void mountProjectPages() {
        mountPage("project/details", ProjectDetailsPage.class);
        mountPage("project/forum", ProjectForumBasePage.class);
        mountPage("project/forum2", ProjectThreadedForumPage.class);
        mountPage("project/forum/thread", ProjectViewForumThreadPage.class);
        mountPage("project/forum/create", ProjectCreateForumThreadPage.class);
        mountPage("project/activityplan", ProjectActivityPlanPage.class);
        mountPage("project/activityplan/editactivity", ProjectEditActivityPage.class);
        mountPage("project/files", ProjectFilePage.class);
        mountPage("project/opposition", ProjectOppositionPage.class);
        mountPage("project/allfinalseminars", FinalSeminarProjectListPage.class);
        mountPage("project", ProjectStartPage.class);
        mountPage("project/checklist/viewchecklist", ProjectViewChecklistPage.class);
        mountPage("projectideas/partner", ProjectPartnerPage.class);
        mountPage("projectideas/newidea", ProjectIdeaSubmissionPage.class);
        mountPage("projectideas/allsupervisor", ProjectNewAllSupervisorIdeasPage.class);
        mountPage("projectideas", ProjectIdeaStartPage.class);
        mountPage("projectideas/details", ProjectIdeaDetailsPage.class);
        mountPage("project/finalseminar", ProjectFinalSeminarPage.class);

        mountPage("project/finalseminardetails", ProjectFinalSeminarDetailsPage.class);

        mountPage("project/milestone", ProjectMileStonePage.class);
        mountPage("groups", AuthorMyGroupsPage.class);
        mountPage("group", AuthorGroupPage.class);
        mountPage("group/createthread", CreateThreadPage.class);
        mountPage("group/viewthread", ViewThreadPage.class);
    }

    private void mountAdminPages() {
        mountPage("admin/start", AdminStartPage.class);
        mountPage("admin/templates", AdminActivityPlanTemplatesPage.class);
        mountPage("admin/templates/edit", AdminEditActivityTemplatePage.class);
        mountPage("admin/maintenance", SystemMaintenancePage.class);
        mountPage("admin/project", ProjectManagementPage.class);
        mountPage("admin/edit", AdminEditProjectPage.class);
        mountPage("admin/finalseminars", AdminFinalSeminarPage.class);
        mountPage("admin/finalseminars/exemptions", AdminFinalSeminarExemptionPage.class);
        mountPage("admin/settings", AdminGeneralSettingsPage.class);
        mountPage("admin/settings/finalseminar", AdminFinalSeminarSettingsPage.class);
        mountPage("admin/settings/reviewerdeadlines", AdminReviewerDeadlineSettingsPage.class);
        mountPage("admin/settings/peer", AdminPeerSettingsPage.class);
        mountPage("admin/settings/projecttype", AdminProjectTypeSettingsPage.class);
        mountPage("admin/settings/notifications", AdminNotificationSettingsPage.class);
        mountPage("admin/settings/milestones", AdminMileStoneSettingsPage.class);
        mountPage("admin/settings/nonworkdays", NonWorkDaysPage.class);
        mountPage("admin/settings/modules", AdminProjectModuleSettingsPage.class);
        mountPage("admin/settings/footer", AdminFooterSettingsPage.class);
        mountPage("admin/settings/language", AdminLanguageSettingsPage.class);
        mountPage("admin/settings/forum", AdminForumSettingsPage.class);
        mountPage("admin/allpeers", AdminPeersPage.class);
        mountPage("admin/peer/review", AdminPeerReviewPage.class);
        mountPage("admin/projectpartner", AdminProjectPartnerPage.class);
        mountPage("admin/checklist", AdminChecklistPage.class);
        mountPage("admin/checklist/edit", AdminEditChecklistTemplatePage.class);
        mountPage("admin/match/projectideas", AdminManageIdeaPage.class);
        mountPage("admin/match/ideapopup", AdminIdeaPopupPage.class);
        mountPage("admin/match/keywords", AdminKeywordPage.class);
        mountPage("admin/match/units", AdminUnitPage.class);
        mountPage("admin/match/program", AdminProgramPage.class);
        mountPage("admin/match/applicationperiods", AdminManageMatchPeriodsPage.class);
        mountPage("admin/match/applicationperiods/create", AdminEditApplicationPeriodPage.class);
        mountPage("admin/mail", AdminMailPage.class);
        mountPage("admin/mail/log", AdminMailLogPage.class);
        mountPage("admin/authors", AdminAuthorsStatisticsPage.class);
        mountPage("admin/statistics", AdminStatisticsPage.class);
        mountPage("admin/statistics/unmettargets", AdminUnmetTargetsStatisticsPage.class);
        mountPage("admin/statistics/statuses", AdminIdeaStatusStatisticsPage.class);
        mountPage("admin/statistics/supervisor", SupervisorStatisticsPage.class);
        mountPage("admin/statistics/external", ProjectExternalStatisticsPage.class);
        mountPage("admin/statistics/units", AdminUnitStatisticsPage.class);
        mountPage("admin/statistics/ideas", AdminIdeaStatisticsPage.class);
        mountPage("admin/statistics/milestones", AdminMilestoneStatisticsPage.class);
        mountPage("admin/statistics/milestonesprogress", AdminMilestoneStatisticsProgressPage.class);
        mountPage("admin/statistics/unfinishedfinalseminars", AdminUnfinishedFinalSeminarsPage.class);
        mountPage("admin/statistics/reviewerdeadlines", AdminReviewerDeadlinesPage.class);
        mountPage("admin/statistics/people", AdminPeopleStatisticsPage.class);
        mountPage("admin/statistics/medianmilestone", AdminMedianMilestonePage.class);
        mountPage("admin/statistics/finalseminar", AdminFinalSeminarStatisticsPage.class);
        mountPage("admin/statistics/grade", AdminGradeStatisticsPage.class);
        mountPage("admin/settings/turnitin", TurnitinSettingsPage.class);
        mountPage("admin/users/roles", AdminRolePage.class);
        mountPage("admin/users/internal", AdminInternalUsersPage.class);
        mountPage("admin/users/internal/edit", AdminEditUserPage.class);
        mountPage("admin/users/internal/changepassword", AdminChangePasswordPage.class);
        mountPage("admin/users/internal/csv", AdminImportUserPage.class);
        mountPage("admin/users/switch", AdminSwitchUserPage.class);
    }

    private void mountNotificationAndSettingsPages() {
        mountPage("notes", NotesPage.class);
        mountPage(NotificationModule.NOTIFICATION_RELATIVE_PAGE_URL, NotificationLandingPage.class);
        mountPage("notifications", NotificationsPage.class);
        mountPage("settings", BasicProfilePage.class);
        mountPage("settings/supervisorprofile", SupervisorProfilePage.class);
        mountPage("settings/notifications", SupervisorNotificationSettingsPage.class);
    }

    private void mountRootPages() {
        mountPage("login", LoginPage.class);
        mountPage("login/local", LocalLogInPage.class);
        mountPage("forgotpassword", ForgotPasswordPage.class);
        mountPage("logout", LogoutPage.class);
        mountPage("setpassword", SetPasswordPage.class);
    }

    @Override
    public Class<? extends Page> getHomePage() {
        return HomePage.class;
    }

    @Override
    public Session newSession(Request request, Response response) {
        return new SciProSession(request);
    }
}
