package se.su.dsv.scipro.activityplan;

import com.google.common.base.Strings;
import org.apache.wicket.Page;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.*;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.checklist.Checklist;
import se.su.dsv.scipro.checklists.ProjectViewChecklistPage;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.files.FileDescriptionPanel;
import se.su.dsv.scipro.finalseminar.ProjectFinalSeminarPage;
import se.su.dsv.scipro.finalseminar.SupervisorFinalSeminarPage;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.peer.PeerPortalPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.pages.ProjectDetailsPage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.SupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

public class ActivityPlanPanel extends GenericPanel<Project> {
    public static final String FILE_UPLOAD = "fileUpload";
    public static final String DATE = "date";

    @Inject
    private ActivityService activityService;
    @Inject
    private ActivityPlanFacade facade;
    @Inject
    private NotificationController notificationController;
    @Inject
    private ProjectService projectService;

    private ModalWindowPlus saveAsTemplateModalWindowPlus;
    private ModalWindowPlus addFromTemplateModalWindowPlus;
    private AddActivitiesFromTemplatePanel addActivitiesFromTemplatePanel;
    private SaveAsTemplatePanel saveAsTemplatePanel;

    private ActivityDataProvider sdp;
    private DefaultDataPanel<Activity, String> table;

    public ActivityPlanPanel(final String id, final IModel<Project> project) {
        super(id, project);

        dialogSetup();
        formSetup();
        newButtonSetup();
        dataViewSetup();
        addModeControls();
        addFeedback();
    }

    private void addFeedback() {
        add(new FencedFeedbackPanel("feedback", this)
                .setOutputMarkupId(true));
    }

    private void newButtonSetup() {

        AjaxLink<Void> createLink = new AjaxCreateLink("createLink");

        add(createLink);

    }

    protected final Project getProject() {
        return getModelObject();
    }

    private void addModeControls() {
        add(new AjaxRadioChoice<ActivityDataProvider.SearchMode>("modeGroup",
                new PropertyModel<ActivityDataProvider.SearchMode>(sdp, "searchMode"),
                Arrays.asList(ActivityDataProvider.SearchMode.values()),
                new EnumChoiceRenderer<ActivityDataProvider.SearchMode>(this)) {
            @Override
            protected void update(AjaxRequestTarget target) {
                target.add(table);
                if (ActivityDataProvider.SearchMode.PROJECT_PAST.equals(getModelObject())) {
                    sdp.setSort(DATE, SortOrder.DESCENDING);
                } else {
                    sdp.setSort(DATE, SortOrder.ASCENDING);
                }
            }
        });
    }

    private void dialogSetup() {
        addFromTemplateModalWindowPlus = new ModalWindowPlus("addDialog");
        addFromTemplateModalWindowPlus.setTitle("Add activities from template");
        addActivitiesFromTemplatePanel = new AddActivitiesFromTemplatePanel(addFromTemplateModalWindowPlus.getContentId(), getModel());
        addFromTemplateModalWindowPlus.setContent(addActivitiesFromTemplatePanel);
        add(addFromTemplateModalWindowPlus);
        saveAsTemplateModalWindowPlus = new ModalWindowPlus("saveAsTemplateDialog");
        saveAsTemplatePanel = new SaveAsTemplatePanel(saveAsTemplateModalWindowPlus.getContentId(), new LoadableDetachableModel<ActivityPlan>() {
            @Override
            protected ActivityPlan load() {
                return getActivityPlan();
            }
        });
        saveAsTemplateModalWindowPlus.setTitle(new ResourceModel("save.as.template"));
        saveAsTemplateModalWindowPlus.setContent(saveAsTemplatePanel);
        add(saveAsTemplateModalWindowPlus);

    }

    private void dataViewSetup() {
        sdp = new ActivityDataProvider(facade, activityService, getProject());
        sdp.setSort(DATE, SortOrder.ASCENDING);
        table = new ExportableDataPanel<>("table", createColumns(), sdp, 100L);
        add(table);
    }

    private List<IColumn<Activity, String>> createColumns() {
        List<IColumn<Activity, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Activity>(new ResourceModel(DATE).wrapOnAssignment(this), DATE, DATE, DateStyle.DATETIME));
        columns.add(new PropertyColumn<Activity, String>(new ResourceModel("title").wrapOnAssignment(this), "title", "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Activity>> cellItem, String componentId, IModel<Activity> rowModel) {
                cellItem.add(new TitleCell(componentId, rowModel));
            }
        });
        columns.add(new ActionColumn(new StringResourceModel("action", this, null, "Action")));
        columns.add(new AbstractExportableColumn<Activity, String, String>(new ResourceModel("checklist").wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<Activity>> cellItem, String componentId, IModel<Activity> rowModel) {
                cellItem.add(new ChecklistCell(componentId, new PropertyModel<Checklist>(rowModel, "checklist")));
            }

            @Override
            public IModel<String> getDataModel(IModel<Activity> rowModel) {
                return new ChecklistNameModel(new PropertyModel<Checklist>(rowModel, "checklist"));
            }
        });
        columns.add(new AbstractColumn<Activity, String>(new ResourceModel("edit")) {
            @Override
            public void populateItem(Item<ICellPopulator<Activity>> cellItem, String componentId, final IModel<Activity> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<Activity>(componentId, rowModel, Model.of(AjaxIconLinkPanel.EDIT)) {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(rowModel.getObject().isEditable());
                    }

                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Activity> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(ActivityPlan.class), model.getObject().getActivityPlan().getId());
                        pp.add(PageParameterKeys.MAP.get(Activity.class), model.getObject().getId());

                        editPage(pp);

                    }
                });
            }
        });
        columns.add(new AbstractColumn<Activity, String>(new ResourceModel("delete")) {
            @Override
            public void populateItem(Item<ICellPopulator<Activity>> cellItem, String componentId, final IModel<Activity> rowModel) {
                cellItem.add(new AjaxIconLinkPanel<Activity>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), getString("delete.confirmation")) {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(rowModel.getObject().isEditable());
                    }

                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Activity> model) {
                        facade.deleteActivity(rowModel.getObject());
                        target.add(table);
                        notifyDeletedActivityPlan(rowModel.getObject().getTitle());
                        success(getString("activity.deleted", model));
                        target.addListener(new AjaxFeedbackPanelUpdater());
                    }
                });
            }
        });
        return columns;
    }

    private void editPage(PageParameters pp) {
        if (hasSupervisingRole()) {
            setResponsePage(SupervisorEditActivityPage.class, pp);
        } else if (getProject().isParticipant(SciProSession.get().getUser())) {
            setResponsePage(ProjectEditActivityPage.class, pp);
        }
    }

    private boolean hasSupervisingRole() {
        return projectService.isSupervisorOrFollower(getProject(), SciProSession.get().getUser());
    }

    private class TitleCell extends Fragment {
        public TitleCell(String id, final IModel<Activity> model) {
            super(id, "titleCell", ActivityPlanPanel.this, model);

            add(new Label("title", new PropertyModel(model, "title")));
            add(new InfoPanel("info", new PropertyModel<String>(model, "description")){
                @Override
                protected void onConfigure() {
                    setVisible(!Strings.isNullOrEmpty(model.getObject().getDescription()));
                }
            });
        }
    }
    private class ResourceCell extends Fragment {
        public ResourceCell(String id, final IModel<Activity> model) {
            super(id, "resourceCell", ActivityPlanPanel.this, model);

            final Activity event = model.getObject();
            if (event.getFileUpload() != null) {

                add(new FileDescriptionPanel(FILE_UPLOAD, new PropertyModel<FileDescription>(model, FILE_UPLOAD)) {
                    @Override
                    public void onDelete(AjaxRequestTarget target) {
                        facade.deleteActivityUpload(model.getObject());
                        target.add(table);
                    }

                    @Override
                    public boolean deleteAllowed() {
                        return true;
                    }
                });

            } else {
                add(new ActivityUploadForm(model) {
                    @Override
                    protected void onFinished(AjaxRequestTarget target) {
                        target.add(table);
                    }
                }.createDefaultWrapperPanel(FILE_UPLOAD));
            }
        }
    }

    private class ChecklistCell extends Fragment {
        public ChecklistCell(String id, IModel<Checklist> model) {
            super(id, "checklistCell", ActivityPlanPanel.this, model);

            Checklist checklist = model.getObject();

            PageParameters pageParameters = new PageParameters();
            if (checklist != null) {
                pageParameters.set(PageParameterKeys.MAP.get(Checklist.class), checklist.getId());
            }
            BookmarkablePageLink<Void> checklistLink = new BookmarkablePageLink<Void>("checklistLink", getChecklistPage(), pageParameters) {
                @Override
                protected void onConfigure() {
                    setEnabled(ChecklistCell.this.getDefaultModelObject() != null);
                }
            };
            checklistLink.setBody(new ChecklistNameModel(model));

            add(checklistLink);
        }
    }

    protected Class<? extends Page> getChecklistPage() {
        return ProjectViewChecklistPage.class;
    }

    private static class ChecklistNameModel extends AbstractReadOnlyModel<String> {
        private final IModel<Checklist> model;

        public ChecklistNameModel(IModel<Checklist> model) {
            this.model = model;
        }

        @Override
        public String getObject() {
            Checklist checklist = model.getObject();
            return checklist != null ? checklist.getName() : "No checklist";
        }
    }

    private void formSetup() {

        addActivitiesFromTemplatePanel.setOutputMarkupId(true);
        add(new AjaxLink<Void>("addFromTemplateLink") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                target.add(addActivitiesFromTemplatePanel);
                addFromTemplateModalWindowPlus.show(target);
            }

        });

        saveAsTemplatePanel.setOutputMarkupId(true);
        final AjaxLink saveAsTemplateLink = new AjaxLink<Void>("saveAsTemplateLink") {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                Project project = ActivityPlanPanel.this.getModelObject();
                User user = SciProSession.get().getUser();
                setVisible(project.isSupervisor(user) || project.isReviewer(user));
            }

            @Override
            public void onClick(AjaxRequestTarget target) {
                target.add(addActivitiesFromTemplatePanel);
                saveAsTemplateModalWindowPlus.show(target);
            }
        };

        add(saveAsTemplateLink);


        AjaxLink deleteAllActivities = new AjaxConfirmationLink("deleteAllActivitiesLink", "Are you sure you want to delete all activities?") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                facade.deleteAllEvents(getActivityPlan());
                target.add(table);
            }
        };
        add(deleteAllActivities);
    }

    private ActivityPlan getActivityPlan() {
        return facade.retrieveActivityPlan(getProject());
    }

    private void notifyDeletedActivityPlan(String name) {
        notifyChangeActivityPlan(name, ProjectEvent.Event.ACTIVITY_REMOVED);
    }

    private void notifyChangeActivityPlan(String name, ProjectEvent.Event event) {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(getProject().getId()));

        NotificationSource source = new NotificationSource();
        source.setMessage(name);

        Project p = projectService.findOne(getProject().getId());
        notificationController.notifyProject(p, event, source);
    }

    private class AjaxCreateLink extends AjaxLink<Void> {

        public AjaxCreateLink(String id) {
            super(id);
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            Activity dummy = new Activity();
            dummy.setDate(new Date());
            dummy.setActivityPlan(getActivityPlan());

            PageParameters pp = new PageParameters();
            pp.add(PageParameterKeys.MAP.get(ActivityPlan.class), dummy.getActivityPlan().getId());

            editPage(pp);
        }

    }

    private class ActionColumn extends AbstractColumn<Activity, String> {
        public ActionColumn(final IModel<String> displayModel) {
            super(displayModel);
        }

        @Override
        public void populateItem(final Item<ICellPopulator<Activity>> cellItem, final String componentId, final IModel<Activity> rowModel) {
            final Activity activity = rowModel.getObject();
            final Action action = activity.getAction();
            final Project project = activity.getActivityPlan().getProject();
            switch (action) {
                case HAND_IN:
                    if (hasSupervisingRole() && rowModel.getObject().getFileUpload() == null) {
                        cellItem.add(new Label(componentId, new ResourceModel("upload.is.for.authors")));
                    }
                    else {
                        cellItem.add(new ResourceCell(componentId, rowModel));
                    }
                    break;
                case PEER:
                    if (hasSupervisingRole()) {
                        cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                            @Override
                            public AbstractLink apply(final String id) {
                                PageParameters pp = SupervisorProjectDetailsPage.getPageParameters(project);
                                BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(id, SupervisorProjectDetailsPage.class, pp);
                                link.setBody(Model.of("View peer status"));
                                link.setAnchor(new WebComponent("unused").setMarkupId("peerStatus"));
                                return link;
                            }
                        }));
                    } else {
                        cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                            @Override
                            public AbstractLink apply(final String id) {
                                PageParameters pp = PeerPortalPage.getPageParameters(project);
                                BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(id, PeerPortalPage.class, pp);
                                link.setBody(Model.of("Submit peer request"));
                                return link;
                            }
                        }));
                    }
                    break;
                case FINAL_THESIS:
                    cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                        @Override
                        public AbstractLink apply(final String id) {
                            final Link<?> link = getProjectBasedLink(id, project, ProjectDetailsPage.class, SupervisorProjectDetailsPage.class);
                            link.setBody(Model.of("Hand in final thesis"));
                            // Thanks Wicket for making it so easy to append an #anchor to a link...
                            link.setAnchor(new WebComponent("unused").setMarkupId("finalThesis"));
                            return link;
                        }
                    }));
                    break;
                case FINAL_SEMINAR:
                    cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                        @Override
                        public AbstractLink apply(final String id) {
                            final Link<?> link = getProjectBasedLink(id, project, ProjectFinalSeminarPage.class, SupervisorFinalSeminarPage.class);
                            link.setBody(Model.of("Hand in final seminar thesis"));
                            return link;
                        }
                    }));
                    break;
                case NONE:
                default:
                    cellItem.add(new Label(componentId, new StringResourceModel("action.not.available", ActivityPlanPanel.this, null, "n/a")));
                    break;
            }
        }

        private BookmarkablePageLink<Void> getProjectBasedLink(final String id,
                                                               final Project project,
                                                               final Class<? extends Page> authorPage,
                                                               final Class<? extends Page> supervisorPage) {
            final PageParameters pp = new PageParameters();
            pp.set(PageParameterKeys.MAP.get(Project.class), project.getId());
            if (project.isParticipant(SciProSession.get().getUser())) {
                return new BookmarkablePageLink<>(id, authorPage, pp);
            } else {
                return new BookmarkablePageLink<>(id, supervisorPage, pp);
            }
        }
    }
}
