package se.su.dsv.scipro.activityplan;

import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ActivityPlanTemplateDetailsPanel extends Panel {

    @Inject
    private UserService userService;

    public ActivityPlanTemplateDetailsPanel(String id, final IModel<ActivityPlanTemplate> model) {
        super(id, model);
        add(new Label("title", new PropertyModel<>(model, "title")));
        add(new Label("description", new PropertyModel<>(model, "description")));

        add(new UserLinkPanel("templateCreator", new DetachableServiceModel<>(userService, model.getObject().getCreator())){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!model.getObject().getIsSysAdminTemplate());
            }
        });

        add(new Label("admin", "System"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(model.getObject().getIsSysAdminTemplate());
            }
        });

        add(new DateLabel("templateCreated", new PropertyModel<Date>(model, "dateCreated")));
        add(new DateLabel("templateUpdated", new PropertyModel<Date>(model, "lastModified")));
        add(new ListView<ActivityTemplate>("eventList", new PropertyModel<List<ActivityTemplate>>(model, "activityTemplates")) {
            @Override
            protected void populateItem(ListItem<ActivityTemplate> item) {
                final ActivityTemplate et = item.getModelObject();
                item.setOutputMarkupId(true);
                item.add(new Label("etTitle", new PropertyModel<>(item.getModel(), "title")));
                item.add(new Label("etDescription", new PropertyModel<>(item.getModel(), "description")));
                item.add(new Label("etTimeConsumption", et.getDaysOffset() + ""));
                item.add(new EnumLabel<>("handinLabel", new PropertyModel<Action>(item.getModel(), "action")));
                add(item);
            }
        });
    }
}
