package se.su.dsv.scipro.activityplan;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorActivityPlan;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

@Authorization(authorizedRoles={Roles.SUPERVISOR})
@ProjectModuleComponent(ProjectModule.ACTIVITY_PLAN)
public class SupervisorEditActivityPage extends AbstractSupervisorProjectDetailsPage implements MenuHighlightSupervisorActivityPlan {

    @Inject
    private ActivityService activityService;

    public SupervisorEditActivityPage(PageParameters pp) {
        super(pp);

        if (pp.get(PageParameterKeys.MAP.get(Activity.class)).isNull()) {
            add(new EditActivityPanel("edit", projectModel, new DetachableServiceModel<>(activityService, new Activity())));
        } else {

            DetachableServiceModel<Activity> activityModel = new DetachableServiceModel<>(activityService, pp.get(PageParameterKeys.MAP.get(Activity.class)).toLong());
            final Project project = activityModel.getObject().getActivityPlan().getProject();
            final User user = SciProSession.get().getUser();
            if (!(project.isSupervisor(user) || project.isCoSupervisor(user) || project.isReviewer(user))) {
                throw new RestartResponseException(AccessDeniedPage.class);
            }
            add(new EditActivityPanel("edit", projectModel, activityModel));

        }
    }
}
