package se.su.dsv.scipro.activityplan;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.StringResourceModel;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

@ProjectModuleComponent(ProjectModule.ACTIVITY_PLAN)
public abstract class UpcomingActivitiesPanel extends Panel {

    @Inject
    private ActivityPlanFacade activityPlanFacade;

    public abstract boolean isAuthorView();

    protected ListView<Activity> listView;

    public UpcomingActivitiesPanel(String id, IModel<Project> model) {
        super(id, model);

        final List<Activity> events = activityPlanFacade.findEventsByProject(model.getObject(), new Date(), null, new PageRequest(0, 5, new Sort(Sort.Direction.ASC, "date")));

        listView = new ListView<Activity>(LIST, events) {
            @Override
            protected void populateItem(ListItem<Activity> item) {
                item.add(new DateLabel(DATE, new Model<>(item.getModelObject().getDate()), DateStyle.DATETIME));
                item.add(new BookmarkablePageLink<Void>(AUTHOR_LINK, ProjectActivityPlanPage.class){
                    @Override
                    protected void onConfigure(){
                        setVisibilityAllowed(isAuthorView());
                    }
                }.add(new Label(AUTHOR_LABEL, item.getModelObject().getTitle())));
                item.add(new BookmarkablePageLink<Void>(EMPLOYEE_LINK, SupervisorActivityPlanPage.class) {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(!isAuthorView() && SciProSession.get().authorizedForRole(Roles.SUPERVISOR));
                    }
                }.add(new Label(EMPLOYEE_LABEL, item.getModelObject().getTitle())));
            }
        };
        add(listView);

        add(new Label(NO, new StringResourceModel("no", this, null)){
            @Override
            protected void onConfigure(){
                setVisibilityAllowed(events.isEmpty());
            }
        });
    }

    public static final String LIST = "list";
    public static final String NO = "no";
    public static final String DATE = "date";
    public static final String AUTHOR_LINK = "authorLink";
    public static final String AUTHOR_LABEL = "authorLabel";
    public static final String EMPLOYEE_LINK = "employeeLink";
    public static final String EMPLOYEE_LABEL = "employeeLabel";
}
