package se.su.dsv.scipro.admin.pages;

import org.apache.wicket.Page;
import se.su.dsv.scipro.basepages.MenuPage;
import se.su.dsv.scipro.components.AbstractMenuPanel;
import se.su.dsv.scipro.generalsystemsettings.AdminGeneralSettingsPage;
import se.su.dsv.scipro.match.AdminManageIdeaPage;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.statistics.AdminStatisticsPage;
import se.su.dsv.scipro.user.AdminRolePage;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Authorization(authorizedRoles={Roles.SYSADMIN, Roles.ADMIN})
public abstract class AbstractAdminPage extends MenuPage {

    @Inject
    private UserProfileService userProfileService;

    public AbstractAdminPage() {
        super();
        userProfileService.setSelectedRole(SciProSession.get().getUser(), Roles.ADMIN);
        add(new AdminTabMenuPanel("tabMenu", this.getClass()));
    }

    private static class AdminTabMenuPanel extends AbstractMenuPanel {

        public AdminTabMenuPanel(String id, Class<? extends Page> containerClass) {
            super(id, AbstractAdminPage.class, containerClass);
        }

        @Override
        protected List<MenuItem> getItemList() {
            List<MenuItem> items = new ArrayList<>();
            items.add(new MenuItem("Start", AdminStartPage.class));
            items.add(new MenuItem("Project management", ProjectManagementPage.class));
            items.add(new MenuItem("Match", AdminManageIdeaPage.class));
            items.add(new MenuItem("System", AdminGeneralSettingsPage.class));
            items.add(new MenuItem("Mail", AdminMailPage.class));
            items.add(new MenuItem("Statistics", AdminStatisticsPage.class));
            items.add(new MenuItem("Users", AdminRolePage.class));
            return items;
        }

        @Override
        protected MenuType getMenuType() {
            return MenuType.TAB_MULTIPLE_ROWS;
        }
    }
}
