package se.su.dsv.scipro.admin.pages;

import com.google.inject.Inject;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminProjectManagement;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;
import se.su.dsv.scipro.util.Pair;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class AdminCreateProjectConfirmationPage extends AbstractAdminProjectPage implements MenuHighlightAdminProjectManagement {
    @Inject
    private NotificationController notificationController;
    @Inject
    private ProjectService projectService;
    private Project project;
    private boolean addMore = false;

    public AdminCreateProjectConfirmationPage(boolean addMore, IModel<Project> projectIModel) {
        project = projectIModel.getObject();
        this.addMore = addMore;
        add(new CreateProjectForm("form", projectIModel));
    }

    private class CreateProjectForm extends Form<Project> {
        public CreateProjectForm(String id, final IModel<Project> model) {
            super(id, model);

            add(new Label("title", new PropertyModel<Project>(model.getObject(), "title")));
            add(new Label("type", new PropertyModel<Project>(model.getObject(), "projectType.name")));
            add(new Label("authors", new LoadableDetachableModel<String>() {
                @Override
                protected String load() {
                    return model.getObject().getAuthorNames();
                }
            }));
            add(new Label("supervisor", new LoadableDetachableModel<String>() {
                @Override
                protected String load() {
                    return model.getObject().getSupervisorName();
                }
            }));
            add(new Label("reviewers", new LoadableDetachableModel<String>() {
                @Override
                protected String load() {
                    return model.getObject().getReviewerName();
                }
            }));
            add(new Label("coSupervisors", new LoadableDetachableModel<String>() {
                @Override
                protected String load() {
                    return model.getObject().getCoSupervisorName();
                }
            }));

            ListView<Pair<User, Project>> activeProjects = new ListView<Pair<User, Project>>("pairs", new LoadableDetachableModel<List<? extends Pair<User, Project>>>() {
                @Override
                protected List<? extends Pair<User, Project>> load() {
                    return getAuthorsWithCurrentActiveProjects();
                }
            }) {
                @Override
                protected void populateItem(ListItem<Pair<User, Project>> item) {
                    User student = item.getModel().getObject().getHead();
                    Project project = item.getModel().getObject().getTail();
                    item.add(new Label("student", new PropertyModel<User>(student, "fullName")));
                    item.add(new Label("project", new PropertyModel<Project>(project, "title")));
                    item.add(new Label("type", new PropertyModel<Project>(project, "projectType.name")));
                }
            };
            add(activeProjects);

            add(new Link("cancel") {
                @Override
                public void onClick() {
                    getSession().info(getString("canceled", Model.of(project)));
                    setResponsePage(AdminEditProjectPage.class);
                }
            });
        }

        @Override
        public void onSubmit() {
            Project project = projectService.save(getModelObject());

            getSession().info(getString("saved", getModel()));
            notificationController.notifyProject(getModelObject(), ProjectEvent.Event.CREATED, new NotificationSource());
            if (addMore) {
                setResponsePage(AdminEditProjectPage.class);
            } else {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), project.getId());
                setResponsePage(AdminEditProjectPage.class, pp);
            }
        }
    }

    private List<Pair<User, Project>> getAuthorsWithCurrentActiveProjects() {
        Set<User> projectParticipants = project.getProjectParticipants();
        List<Pair<User, Project>> activeProjects = new ArrayList<>();
        for (User student : projectParticipants) {
            List<Project> byUserAndStatus = projectService.findByUserAndStatus(student, ProjectStatus.ACTIVE);
            for (Project project : byUserAndStatus) {
                Pair<User, Project> activeProject = new Pair<>(student, project);
                activeProjects.add(activeProject);
            }
        }
        return activeProjects;
    }
}
