package se.su.dsv.scipro.admin.panels;

import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.system.FooterColumn;
import se.su.dsv.scipro.system.FooterLink;
import se.su.dsv.scipro.system.FooterLinkService;

import javax.inject.Inject;
import java.util.List;

public class AdminEditFooterColumnPanel extends Panel {

    @Inject
    private FooterLinkService footerLinkService;

    private final AutoHidingListView<FooterLink> links;

    public AdminEditFooterColumnPanel(final String id, final FooterColumn column) {
        super(id);

        final AddFooterLinkForm addForm = new AddFooterLinkForm("new", column);
        add(addForm);

        IModel<List<? extends FooterLink>> current = new LoadableDetachableModel<List<? extends FooterLink>>() {
            @Override
            protected List<? extends FooterLink> load() {
                return footerLinkService.getLinks(column);
            }
        };
        links = new AutoHidingListView<FooterLink>("current", current) {
            @Override
            protected void populateItem(final ListItem<FooterLink> item) {
                item.add(new ExternalLink("link", new PropertyModel<String>(item.getModel(), "url"), new PropertyModel<String>(item.getModel(), "title")));
                // Using a form so the user stays on the correct tab after deleting.
                item.add(new Form<FooterLink>("delete", item.getModel()) {
                    @Override
                    public void onSubmit() {
                        if (addForm.getModelObject().equals(item.getModelObject())){
                            FooterLink footerLink = new FooterLink();
                            footerLink.setFooterColumn(column);
                            footerLink.setTitle("");
                            footerLink.setUrl("");
                            addForm.setModel(new CompoundPropertyModel<>(footerLink));
                        }
                        footerLinkService.delete(getModelObject());
                        AdminEditFooterColumnPanel.this.info(getString("deleted", getModel()));
                        refreshLinks();
                    }
                });
                item.add(new Form<FooterLink>("edit", item.getModel()) {
                    @Override
                    protected void onSubmit() {
                        addForm.setModel(new CompoundPropertyModel<>(getModel()));
                    }
                });
                item.add(new Form<FooterLink>("up", item.getModel()) {
                    @Override
                    protected void onSubmit() {
                        footerLinkService.moveUp(getModelObject());
                        refreshLinks();
                    }
                });
                item.add(new Form<FooterLink>("down", item.getModel()) {
                    @Override
                    protected void onSubmit() {
                        footerLinkService.moveDown(getModelObject());
                        refreshLinks();
                    }
                });
            }
        };
        add(links);
    }

    private void refreshLinks() {
        links.detach();
        links.removeAll();
    }

    private class AddFooterLinkForm extends Form<FooterLink> {
        private FooterColumn footerColumn;

        public AddFooterLinkForm(String id, FooterColumn footerColumn) {
            super(id);
            this.footerColumn = footerColumn;
            setModel(new CompoundPropertyModel<>(new FooterLink()));
            add(new RequiredTextField<String>("title"));
            add(new RequiredTextField<String>("url"));

            WebMarkupContainer help = new WebMarkupContainer("urlHelp");
            add(help);
            WebMarkupContainer toggle = new WebMarkupContainer("urlHelpToggle");
            toggle.add(AttributeAppender.replace("onclick", String.format("$('#%s').toggle();", help.getMarkupId())));
            add(toggle);
        }

        @Override
        protected void onSubmit() {
            FooterLink modelObject = getModelObject();
            modelObject.setFooterColumn(footerColumn);
            footerLinkService.save(modelObject);
            info(getString("added"));
            setModel(new CompoundPropertyModel<>(new FooterLink()));
        }
    }
}
