package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.activityplan.ActivityPlanTemplate;
import se.su.dsv.scipro.activityplan.ActivityPlanTemplateService;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodProjectType;
import se.su.dsv.scipro.match.ApplicationPeriodService;

import javax.inject.Inject;
import java.util.List;
import java.util.Set;

public class EditApplicationPeriodActivityPlanTemplatePanel extends Panel {
    public static final String LIST = "list";
    public static final String PC = "pc";
    public static final String DROPDOWN = "dropdown";
    public static final String FORM = "form";

    @Inject
    private ActivityPlanTemplateService activityPlanTemplateService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;

    protected ListView<ApplicationPeriodProjectType> list;

    public EditApplicationPeriodActivityPlanTemplatePanel(String id, final IModel<ApplicationPeriod> model) {
        super(id, model);
        add(new EditForm(FORM, model));
    }

    protected class EditForm extends Form<ApplicationPeriod> {
        public EditForm(String id, IModel<ApplicationPeriod> model) {
            super(id, model);
            list = new ListView<ApplicationPeriodProjectType>(LIST, getProjectTypes(model)) {
                @Override
                protected void populateItem(ListItem<ApplicationPeriodProjectType> item) {
                    item.add(new Label(PC, new PropertyModel<String>(item.getModel(), "projectType.name")));
                    item.add(new DropDownChoice<>(DROPDOWN,
                            new PropertyModel<ActivityPlanTemplate>(item.getModel(), "activityPlanTemplate"),
                            new ListAdapterModel<>(getTemplates()),
                            new ChoiceRenderer<>("title", "id"))
                            .setNullValid(true));
                }
            };
            add(list);
        }

        private IModel<Iterable<ActivityPlanTemplate>> getTemplates() {
            return new LoadableDetachableModel<Iterable<ActivityPlanTemplate>>() {
                @Override
                protected Iterable<ActivityPlanTemplate> load() {
                    return activityPlanTemplateService.findAll();
                }
            };
        }

        private IModel<List<ApplicationPeriodProjectType>> getProjectTypes(IModel<ApplicationPeriod> model) {
            return new ListAdapterModel<>(new PropertyModel<Set<ApplicationPeriodProjectType>>(model, "applicationPeriodProjectTypes"));
        }

        @Override
        protected void onSubmit(){
            applicationPeriodService.save(getModelObject());
            success(getString("saved"));
        }
    }
}