package se.su.dsv.scipro.basepanels;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.loginlogout.pages.LogoutPage;
import se.su.dsv.scipro.notes.NotesPage;
import se.su.dsv.scipro.profile.UserProfileImage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.pages.BasicProfilePage;
import se.su.dsv.scipro.system.User;

public class HeaderPanel extends Panel {

	public HeaderPanel(String id, Class<? extends Page> containerClass) {
		super(id);
		add(new BookmarkablePageLink<Void>("homeLink", getApplication().getHomePage()));
		add(new RolesMenuPanel("mainMenuPanel", containerClass));
        add(new BookmarkablePageLink<>("notes", NotesPage.class));
        final IModel<User> currentUser = new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return SciProSession.get().getUser();
            }
        };
        add(new Label("fullName", new PropertyModel<String>(currentUser, "fullName")));
        add(new UserProfileImage("picture", currentUser, UserProfileImage.Size.TINY));
        add(new NotificationsMenuPanel("notificationsMenuPanel"));
        add(new BookmarkablePageLink<Void>("editProfile", BasicProfilePage.class));
        add(new BookmarkablePageLink<Void>("logOut", LogoutPage.class) {
            @Override
            protected void onConfigure() {
                User user = currentUser.getObject();
                // How the fuck is user null here? Why doesn't Wicket's authorization work?
                setVisible(user != null && user.getIdentifier() == null);
            }
        });
        setOutputMarkupPlaceholderTag(true);
	}
}
