package se.su.dsv.scipro.checklists;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.image.ContextImage;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.checklist.ChecklistAnswer;
import se.su.dsv.scipro.checklist.ChecklistAnswerEnum;
import se.su.dsv.scipro.checklist.ChecklistQuestion;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import java.util.EnumMap;
import java.util.Map;

public abstract class AnswerPanel extends Panel {

    public abstract void hook();

    static final Map<ChecklistAnswerEnum, String> ANSWER_IMAGES;

    static {
        ANSWER_IMAGES = new EnumMap<>(ChecklistAnswerEnum.class);
        ANSWER_IMAGES.put(ChecklistAnswerEnum.GREEN, "images/icons/bullet_ball_glass_green.png");
        ANSWER_IMAGES.put(ChecklistAnswerEnum.YELLOW, "images/icons/bullet_ball_glass_yellow.png");
        ANSWER_IMAGES.put(ChecklistAnswerEnum.RED, "images/icons/bullet_ball_glass_red.png");
        ANSWER_IMAGES.put(ChecklistAnswerEnum.NOT_APLICABLE, "images/icons/bullet_ball_glass_grey.png");
    }

    public static final String CLICK_ICON = "images/icons/bullet_ball_glass_click.png";
    public static final String NO_ANSWER = "images/icons/bullet_ball_glass_no_answer.png";

    private ModalWindowPlus dialog;

    public AnswerPanel(String id, final IModel<ChecklistQuestion> question, final IModel<User> user) {
        super(id);
        dialog = new ModalWindowPlus("dialog");
        add(dialog);

        final IModel<ChecklistAnswer> answer = new LoadableDetachableModel<ChecklistAnswer>() {
            @Override
            public ChecklistAnswer load() {
                return question.getObject().getAnswerFromUser(user.getObject());
            }
        };

        final AjaxLink<ChecklistAnswer> answerLink = new AjaxLink<ChecklistAnswer>("answerLink", answer) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                dialog.setContent(new EditAnswerPanel(dialog.getContentId(), question, answer) {
                    @Override
                    public void submitHook() {
                        hook();
                    }
                });
                dialog.show(target);
            }

            @Override
            protected void onConfigure() {
                setEnabled(user.getObject().equals(SciProSession.get().getUser()));
            }
        };
        answerLink.add(new ContextImage("answer", new AbstractReadOnlyModel<String>() {
            @Override
            public String getObject() {
                return getAnswerImage(answer.getObject());
            }
        }));
        add(answerLink);
    }

    private String getAnswerImage(ChecklistAnswer answer) {
        if (ANSWER_IMAGES.containsKey(answer.getAnswer())) {
            return ANSWER_IMAGES.get(answer.getAnswer());
        }
        if (SciProSession.get().getUser().equals(answer.getUser())) {
            return CLICK_ICON;
        } else {
            return NO_ANSWER;
        }
    }
}
