package se.su.dsv.scipro.components;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

public abstract class AjaxIconLinkPanel<T> extends GenericPanel<T> {

    public static final String DELETE = "glyphicon glyphicon-remove";
    public static final String EDIT = "glyphicon glyphicon-edit";

    public static final String LINK = "link";
    public static final String LABEL = "label";

    public abstract void onClick(AjaxRequestTarget target, IModel<T> model);

    private IModel<String> iconString;
    private String confirmationMessage;

    public AjaxIconLinkPanel(String id, final IModel<T> model, IModel<String> iconString) {
        super(id, model);
        this.iconString = iconString;
    }

    public AjaxIconLinkPanel(String id, final IModel<T> model, IModel<String> iconString, String confirmationMessage) {
        super(id, model);
        this.iconString = iconString;
        this.confirmationMessage = confirmationMessage;
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        addLink();
    }

    protected Component newLabelComponent(String id, IModel<String> iconString) {
        Label label = new Label(id);
        label.add(new AttributeAppender("class", Model.of(iconString)));
        return label;
    }

    private void addLink() {
        AjaxLink<T> link = newLinkComponent(LINK, getModel());
        link.add(newLabelComponent(LABEL, iconString));
        add(link);
    }

    protected AjaxLink<T> newLinkComponent(final String id, IModel<T> model) {
        return new AjaxLink<T>(id, model) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                AjaxIconLinkPanel.this.onClick(target, getModel());
            }

            @Override
            protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                super.updateAjaxAttributes(attributes);
                if (confirmationMessage!=null){
                    AjaxCallListener ajaxCallListener = new AjaxCallListener();
                    ajaxCallListener.onPrecondition("return confirm('" + confirmationMessage + "');");
                    attributes.getAjaxCallListeners().add(ajaxCallListener);
                }
            }
        };
    }
}
