package se.su.dsv.scipro.components;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.form.AjaxFormChoiceComponentUpdatingBehavior;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.RadioChoice;
import org.apache.wicket.model.IModel;

import java.util.List;

public abstract class AjaxRadioChoice<T> extends RadioChoice<T> {

    private String confirmationMessage;

    public AjaxRadioChoice(String id, IModel<T> model, List<? extends T> choices) {
        super(id, model, choices);
    }

    public AjaxRadioChoice(String id, IModel<T> model, List<? extends T> choices, IChoiceRenderer<? super T> renderer) {
        super(id, model, choices, renderer);
    }

    public AjaxRadioChoice(String id, IModel<T> model, List<? extends T> choices, IChoiceRenderer<? super T> renderer, String confirmationMessage) {
        super(id, model, choices, renderer);
        this.confirmationMessage = confirmationMessage;
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        setLabelPosition(LabelPosition.WRAP_AFTER);
        add(new AjaxFormChoiceComponentUpdatingBehavior() {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                update(target);
            }

            @Override
            protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                super.updateAjaxAttributes(attributes);
                if (confirmationMessage != null) {
                    addConfirmation(attributes);
                }
            }
        });
    }

    protected abstract void update(AjaxRequestTarget target);

    protected void addConfirmation(AjaxRequestAttributes attributes) {
            AjaxCallListener ajaxCallListener = new AjaxCallListener();
            ajaxCallListener.onPrecondition("if (!confirm('" + confirmationMessage.replaceAll("'", "\\\\'") + "')) {window.location.reload(); return false;}");
            attributes.getAjaxCallListeners().add(ajaxCallListener);
    }
}
