package se.su.dsv.scipro.components;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.joda.time.LocalDate;
import org.joda.time.LocalTime;

import java.util.Date;

public class BootstrapDateTimeComponent extends FormComponentPanel<Date> {

    private FormComponent<Date> dateField;
    private FormComponent<Date> timeField;

    public BootstrapDateTimeComponent(final String id) {
        super(id);
    }

    public BootstrapDateTimeComponent(final String id, final IModel<Date> model) {
        super(id, model);
    }

    @Override
    protected void onInitialize() {
        super.onInitialize();
        dateField = new BootstrapDateField("date") {
            @Override
            protected void onConfigure() {
                setRequired(BootstrapDateTimeComponent.this.isRequired());
            }
        };
        dateField.setModel(getModel());
        dateField.add(new BootstrapDatePicker());
        add(dateField);

        add(new Label("dateHelp", dateHelpText()) {
            @Override
            protected void onConfigure() {
                setVisible(!getDefaultModelObjectAsString().isEmpty());
            }
        });

        add(new WebMarkupContainer("dateRequired") {
            @Override
            protected void onConfigure() {
                setVisible(BootstrapDateTimeComponent.this.isRequired());
            }
        });

        timeField = new BootstrapTimeField("time") {
            @Override
            protected void onConfigure() {
                setRequired(BootstrapDateTimeComponent.this.isRequired());
            }
        };
        timeField.setModel(getModel());
        add(timeField);

        add(new WebMarkupContainer("timeRequired") {
            @Override
            protected void onConfigure() {
                setVisible(BootstrapDateTimeComponent.this.isRequired());
            }
        });
    }

    protected IModel<String> dateHelpText() {
        return new Model<>();
    }

    @Override
    protected void convertInput() {
        LocalDate date = new LocalDate(dateField.getConvertedInput());
        LocalTime time = new LocalTime(timeField.getConvertedInput());
        Date datetime = date.toLocalDateTime(time).toDate();
        setConvertedInput(datetime);
    }
}
