package se.su.dsv.scipro.components;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.validation.validator.StringValidator;

public class CounterTextArea extends FormComponentPanel<String> {

    public static final String INPUT = "input";
    public static final String COUNT_LABEL = "countLabel";

    private TextArea<String> textArea;
    private final int max;

    public CounterTextArea(String id, IModel<String> model, int max) {
        super(id, model);
        this.max = max;
        textArea = new TextArea<>(INPUT, getModel());
        add(textArea);
        add(StringValidator.maximumLength(max));
        addFeedbackPanel();
        addCounter();
    }

    @Override
    protected void convertInput() {
        setConvertedInput(textArea.getConvertedInput());
    }

    private void addFeedbackPanel() {
        ComponentFeedbackPanel componentFeedbackPanel = new ComponentFeedbackPanel("feedback", this);
        add(componentFeedbackPanel.setOutputMarkupId(true));
    }

    private void addCounter() {
        final Label countLabel = new Label(COUNT_LABEL, new AbstractReadOnlyModel<Integer>() {
            @Override
            public Integer getObject() {
                String text = textArea.getInput();
                return max - (text != null ? text.length() : 0);
            }
        });
        countLabel.setOutputMarkupId(true);
        AjaxFormComponentUpdatingBehavior behavior = new AjaxFormComponentUpdatingBehavior("onkeyup") {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                updateCount(target);
            }
            @Override
            protected void onError(AjaxRequestTarget target, RuntimeException e) {
                updateCount(target);
            }

            private void updateCount(AjaxRequestTarget target) {
                target.add(countLabel);
            }
        };
        add(countLabel);
        textArea.add(behavior);
    }
}
