package se.su.dsv.scipro.components;

import org.apache.commons.lang3.StringUtils;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.IModel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

public class ListField<T> extends TextField<List<? extends T>> {

    public static final String SEPARATOR = ", ";

    private Class<T> type;

    public ListField(String id) {
        super(id);
    }

    public ListField(String id, IModel<List<? extends T>> model, Class<T> type) {
        super(id, model);

        this.type = type;
    }

    private String convertToString(T object, Locale locale) {
        return getConverter(type).convertToString(object, locale);
    }

    private T convertToObject(String string, Locale locale) {
        return getConverter(type).convertToObject(string, locale);
    }

    @Override
    protected String getModelValue() {
        StringBuilder value = new StringBuilder();
        for (Iterator<? extends T> iterator = getModelObject().iterator(); iterator.hasNext(); ) {
            value.append(convertToString(iterator.next(), getLocale()));
            if (iterator.hasNext()) {
                value.append(SEPARATOR);
            }
        }
        return value.toString();
    }

    @Override
    protected void convertInput() {
        List<T> values = new ArrayList<>();

        if (getInput() != null) {
            String[] inputs = getInput().split(SEPARATOR.trim()); // trim spaces from separator since users may not add it
            for (String input1 : inputs) {
                String input = input1.trim();
                if (!StringUtils.isEmpty(input)) {
                    values.add(convertToObject(input1.trim(), getLocale()));
                }
            }
        }

        setConvertedInput(values);
    }
}
