package se.su.dsv.scipro.components;

import com.vaynberg.wicket.select2.Response;
import com.vaynberg.wicket.select2.TextChoiceProvider;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class UserAutoComplete extends AutoCompleteChoice<User> {

    @Inject
    private UserService userService;

    public UserAutoComplete(String id) {
        super(id);
    }

    public UserAutoComplete(String id, IModel<User> userModel) {
        super(id);
        setModel(userModel);
    }

    @Override
    protected TextChoiceProvider<User> getChoiceProvider() {
        return new UserProvider();
    }

    private class UserProvider extends TextChoiceProvider<User> {

        @Override
        protected String getDisplayText(User user) {
            StringBuilder sb = new StringBuilder();
            sb.append(user.getFullName());
            if (user.getEmailAddress() != null) {
                sb.append(" <");
                sb.append(user.getEmailAddress());
                sb.append(">");
            }
            return sb.toString();
        }

        @Override
        protected Object getId(User choice) {
            return choice.getId();
        }

        @Override
        public void query(String term, int page, Response<User> response) {
            response.addAll(userService.getAutoCompleteUsers(term, MAX_RESULTS));
        }

        @Override
        public Collection<User> toChoices(Collection<String> ids) {
            List<User> users = new ArrayList<>();
            for (String id : ids) {
                users.add(userService.findOne(Long.valueOf(id)));
            }
            return users;
        }
    }
}
