package se.su.dsv.scipro.datatables;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

public abstract class ClickableIconColumn<T> extends AbstractColumn<T, String> {
    public static final String DELETE = "glyphicon glyphicon-remove";
    public static final String EDIT = "glyphicon glyphicon-edit";

    private final String iconString;
    private final String confirmMessage;

    public ClickableIconColumn(IModel<String> displayModel, String sort, String iconString) {
       this(displayModel, sort, iconString, null);
    }
    
    public ClickableIconColumn(IModel<String> displayModel, String sort, String iconString, String confirmMessage) {
    	super(displayModel, sort);
        this.iconString = iconString;
    	this.confirmMessage = confirmMessage;
    }

    @Override
    public void populateItem(Item<ICellPopulator<T>> cellItem,
                             String componentId, IModel<T> rowModel) {
        cellItem.add(new LinkPanel(componentId, rowModel,
                getIconString()));
    }

    protected abstract void onClick(IModel<T> clicked, AjaxRequestTarget target);

    /**
     * Override this if you want to customize the visibility of the icon. 
     * e.g. the case where you want some specific rows editable based on certain conditions.
     */
    protected boolean shouldBeVisible(IModel<T> rowModel) {
    	return true;
    }

    protected String getIconString() {
        return iconString;
    }
    
    private class LinkPanel extends Panel {

        public LinkPanel(String id, IModel<T> rowModel, String iconString) {
            super(id);

            AjaxLink<T> link;
            
            if(confirmMessage!=null) {
            	link = new AjaxConfirmationLink<T>("link", rowModel, confirmMessage) {

                    @Override
					public void onClick(AjaxRequestTarget target) {
	                    ClickableIconColumn.this.onClick(getModel(), target);
					}
				};
            } else {
            	link = new AjaxLink<T>("link", rowModel) {

                    @Override
					public void onClick(AjaxRequestTarget target) {
						ClickableIconColumn.this.onClick(getModel(), target);
					}
            	};
            }
            add(link);
            Label label = new Label("label");
            label.add(new AttributeAppender("class", Model.of(iconString)));
            link.add(label);
            link.setVisible(shouldBeVisible(rowModel));
        }
    }
}