package se.su.dsv.scipro.datatables.project;

import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.supervisor.pages.SupervisorProjectDetailsPage;
import se.su.dsv.scipro.util.PageParameterKeys;

public class ProjectTitleColumn extends AbstractExportableColumn<Project, String, String> {

    public ProjectTitleColumn(IModel<String> displayModel, String sort) {
        super(displayModel, sort);
    }

    @Override
    public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
        cellItem.add(new LinkPanel(componentId, rowModel));
    }

    @Override
    public IModel<String> getDataModel(IModel<Project> projectIModel) {
        return Model.of(projectIModel.getObject().getTitle());
    }

    private static class LinkPanel extends Panel {
        public LinkPanel(String id, IModel<Project> rowModel) {
            super(id);

            Project project = rowModel.getObject();

            PageParameters pp = new PageParameters();
            pp.add(PageParameterKeys.MAP.get(Project.class), project.getId());
            BookmarkablePageLink<Void> projectLink = new BookmarkablePageLink<>("link", SupervisorProjectDetailsPage.class, pp);

            Label title = new Label("label", project.getTitle().length() < 31 ? project.getTitle() : project.getTitle().substring(0, 31) + "...");
            title.add(new AttributeAppender("title", Model.of(project.getTitle())));
            projectLink.add(title);
            add(projectLink);
        }
    }
}