package se.su.dsv.scipro.date;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.BootstrapDatePicker;

import javax.inject.Inject;
import java.util.Date;

public class DatePickerPanel extends Panel {

    public static final String DATE_FORM = "dateForm";
    public static final String START_DATE = "startDate";
    public static final String END_DATE = "endDate";

    @Inject
    DateService dateService;

    private FormComponent<Date> startDtf;
    private FormComponent<Date> endDtf;
    private CompoundPropertyModel<DatePickerModel> model;
    private DatePickerModel dpm;

    public Date getStartDate() {
        return dpm.getStartDate();
    }

    public Date getEndDate() {
        return dpm.getEndDate();
    }

    /**
     * @param id        markup id
     * @param startDate the start date to be set, end date will be todays date.
     */
    public DatePickerPanel(String id, Date startDate) {
        this(id, startDate, null);
    }

    /**
     * Creates a DatePickerPanel with startDate as start date and endDate as endDate.
     *
     * @param id        the wicket markup id
     * @param startDate the start date to be set
     * @param endDate   the end date to be set
     */
    public DatePickerPanel(String id, Date startDate, Date endDate) {
        this(id, Model.of(startDate), Model.of(endDate));
    }

    public DatePickerPanel(String id, IModel<Date> startDate, IModel<Date> endDate) {
        super(id);


        dpm = new DatePickerModel(startDate.getObject(), endDate.getObject());
        model = new CompoundPropertyModel<>(dpm);


        Form<Date> dateForm = new Form<Date>(DATE_FORM) {

            @Override
            protected void onSubmit() {
                model.getObject().setStartDate(startDtf.getModelObject());
                model.getObject().setEndDate(endDtf.getModelObject());

            }
        };

        startDtf = new BootstrapDateField(START_DATE, startDate);
        endDtf = new BootstrapDateField(END_DATE, endDate);


        OnChangeAjaxBehavior startDtfOnChange = new OnChangeAjaxBehavior() {

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                setStartDate(startDtf.getModelObject());
                updateStartDate(target);
            }
        };

        OnChangeAjaxBehavior endDtfOnChangeAjaxBehavior = new OnChangeAjaxBehavior() {


            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                setEndDate(endDtf.getModelObject());
                updateEndDate(target);
            }
        };

        startDtf.add(startDtfOnChange);
        startDtf.add(new BootstrapDatePicker());
        endDtf.add(endDtfOnChangeAjaxBehavior);
        endDtf.add(new BootstrapDatePicker());

        dateForm.add(startDtf);

        dateForm.add(endDtf);

        add(dateForm);

    }

    public void setStartDate(Date startDate) {
        dpm.setStartDate(startDate);
        model.getObject().setStartDate(dpm.getStartDate());
        startDtf.setModelObject(dpm.getStartDate());
    }

    public void setEndDate(Date endDate) {

        dpm.setEndDate(endDate);
        endDtf.setModelObject(dpm.getEndDate());
    }

    public void updateStartDate(AjaxRequestTarget target) {}

    public void updateEndDate(AjaxRequestTarget target) {}
}
