package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.*;
import org.apache.wicket.validation.validator.UrlValidator;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.ProjectTypeSettings;

import javax.inject.Inject;
import java.util.List;

public class AdminFinalSeminarSettingsPage extends AbstractAdminSystemPage {
    static final String SETTINGS_FORM = "seminarSettingsForm";
    static final String SEMINAR_CREATE_DAYS_AHEAD = "daysAheadToCreate";
    static final String SEMINAR_MIN_DAYS_REGISTER_ACTIVE = "daysAheadToRegisterParticipation";
    static final String SEMINAR_MIN_DAYS_REGISTER_OPPONENT = "daysAheadToRegisterOpposition";
    static final String SEMINAR_THESIS_DAYS_AHEAD_LABEL = "finalSeminarThesisDaysAheadLabel";
    static final String SEMINAR_THESIS_DAYS_AHEAD = "daysAheadToUploadThesis";
    static final String SEMINAR_MAX_OPPONENTS = "maxOpponentsOnFinalSeminar";
    static final String SEMINAR_MAX_PARTICIPANTS = "maxFinalSeminarActiveParticipation";
    static final String SEMINAR_MIN_OPPONENTS = "minOpponentsOnFinalSeminar";
    static final String SEMINAR_MIN_PARTICIPANTS = "minFinalSeminarActiveParticipation";
    static final String SEMINAR_PDF = "thesisMustBePDF";
    static final String PROJECT_TYPE_LIST = "pcListView";
    static final String PROJECT_TYPE = "pcName";
    static final String PROJECT_TYPE_FORM = "pcForm";
    static final String EVALUATION_URL = "evaluationURL";

    @Inject
    private ProjectTypeService projectTypeService;

    @Inject
    private FinalSeminarSettingsService finalSeminarSettingsService;

    public AdminFinalSeminarSettingsPage() {
        FinalSeminarSettingsForm form = new FinalSeminarSettingsForm(SETTINGS_FORM, getFinalSeminarSettings());
        form.add(new ListView<ProjectType>(PROJECT_TYPE_LIST, getProjectTypes()) {
            @Override
            protected void populateItem(ListItem<ProjectType> item) {
                item.add(new Label(PROJECT_TYPE, item.getModelObject().getName()));
                item.add(new SeminarLevelSettingsForm(PROJECT_TYPE_FORM, item.getModel()));
            }
        });
        add(form);
    }

    private IModel<FinalSeminarSettings> getFinalSeminarSettings() {
        return new CompoundPropertyModel<>(new LoadableDetachableModel<FinalSeminarSettings>() {
            @Override
            protected FinalSeminarSettings load() {
                return finalSeminarSettingsService.getInstance();
            }
        });
    }

    private LoadableDetachableModel<List<? extends ProjectType>> getProjectTypes() {
        return new LoadableDetachableModel<List<? extends ProjectType>>() {
            @Override
            protected List<? extends ProjectType> load() {
                return projectTypeService.findAllActive();
            }
        };
    }

    private class FinalSeminarSettingsForm extends Form<FinalSeminarSettings> {

        public FinalSeminarSettingsForm(String id, IModel<FinalSeminarSettings> model) {
            super(id, model);
            add(new RequiredTextField<>(SEMINAR_MIN_DAYS_REGISTER_OPPONENT, Integer.class));
            add(new RequiredTextField<>(SEMINAR_MIN_DAYS_REGISTER_ACTIVE, Integer.class));
            add(new RequiredTextField<>(SEMINAR_CREATE_DAYS_AHEAD, Integer.class));
            add(new Label(SEMINAR_THESIS_DAYS_AHEAD_LABEL, new StringResourceModel(SEMINAR_THESIS_DAYS_AHEAD_LABEL, this, null)));
            add(new RequiredTextField<>(SEMINAR_THESIS_DAYS_AHEAD, Integer.class));
            add(new CheckBox(SEMINAR_PDF));
            add(new RequiredTextField<>("oppositionPriorityDays", Integer.class));
            TextField<String> evaluationURL = new TextField<>(EVALUATION_URL);
            evaluationURL.add(new UrlValidator(new String[]{"http", "https"}));
            add(evaluationURL);
        }

        @Override
        protected void onSubmit() {
            finalSeminarSettingsService.save(getModelObject());
            info("Settings saved");
        }

    }

    private class SeminarLevelSettingsForm extends Form<ProjectTypeSettings> {

        public SeminarLevelSettingsForm(String id, IModel<ProjectType> model) {
            super(id, new CompoundPropertyModel<>(new PropertyModel<ProjectTypeSettings>(model, "projectTypeSettings")));
            add(new RequiredTextField<Integer>(SEMINAR_MAX_OPPONENTS));
            add(new RequiredTextField<Integer>(SEMINAR_MAX_PARTICIPANTS));
            add(new RequiredTextField<Integer>(SEMINAR_MIN_OPPONENTS));
            add(new RequiredTextField<Integer>(SEMINAR_MIN_PARTICIPANTS));
        }

        @Override
        protected void onSubmit() {
            projectTypeService.save(getModelObject().getProjectType());
        }

    }
}