package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;

@ProjectModuleComponent(ProjectModule.FINAL_SEMINAR)
public class OverviewSeminarPanel extends Panel {

    public static final String NOSEMINAR = "noSeminar";
    public static final String CREATE = "create";
    public static final String HASSEMINAR = "hasSeminar";
    public static final String DATE = "date";
    public static final String ROOM = "room";
    public static final String VIEW = "view";

    @Inject
    private FinalSeminarService finalSeminarService;

    public OverviewSeminarPanel(String id, final IModel<Project> model) {
        super(id, model);

        final FinalSeminar seminar = finalSeminarService.findByProject(model.getObject());

        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(model.getObject().getId()));

        WebMarkupContainer noSeminar;
        noSeminar = new WebMarkupContainer(NOSEMINAR) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(seminar == null);
            }
        };
        add(noSeminar);

        BookmarkablePageLink<Void> create = new BookmarkablePageLink<Void>(CREATE, SupervisorFinalSeminarPage.class, pp) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(SciProSession.get().getUser().equals(model.getObject().getHeadSupervisor()));
            }
        };
        noSeminar.add(create);

        WebMarkupContainer hasSeminar;
        hasSeminar = new WebMarkupContainer(HASSEMINAR) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(seminar != null);
            }
        };
        add(hasSeminar);

        if (seminar != null) {
            hasSeminar.add(new DateLabel(DATE, new PropertyModel<Date>(seminar, "startDate"), DateStyle.DATETIME));
            hasSeminar.add(new Label(ROOM, finalSeminarService.findByProject(model.getObject()).getRoom()));
        } else {
            hasSeminar.add(new EmptyPanel(DATE));
            hasSeminar.add(new EmptyPanel(ROOM));
        }

        BookmarkablePageLink<Void> view;
        if (!model.getObject().isParticipant(SciProSession.get().getUser())) {
            view = new BookmarkablePageLink<>(VIEW, SupervisorFinalSeminarPage.class, pp);
        } else {
            view = new BookmarkablePageLink<>(VIEW, ProjectFinalSeminarPage.class, pp);
        }
        hasSeminar.add(view);
    }
}
