package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import java.util.List;
import java.util.Set;

public class ProjectDetailsPanel extends Panel {

    public static final String PROJECT_TYPE = "projectType";
    public static final String PROJECT_HEAD_SUPERVISOR = "headSupervisor";
    public static final String PROJECT_CO_SUPERVISOR = "coSupervisor";
    public static final String PROJECT_CO_SUPERVISORS = "coSupervisors";
    public static final String PROJECT_REVIEWER = "reviewer";
    public static final String PROJECT_REVIEWERS = "reviewers";

    public ProjectDetailsPanel(String id, IModel<Project> projectModel) {
        super(id, projectModel);

        add(new Label(PROJECT_TYPE, new PropertyModel<>(projectModel, "projectType.name")));
        add(new UserLinkPanel(PROJECT_HEAD_SUPERVISOR, new PropertyModel<User>(projectModel, "headSupervisor")));

        IModel<List<User>> coSupervisors = new ListAdapterModel<>(new PropertyModel<Set<User>>(projectModel, "coSupervisors"));
        add(new AutoHidingListView<User>(PROJECT_CO_SUPERVISORS, coSupervisors) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new UserLinkPanel(PROJECT_CO_SUPERVISOR, item.getModel()));
            }
        });

        IModel<List<User>> reviewers = new ListAdapterModel<>(new PropertyModel<Set<User>>(projectModel, "reviewers"));
        add(new AutoHidingListView<User>(PROJECT_REVIEWERS, reviewers) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new UserLinkPanel(PROJECT_REVIEWER, item.getModel()));
            }
        });
    }
}
