package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.LinkWrapper;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ProjectModuleComponent(ProjectModule.FINAL_SEMINAR)
public class ProjectOppositionPage extends AbstractProjectDetailsPage implements MenuHighlightAuthorMyProjects {

    @Inject
    private FinalSeminarService finalSeminarService;

    private FinalSeminarService.Filter filter;

    public ProjectOppositionPage(PageParameters pp) {
        super(pp);

        filter = new FinalSeminarService.Filter();
        filter.setFromDate(new Date());
        filter.setLazyDeleted(false);
        filter.setProjectType(getActiveProject().getProjectType());
        filter.setOnlyActiveProjects(true);

        add(new FeedbackPanel("feedback"));

        createDataPanel();

        add(new AttendingPanel("attending", projectModel));
    }

    private void createDataPanel() {
        SortableDataProvider<FinalSeminar, String> provider = new FilteredDataProvider<>(finalSeminarService, Model.of(this.filter));
        provider.setSort("startDate", SortOrder.DESCENDING);

        add(new ExportableDataPanel<>("select", createColumns(), provider));
    }

    public void oppose(FinalSeminar finalSeminar) {
        finalSeminarService.oppose(SciProSession.get().getUser(), finalSeminar, getActiveProject());
    }

    public void participate(FinalSeminar finalSeminar) {
        finalSeminarService.participate(SciProSession.get().getUser(), finalSeminar, getActiveProject());
    }

    private List<IColumn<FinalSeminar, String>> createColumns() {
        List<IColumn<FinalSeminar, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<FinalSeminar>(Model.of("Date"), "startDate", "startDate", DateStyle.DATETIME));
        columns.add(new PropertyColumn<FinalSeminar, String>(Model.of("Type"), "project.projectType.name", "project.projectType.name"));

        columns.add(new AbstractColumn<FinalSeminar, String>(Model.of("Title"), "project.title") {
            @Override
            public void populateItem(Item<ICellPopulator<FinalSeminar>> cellItem, String componentId, final IModel<FinalSeminar> rowModel) {
                cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                    @Override
                    public AbstractLink apply(final String id) {
                        final PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Project.class), rowModel.getObject().getProject().getId());
                        final BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(id, ProjectFinalSeminarDetailsPage.class, pp);
                        link.setBody(new PropertyModel<>(rowModel, "project.title"));
                        return link;
                    }
                }));
            }
        });

        columns.add(new AbstractColumn<FinalSeminar, String>(Model.of("Oppose"), null) {
            @Override
            public void populateItem(Item<ICellPopulator<FinalSeminar>> item, String s, final IModel<FinalSeminar> iModel) {
                item.add(new OpposeColumnPanel(s, iModel, projectModel) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<FinalSeminar> clickedModel) {
                        oppose(clickedModel.getObject());
                        getSession().success(getString("oppose", clickedModel));
                        setResponsePage(ProjectOppositionPage.class);
                    }
                });
            }
        });
        columns.add(new AbstractColumn<FinalSeminar, String>(Model.of("Participate"), null) {
            @Override
            public void populateItem(Item<ICellPopulator<FinalSeminar>> item, String s, final IModel<FinalSeminar> iModel) {
                item.add(new ParticipateColumnPanel(s, iModel) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<FinalSeminar> clickedModel) {
                        participate(clickedModel.getObject());
                        getSession().success(getString("participate", clickedModel));
                        setResponsePage(ProjectOppositionPage.class);
                    }
                });
            }
        });
        return columns;
    }

}
