package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorFinalSeminars;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorPage;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

public class SupervisorFinalSeminarDetailsPage extends AbstractSupervisorPage implements MenuHighlightSupervisorFinalSeminars {

    public static final String SEMINAR_PANEL = "seminarPanel";
    @Inject
    private ProjectService projectService;
    @Inject
    private FinalSeminarService finalSeminarService;

    @SuppressWarnings("UnusedDeclaration")
    public SupervisorFinalSeminarDetailsPage(PageParameters pp) {
        super(pp);
        DetachableServiceModel<FinalSeminar> seminarModel = new DetachableServiceModel<>(finalSeminarService, getSeminarFromProject(pp));
        init(seminarModel);
    }

    public SupervisorFinalSeminarDetailsPage(IModel<FinalSeminar> seminarModel) {
        init(seminarModel);
    }

    private void init(final IModel<FinalSeminar> seminarModel) {
        add(new SeminarPanel(SEMINAR_PANEL, seminarModel));
    }

    private FinalSeminar getSeminarFromProject(PageParameters pp) {
        StringValue projectId = pp.get(PageParameterKeys.MAP.get(Project.class));
        if (!projectId.isNull()) {
            Project project = projectService.findOne(projectId.toLong());
            if (project != null) {
                return finalSeminarService.findByProject(project);
            }
        }
        throw new RestartResponseException(getApplication().getHomePage());
    }
}
