package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightFinalSeminar;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMyProjects;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;

@Authorization(authorizedRoles = {Roles.SUPERVISOR})
@ProjectModuleComponent(ProjectModule.FINAL_SEMINAR)
public class SupervisorFinalSeminarPage extends AbstractSupervisorProjectDetailsPage implements MenuHighlightSupervisorMyProjects, MenuHighlightFinalSeminar {
    static final String PANEL = "panel";

    @Inject
    private FinalSeminarService finalSeminarService;

    public SupervisorFinalSeminarPage(final PageParameters pp) {
        super(pp);

        add(getSeminar().map(new AbstractFunction1<FinalSeminar, Component>() {
            @Override
            public Component apply(final FinalSeminar finalSeminar) {
                return new SeminarPanel(PANEL, new DetachableServiceModel<>(finalSeminarService, finalSeminar));
            }
        }).getOrElse(new AbstractFunction0<Component>() {
            @Override
            public Component apply() {
                return missingFinalSeminarComponent();
            }
        }));
    }

    private Option<FinalSeminar> getSeminar() {
        return Option.apply(finalSeminarService.findByProject(projectModel.getObject()));
    }

    private Component missingFinalSeminarComponent() {
        if (isHeadSupervisor()) {
            return createSeminarComponent();
        }
        else {
            return new Label(PANEL, new ResourceModel("no.seminar.created.yet"));
        }
    }

    private boolean isHeadSupervisor() {
        return projectModel.getObject().isSupervisor(SciProSession.get().getUser());
    }

    private Component createSeminarComponent() {
        Project project = projectModel.getObject();
        FinalSeminar finalSeminar = new FinalSeminar(project);
        return new SeminarPanel(PANEL, new DetachableServiceModel<>(finalSeminarService, finalSeminar));
    }
}