package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.plagiarism.PlagiarismControl;
import se.su.dsv.scipro.plagiarism.TurnItInCheckService;
import se.su.dsv.scipro.plagiarism.TurnitinSettings;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsService;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;

public class TurnItInPanel extends Panel {

    static final String TURN_IT_IN_LINK = "link";
    public static final String CONTAINER = "container";
    public static final String TOO_BIG_FILE_LABEL = "tooBigFileLabel";

    @Inject
    private TurnItInCheckService turnItInService;
    @Inject
    private PlagiarismControl plagiarismControl;
    @Inject
    private TurnitinSettingsService turnItInSettingsService;

    public TurnItInPanel(String id, final IModel<FinalSeminar> seminar) {
        super(id, seminar);

        WebMarkupContainer container = new WebMarkupContainer(CONTAINER) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(SciProSession.get().hasActualRole(Roles.SUPERVISOR) &&
                        seminar.getObject().getDocument() != null &&
                        turnItInService.findByFileDescription(seminar.getObject().getDocument()) != null &&
                        turnItInService.findByFileDescription(seminar.getObject().getDocument()).getTurnItInId() != null);
            }
        };

        add(new InfoPanel("info", new ResourceModel("info")));

        Link<Void> link = new Link<Void>(TURN_IT_IN_LINK) {
            @Override
            public void onClick() {
                String turnItInId = turnItInService.findByFileDescription(seminar.getObject().getDocument()).getTurnItInId();
                String url = plagiarismControl.returnReport(turnItInId);
                setResponsePage(new TurnitinPopupPage(url));
            }
        };
        container.add(link);

        add(container);

        add(new Label(TOO_BIG_FILE_LABEL, new ResourceModel("too.big")) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                TurnitinSettings settings = turnItInSettingsService.getSettings();
                FileDescription document = seminar.getObject().getDocument();
                setVisibilityAllowed(document != null && document.getSize() > settings.getMaxFileSizeInBytes() && isSupervisor(seminar));
            }
        });
    }

    private boolean isSupervisor(IModel<FinalSeminar> seminar) {
        return SciProSession.get().getUser().equals(seminar.getObject().getProject().getHeadSupervisor());
    }
}
