package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.files.FileDescriptionPanel;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.forum.pages.ProjectForumBasePage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;

import static se.su.dsv.scipro.finalthesis.FinalThesis.Status;

@ProjectModuleComponent(ProjectModule.GRADING)
public class FinalThesisPanel extends GenericPanel<Project> {
    public static final String UPLOAD_PANEL = "formPanel";
    public static final String DOWNLOAD = "download";
    public static final String REVISION_PANEL = "revisionPanel";
    public static final String REJECTED_DATE_LABEL = "rejectedDateLabel";
    public static final String FORUM_LINK = "forumLink";
    public static final String APPROVED_PANEL = "approvedPanel";
    public static final String APPROVED_DATE = "approvedDate";
    public static final String NO_DECISION_PANEL = "noDecisionPanel";

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private FinalSeminarService finalSeminarService;

    public FinalThesisPanel(String id, IModel<Project> project) {
        super(id, project);

        add(new FileDescriptionPanel(DOWNLOAD, getFinalThesisFileDescription()) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(isFinalThesisStatus(Status.NO_DECISION));
            }
        });
        add(new RevisionPanel(REVISION_PANEL));
        add(new ApprovedPanel(APPROVED_PANEL));
        add(new WebMarkupContainer(NO_DECISION_PANEL) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(isFinalThesisStatus(Status.NO_DECISION));
            }
        });
        add(new FinalThesisUploadFormPanel(UPLOAD_PANEL, project));
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(finalSeminarService.hasHadFinalSeminar(getModelObject()));
    }

    private class RevisionPanel extends Panel {
        public RevisionPanel(String id) {
            super(id);
            add(new DateLabel(REJECTED_DATE_LABEL, new PropertyModel<Date>(getFinalThesis(), "dateRejected")));
            PageParameters parameters = new PageParameters();
            parameters.set(PageParameterKeys.MAP.get(Project.class), getModelObject().getId());
            add(new BookmarkablePageLink<Void>(FORUM_LINK, ProjectForumBasePage.class, parameters));
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(isFinalThesisStatus(Status.REJECTED));
        }
    }

    private boolean isFinalThesisStatus(Status status) {
        return getFinalThesis().getObject() != null && getFinalThesis().getObject().getStatus().equals(status);
    }

    private class ApprovedPanel extends Panel {
        public ApprovedPanel(String id) {
            super(id);

            add(new DateLabel(APPROVED_DATE, new PropertyModel<Date>(getFinalThesis(), "dateApproved")));
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(isFinalThesisStatus(Status.APPROVED));
        }
    }

    private LoadableDetachableModel<FinalThesis> getFinalThesis() {
        return new LoadableDetachableModel<FinalThesis>() {
            @Override
            protected FinalThesis load() {
                return finalThesisService.findByProject(getModelObject());
            }
        };
    }

    private LoadableDetachableModel<FileDescription> getFinalThesisFileDescription() {
        return new LoadableDetachableModel<FileDescription>() {
            @Override
            protected FileDescription load() {
                return finalThesisService.getFinalThesisFileDescription(getModelObject());
            }
        };
    }

}
