package se.su.dsv.scipro.firstmeeting;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;

public abstract class FirstMeetingColumnPanel extends GenericPanel<Idea> {

    @Inject
    private DateService dateService;

	public FirstMeetingColumnPanel(String id, final IModel<Idea> model) {
		super(id, model);
		if (getModelObject().getMatchStatus().equals(Idea.Status.UNMATCHED)) {
			add(new Label("label", "-"));
		} else {
			add(new AjaxLinkPanel<Idea>("label", getModel(), getDisplayModel()) {
				@Override
				protected void onConfigure() {
					setEnabled(!(model.getObject().isInactive() || (!isSupervisor(getModelObject()) && getModelObject().getFirstMeeting() == null)));
				}

				@Override
				public void onClick(AjaxRequestTarget target, final IModel<Idea> model) {
					onColumnClick(target, model);
				}
			});
		}
	}

	private IModel<String> getDisplayModel() {
		if (getModelObject().getFirstMeeting() == null) {
			if (isSupervisor(getModelObject()) && !getModelObject().isInactive()) {
				return new ResourceModel("none_create_one");
			}
		} else {
            return new Model<>(dateService.format(getModelObject().getFirstMeeting().getFirstMeetingDate(), DateStyle.DATETIME));
		}
		return new ResourceModel("none");
	}

	protected abstract void onColumnClick(AjaxRequestTarget target, IModel<Idea> model);

	private boolean isSupervisor(Idea idea) {
		return SciProSession.get().getUser().equals(idea.getMatch().getSupervisor());
	}
}
