package se.su.dsv.scipro.forum.panels.threaded;

import lombok.experimental.Delegate;
import se.su.dsv.scipro.file.*;
import se.su.dsv.scipro.forum.ForumThread;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ProjectForumThread implements ForumThread<ProjectThread> {
    private final ProjectForumService projectForumService;
    private final FileDescriptionService fileDescriptionService;

    public ProjectForumThread(final ProjectForumService projectForumService, final FileDescriptionService fileDescriptionService) {
        this.projectForumService = projectForumService;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public List<ForumPost> getPosts(final ProjectThread projectThread) {
        return projectForumService.getPosts(projectThread);
    }

    @Override
    public ForumPost reply(final ProjectThread projectThread, final User poster, final String content, final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = store(projectThread.getProject(), attachments);
        return projectForumService.createReply(projectThread, poster, content, storedAttachments);
    }

    private Set<FileDescription> store(final Project project, final Set<FileUpload> attachments) {
        Set<FileDescription> storedAttachments = new HashSet<>(attachments.size());
        for (FileUpload attachment : attachments) {
            storedAttachments.add(fileDescriptionService.storeFile(new ForumFile(project, attachment), FileSource.FORUM));
        }
        return storedAttachments;
    }

    @Override
    public String getSubject(final ProjectThread projectThread) {
        return projectThread.getForumThread().getSubject();
    }

    @Override
    public void setRead(User user, ProjectThread thread) {
        projectForumService.markRead(user, thread);
    }

    private static class ForumFile implements ProjectFileUpload {
        private final Project project;
        @SuppressWarnings("UnusedDeclaration")
        @Delegate
        private final FileUpload fileUpload;

        public ForumFile(Project project, FileUpload fileUpload) {
            this.project = project;
            this.fileUpload = fileUpload;
        }

        @Override
        public Project getProject() {
            return project;
        }

        @Override
        public boolean isDeletable() {
            return false;
        }
    }
}
