package se.su.dsv.scipro.forum.panels.threaded;

import com.google.common.base.Function;
import com.google.common.collect.FluentIterable;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.forum.ForumThread;
import se.su.dsv.scipro.session.SciProSession;

import java.util.Set;

public class SubmitForumReplyPanel<A> extends GenericPanel<A> {

    final ForumThread<A> wicketForumService;

    protected void updateOnSubmit() { }

    public SubmitForumReplyPanel(String id, IModel<A> threadModel, final ForumThread<A> wicketForumService) {
        super(id, threadModel);
        this.wicketForumService = wicketForumService;
        add(new ForumForm("form"));
    }

    private class ForumForm extends StatelessForm<Void> {

        private final FileUploadField upload;
        private final FormComponent<String> contentArea;

        public ForumForm(String id) {
            super(id);

            final FeedbackPanel feedbackPanel = new FeedbackPanel("feedback");
            feedbackPanel.setOutputMarkupId(true);
            add(feedbackPanel);

            contentArea = new TextArea<>("content", new Model<String>()).setRequired(true);
            add(contentArea);

            upload = new FileUploadField("attachment");
            add(upload);
        }

        @Override
        protected void onSubmit() {
            Set<FileUpload> attachments = FluentIterable.from(upload.getFileUploads())
                    .transform(new Function<org.apache.wicket.markup.html.form.upload.FileUpload, FileUpload>() {
                        @Override
                        public FileUpload apply(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
                            return WicketFileUpload.from(fileUpload);
                        }
                    })
                    .toSet();
            wicketForumService.reply(SubmitForumReplyPanel.this.getModelObject(), SciProSession.get().getUser(), contentArea.getModelObject(), attachments);
            contentArea.setModel(Model.of(""));
            updateOnSubmit();
        }
    }
}
