package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.forum.Discussable;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

public class ThreadsOverviewPanel<A> extends Panel {
    public ThreadsOverviewPanel(final String id, final IModel<List<A>> model, final Discussable<A> discussable, final ThreadLinkSupplier<A> threadLinkSupplier) {
        super(id, model);

        add(new ListView<A>("threads", model) {
            @Override
            protected void populateItem(final ListItem<A> item) {
                final IModel<ForumThread> discussion = discussable.discussion(item.getModel());
                item.add(new ThreadReadStatePanel("readState", discussion));
                item.add(new WebMarkupContainer("attachmentIcon") {
                    @Override
                    protected void onConfigure() {
                        setVisibilityAllowed(discussion.getObject().hasAttachments());
                    }
                });
                item.add(new ThreadTitleColumn<>("threadTitle", discussion, item.getModel(), threadLinkSupplier));
                item.add(new Label("replies", new PropertyModel<Integer>(discussion, "postCount")));
                item.add(new LastPostColumn("lastPost", discussion));
            }
        });
    }

    private static class ThreadTitleColumn<A> extends WebMarkupContainer {
        public ThreadTitleColumn(final String id, final IModel<ForumThread> thread, final IModel<A> model, final ThreadLinkSupplier<A> threadLinkSupplier) {
            super(id);
            final BookmarkablePageLink<Void> threadLink = threadLinkSupplier.newThreadLink("threadLink", model);
            threadLink.setBody(new PropertyModel<String>(thread, "subject"));
            add(threadLink);
            add(new Label("createdBy", new PropertyModel<User>(thread, "createdBy.fullName")));
            add(new DateLabel("dateCreated", new PropertyModel<Date>(thread, "dateCreated"), DateStyle.DATETIME));
        }
    }

    public interface ThreadLinkSupplier<A> extends Serializable {
        BookmarkablePageLink<Void> newThreadLink(String id, IModel<A> thread);
    }

    private static class LastPostColumn extends WebMarkupContainer {
        public LastPostColumn(String id, final IModel<ForumThread> model) {
            super(id);
            IModel<ForumPost> postModel = getLastPost(model);
            add(new UserLinkPanel("postedBy", new PropertyModel<User>(postModel, "postedBy")));
            add(new DateLabel("lastPostDate", new PropertyModel<Date>(postModel, "dateCreated"), DateStyle.DATETIME));
        }

        private IModel<ForumPost> getLastPost(final IModel<ForumThread> model) {
            return new LoadableDetachableModel<ForumPost>() {
                @Override
                protected ForumPost load() {
                    return Collections.max(model.getObject().getPosts(), new Comparator<ForumPost>() {
                        @Override
                        public int compare(final ForumPost o1, final ForumPost o2) {
                            return new CompareToBuilder()
                                    .append(o1.getDateCreated(), o2.getDateCreated())
                                    .append(o1.getId(), o2.getId())
                                    .toComparison();
                        }
                    });
                }
            };
        }
    }
}
