package se.su.dsv.scipro.forum.panels.unthreaded;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxFallbackButton;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import scala.Option;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.grading.GradingReportPdfResourceFileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.Set;

public class GradingCreateForumThreadPanel extends Panel {
    @Inject
    private ProjectForumService projectForumService;

    private final IModel<GradingReportPdfResourceFileUpload> gradingReport;

    public GradingCreateForumThreadPanel(String id, IModel<Project> model, IModel<GradingReportPdfResourceFileUpload> gradingReport) {
        super(id, model);
        this.gradingReport = gradingReport;
        add(new GradingCreateForumThreadForm("form", model));
    }

    protected void threadCreated(ProjectThread projectThread) {
    }

    private class GradingCreateForumThreadForm extends Form<Project> {
        private final FileUploadField upload;
        private final FeedbackPanel feedbackPanel;
        private IModel<String> subject = new Model<>();
        private IModel<String> content = new Model<>();

        public GradingCreateForumThreadForm(final String id, final IModel<Project> model) {
            super(id, model);

            feedbackPanel = new FeedbackPanel("feedback");
            feedbackPanel.setOutputMarkupId(true);
            add(feedbackPanel);

            add(new RequiredTextField<>("subject", subject));
            add(new TextArea<>("content", content).setRequired(true));

            upload = new FileUploadField("attachment");
            add(upload);

            add(new AjaxFallbackButton("create", this) {
                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedbackPanel);
                }
            });
        }

        @Override
        protected void onSubmit() {
            User user = SciProSession.get().getUser();

            Option<FileUpload> fileUpload = WicketFileUpload.ofOptional(upload.getFileUpload());

            Set<FileUpload> uploads = new HashSet<>();
            uploads.add(gradingReport.getObject());
            if (fileUpload.isDefined()) {
                uploads.add(fileUpload.get());
            }

            ProjectThread projectThread =
                    projectForumService.createThread(getModelObject(), user, subject.getObject(), content.getObject(), uploads);

            threadCreated(projectThread);
        }

    }
}
