package se.su.dsv.scipro.grading;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.AbstractTextComponent;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import se.su.dsv.scipro.components.AutoSavingBehaviour;
import se.su.dsv.scipro.report.GradingReport;
import se.su.dsv.scipro.report.GradingReportService;

import javax.inject.Inject;

public class AutoSavingCriteriaPanel extends CriteriaPanel {

    @Inject
    private GradingReportService gradingReportService;

    public AutoSavingCriteriaPanel(String id, final IModel<? extends GradingReport> gradingReport) {
        super(id, gradingReport);
    }

    @Override
    protected void onBeforeRender() {
        super.onBeforeRender();
        visitChildren(AbstractTextComponent.class, new IVisitor<FormComponent, Void>() {
            @Override
            public void component(final FormComponent object, final IVisit<Void> visit) {
                object.add(new GradingReportAutoSavingBehavior(gradingReportService, getModel()));
            }
        });
    }

    // Has to be a standalone class or we get infinite looping serialization problems
    private static final class GradingReportAutoSavingBehavior extends AutoSavingBehaviour {
        private final GradingReportService gradingReportService;
        private final IModel<? extends GradingReport> report;

        private GradingReportAutoSavingBehavior(
                final GradingReportService gradingReportService,
                final IModel<? extends GradingReport> report)
        {
            this.gradingReportService = gradingReportService;
            this.report = report;
        }

        @Override
        protected void onUpdate(AjaxRequestTarget target) {
            gradingReportService.save(report.getObject());
        }
    }
}
