package se.su.dsv.scipro.grading;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.SubmitLink;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.link.ResourceLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.*;
import org.apache.wicket.request.handler.resource.ResourceRequestHandler;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AutoSavingBehaviour;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.forum.pages.threaded.SupervisorViewForumThreadPage;
import se.su.dsv.scipro.forum.panels.unthreaded.GradingCreateForumThreadPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.*;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.util.*;

public class FillOutGradingReportPanel extends GenericPanel<Project> {
    public static final String FORM = "form";
    public static final String PDF_DOWNLOAD = "pdfDownload";
    public static final String GRADING_REPORTS = "gradingReports";
    public static final String NAVIGATION_TABS = "navigationTabs";
    public static final String TAB_LINK = "tabLink";
    public static final String REPORT_TAB = "reportTab";
    public static final String PLAGIARISM_FEEDBACK = "plagiarismFeedback";
    public static final String COPY = "copy";
    public static final String CRITERIA_PANEL = "criteriaPanel";
    public static final String SAVE = "save";
    public static final String POINTS_PANEL = "pointsPanel";

    public static final String OTHER_REPORTS = "otherReports";
    public static final String TEMPLATE_PANEL = "templatePanel";

    @Inject
    private GradingReportService gradingReportService;

    private ModalWindowPlus modalWindowPlus;

    private final IModel<Project> project;

    public FillOutGradingReportPanel(String id, final IModel<Project> project) {
        super(id, project);

        this.project = project;

        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);

        final IModel<List<SupervisorGradingReport>> gradingReports = getGradingReports();

        add(new ListView<SupervisorGradingReport>(GRADING_REPORTS, gradingReports) {
            {
                setReuseItems(true);
            }

            @Override
            protected void populateItem(ListItem<SupervisorGradingReport> item) {
                item.setMarkupId("author" + item.getIndex());
                if (item.getIndex() == 0) {
                    item.add(AttributeModifier.append("class", "active"));
                }
                item.add(new GradingReportForm(FORM, item.getModel()));
            }
        });

        add(new ListView<User>(NAVIGATION_TABS, getStudents()) {
            @Override
            protected void populateItem(final ListItem<User> item) {
                if (item.getIndex() == 0) {
                    item.add(AttributeModifier.append("class", "active"));
                }
                item.add(new ExternalLink(TAB_LINK, Model.of("#author" + item.getIndex()), new PropertyModel<String>(item.getModel(), "fullName")));
            }
        });
    }

    public class GradingReportForm extends Form<SupervisorGradingReport> {
        public GradingReportForm(String id, IModel<SupervisorGradingReport> gradingReports) {
            super(id, gradingReports);
            add(new ReportTab(REPORT_TAB, getModel()));

            final Button save = new Button("save") {
                @Override
                public void onAfterSubmit() {
                    String dateString = new SimpleDateFormat("HH:mm:ss").format(new Date());
                    success(getString("saved") + " " + dateString);
                }
            };
            add(save);

            final FeedbackPanel feedbackPanel = new FencedFeedbackPanel("feedbackPanel", this);
            feedbackPanel.setMaxMessages(1);
            add(feedbackPanel);
        }

        @Override
        protected void onSubmit() {
            if (!getModelObject().isSubmitted()) {
                gradingReportService.save(getModelObject());
            }
        }
    }

    private List<? extends User> getStudents() {
        SortedSet<User> projectParticipants = getModelObject().getProjectParticipants();
        return new ArrayList<>(projectParticipants);
    }

    private IModel<List<SupervisorGradingReport>> getGradingReports() {
        return new LoadableDetachableModel<List<SupervisorGradingReport>>() {
            @Override
            protected List<SupervisorGradingReport> load() {
                return gradingReportService.getSupervisorGradingReports(getModelObject());
            }
        };
    }

    private class ReportTab extends WebMarkupContainer {

        public ReportTab(String id, final IModel<SupervisorGradingReport> gradingReportIModel) {
            super(id, new CompoundPropertyModel<>(gradingReportIModel));

            addCopyLink(gradingReportIModel);

            add(new AjaxSubmitLink("postInForum") {
                @Override
                public void onAfterSubmit(AjaxRequestTarget target, final Form<?> form) {

                    GradingReportPdfResourceFileUpload gradingReportPdfResourceFileUpload = new GradingReportPdfResourceFileUpload(
                            new SupervisorGradingReportPdf(gradingReportIModel, getGradeCalculator()),
                            SciProSession.get().getUser());

                    modalWindowPlus.setContent(new GradingCreateForumThreadPanel(modalWindowPlus.getContentId(), project, Model.of(gradingReportPdfResourceFileUpload)){
                        @Override
                        protected void threadCreated(ProjectThread thread) {
                            PageParameters parameters = new PageParameters().set(PageParameterKeys.MAP.get(ForumThread.class), thread.getId());
                            setResponsePage(SupervisorViewForumThreadPage.class, parameters);
                        }
                    });

                    modalWindowPlus.show(target);
                }
            });

            add(new AutoSavingCriteriaPanel(CRITERIA_PANEL, gradingReportIModel) {
                @Override
                protected void onConfigure() {
                    setEnabled(!gradingReportIModel.getObject().isSubmitted());
                }
            });

            add(new TemplatePanel(TEMPLATE_PANEL, getModel()));

            add(new GradingReportPointsPanel(POINTS_PANEL, gradingReportIModel, getGradeCalculator()));

            TextArea<String> textArea = new TextArea<String>(PLAGIARISM_FEEDBACK) {
                @Override
                protected void onConfigure() {
                    setEnabled(!gradingReportIModel.getObject().isSubmitted());
                }
            };
            textArea.add(new AutoSavingBehaviour() {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    gradingReportService.save(gradingReportIModel.getObject());
                }
            });
            add(textArea);

            add(new SubmitLink(PDF_DOWNLOAD, gradingReportIModel){
                @Override
                public void onAfterSubmit() {
                    super.onAfterSubmit();
                    getRequestCycle().scheduleRequestHandlerAfterCurrent(new ResourceRequestHandler(new GradingReportPdfResource(
                            new SupervisorGradingReportPdf(gradingReportIModel, getGradeCalculator())), null));
                }
            });

            add(new ResourceLink<>("washedReport",
                    new GradingReportPdfResource(
                            new NoPointNoFeedbackGradingReportPdf(gradingReportIModel))));

            addNextStep(gradingReportIModel);

            add(new ExternalLink("daisy", "https://daisy.dsv.su.se/anstalld/uppsats/visaBetyg.jspa?uppsatstillfID=" + project.getObject().getIdentifier(), "Grading in Daisy") {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(project.getObject().getIdentifier()!=null);
                }
            });
        }

        private void addNextStep(final IModel<SupervisorGradingReport> gradingReportIModel) {
            switch (gradingReportIModel.getObject().getState()) {
                case INITIAL:
                    addOrReplace(new InitialFragment("nextStep", gradingReportIModel));
                    break;
                case REVIEWING:
                    addOrReplace(new ReviewingFragment("nextStep", gradingReportIModel));
                    break;
                case FINALIZED:
                    addOrReplace(new EmptyPanel("nextStep"));
                    break;
            }
        }

        private class InitialFragment extends Fragment {
            public InitialFragment(final String id, final IModel<SupervisorGradingReport> reportModel) {
                super(id, "initialStep", FillOutGradingReportPanel.this, reportModel);

                WebMarkupContainer dialog = new WebMarkupContainer("dialog");
                add(dialog);
                WebMarkupContainer show = new WebMarkupContainer("show");
                show.add(AttributeModifier.replace("data-target", "#" + dialog.getMarkupId(true)));
                add(show);

                dialog.add(new Button("initialAssessmentDone") {
                    @Override
                    public void onAfterSubmit() {
                        try {
                            gradingReportService.initialAssessmentDone(reportModel.getObject());
                            getForm().success("Initial assessment done for " + reportModel.getObject().getUser().getFullName());
                            addNextStep(reportModel);
                        }
                        catch (IllegalStateException ignored) {
                            getForm().error(getString("need.to.assess.all.criteria"));
                        }
                    }
                });
            }
        }

        private class ReviewingFragment extends Fragment {
            public ReviewingFragment(final String id, final IModel<SupervisorGradingReport> reportModel) {
                super(id, "reviewingStep", FillOutGradingReportPanel.this, reportModel);

                WebMarkupContainer dialog = new WebMarkupContainer("dialog");
                add(dialog);
                WebMarkupContainer show = new WebMarkupContainer("show");
                show.add(AttributeModifier.replace("data-target", "#" + dialog.getMarkupId(true)));
                add(show);

                dialog.add(new Button("finalize") {
                    @Override
                    public void onAfterSubmit() {
                        try {
                            gradingReportService.submitReport(reportModel.getObject());
                            getForm().success("Finalized grading report for " + reportModel.getObject().getUser().getFullName());
                            addNextStep(reportModel);
                        }
                        catch (IllegalStateException ignored) {
                            getForm().error(getString("not.finished"));
                        }
                    }
                });
            }
        }

        private void addCopyLink(final IModel<SupervisorGradingReport> gradingReportIModel) {
            add(new ListView<User>(OTHER_REPORTS, new ListAdapterModel<>(new PropertyModel<Set<User>>(getModel(), "projectParticipants"))) {
                @Override
                protected void populateItem(final ListItem<User> item) {
                    SubmitLink link = new SubmitLink(COPY, item.getModel()) {
                        @Override
                        protected void onConfigure() {
                            super.onConfigure();
                            setVisible(!item.getModelObject().equals(gradingReportIModel.getObject().getUser()) &&
                                    !gradingReportIModel.getObject().isSubmitted());
                        }

                        @Override
                        public void onAfterSubmit() {
                            SupervisorGradingReport otherReport = gradingReportService.getSupervisorGradingReport(FillOutGradingReportPanel.this.getModelObject(), item.getModelObject());
                            gradingReportService.copyReport(gradingReportIModel.getObject(), otherReport);
                            ReportTab.this.success(getString("copy"));
                        }
                    };
                    link.add(new JavascriptEventConfirmation("onclick", new StringResourceModel("confirmCopy", gradingReportIModel)));
                    link.setBody(new StringResourceModel("copyLabel", item.getModel()));
                    item.add(link);
                }
            });
        }
    }

    private GradeCalculator getGradeCalculator() {
        DegreeLevel degreeLevel = getModelObject().getProjectType().getDegreeLevel();
        if (degreeLevel.equals(DegreeLevel.FIRST_CYCLE)) {
            return new SupervisorBachelorGradeCalculator();
        } else if (degreeLevel.equals(DegreeLevel.SECOND_CYCLE)) {
            return getMasterGradeCalculator();
        }
        throw new IllegalStateException("No grade calculation implemented for project type");
    }

    private GradeCalculator getMasterGradeCalculator() {
        int credits = getModelObject().getCredits();
        if (credits == ProjectType.HP_15) {
            return new SupervisorMaster15GradeCalculator();
        } else if (credits == ProjectType.HP_30 || credits == 0) {
            return new SupervisorMaster30GradeCalculator();
        }
        throw new IllegalStateException("No grade calculation implemented for master project with " + credits + " credits");
    }
}
