package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.report.GradeCalculator;
import se.su.dsv.scipro.report.GradingReport;

public class GradingReportPointsPanel extends Panel {

    public static final String GRADE = "grade";
    public static final String POINTS_LABEL = "pointsLabel";
    public static final String NO_GRADE_EXPLANATION = "noGradeExplanation";

    public GradingReportPointsPanel(String id, final IModel<? extends GradingReport> gradingReportIModel, final GradeCalculator gradeCalculator) {
        super(id, gradingReportIModel);

        final IModel<GradingReport.Grade> gradeModel = new LoadableDetachableModel<GradingReport.Grade>() {
            @Override
            protected GradingReport.Grade load() {
                return gradingReportIModel.getObject().getGrade(gradeCalculator);
            }
        };
        final Label grade = new Label(GRADE, gradeModel) {
            @Override
            protected void onConfigure() {
                setVisible(getDefaultModelObject() != null);
            }
        };
        add(grade);

        final IModel<Long> points = new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return gradingReportIModel.getObject().getPoints(gradeCalculator);
            }
        };
        add(new Label(POINTS_LABEL, points));

        add(new WebMarkupContainer(NO_GRADE_EXPLANATION)
                .add(new OppositeVisibility(grade)));
    }
}
