package se.su.dsv.scipro.grading;

import com.google.common.base.Joiner;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.report.GradeCalculator;
import se.su.dsv.scipro.report.GradingCriterion;
import se.su.dsv.scipro.report.GradingReport;
import se.su.dsv.scipro.report.SupervisorGradingReport;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class SupervisorGradingReportPdf implements GradingReportPdf {

    private final IModel<SupervisorGradingReport> supervisorGradingReport;
    private GradeCalculator gradeCalculator;

    public SupervisorGradingReportPdf(IModel<SupervisorGradingReport> supervisorGradingReport, GradeCalculator gradeCalculator) {
        this.supervisorGradingReport = supervisorGradingReport;
        this.gradeCalculator = gradeCalculator;
    }

    @Override
    public String getProjectTitle() {
        return supervisorGradingReport.getObject().getProject().getTitle();
    }

    @Override
    public String getProjectType() {
        return supervisorGradingReport.getObject().getProject().getProjectType().getName();
    }

    @Override
    public String getAuthorName() {
        return supervisorGradingReport.getObject().getUser().getFullName();
    }

    @Override
    public String getAuthorNames() {
        List<String> names = new ArrayList<>();
        for (User student : supervisorGradingReport.getObject().getProject().getProjectParticipants()) {
            names.add(student.getFullName());
        }
	    return Joiner.on(", ").join(names);
    }

    @Override
    public String getSupervisorName() {
        return supervisorGradingReport.getObject().getProject().getSupervisorName();
    }

    @Override
    public String getReviewerName() {
        return supervisorGradingReport.getObject().getProject().getReviewerName();
    }

    @Override
    public String getPlagiarismFeedback() {
        return supervisorGradingReport.getObject().getPlagiarismFeedback();
    }

    @Override
    public String getGrade() {
        GradingReport.Grade grade = supervisorGradingReport.getObject().getGrade(gradeCalculator);
        if (grade == null){
            return "Betyget räknas ut när alla kriterier har poängsatts";
        }
        return grade.name();
    }

    @Override
    public Date getLastModified() {
        return supervisorGradingReport.getObject().getLastModified();
    }

    @Override
    public List<GradingCriterion> getGradingCriteria() {
        return new ArrayList<>(supervisorGradingReport.getObject().getGradingCriteria());
    }

    @Override
    public String getProjectTypeName() {
        return supervisorGradingReport.getObject().getProjectType().getName();
    }
}
