package se.su.dsv.scipro.group;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.*;

public class EditGroupPanel extends Panel {

    @Inject
    private ProjectService projectService;
    @Inject
    private GroupService groupService;
    @Inject
    private ProjectTypeService projectTypeService;

    public EditGroupPanel(String id, final IModel<Group> model) {
        super(id, model);
        add(new GroupForm("form", model));
    }

    private class GroupForm extends Form<Group> {

        private final AjaxDropDownChoice<Project> addProjects;
        private final ListView<Project> projects;
        private final List<Project> currentProjects;
        private final AjaxCheckBoxMultipleChoice<ProjectType> projectTypes;

        public GroupForm(String form, final IModel<Group> model) {
            super(form, model);

            final FeedbackPanel feedbackPanel = new FeedbackPanel("feedback");
            feedbackPanel.setOutputMarkupId(true);
            add(feedbackPanel);

            currentProjects = new ArrayList<>(getModelObject().getProjects());

            add(new RequiredTextField<>("title", new PropertyModel<>(model, "title")));
            add(new TextArea<>("description", new PropertyModel<>(model, "description")));
            add(new CheckBox("active", new PropertyModel<Boolean>(model, "active")));

            final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);

            projectTypes = projectTypeSelection(wmc);
            wmc.add(projectTypes);

            addProjects = new AjaxDropDownChoice<Project>("addProjects", new Model<Project>(), getSelectableProjects(currentProjects), new ChoiceRenderer<Project>("title", "id")) {
                @Override
                public void onNewSelection(AjaxRequestTarget target, Project objectSelected) {
                    if (objectSelected!=null && !currentProjects.contains(objectSelected)){
                        currentProjects.add(objectSelected);
                        projects.setList(currentProjects);
                        addProjects.setChoices(getSelectableProjects(currentProjects));
                        target.add(wmc);
                    }
                }
            };
            addProjects.setRequired(false);
            addProjects.setNullValid(true);
            wmc.add(addProjects);

            projects = new ListView<Project>("projects", new ArrayList<>(currentProjects)) {
                @Override
                protected void populateItem(final ListItem<Project> item) {
                    item.add(new Label("type", item.getModelObject().getProjectTypeName()));
                    item.add(new Label("title", new PropertyModel<>(item.getModel(), "title")));
                    item.add(new ListView<User>("authors", new ListAdapterModel<>(new PropertyModel<Set<User>>(getLoaded(item.getModelObject()), "projectParticipants"))) {
                        @Override
                        public void populateItem(ListItem<User> item) {
                            item.add(new UserLinkPanel("author", item.getModel()));
                        }
                    });
                    item.add(new AjaxLink<Project>("remove", item.getModel()) {
                        @Override
                        public void onClick(AjaxRequestTarget target) {
                            currentProjects.remove(item.getModelObject());
                            projects.setList(currentProjects);
                            addProjects.setChoices(getSelectableProjects(currentProjects));
                            target.add(wmc);
                        }
                    });
                }
            };
            wmc.add(projects);

            wmc.add(new Label("noProjects", "None"){
                @Override
                protected void onConfigure() {
                    super.onConfigure();
                    setVisibilityAllowed(currentProjects.isEmpty());
                }
            });

            add(wmc);

        }

        private AjaxCheckBoxMultipleChoice<ProjectType> projectTypeSelection(final WebMarkupContainer wmc) {
            return new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypes", projectTypeService.findAllActive(),
                    projectTypeService.findAllActive(), new ChoiceRenderer<ProjectType>("name", "id")
                    ) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    addProjects.setChoices(getSelectableProjects(currentProjects));
                    target.add(wmc);
                }
            };
        }

        @Override
        protected void onSubmit() {
            Group group = getModelObject();
            group.setProjects(new HashSet<>(currentProjects));
            groupService.save(group);
            info(getString("saved"));
        }

        private ListModel<Project> getSelectableProjects(List<Project> currentProjects){
            final ProjectService.Filter filter = new ProjectService.Filter();
            filter.setSupervisor(SciProSession.get().getUser());
            filter.setRoles(Collections.singleton(ProjectTeamMemberRoles.CO_SUPERVISOR));
            filter.setStatuses(Collections.singletonList(ProjectStatus.ACTIVE));
            filter.setProjectTypes(projectTypes.getModelObject());
            List<Project> all = projectService.findAll(filter);
            all.removeAll(currentProjects);
            all.remove(null);
            return new ListModel<>(all);
        }

        private LoadableDetachableModel<Project> getLoaded(final Project project){
            return new LoadableDetachableModel<Project>() {
                @Override
                protected Project load() {
                    return projectService.findOne(project.getId());
                }
            };
        }
    }
}

