package se.su.dsv.scipro.group;

import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.SystemModuleComponent;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.io.Serializable;
import java.util.List;

@SystemModuleComponent(SystemModule.GROUP)
public class GroupsOverviewPanel extends GenericPanel<Project> {

    @Inject
    private GroupService groupService;

    public GroupsOverviewPanel(String id, IModel<Project> model, final GroupLinkSupplier linkSupplier) {
        super(id, model);

        add(new AutoHidingListView<Group>("groupList", getGroups()) {
            @Override
            protected void populateItem(ListItem<Group> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Group.class), item.getModelObject().getId());
                BookmarkablePageLink<Void> link = linkSupplier.newGroupLink("groupLink", pp);
                link.setBody(new PropertyModel(item.getModel(), "title"));
                item.add(link);
            }
        });
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(!getGroups().getObject().isEmpty());
    }

    private LoadableDetachableModel<List<Group>> getGroups() {
        return new LoadableDetachableModel<List<Group>>() {
            @Override
            protected List<Group> load() {
                GroupService.Filter filter = new GroupService.Filter();
                filter.setProject(getModelObject());
                return groupService.findAll(filter);
            }
        };
    }

    public interface GroupLinkSupplier extends Serializable {
        BookmarkablePageLink<Void> newGroupLink(String id, PageParameters pp);
    }
}
