package se.su.dsv.scipro.group;

import org.apache.wicket.Component;
import org.apache.wicket.RestartResponseException;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyGroups;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.GroupForumService;
import se.su.dsv.scipro.forum.dataobjects.GroupThread;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.panels.threaded.ViewForumThreadPanel;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

public class ViewThreadPage extends AbstractAuthorGroupPage implements MenuHighlightAuthorMyGroups {

    @Inject
    private GroupForumService groupForumService;
    @Inject
    private BasicForumService basicForumService;
    @Inject
    private FileDescriptionService fileDescriptionService;

    public ViewThreadPage(final PageParameters parameters) {
        super(parameters);

        final long threadId = parameters.get(PageParameterKeys.MAP.get(ForumThread.class)).toLong(0L);
        final IModel<GroupThread> groupThreadModel = new LoadableDetachableModel<GroupThread>() {
            @Override
            protected GroupThread load() {
                return groupForumService.findOne(threadId);
            }
        };

        GroupThread groupThread = groupThreadModel.getObject();
        if (groupThread == null || !groupThread.getGroup().equals(getGroup().getObject())) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        add(new FeedbackPanel("feedback"));

        basicForumService.setThreadRead(getUser(), groupThread.getForumThread(), true);

        add(new ViewForumThreadPanel<GroupThread>("thread", groupThreadModel, new GroupForumThread(groupForumService, fileDescriptionService)) {
            @Override
            protected Component newBackLink(final String id) {
                PageParameters pageParameters = new PageParameters(parameters)
                        .remove(PageParameterKeys.MAP.get(ForumThread.class));
                return new BookmarkablePageLink<Void>(id, AuthorGroupPage.class, pageParameters);
            }
        });
    }
}
