package se.su.dsv.scipro.icons;

import org.apache.wicket.markup.html.image.ContextImage;

/**
 *         An image class that
 *         1. Centralizes the management of images, thus making it more object oriented.
 *         Should developers/designers want to change an icon, it needs to be done only
 *         once here and in the centralized folders, rather than countless times elsewhere
 *         in the code.
 *         2. Allows the separation of code and static resources (images in this case).
 *         3. Allows for easy swapping between images of different sizes in the code.
 */
public class ImageObject extends ContextImage {
    /**
     * Path to icon folder in webapp
     */
    public static final String PATH = "images/icons/";

    /**
     * Paths to subfolders for different size images.
     * Not all images come in different sizes, in that
     * case insert ImageObject.PATH before the image name
     * or use the convenience factory method (recommended).
     */
    public static final String TWENTYFOUR = PATH + "24/";

    /**
     * Names of image files
     */
    public static final String LIGHT_GREEN = "bullet_ball_glass_green.png";
    public static final String LIGHT_YELLOW = "bullet_ball_glass_yellow.png";
    public static final String LIGHT_RED = "bullet_ball_glass_red.png";

    /**
     * Default constructor. Send the wicket id and the path to the image.
     * Use **ONLY** the static strings provided in this class for paths.
     * Examples:
     * wicketComponent.add(new ImageObject("deleteIcon", ImageObject.SIXTEEN + ImageObject.DELETE));
     * wicketComponent.add(new ImageObject("greenLightIcon", ImageObject.PATH + ImageObject.LIGHT_GREEN));
     */
    public ImageObject(String id, String imagePath) {
        super(id, imagePath);
    }

    public ImageObject(String id, String path, String image) {
        this(id, path + image);
    }
}
