package se.su.dsv.scipro.io;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractDataExporter;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.IExportableColumn;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;

/**
 * Using IDataExporter does not currently work as {@link org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn#getHeader(String)}
 * does not handle {@link org.apache.wicket.model.IComponentAssignedModel}
 * correctly, most notably {@link org.apache.wicket.model.ResourceModel}.
 *
 * Therefore it throws {@code NullPointerException} when trying to render
 * headers as the {@code ResourceModel} has no connection to the component and
 * can therefore not resolve the localized property.
 */
public class ExcelExporter extends AbstractDataExporter {

    public static final int HEADER_ROW = 0;

    public ExcelExporter() {
        super(Model.of("Excel"), "application/vnd.ms-excel", "xls");
    }

    @Override
    public <T> void exportData(IDataProvider<T> dataProvider, List<IExportableColumn<T, ?, ?>> columns, OutputStream outputStream) throws IOException {
        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet();
        addHeaders(sheet, columns);
        addData(sheet, dataProvider, columns);
        workbook.write(outputStream);
        outputStream.close();
    }

    private <T> void addHeaders(HSSFSheet sheet, List<IExportableColumn<T, ?, ?>> columns) {
        HSSFRow row = sheet.createRow(HEADER_ROW);
        for (int i = 0; i < columns.size(); i++) {
            IExportableColumn<T, ?, ?> exportableColumn = columns.get(i);
            String heading = exportableColumn.getDisplayModel().getObject();
            HSSFCell cell = row.createCell(i);
            cell.setCellValue(heading);
        }
    }

    private <T> void addData(HSSFSheet sheet, IDataProvider<T> dataProvider, List<IExportableColumn<T, ?, ?>> columns) {
        int row = HEADER_ROW + 1;
        Iterator<? extends T> iterator = dataProvider.iterator(0, dataProvider.size());
        while (iterator.hasNext()) {
            T next = iterator.next();
            HSSFRow sheetRow = sheet.createRow(row);
            fillRow(sheetRow, dataProvider.model(next), columns);
            row++;
        }
    }

    private <T> void fillRow(HSSFRow row, IModel<T> data, List<IExportableColumn<T, ?, ?>> columns) {
        for (int i = 0; i < columns.size(); i++) {
            Object cellValue = columns.get(i).getDataModel(data).getObject();
            HSSFCell cell = row.createCell(i);
            cell.setCellValue(String.valueOf(cellValue));
        }
    }
}
