package se.su.dsv.scipro.match;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.system.ResearchArea;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AdminResearchAreaKeywordPanel extends Panel {

    public static final String DATA_TABLE = "dataTable";
    @Inject
    private KeywordService keywordService;

    public AdminResearchAreaKeywordPanel(String id, IModel<ResearchArea> areaModel) {
        super(id);
        addTable(areaModel);
    }

    private void addTable(final IModel<ResearchArea> areaModel) {
        SortableDataProvider<Keyword, String> provider = new SortableDataProvider<Keyword, String>() {
            @Override
            public Iterator<Keyword> iterator(long first, long count) {
                return keywordService.findByResearchArea(areaModel.getObject(), new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return keywordService.countByResearchArea(areaModel.getObject());
            }

            @Override
            public IModel<Keyword> model(Keyword object) {
                return new DetachableServiceModel<>(keywordService, object);
            }
        };
        provider.setSort("keyword", SortOrder.DESCENDING);
        List<IColumn<Keyword, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Keyword, String>(Model.of("Keyword"), "keyword", "keyword"));
        add(new ExportableDataPanel<>(DATA_TABLE, columns, provider));
    }

}
