package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.ResearchAreaService;

import javax.inject.Inject;

public abstract class EditResearchAreaPanel extends Panel {

    @Inject
    private ResearchAreaService researchAreaService;

    public abstract void onSave(AjaxRequestTarget target);

    public EditResearchAreaPanel(String id, final IModel<ResearchArea> model) {
        super(id, model);
        add(new ResearchAreaForm("form", model));
    }

    private class ResearchAreaForm extends Form<ResearchArea> {

        public ResearchAreaForm(String id, IModel<ResearchArea> model) {
            super(id, model);

            final WebMarkupContainer wmc = new WebMarkupContainer("wmc");
            wmc.setOutputMarkupId(true);

            RequiredTextField<String> researchArea = new RequiredTextField<>("researchArea", new PropertyModel<String>(model, "title"));
            wmc.add(researchArea);
            wmc.add(new ComponentFeedbackPanel("researchAreaFeedback", researchArea));

            FeedbackPanel feedback = new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            });
            feedback.setOutputMarkupId(true);
            wmc.add(feedback);

            add(wmc);

            add(new AjaxButton("save") {
                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    super.onSubmit(target, form);
                    ResearchAreaForm.this.setModelObject(researchAreaService.save(ResearchAreaForm.this.getModelObject()));
                    info("Saved");
                    target.add(wmc);
                    onSave(target);
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    super.onError(target, form);
                    target.add(wmc);
                }
            });
        }
    }
}