package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.NullReplacementModel;
import se.su.dsv.scipro.system.Language;

import java.util.Set;

public class IdeaDetailsPanel extends Panel {

    public static final String TITLE = "title";

    public IdeaDetailsPanel(String id, IModel<Idea> model, boolean showDescriptionAndPrereqs) {
		super(id, model);
        Idea idea = model.getObject();
        add(new Label("projectType", new PropertyModel<>(idea, "projectType.name")));
        add(new PropertyListView<Language>("languages",
                new ListAdapterModel<>(new PropertyModel<Set<Language>>(model, "languages"))) {
            @Override
            protected void populateItem(final ListItem<Language> item) {
                item.add(new Label("name"));
            }
        });
		add(new Label("area", new NullReplacementModel(
                new PropertyModel<>(idea, "researchArea.title"), "No research area selected")));
		add(new Label("applicationPeriod", new NullReplacementModel(
                new PropertyModel<>(idea, "applicationPeriod.name"), "Not connected to an application period")));
        addDescriptionAndPrerequisites(model, showDescriptionAndPrereqs);
		addKeywords(model);
	}

	private void addDescriptionAndPrerequisites(IModel<Idea> model, boolean show) {
        MultiLineLabel descLabel = new MultiLineLabel("description", new PropertyModel<>(model, "description"));
        MultiLineLabel reqsLabel = new MultiLineLabel("reqs", new NullReplacementModel(
                new PropertyModel<>(model, "prerequisites"), "No prerequisites added"));
		descLabel.setVisible(show);
		add(descLabel);
		add(reqsLabel);
	}

	private void addKeywords(IModel<Idea> model) {
		add(new Label("noKeysLabel", "No keywords available").setVisible(model.getObject().getKeywords().isEmpty()));
		add(new Label("keywords", new KeywordsModel(model)));
	}

}
