package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.StringResourceModel;
import org.apache.wicket.validation.validator.StringValidator;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;

public class IdeaWatsonCompletionPanel extends Panel {

    public static final String MAX_CHARS_LABEL = "maxCharsLabel";
    @Inject
    private IdeaService ideaService;
    @Inject
    private ApplicationPeriodService periodService;

    public IdeaWatsonCompletionPanel(String id, IModel<Idea> model) {
        super(id, model);

        boolean isEditable = !periodService.endDateIsReached(model.getObject().getApplicationPeriod());
        add(new WatsonCompletionForm("form", new CompoundPropertyModel<>(model), isEditable));
    }

    private class WatsonCompletionForm extends StatelessForm<Idea> {

        public WatsonCompletionForm(String id, CompoundPropertyModel<Idea> model, boolean isEditable) {
            super(id, model);

            add(new Label(MAX_CHARS_LABEL, new StringResourceModel("max.chars.label", this, null, Watson.MAX_CHARS)));
            add(new TextArea<String>("watson.what").add(StringValidator.maximumLength(Watson.MAX_CHARS)));
            add(new TextArea<String>("watson.why").add(StringValidator.maximumLength(Watson.MAX_CHARS)));
            add(new TextArea<String>("watson.theoryHow").add(StringValidator.maximumLength(Watson.MAX_CHARS)));
            add(new TextArea<String>("watson.practicalHow").add(StringValidator.maximumLength(Watson.MAX_CHARS)));
            setEnabled(isEditable);
            addSaveButton();
        }

        private void addSaveButton() {
            Button saveLink = new Button("saveLink");
            saveLink.add(new JavascriptEventConfirmation("click", "Are you sure you want to save your changes?"));
            add(saveLink);
        }

        @Override
        protected void onSubmit() {
            ideaService.save(getModelObject());
            info("Idea updated");
        }
    }

}
