package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;

import javax.inject.Inject;

/**
 * @author : emil-siv
 *         Date: 2013-08-16
 */
public class EditMileStonePhasePanel extends Panel {

    static final String EDIT_FORM = "editForm";
    static final String FEEDBACK = "feedback";
    static final String TITLE = "title";
    static final String DESCRIPTION = "description";
    static final String SAVE = "save";

    @Inject
    MilestonePhaseTemplateService phaseService;

    private final FeedbackPanel feedback;

    public EditMileStonePhasePanel(String id, IModel<MilestonePhaseTemplate> model) {
        super(id);

        feedback = new FeedbackPanel(FEEDBACK);
        add(feedback.setOutputMarkupId(true));

        add(new EditForm(EDIT_FORM, new CompoundPropertyModel<>(model)));

    }

    private class EditForm extends Form<MilestonePhaseTemplate> {

        public EditForm(String id, IModel<MilestonePhaseTemplate> model) {
            super(id, model);


            add(new RequiredTextField<String>(TITLE));
            add(new TextArea<String>(DESCRIPTION).setRequired(true));
            add(new AjaxSubmitLink(SAVE) {

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedback);
                }

                @Override
                protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
                    activitySaved(target);
                }
            });
        }
        @Override
        protected void onSubmit() {
            phaseService.save(getModelObject());
        }

    }

    protected void activitySaved(AjaxRequestTarget target) {
    }

}
