package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.FilteredListModel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.List;

@ProjectModuleComponent(ProjectModule.MILESTONES)
public class MilestoneSummaryPanel extends Panel {

	@Inject
	private MileStoneService mileStoneService;
	@Inject
	private MilestoneActivityTemplateService activityService;

	private final IModel<Project> projectModel;

    public MilestoneSummaryPanel(String id, IModel<Project> projectModel) {
		super(id);
		this.projectModel = projectModel;

		addProjectProgress();
		addStudentProgress();
		addUpcomingActivities();
	}

	private void addProjectProgress() {
        add(new MileStoneProgressPanel(PROJECT_PROGRESS, countProject(), MilestoneActivityTemplate.Type.PROJECT, projectModel));
	}

	private void addStudentProgress() {
		add(new ListView<User>(STUDENT_LIST, new ListAdapterModel<>(new PropertyModel<Iterable<User>>(projectModel, "projectParticipants"))) {
			@Override
			protected void populateItem(ListItem<User> item) {
				item.add(new Label(STUDENT_NAME, item.getModelObject().getFullName()));
				item.add(new MileStoneProgressPanel(STUDENT_PROGRESS, countStudent(item.getModel()), MilestoneActivityTemplate.Type.STUDENT, projectModel));
			}
		});
	}

	private void addUpcomingActivities() {
		final FilteredListModel<MilestoneActivityTemplate> upcoming = new FilteredListModel<MilestoneActivityTemplate>(getActivitiesInOrder()) {
			@Override
			protected boolean accept(MilestoneActivityTemplate mileStoneActivity) {
				return !mileStoneService.isConfirmed(projectModel.getObject(), mileStoneActivity);
			}
		};
		ListView<MilestoneActivityTemplate> listView = new ListView<MilestoneActivityTemplate>(UPCOMING_LIST, upcoming) {
			@Override
			protected void populateItem(final ListItem<MilestoneActivityTemplate> item) {
                item.add(new Label(TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
			}
		};
        listView.setViewSize(ACTIVITIES_TO_SHOW);
		add(listView);
	}

    private IModel<Long> countStudent(final IModel<User> studentModel) {
		return Model.of(mileStoneService.countMileStones(projectModel.getObject(), studentModel.getObject(), true));
	}

	private IModel<Long> countProject() {
		return new LoadableDetachableModel<Long>() {
			@Override
			protected Long load() {
				return mileStoneService.countProjectMilestones(projectModel.getObject(), true);
			}
		};
	}

	private IModel<List<? extends MilestoneActivityTemplate>> getActivitiesInOrder() {
		return new LoadableDetachableModel<List<? extends MilestoneActivityTemplate>>() {
			@Override
			protected List<MilestoneActivityTemplate> load() {
				return activityService.getActivitiesInOrder(MilestoneActivityTemplate.Type.PROJECT, projectModel.getObject());
			}
		};
	}

	@Override
	protected void onConfigure() {
		setVisibilityAllowed(activityService.countActivities(projectModel.getObject().getProjectType(), projectModel.getObject().getDateCreated()) != 0);
	}

	static final String STUDENT_LIST = "studentList";
	static final String STUDENT_NAME = "studentName";
	static final String STUDENT_PROGRESS = "studentProgress";
	static final String PROJECT_PROGRESS = "projectProgress";
	static final String UPCOMING_LIST = "upcomingList";
	static final String TITLE = "title";

	static final int ACTIVITIES_TO_SHOW = 3;

}
