package se.su.dsv.scipro.nonworkdays;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.admin.pages.AbstractAdminProjectPage;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriod;
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class NonWorkDaysPage extends AbstractAdminProjectPage {
    public static final String DIALOG = "modalWindowPlus";
    public static final String ADD = "add";
    public static final String FEEDBACK = "feedback";
    public static final String DATA_PANEL = "dp";

    @Inject
    private NonWorkDayPeriodService nonWorkDayPeriodService;

    private ModalWindowPlus modalWindowPlus;
    private FeedbackPanel feedback;
    private ExportableDataPanel<NonWorkDayPeriod, String> dp;

    public NonWorkDaysPage() {
        addFeedbackPanel();
        addModalWindow();
        addAddNewLink();
        addDataTable();
    }

    private void addFeedbackPanel() {
        feedback = new FeedbackPanel(FEEDBACK);
        add(feedback);
        feedback.setOutputMarkupId(true);
    }

    private void addModalWindow() {
        modalWindowPlus = new ModalWindowPlus(DIALOG);
        modalWindowPlus.setOutputMarkupId(true);
        add(modalWindowPlus);
    }

    private void addAddNewLink() {
        add(new AjaxLink<Void>(ADD) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Add new non work days period");
                NonWorkDayPeriod nonWorkDayPeriod = new NonWorkDayPeriod();
                nonWorkDayPeriod.setStartDate(new Date());
                nonWorkDayPeriod.setEndDate(new Date());
                nonWorkDayPeriod.setComment("");

                dialog(new Model<>(nonWorkDayPeriod), target);
            }
        });
    }

    private void addDataTable() {
        SortableDataProvider<NonWorkDayPeriod, String> provider = new GenericDataProvider<>(nonWorkDayPeriodService);
        provider.setSort("startDate", SortOrder.DESCENDING);
        dp = new ExportableDataPanel<>(DATA_PANEL, createColumns(), provider);
        add(dp);
    }

    private List<IColumn<NonWorkDayPeriod, String>> createColumns() {
        List<IColumn<NonWorkDayPeriod, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<NonWorkDayPeriod>(Model.of("From"), "startDate", "startDate", DateStyle.DATETIME));
        columns.add(new DateColumn<NonWorkDayPeriod>(Model.of("To"), "endDate", "endDate", DateStyle.DATETIME));
        columns.add(new PropertyColumn<NonWorkDayPeriod, String>(Model.of("Comment"), "comment"));
        columns.add(new ClickableIconColumn<NonWorkDayPeriod>(Model.of("Edit"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<NonWorkDayPeriod> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit non work days period");
                dialog(clicked, target);
            }
        });
        columns.add(new ClickableIconColumn<NonWorkDayPeriod>(Model.of("Delete"), null, ClickableIconColumn.DELETE) {
            @Override
            protected void onClick(IModel<NonWorkDayPeriod> clicked, AjaxRequestTarget target) {
                nonWorkDayPeriodService.delete(clicked.getObject().getId());
                info("Deleted non work days period");
                target.add(dp);
                target.add(feedback);
            }
        });
        return columns;
    }

    public void dialog(IModel<NonWorkDayPeriod> model, AjaxRequestTarget target) {
        modalWindowPlus.setContent(new NonWorkDaysCRUDPanel(modalWindowPlus.getContentId(), model) {
            @Override
            public void handleSave(AjaxRequestTarget target) {
                modalWindowPlus.close(target);
                info("Saved non work days period");
                target.add(dp);
                target.add(feedback);
            }
        });
        modalWindowPlus.show(target);
    }
}
