package se.su.dsv.scipro.panels;

import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.peer.*;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.util.List;

public class CommentThreadPanel extends GenericPanel<PeerReview> {

    public static final String COMMENTS = "comments";
    public static final String DATE = "dateCreated";
    public static final String FROM = "creator";
    public static final String MESSAGE = "comment";
    public static final String COMMENT_FORM = "commentForm";
    public static final String COMMENT_TEXT_AREA = "commentTextArea";
    public static final String DELETE_LINK = "deleteLink";
    public static final String FEEDBACK_PANEL = "feedbackPanel";
    private final IModel<? extends List<? extends Comment>> comments;

    @Inject
    private CommentThreadService commentThreadService;
    @Inject
    private CommentService commentService;

    private CommentThread commentThread;

    public CommentThreadPanel(String id, IModel<PeerReview> peerReviewIModel) {
        super(id, peerReviewIModel);

        commentThread = commentThreadService.getCommentThread(peerReviewIModel.getObject());
        if (commentThread == null) {
            CommentThread ct = new CommentThread(peerReviewIModel.getObject());
            commentThread = commentThreadService.save(ct);
        }

        comments = findComments();
        add(new PropertyListView<Comment>(COMMENTS, comments) {
            @Override
            protected void populateItem(ListItem<Comment> item) {
                item.add(new DateLabel(DATE, DateStyle.DATETIME));
                item.add(new UserLinkPanel(FROM));
                item.add(new MultiLineLabel(MESSAGE));
                item.add(new DeleteLink(DELETE_LINK, item.getModel()));
            }
        });

        CommentForm commentForm = new CommentForm(COMMENT_FORM, findComment());
        add(commentForm);
        add(new FencedFeedbackPanel(FEEDBACK_PANEL, commentForm));
    }

    @Override
    protected void onConfigure() {
        setVisible(getModelObject().isCompleted());
    }

    private IModel<Comment> findComment() {
        return new LoadableDetachableModel<Comment>() {
            @Override
            protected Comment load() {
                return new Comment(SciProSession.get().getUser(), commentThread);
            }
        };
    }

    private IModel<? extends List<? extends Comment>> findComments() {
        return new LoadableDetachableModel<List<? extends Comment>>() {
            @Override
            protected List<? extends Comment> load() {
                return commentService.findByCommentThread(commentThread);
            }
        };
    }

    private class CommentForm extends Form<Comment> {

        @Override
        protected void onSubmit() {
            commentService.save(getModelObject());
        }

        public CommentForm(String id, IModel<Comment> commentIModel) {
            super(id, commentIModel);
            final TextArea<String> comment = new TextArea<>(COMMENT_TEXT_AREA, new PropertyModel<String>(commentIModel, "comment"));
            add(comment.setRequired(true));
        }
    }

    private class DeleteLink extends Link<Comment> {
        public DeleteLink(String id, IModel<Comment> commentIModel) {
            super(id, commentIModel);
            add(new JavascriptEventConfirmation("onclick", new ResourceModel("delete.confirmation")));
        }

        @Override
        public void onClick() {
            commentService.delete(getModelObject());
            comments.detach();
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(getModelObject().getCreator().equals(SciProSession.get().getUser()));
        }
    }

}
