package se.su.dsv.scipro.profile;

import org.apache.wicket.extensions.markup.html.basic.SmartLinkMultiLineLabel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.components.NullReplacementModel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Set;

public class DisplayUserProfilePanel extends GenericPanel<User> {

    @Inject
    private UserProfileService userProfileService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    public DisplayUserProfilePanel(String id, final IModel<User> model) {
        super(id, model);

        add(new UserProfileImage("image", model, UserProfileImage.Size.LARGE));

        add(new ExternalLink(MAIL, "mailto:" + model.getObject().getEmailAddress(), model.getObject().getEmailAddress()));

        add(new Label(SKYPE, new NullReplacementModel(new PropertyModel<>(getProfile(), "skypeId"), "Not entered")));

        add(new Label(PHONE, new NullReplacementModel(new PropertyModel<>(getProfile(), "phoneNumber"), "Not entered")));

        add(new SmartLinkMultiLineLabel(OTHER, new NullReplacementModel(new PropertyModel<>(getProfile(), "otherInfo"), "Not entered")));

        WebMarkupContainer employeeWMC = new WebMarkupContainer(EMPLOYEE) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(model.getObject().hasRole(Roles.SUPERVISOR));
            }
        };
        add(employeeWMC);

        employeeWMC.add(new Label(UNIT, new NullReplacementModel(new PropertyModel<>(model, "unit.title"), "None")));

        ListView<ResearchArea> areas = new AutoHidingListView<ResearchArea>(AREAS,
                new ListAdapterModel<>(new PropertyModel<Set<ResearchArea>>(model, "activeResearchAreas"))) {
            @Override
            protected void populateItem(ListItem<ResearchArea> item) {
                item.add(new Label(AREA, item.getModelObject().getTitle()));
            }
        };
        employeeWMC.add(areas);
        employeeWMC.add(new Label(NOAREAS, "None")
                .add(new OppositeVisibility(areas)));

        ListView<Language> languages = new AutoHidingListView<Language>(LANGUAGES, new ListAdapterModel<>(new PropertyModel<Set<Language>>(model, "languages"))) {
            @Override
            protected void populateItem(ListItem<Language> item) {
                item.add(new Label(LANGUAGE, new PropertyModel<>(item.getModel(), "name")));
            }
        };
        employeeWMC.add(languages);
        employeeWMC.add(new Label(NOLANGUAGES, "None")
                .add(new OppositeVisibility(languages)));

        GeneralSystemSettings generalSystemSettings = generalSystemSettingsService.getGeneralSystemSettingsInstance();
        String profile = "";
        if (generalSystemSettings.getDaisyProfileLinkBaseURL() != null && !generalSystemSettings.getDaisyProfileLinkBaseURL().isEmpty() && model.getObject().getIdentifier() != null) {
            profile = generalSystemSettings.getDaisyProfileLinkBaseURL() + model.getObject().getIdentifier();
        }
        final String finalProfile = profile;
        employeeWMC.add(new ExternalLink(DAISYPROFILE, finalProfile) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!finalProfile.isEmpty());
            }
        });
    }

    public static final String MAIL = "mail";
    public static final String SKYPE = "skype";
    public static final String PHONE = "phone";
    public static final String OTHER = "other";
    public static final String EMPLOYEE = "employee";
    public static final String UNIT = "unit";
    public static final String AREAS = "areas";
    public static final String AREA = "area";
    public static final String NOAREAS = "noareas";
    public static final String LANGUAGES = "languages";
    public static final String LANGUAGE = "language";
    public static final String NOLANGUAGES = "nolanguages";
    public static final String DAISYPROFILE = "daisyprofile";

    private IModel<UserProfile> getProfile() {
        return new LoadableDetachableModel<UserProfile>() {
            @Override
            protected UserProfile load() {
                return userProfileService.findByUser(getModelObject());
            }
        };
    }
}
