package se.su.dsv.scipro.profile

import javax.inject.Inject

import org.apache.wicket.injection.Injector
import org.apache.wicket.request.resource.IResource.Attributes
import se.su.dsv.scipro.StringValueConversions
import se.su.dsv.scipro.file.FileDescription
import se.su.dsv.scipro.repository.components.FileDescriptionResourceReference
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.settings.dataobjects.UserProfile
import se.su.dsv.scipro.springdata.services.UserProfileService
import se.su.dsv.scipro.system.UserService

class ProfilePictureResource extends FileDescriptionResourceReference("profilePicture") {

  @Inject private var userProfileService: UserProfileService = _
  @Inject private var userService: UserService = _

  Injector.get().inject(this)

  override def fileDescription(attributes: Attributes): Option[FileDescription] = for {
    id ← attributes.getParameters.get("id").asLong
    if SciProSession.get().isLoggedIn
    user ← Option(userService.findOne(id))
    profile = userProfileService.findByUser(user)
    if profile.getPicture == UserProfile.Picture.FILE
    picture ← Option(profile.getProfilePicture)
  } yield picture
}
