package se.su.dsv.scipro.project.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import org.apache.wicket.util.string.StringValueConversionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.panels.ProjectHeaderPanel;
import se.su.dsv.scipro.project.panels.ProjectSubTabMenuPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import javax.servlet.http.Cookie;

import static se.su.dsv.scipro.components.AbstractMenuPanel.MenuType.HORIZONTAL;

public abstract class AbstractProjectDetailsPage extends ProjectPage {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractProjectDetailsPage.class);
    public static final String COOKIE_KEY = "authorLastVisitedPID";
    public static final int MAX_AGE = 2592000;

    @Inject
    protected ProjectService projectService;

    protected DetachableServiceModel<Project> projectModel;

    public AbstractProjectDetailsPage(PageParameters pp) {
        super(pp);

        Long projectId = null;
        StringValue projectValue = pp.get(PageParameterKeys.MAP.get(Project.class));

        if (!projectValue.isNull()) {
            try {
                projectId = projectValue.toLongObject();
            } catch (StringValueConversionException e){
                LOGGER.warn("Project ID is not valid.", e);
                throw new RestartResponseException(NotFoundPage.class);
            }
            /*
             * Cookie is set when the link to visit this details page is clicked but for it to work
			 * if a URL is entered directly we set it here too
			 */
            Cookie c = new Cookie(COOKIE_KEY, projectId.toString());
            c.setMaxAge(MAX_AGE);
            c.setPath("/");
            ((WebResponse) getRequestCycle().getResponse()).addCookie(c);
        } else {
            Cookie cookie = ((WebRequest) this.getRequest()).getCookie(COOKIE_KEY);
            if (cookie != null && cookie.getValue() != null) {
                try {
                    projectId = Long.parseLong(cookie.getValue());
                } catch (RuntimeException e) {/* Ignore */
                    LOGGER.error("Could not initialize project", e);
                }
            }
        }

        if (projectId == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        Project project = projectService.findOne(projectId);
        if (project == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }
        if (!projectService.isPartOfProject(project, SciProSession.get().getUser())) {
            throw new RestartResponseException(AccessDeniedPage.class);
        }

        SciProSession.get().setProjectModules(project.getAuthorizedModules());
        projectModel = new DetachableServiceModel<>(projectService, projectId);
        setDefaultModel(projectModel);

        add(new ProjectSubTabMenuPanel("projectSub", this.getClass(), getPageParameters(project), projectModel.getObject()) {
            @Override
            protected MenuType getMenuType() {
                return HORIZONTAL;
            }
        });

        add(new ProjectHeaderPanel("projectHeader", projectModel));
    }

    protected Project getActiveProject() {
        return projectModel.getObject();
    }

    public static PageParameters getPageParameters(Project project) {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), project.getId());
        return pp;
    }
}