package se.su.dsv.scipro.project.panels;

import org.apache.wicket.ajax.AbstractAjaxTimerBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.util.time.Duration;
import se.su.dsv.scipro.components.AuthorAutoComplete;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.match.IdeaParticipation;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import java.util.Set;

public class PartnerAdditionPanel extends Panel {

    public static final String CO_AUTHOR_LABEL = "coAuthorLabel";
    private User coAuthor;
    private AuthorAutoComplete coAuthorField;
    private WebMarkupContainer coAuthorContainer;
    private AjaxLink<Void> removeLink;

    public PartnerAdditionPanel(String id, boolean reFocusNeeded) {
        this(id, null, reFocusNeeded);
    }

    public PartnerAdditionPanel(String id, Set<IdeaParticipation> participants, boolean reFocusNeeded) {
        super(id);
        if (participants != null && participants.size() > 1) {
            for (IdeaParticipation ip : participants) {
                if (ip.getUser().equals(SciProSession.get().getUser())) {
                    coAuthor = ip.getUser();
                }
            }
        }
        coAuthorContainer = new WebMarkupContainer("container");
        coAuthorContainer.setOutputMarkupId(true);
        add(coAuthorContainer);
        addAutoCompleteField(reFocusNeeded);
        addCurrentCoAuthorLabel();
        addRemoveAuthorIcon();
        addInfo();
    }

    private void addInfo() {
        coAuthorContainer.add(new InfoPanel("partnerInfo", new ResourceModel("partnerInfo")));
    }

    private void addAutoCompleteField(boolean reFocusNeeded) {
        coAuthorField = new AuthorAutoComplete("inputField") {

            @Override
            protected void action(AjaxRequestTarget pTarget, User newSelection) {
                coAuthor = newSelection;
                setModelObject(null);
                setVisible(false);
                coAuthorContainer.replace(new UserLinkPanel(CO_AUTHOR_LABEL, new Model<>(coAuthor)));
                removeLink.setVisible(true);
                pTarget.add(coAuthorContainer);
            }
        };
        coAuthorField.setVisible(!coAuthorExists());
        coAuthorContainer.add(coAuthorField);
        if (reFocusNeeded) {
            reFocus();
        }
    }

    private void addCurrentCoAuthorLabel() {
        if (coAuthorExists()){
            coAuthorContainer.add(new UserLinkPanel(CO_AUTHOR_LABEL, new Model<>(coAuthor)));
        } else {
            coAuthorContainer.add(new EmptyPanel(CO_AUTHOR_LABEL));
        }

    }

    private void addRemoveAuthorIcon() {
        removeLink = new AjaxLink<Void>("link") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                removeCoAuthor();
                target.add(coAuthorContainer);
            }

        };
        removeLink.setVisible(coAuthorExists());
        coAuthorContainer.add(removeLink);
    }

    public final void removeCoAuthor() {
        if (coAuthorExists()) {
            coAuthor = null;
            removeLink.setVisible(false);
            coAuthorContainer.replace(new EmptyPanel(CO_AUTHOR_LABEL));
            coAuthorField.setVisible(true);
        }
    }

    public User getCoAuthor() {
        return coAuthor;
    }

    private boolean coAuthorExists() {
        return coAuthor != null;
    }

    /**
     * Method to trick the auto complete components focus that bugs out if we have it focused on opening dialog.
     * This makes it still focused and work properly.
     */
    private void reFocus() {
        add(new AbstractAjaxTimerBehavior(Duration.milliseconds(1)) {

            @Override
            protected void onTimer(AjaxRequestTarget target) {
                target.focusComponent(coAuthorField);
                stop(target);
            }
        });
    }
}
