package se.su.dsv.scipro.project.panels;

import org.apache.wicket.Page;
import se.su.dsv.scipro.components.AbstractMenuPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorFinalSeminars;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyGroups;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightProjectIdeas;
import se.su.dsv.scipro.finalseminar.FinalSeminarProjectListPage;
import se.su.dsv.scipro.group.AuthorMyGroupsPage;
import se.su.dsv.scipro.match.ProjectIdeaStartPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.pages.ProjectPage;
import se.su.dsv.scipro.project.pages.ProjectStartPage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ProjectTabMenuPanel extends AbstractMenuPanel {

    public static final String ALL_FINAL_SEMINARS = "All Final Seminars";
    public static final String IDEAS = "Ideas";
    public static final String MY_PROJECT_S = "My projects";
    @Inject
    private ProjectService projectService;
    @Inject
    private ProjectTypeService projectTypeService;

    public ProjectTabMenuPanel(final String id, final Class<? extends Page> containerClass) {
        super(id, ProjectPage.class, containerClass);
    }

    @Override
    protected List<MenuItem> getItemList() {
        List<MenuItem> itemList = new ArrayList<>();
        itemList.add(new MenuItem(MY_PROJECT_S, ProjectStartPage.class, MenuHighlightAuthorMyProjects.class));
        itemList.add(new MenuItem("My groups", AuthorMyGroupsPage.class, MenuHighlightAuthorMyGroups.class));
        if (matchIsEnabled()) {
            itemList.add(new MenuItem(IDEAS, ProjectIdeaStartPage.class, MenuHighlightProjectIdeas.class));
        }
        if (studentHasProjectWithDegreeLevel()) {
            itemList.add(new MenuItem(ALL_FINAL_SEMINARS, FinalSeminarProjectListPage.class, MenuHighlightAuthorFinalSeminars.class));
        }
        return itemList;
    }

    private boolean matchIsEnabled() {
        return projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH);
    }

    private boolean studentHasProjectWithDegreeLevel() {
        List<DegreeLevel> degreeLevels = Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE);
        for (Project project : projectService.getProjectsByUser(SciProSession.get().getUser())) {
            if (degreeLevels.contains(project.getProjectTypeDegreeLevel())) {
                return true;
            }
        }
        return false;
    }

    @Override
    protected MenuType getMenuType() {
        return MenuType.TAB_MULTIPLE_ROWS;
    }

}
