package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.navigation.paging.PagingNavigator;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightProjectIdeas;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.match.AbstractIdeaProjectPage;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;
import java.util.Date;

public class ProjectPartnerPage extends AbstractIdeaProjectPage implements MenuHighlightProjectIdeas {

    public static final int ITEMS_PER_PAGE = 5;
    @Inject
    private ProjectPartnerService projectPartnerService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    public ProjectPartnerPage(PageParameters pp) {
        super(pp);

        GeneralSystemSettings settings = generalSystemSettingsService.getGeneralSystemSettingsInstance();
        add(new FeedbackPanel("feedback"));
        ProjectPartnerDataProvider projectPartnerDataProvider = new ProjectPartnerDataProvider(settings.getProjectPartnerDaysToLive());
        final DataView<ProjectPartner> dataView = new DataView<ProjectPartner>("list", projectPartnerDataProvider, ITEMS_PER_PAGE) {
            @Override
            protected void populateItem(Item<ProjectPartner> item) {
                item.add(new Label("exname", new PropertyModel<String>(item.getModel(), "user.fullName")));
                item.add(new Label("exmail", new PropertyModel<String>(item.getModel(), "user.emailAddress")));
                item.add(new DateLabel("exdate", new PropertyModel<Date>(item.getModel(), "dateCreated")));
                item.add(new MultiLineLabel("exinfoText", new PropertyModel<String>(item.getModel(), "infotext")));
            }
        };

        add(new InfoPanel("infoPanel", new ResourceModel("info.text")));

        Form<ProjectPartner> form = new Form<ProjectPartner>("projectPartnerForm", new CompoundPropertyModel<>(new ProjectPartner(SciProSession.get().getUser()))){
            @Override
            protected void onSubmit() {
                info(getString("saved"));
                projectPartnerService.save(getModelObject());
                setResponsePage(ProjectPartnerPage.class);
            }
        };
        TextArea<String> infoText = new TextArea<>("infotext");
        infoText.setRequired(true);
        form.add(infoText);
        add(form);

        WebMarkupContainer container = new WebMarkupContainer("listContainer");
        container.add(dataView);
        container.setOutputMarkupId(true);
        PagingNavigator pagingNavigator = new PagingNavigator("navigator", dataView);
        container.add(pagingNavigator);
        add(container);
    }
}