package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.navigation.paging.PagingNavigator;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class ProjectPartnerPanel extends Panel {


    public static final int ITEMS_PER_PAGE = 5;
    @Inject
    private ProjectPartnerService projectPartnerService;

    private IModel<List<ProjectPartner>> listModel;
    private WebMarkupContainer wmc;
    private DataView<ProjectPartner> dataView;

    public ProjectPartnerPanel(String id) {
        super(id);

        ProjectPartnerDataProvider projectPartnerDataProvider = new ProjectPartnerDataProvider(0);

        wmc = new WebMarkupContainer("listContainer");

        listModel = new LoadableDetachableModel<List<ProjectPartner>>() {
            @Override
            protected List<ProjectPartner> load() {
                return projectPartnerService.getProjectPartnerInSpan(0, new PageRequest(dataView.getCurrentPage(), dataView.getItemsPerPage()));
            }

        };

        wmc.setOutputMarkupId(true);

        loadDataView(projectPartnerDataProvider);
        wmc.add(dataView);
        wmc.add(new PagingNavigator("navigator", dataView));

        add(wmc);
    }

    private void loadDataView(ProjectPartnerDataProvider projectPartnerDataProvider) {
        dataView = new DataView<ProjectPartner>("list", projectPartnerDataProvider, ITEMS_PER_PAGE) {
            /**
             *
             */

            @Override
            protected void populateItem(final Item<ProjectPartner> item) {
                item.add(new Label("exname", new PropertyModel<String>(item.getModel(), "user.toString")));
                item.add(new Label("exmail", new PropertyModel<String>(item.getModel(), "user.emailAddress")));
                item.add(new DateLabel("exdate", new PropertyModel<Date>(item.getModel(), "dateCreated")));
                item.add(new MultiLineLabel("exinfoText", new PropertyModel<String>(item.getModel(), "infotext")));

                AjaxConfirmationLink<Void> deleteLink = new AjaxConfirmationLink<Void>("delete", "Are you sure you want to remove this project partner application?") {


                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        ProjectPartner temp = item.getModelObject();

                        projectPartnerService.delete(temp.getId());

                        listModel.detach();

                        target.add(wmc);

                    }

                };
                item.add(deleteLink);

            }


        };
    }
}
