package se.su.dsv.scipro.reviewer;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.dataobjects.ReviewerThread;
import se.su.dsv.scipro.forum.panels.threaded.SubmitForumReplyPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.panels.ProjectHeaderPanel;
import se.su.dsv.scipro.reviewer.timeline.TimelinePanel;
import se.su.dsv.scipro.reviewing.ReviewerInteractionService;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

public class ReviewerInteractionPage extends ReviewerPage {

    @Inject
    private ReviewerInteractionService reviewerInteractionService;
    @Inject
    private ProjectService projectService;
    @Inject
    private FileDescriptionService fileDescriptionService;
    @Inject
    private BasicForumService basicForumService;

    public ReviewerInteractionPage(PageParameters pp) {
        super(pp);
        IModel<Project> projectModel = validatePageLoad(pp);
        add(new Label("projectTitle", new PropertyModel<String>(projectModel, "title")));
        add(new ProjectHeaderPanel("header", projectModel));
        add(new InfoPanel("info", new ResourceModel("info")));

        ReviewerThread reviewerThread = reviewerInteractionService.getReviewerThread(projectModel.getObject());
        if (reviewerThread == null || !reviewerThread.getProject().equals(projectModel.getObject())) {
            throw new RestartResponseException(getApplication().getHomePage());
        }
        basicForumService.setThreadRead(SciProSession.get().getUser(), reviewerThread.getForumThread(), true);

        add(new SubmitForumReplyPanel<>("communication", projectModel, new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)));
        add(new TimelinePanel("timeline", projectModel));
    }

    private IModel<Project> validatePageLoad(PageParameters pp) {
        final StringValue value = pp.get(PageParameterKeys.MAP.get(Project.class));
        if(value.isNull()) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        IModel<Project> projectModel = new LoadableDetachableModel<Project>() {
            @Override
            protected Project load() {
                return projectService.findOne(value.toLong());
            }
        };
        if (projectModel.getObject() == null || !projectModel.getObject().isReviewer(SciProSession.get().getUser())) {
            throw new RestartResponseException(AccessDeniedPage.class);
        }
        return projectModel;
    }
}
