package se.su.dsv.scipro.reviewer;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.components.datatables.UserColumn;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.finalthesis.FinalThesisRepo;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.forum.BasicForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.reviewing.MyReviewService;
import se.su.dsv.scipro.reviewing.ReviewerDeadlineSettingsService;
import se.su.dsv.scipro.reviewing.ReviewerInteractionService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.*;

import static java.util.Arrays.asList;

@Authorization(authorizedRoles = {Roles.REVIEWER})
public class ReviewerStartPage extends ReviewerPage {
    public static final String MAIN_MENU_LABEL = "Reviewer";

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private FinalThesisRepo finalThesisRepo;
    @Inject
    private MyReviewService myReviewService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;
    @Inject
    private ReviewerInteractionService reviewerInteractionService;
    @Inject
    private ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    @Inject
    private DaysService daysService;
    @Inject
    private BasicForumService basicForumService;

    public ReviewerStartPage() {
        addFinalGradingReports();
        addRoughDraftApprovals();
        addFinalSeminarApprovals();
        addInteractWithReviewer();
    }

    private void addFinalGradingReports() {
        final Model<Boolean> showAllModel = Model.of(false);

        SortableDataProvider<FinalThesis, String> finalGradingProvider = new SortableDataProvider<FinalThesis, String>() {
            @Override
            public Iterator<? extends FinalThesis> iterator(long first, long count) {
                return finalThesisRepo.reviewers(SciProSession.get().getUser(), showAllModel.getObject(), new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return finalThesisRepo.countReviewers(SciProSession.get().getUser(), showAllModel.getObject());
            }

            @Override
            public IModel<FinalThesis> model(FinalThesis object) {
                return new DetachableServiceModel<>(finalThesisService, object);
            }
        };
        finalGradingProvider.setSort("dateCreated", SortOrder.ASCENDING);
        add(new Label("noUnfinishedFinal", new AbstractReadOnlyModel<Object>() {
            @Override
            public Object getObject() {
                long unfinished = finalThesisRepo.countReviewers(SciProSession.get().getUser(), false);
                return unfinished == 0 ? "" : unfinished;
            }
        }));
        final DefaultDataPanel<FinalThesis, String> dataPanel = new DefaultDataPanel<>("finalGrading", createFinalGradingColumns(), finalGradingProvider);
        dataPanel.setOutputMarkupId(true);
        add(dataPanel);

        final AjaxCheckBox showAll = new AjaxCheckBox("showAll", showAllModel) {
            @Override
            protected void onUpdate(AjaxRequestTarget ajaxRequestTarget) {
                ajaxRequestTarget.add(dataPanel);
            }
        };
        add(showAll);
    }

    private void addRoughDraftApprovals() {
        add(new Label("undecidedRoughDraftApprovals", new AbstractReadOnlyModel<Object>() {
            @Override
            public Object getObject() {
                long undecided = myReviewService.countUndecidedRoughDraft(SciProSession.get().getUser());
                return undecided == 0 ? "" : undecided;
            }
        }));
        add(new RoughDraftApprovalReviewerPanel("RoughDraftApprovals"));
    }

    private void addFinalSeminarApprovals() {
        add(new Label("undecidedFinalSeminarApprovals", new AbstractReadOnlyModel<Object>() {
            @Override
            public Object getObject() {
                long undecided = myReviewService.countUndecidedFinalSeminar(SciProSession.get().getUser());
                return undecided == 0 ? "" : undecided;
            }
        }));
        add(new FinalSeminarApprovalReviewerPanel("FinalSeminarApprovals"));
    }

    private void addInteractWithReviewer() {
        final ProjectService.Filter filter = new ProjectService.Filter();
        filter.setFilterSupervisor(false);
        filter.setSupervisor(SciProSession.get().getUser());
        filter.setStatuses(Collections.singletonList(ProjectStatus.ACTIVE));
        filter.setRoles(Collections.singletonList(ProjectTeamMemberRoles.REVIEWER));
        filter.setProjectTypes(projectTypeService.findAllActive());

        final SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(filter));

        DefaultDataTable<Project, String> table = new DefaultDataTable<>("dataPanel", createColumns(), provider, 1000);
        add(table);

        addFilteringOptions(filter, table);

        add(new Label("unreadSupervisorsPosts", new AbstractReadOnlyModel<Object>() {
            @Override
            public Object getObject() {
                List<Project> projects = projectService.findAll(filter);
                int totalNumberOfUnreadPosts = 0;
                for (Project project : projects) {
                    User user = SciProSession.get().getUser();
                    totalNumberOfUnreadPosts += getUnreadPosts(user, project);
                }
                return (totalNumberOfUnreadPosts == 0) ? "" : totalNumberOfUnreadPosts;
            }
        }));
    }

    private int getUnreadPosts(User user, Project project) {
        int unread = 0;
        for (ForumPost post : reviewerInteractionService.getPosts(project)) {
            if (!basicForumService.isRead(user, post)) {
                unread++;
            }
        }
        return unread;
    }

    private void addFilteringOptions(final ProjectService.Filter filter, final Component table) {
        table.setOutputMarkupId(true);
        add(new AjaxCheckBoxMultipleChoice<ProjectStatus>("status",
                new PropertyModel<Collection<ProjectStatus>>(filter, "statuses"),
                asList(ProjectStatus.values()),
                new EnumChoiceRenderer<ProjectStatus>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(table);
            }
        });
        add(new AjaxCheckBoxMultipleChoice<ProjectType>("type",
                new PropertyModel<Collection<ProjectType>>(filter, "projectTypes"),
                projectTypeService.findBySupervisorProjects(SciProSession.get().getUser()),
                new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(table);
            }
        });
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Project>(Model.of("Started"), "dateCreated"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name"));
        columns.add(new AbstractExportableColumn<Project, String, String>(Model.of("Title")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, final IModel<Project> rowModel) {
                cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                    @Override
                    public AbstractLink apply(final String id) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Project.class), rowModel.getObject().getId());
                        AbstractLink link = new BookmarkablePageLink<>(id, ReviewerInteractionPage.class, pp);
                        link.setBody(new PropertyModel<>(rowModel, "title"));
                        return link;
                    }
                }));
            }

            @Override
            public IModel<String> getDataModel(IModel<Project> rowModel) {
                return new PropertyModel<>(rowModel, "title");
            }
        });
        columns.add(new MultipleUsersColumn<Project, String>(Model.of("Authors")) {
            @Override
            public IModel<? extends List<? extends User>> getUsers(final IModel<Project> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<Iterable<User>>(rowModel, "ProjectParticipants"));
            }
        });
        columns.add(new AbstractExportableColumn<Project, String, String>(Model.of("Head supervisor")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "headSupervisor")));
            }

            @Override
            public IModel<String> getDataModel(IModel<Project> rowModel) {
                return new PropertyModel<>(rowModel, "headSupervisor.fullName");
            }
        });
        columns.add(new AbstractExportableColumn<Project, String, Integer>(Model.of("Unread posts")) {
            @Override
            public IModel<Integer> getDataModel(final IModel<Project> rowModel) {
                return new AbstractReadOnlyModel<Integer>() {
                    @Override
                    public Integer getObject() {
                        return getUnreadPosts(SciProSession.get().getUser(), rowModel.getObject());
                    }
                };
            }
        });
        return columns;
    }

    private List<? extends IColumn<FinalThesis, String>> createFinalGradingColumns() {
        List<IColumn<FinalThesis, String>> columns = new ArrayList<>();

        columns.add(new AbstractColumn<FinalThesis, String>(Model.of("Requested"),"dateCreated") {
            @Override
            public void populateItem(Item<ICellPopulator<FinalThesis>> cellItem, String componentId, IModel<FinalThesis> rowModel) {
                cellItem.add(new DateLabel(componentId, new PropertyModel<Date>(rowModel, "dateApproved"), DateStyle.DATE));
            }
        });
        columns.add(new AbstractColumn<FinalThesis, String>(Model.of("Deadline")) {
            @Override
            public void populateItem(Item<ICellPopulator<FinalThesis>> cellItem, String componentId, final IModel<FinalThesis> rowModel) {
                final int daysToFinish = reviewerDeadlineSettingsService.getInstance().getFinalGrading();
                IModel<Date> deadline = new AbstractReadOnlyModel<Date>() {
                    @Override
                    public Date getObject() {
                        return daysService.workDaysAfter(rowModel.getObject().getDateApproved(), daysToFinish);
                    }
                };
                cellItem.add(new DateLabel(componentId, deadline, DateStyle.DATE));
            }
        });
        columns.add(new AbstractColumn<FinalThesis, String>(Model.of("Title"), "project.title") {
            @Override
            public void populateItem(final Item<ICellPopulator<FinalThesis>> cellItem, final String componentId, final IModel<FinalThesis> rowModel) {
                cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                    @Override
                    public AbstractLink apply(final String id) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(Project.class), rowModel.getObject().getProject().getId());
                        BookmarkablePageLink<Void> link = new BookmarkablePageLink<>(id, ReviewerGradingPage.class, pp);
                        link.setBody(new PropertyModel<String>(rowModel, "project.title"));
                        return link;
                    }
                }));
            }
        });
        columns.add(new MultipleUsersColumn<FinalThesis, String>(Model.of("Authors")) {
            @Override
            public IModel<? extends List<? extends User>> getUsers(final IModel<FinalThesis> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<List<User>>(rowModel, "project.projectParticipants"));
            }
        });
        columns.add(new UserColumn<FinalThesis, String>(Model.of("Supervisor")) {
            @Override
            protected IModel<User> getUser(final IModel<FinalThesis> rowModel) {
                return new PropertyModel<>(rowModel, "project.headSupervisor");
            }
        });
        return columns;
    }
}
