package se.su.dsv.scipro.security.auth;

import org.apache.wicket.Component;
import org.apache.wicket.authorization.Action;
import org.apache.wicket.authorization.IAuthorizationStrategy;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.request.component.IRequestableComponent;
import se.su.dsv.scipro.session.SciProSession;

public class ProjectModuleBasedAuthorizationStrategy implements IAuthorizationStrategy {

    @Override
    public <T extends IRequestableComponent> boolean isInstantiationAuthorized(Class<T> componentClass) {
        return true;
    }

    @Override
    public boolean isActionAuthorized(Component component, Action action) {
        ProjectModuleComponent annotation = getAnnotation(component);
        return annotation == null || isAuthorized(annotation);
    }

    private boolean isAuthorized(ProjectModuleComponent annotation) {
        return SciProSession.get().getProjectModules().contains(annotation.value());
    }

    private ProjectModuleComponent getAnnotation(Component component) {
        Class<? extends Component> componentClass;

        if (component instanceof BookmarkablePageLink<?>) {
            BookmarkablePageLink<?> bookmarkablePageLink = (BookmarkablePageLink<?>) component;
            componentClass = bookmarkablePageLink.getPageClass();
        } else {
            componentClass = component.getClass();
        }

        return internalGetAnnotation(componentClass);
    }

    ProjectModuleComponent internalGetAnnotation(Class<? extends Component> authRequired) {
        return authRequired.getAnnotation(ProjectModuleComponent.class);
    }

}
