package se.su.dsv.scipro.security.auth;

import org.apache.wicket.Component;
import org.apache.wicket.Page;
import org.apache.wicket.RestartResponseAtInterceptPageException;
import org.apache.wicket.authorization.Action;
import org.apache.wicket.authorization.IAuthorizationStrategy;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.request.component.IRequestableComponent;
import se.su.dsv.scipro.loginlogout.pages.LoginPage;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;

public class RoleBasedAuthorizationStrategy implements IAuthorizationStrategy {

    protected boolean instantiationAuthorized(Authorization annotation, SciProSession session) {
        //Check the users role for authorization to instantiate the component
        for(Roles role : annotation.authorizedRoles()){
            if( session.authorizedForRole(role) ) {
                return true;
            }
        }
        //No roles were added to the annotation or user was not authorized for the roles that were added
        return false;
    }

    @Override
    public <T extends IRequestableComponent> boolean isInstantiationAuthorized(Class<T> componentClass) {
        Authorization annotation = componentClass.getAnnotation(Authorization.class);
        if (annotation != null) {
            if( !annotation.requiresLoggedInUser() ) {
                return true;
            }
            SciProSession session = SciProSession.get();
            //If page requires login and user isn't logged in, send them to login page first
            if( annotation.requiresLoggedInUser() && !session.isLoggedIn() ){
                throw new RestartResponseAtInterceptPageException(LoginPage.class);
            }
            //We only check pages for instantiation authorization, other components are handled via ActionAuthorization
            return !(Page.class.isAssignableFrom(componentClass)) || instantiationAuthorized(annotation, session);


        }
        return true;
    }

    @Override
    public boolean isActionAuthorized(Component component, Action action) {
        Authorization annotation = getAnnotation(component);

        if( annotation ==  null || !annotation.requiresLoggedInUser()) {
            return true;
        }

        SciProSession session = SciProSession.get();
        if (!session.isLoggedIn()) {
            return false;
        }

        return instantiationAuthorized(annotation, session) || allowed(action);

    }

    private boolean allowed(Action action) {
        return !action.equals(Component.RENDER);
    }

    private Authorization getAnnotation(Component component) {
        Class<? extends Component> componentClass;

        if (component instanceof BookmarkablePageLink<?>) {
            BookmarkablePageLink<?> bookmarkablePageLink = (BookmarkablePageLink<?>) component;
            componentClass = bookmarkablePageLink.getPageClass();
        } else {
            componentClass = component.getClass();
        }

        return internalGetAnnotation(componentClass);
    }

    Authorization internalGetAnnotation(Class<? extends Component> authRequired) {
        return authRequired.getAnnotation(Authorization.class);
    }
}
