package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;

import javax.inject.Inject;
import java.util.Iterator;
import java.util.List;

public abstract class AbstractMilestoneDetailsPanel extends GenericPanel<MilestoneActivityTemplate> {

    public static final String TITLE = "title";
    public static final String TABLE = "table";

    @Inject
    private MileStoneService mileStoneService;

    private MileStoneService.Filter filter;

    public AbstractMilestoneDetailsPanel(String id, IModel<MilestoneActivityTemplate> model, MileStoneService.Filter filter) {
        super(id, model);
        this.filter = filter;

        add(new MilestoneFilterDetailsPanel("filterDetails", filter));
        add(new Label(TITLE, new PropertyModel<>(getModel(), TITLE)));

        addProjectTable();
    }

    private void addProjectTable() {
        SortableDataProvider<Milestone, String> provider = new SortableDataProvider<Milestone, String>() {

            @Override
            public Iterator<? extends Milestone> iterator(long first, long count) {
                return mileStoneService.findCompletedMilestones(getModelObject(), filter, new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return mileStoneService.countCompletedMileStones(getModelObject(), filter);
            }

            @Override
            public IModel<Milestone> model(Milestone object) {
                return new DetachableServiceModel<>(mileStoneService, object);
            }
        };
        provider.setSort("lastModified", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>(TABLE, createColumns(), provider));
    }

    protected abstract List<IColumn<Milestone,String>> createColumns();

}
