package se.su.dsv.scipro.statistics;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminAuthorStatistics;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;

import static java.util.Arrays.asList;

public class AdminAuthorsStatisticsPage extends AbstractAdminStatisticsPage implements MenuHighlightAdminAuthorStatistics {

    protected static final String CONTAINER = "container";
    protected static final String PROJECT_TYPE_FILTER = "projectTypeFilter";
    protected static final String STATUS_FILTER = "statusFilter";
    public static final String AUTHORS = "authors";
    public static final String DATE_FILTER = "createdBetween";
    public static final String MANY_PROJECTS = "authorsWithManyProjects";
    public static final String DETAILS_LINK = "detailsLink";

    @Inject
    private ProjectService projectService;
    @Inject
    private ProjectTypeService projectTypeService;

    private ProjectService.Filter params;
    private WebMarkupContainer container;

    public AdminAuthorsStatisticsPage() {
        params = new ProjectService.Filter();
        container = new WebMarkupContainer(CONTAINER);
        container.setOutputMarkupId(true);
        add(container);

        addDateFilter();
        addProjectTypeFilter();
        addProjectStatusFilter();

        container.add(new Label(AUTHORS, getCount()));
        Link<Void> detailsLink = new Link<Void>(DETAILS_LINK) {
            @Override
            public void onClick() {
                setResponsePage(new AdminMultipleAuthorsStatisticsPage(params));
            }
        };
        detailsLink.add(new Label(MANY_PROJECTS, getMultiAuthorCount()));
        container.add(detailsLink);
    }

    private LoadableDetachableModel<String> getMultiAuthorCount() {
        return new LoadableDetachableModel<String>() {
            @Override
            protected String load() {
                return String.valueOf(projectService.countMultiProjectAuthors(params));
            }
        };
    }

    private LoadableDetachableModel<String> getCount() {
        return new LoadableDetachableModel<String>() {
            @Override
            protected String load() {
                return String.valueOf(projectService.countAuthors(params));
            }
        };
    }

    private void addDateFilter() {
        container.add(new DeactivatableDatePickerPanel(DATE_FILTER,
                new PropertyModel<Date>(params, "createdAfter"),
                new PropertyModel<Date>(params, "createdBefore")) {
            @Override
            protected void datesChanged(AjaxRequestTarget target) {
                target.add(container);
            }
        });
    }

    private void addProjectTypeFilter() {
        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>(PROJECT_TYPE_FILTER,
                projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                projectTypeService.findAllActive(),
                new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                if (!getModelObject().isEmpty()) {
                    params.setProjectTypes(getModelObject());
                }
                target.add(container);
            }
        };
        params.setProjectTypes(projectType.getModelObject());
        container.add(projectType);
    }

    private void addProjectStatusFilter() {
        AjaxCheckBoxMultipleChoice<ProjectStatus> projectStatus = new AjaxCheckBoxMultipleChoice<ProjectStatus>(STATUS_FILTER,
                new ArrayList<>(asList(ProjectStatus.ACTIVE, ProjectStatus.COMPLETED)),
                asList(ProjectStatus.values()),
                new EnumChoiceRenderer<ProjectStatus>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                params.setStatuses(new HashSet<>(getModelObject()));
                target.add(container);
            }
        };
        container.add(projectStatus);
        params.setStatuses(new HashSet<>(projectStatus.getModelObject()));
    }
}