package se.su.dsv.scipro.statistics;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.joda.time.LocalDate;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.date.DatePickerPanel;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalService;
import se.su.dsv.scipro.reviewing.ProjectFinalSeminarStatisticsService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.*;

import static java.util.Arrays.asList;

public class AdminFinalSeminarStatisticsPage extends AbstractAdminStatisticsPage {

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectFinalSeminarStatisticsService projectFinalSeminarStatisticsService;
    @Inject
    private FinalSeminarApprovalService finalSeminarApprovalService;
    @Inject
    private FinalSeminarService finalSeminarService;

    private ProjectFinalSeminarStatisticsService.Filter filter;
    private final WebMarkupContainer wmc;

    public AdminFinalSeminarStatisticsPage() {

        add(new FilterForm("filterForm"));

        wmc = new WebMarkupContainer("wmc");
        wmc.setOutputMarkupId(true);
        add(wmc);

        SortableDataProvider<Project, String> fsNoOkProvider = new SortableDataProvider<Project, String>() {
            @Override
            public Iterator<? extends Project> iterator(long first, long count) {
                PageRequest pageRequest = new PageAdapter(first, count, getSort());
                return projectFinalSeminarStatisticsService.finalSeminarStatisticsNoOkFromReviewer(filter, pageRequest).iterator();
            }
            @Override
            public long size() {
                return projectFinalSeminarStatisticsService.countFinalSeminarStatisticsNoOkFromReviewer(filter);
            }
            @Override
            public IModel<Project> model(Project object) {
                return new DetachableServiceModel<>(projectFinalSeminarStatisticsService, object);
            }
        };
        fsNoOkProvider.setSort("dateCreated", SortOrder.DESCENDING);
        wmc.add(new ExportableDataPanel<>("fsNoOk", createColumns(), fsNoOkProvider));
        
        wmc.add(new Label("finalSeminars", getFinalSeminars()));
        wmc.add(new Label("finalSeminarsOkFromReviewer", getFinalSeminarsOkFromReviewer()));

    }

    private LoadableDetachableModel<Long> getFinalSeminars(){
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectFinalSeminarStatisticsService.countFinalSeminarStatistics(filter);
            }
        };
    }

    private LoadableDetachableModel<Long> getFinalSeminarsOkFromReviewer(){
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectFinalSeminarStatisticsService.countFinalSeminarStatisticsOkFromReviewer(filter);
            }
        };
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Project>(Model.of("Project started"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Title"), "title", "title"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectTypeName"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Status"), "projectStatus"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Supervisor"), "headSupervisor.fullName", "headSupervisor.firstName"));

        columns.add(new AbstractColumn<Project, String>(Model.of("Current final seminar approval request status")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                FinalSeminarApproval byProject = finalSeminarApprovalService.findByProject(rowModel.getObject());
                cellItem.add(new Label(componentId, byProject == null ? "-" : byProject.getCurrentStatus()));
            }
        });
        columns.add(new AbstractColumn<Project, String>(Model.of("Current final seminar approval request reason")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                FinalSeminarApproval byProject = finalSeminarApprovalService.findByProject(rowModel.getObject());
                cellItem.add(new Label(componentId, byProject == null ? "-" : byProject.getCurrentReason()));
            }
        });
        columns.add(new AbstractColumn<Project, String>(Model.of("Supervisor reason for creating final seminar")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                cellItem.add(new Label(componentId, finalSeminarService.findByProject(rowModel.getObject()).getCreationReason()));
            }
        });
        return columns;
    }

    private class FilterForm extends Form<Void> {

        public FilterForm(String id) {
            super(id);

            filter = new ProjectFinalSeminarStatisticsService.Filter();
            filter.setFinalSeminarCreatedBefore(new Date());
            filter.setFinalSeminarCreatedAfter(new LocalDate().minusMonths(6).toDate());

            AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                    projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                    projectTypeService.findAllActive(),
                    new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    if (!getModelObject().isEmpty()) {
                        filter.setProjectTypes(getModelObject());
                    }
                    target.add(wmc);
                }
            };
            filter.setProjectTypes(projectType.getModelObject());
            add(projectType);

            add(new DeactivatableDatePickerPanel("dateFilter",
                    new PropertyModel<Date>(filter, "createdAfter"),
                    new PropertyModel<Date>(filter, "createdBefore")) {
                @Override
                protected void datesChanged(AjaxRequestTarget target) {
                    target.add(wmc);
                }
            });

            add(new DatePickerPanel("fsCreatedBetween",
                    new PropertyModel<Date>(filter, "finalSeminarCreatedAfter"),
                    new PropertyModel<Date>(filter, "finalSeminarCreatedBefore")) {
                @Override
                public void updateStartDate(final AjaxRequestTarget target) {
                    target.add(wmc);
                }

                @Override
                public void updateEndDate(final AjaxRequestTarget target) {
                    target.add(wmc);
                }
            });

            AjaxCheckBoxMultipleChoice<ProjectStatus> projectStatus = new AjaxCheckBoxMultipleChoice<ProjectStatus>("statusFilter",
                    asList(ProjectStatus.ACTIVE, ProjectStatus.COMPLETED),
                    asList(ProjectStatus.values()),
                    new EnumChoiceRenderer<ProjectStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    filter.setStatuses(new HashSet<>(getModelObject()));
                    target.add(wmc);
                }
            };
            add(projectStatus);
            filter.setStatuses(new HashSet<>(projectStatus.getModelObject()));
        }
    }
}