package se.su.dsv.scipro.statistics;

import com.google.common.base.Joiner;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminAuthorStatistics;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class AdminMultipleAuthorsStatisticsPage extends AbstractAdminStatisticsPage implements MenuHighlightAdminAuthorStatistics {

    public static final String AUTHOR_LIST = "authorList";
    public static final String AUTHOR_NAME = "fullName";
    public static final String PROJECTS = "projects";

    @Inject
    private ProjectService projectService;

    private ProjectService.Filter filter;

    public AdminMultipleAuthorsStatisticsPage(ProjectService.Filter filter) {
        this.filter = filter;
        add(new PropertyListView<User>(AUTHOR_LIST, getMultipleAuthors()) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new Label(AUTHOR_NAME));
                item.add(new Label(PROJECTS, getProjects(item.getModel())));
            }
        });
    }

    private String getProjects(IModel<User> model) {
        filter.setAuthorUser(model.getObject());
        List<String> result = new ArrayList<>();
        for (Project project : projectService.findAll(filter)) {
            result.add(project.getTitle());
        }
        return Joiner.on(", ").join(result);
    }

    private LoadableDetachableModel<List<? extends User>> getMultipleAuthors() {
        return new LoadableDetachableModel<List<? extends User>>() {
            @Override
            protected List<? extends User> load() {
                return projectService.findMultipleAuthors(filter);
            }
        };
    }
}
