package se.su.dsv.scipro.statistics;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.project.ProjectPeopleStatisticsService;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

public class AdminPeopleStatisticsPage extends AbstractAdminStatisticsPage {

    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectPeopleStatisticsService projectPeopleStatisticsService;

    private ProjectService.Filter params;

    public AdminPeopleStatisticsPage() {

        final WebMarkupContainer container = new WebMarkupContainer("container"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(params.getProjectTypes()!=null && !params.getProjectTypes().isEmpty());
            }
        };
        container.setOutputMarkupPlaceholderTag(true);
        add(container);

        params = new ProjectService.Filter();
        params.setProjectTypes(projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)));

        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                new PropertyModel<List<ProjectType>>(params, "projectTypes"),
                projectTypeService.findAllActive(),
                new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(container);
            }
        };

        params.setProjectTypes(projectType.getModelObject());
        add(projectType);

        container.add(new InfoPanel("info", new ResourceModel("info")));

        container.add(new Label("students", new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectPeopleStatisticsService.peopleStudents(params);
            }
        }));

        container.add(new Label("headSupervisors", new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectPeopleStatisticsService.peopleHeadSupervisors(params);
            }
        }));

        container.add(new Label("coSupervisors", new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectPeopleStatisticsService.peopleCoSupervisors(params);
            }
        }));

        container.add(new Label("reviewers", new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return projectPeopleStatisticsService.peopleReviewers(params);
            }
        }));
    }
}

