package se.su.dsv.scipro.statistics;

import com.google.common.base.Joiner;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.StringResourceModel;
import org.apache.wicket.util.io.IClusterable;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.date.DateService;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class MilestoneFilterDetailsPanel extends Panel {

    @Inject
    private DateService dateService;

    private final MileStoneService.Filter filter;

    public MilestoneFilterDetailsPanel(String id, MileStoneService.Filter filter) {
        super(id);
        this.filter = filter;
        add(new Label("filterInfo",
                new StringResourceModel("filterInfo",
                        Model.of(new FilterStringFormatter()))).setEscapeModelStrings(false));

    }

    protected class FilterStringFormatter implements IClusterable {

        public String projectTypeNames() {
            List<String> projectTypes = new ArrayList<>();
            for (ProjectType projectType : filter.getProjectTypes()) {
                projectTypes.add(projectType.getName());
            }
            return Joiner.on(", ").join(projectTypes);
        }

        public String projectStatusNames() {
            return Joiner.on(", ").join(filter.getProjectStatuses());
        }

        public String startedBetween() {
            if (filter.getProjectStartedAfter() == null) {
                return getString("notAvailable");
            }
            String startedAfter = dateService.format(filter.getProjectStartedAfter(), DateStyle.DATE);
            String startedBefore = dateService.format(filter.getProjectStartedBefore(), DateStyle.DATE);
            return startedAfter + " and " + startedBefore;
        }

        public String completedBefore() {
            return dateService.format(filter.getMileStoneCompletedBefore(), DateStyle.DATE);
        }
    }
}
