package se.su.dsv.scipro.statistics;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class SupervisorIdeaStatisticsPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    private final IdeaService.Filter param = new IdeaService.Filter();

    public SupervisorIdeaStatisticsPanel(String id) {
        super(id);
        addFilters();
        addIdeaTable();
    }

    private void addFilters() {
        param.setStatuses(Collections.singletonList(Idea.Status.MATCHED));
        param.setNoSupervisor(true);
    }

    private void addIdeaTable() {
        List<? extends IColumn<Idea, String>> columns = Arrays.asList(
                new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"),
                new PropertyColumn<Idea, String>(Model.of("Idea"), "type", "type"),
                new PropertyColumn<Idea, String>(Model.of("Title"), "title", "title"),
                new PropertyColumn<Idea, String>(Model.of("Authors"), "authorNames")
        );
        FilteredDataProvider<Idea, IdeaService.Filter> provider = new FilteredDataProvider<>(ideaService, Model.of(param));
        provider.setSort("title", SortOrder.ASCENDING);
        add(new ExportableDataPanel<>(IDEAS, columns, provider));
    }

    @Override
    protected void onConfigure() {
        setVisible(param.getSupervisor() != null);
    }

    protected void updateSupervisor(User newSupervisor) {
        param.setSupervisor(newSupervisor);
    }

    static final String IDEAS = "ideaTable";
}
