package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorMyGroups;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.group.Group;
import se.su.dsv.scipro.group.GroupDetailsPanel;
import se.su.dsv.scipro.group.GroupFacade;
import se.su.dsv.scipro.group.GroupService;
import se.su.dsv.scipro.security.auth.SystemModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.SystemModule;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import javax.servlet.http.Cookie;

@SystemModuleComponent(SystemModule.GROUP)
public abstract class AbstractSupervisorGroupPage extends AbstractSupervisorPage implements MenuHighlightSupervisorMyGroups {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractSupervisorGroupPage.class);
    public static final String COOKIE_KEY = "supervisorLastVisitedGID";
    public static final int MAX_AGE = 2592000;

    @Inject
    protected GroupService groupService;
    @Inject
    protected GroupFacade groupFacade;

    protected DetachableServiceModel<Group> groupModel;

    public AbstractSupervisorGroupPage(PageParameters pp) {
        super(pp);

        Long groupId = null;
        StringValue groupValue = pp.get(PageParameterKeys.MAP.get(Group.class));

        if (!groupValue.isNull()) {
            try {
                groupId = groupValue.toLongObject();
            } catch (NumberFormatException e){
                throw new RestartResponseException(NotFoundPage.class);
            }
            /*
             * Cookie is set when the link to visit this details page is clicked but for it to work
			 * if a URL is entered directly we set it here too
			 */
            Cookie c = new Cookie(COOKIE_KEY, groupId.toString());
            c.setMaxAge(MAX_AGE);
            c.setPath("/");
            ((WebResponse) getRequestCycle().getResponse()).addCookie(c);
        } else {
            Cookie cookie = ((WebRequest) this.getRequest()).getCookie(COOKIE_KEY);
            if (cookie != null && cookie.getValue() != null) {
                try {
                    groupId = Long.parseLong(cookie.getValue());
                    pp.add(PageParameterKeys.MAP.get(Group.class), groupId);
                } catch (RuntimeException e) {/* Ignore */
                    LOGGER.error("Could not initialize group", e);
                }
            }
        }

        if (groupId == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        Group group = groupService.findOne(groupId);
        if (group == null) {
            throw new RestartResponseException(getApplication().getHomePage());
        }
        if (!groupFacade.isSupervisorOrFollower(group, SciProSession.get().getUser())) {
            throw new RestartResponseException(AccessDeniedPage.class);
        }

        groupModel = new DetachableServiceModel<>(groupService, groupId);
        setDefaultModel(groupModel);

        add(new GroupDetailsPanel("groupDetails", groupModel));
    }

    public static PageParameters getPageParameters(Group group) {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Group.class), group.getId());
        return pp;
    }
}
