package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.AbstractChoice;
import org.apache.wicket.markup.html.form.CheckBoxMultipleChoice;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.pages.AbstractSettingsPage;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.Set;

import static se.su.dsv.scipro.security.auth.roles.Roles.SUPERVISOR;

@Authorization(authorizedRoles = {SUPERVISOR})
public class SupervisorProfilePage extends AbstractSettingsPage {

    @Inject
    UserService userService;
    @Inject
    private LanguageService languageService;

    public SupervisorProfilePage() {
        add(new FeedbackPanel("feedback"));
        add(new ProfileForm("form", getUserModel()));
    }

    public class ProfileForm extends Form<User> {

        public ProfileForm(String id, final IModel<User> model) {
            super(id, model);

            add(new CheckBoxMultipleChoice<>("languages", new PropertyModel<Set<Language>>(getModel(), "languages"), languageService.findAll(),
                    new ChoiceRenderer<>("name", "id")).setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER));

            add(new Label("unit", new PropertyModel<>(getModel(), "unit.title")));
            add(new Label("noUnit", new ResourceModel("noUnit")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(getModelObject().getUnit() == null);
                }
            });

            add(new ListView<ResearchArea>("researchAreas",
                    new ListAdapterModel<>(getActiveResearchAreas())) {
                @Override
                protected void populateItem(ListItem<ResearchArea> item) {
                    item.add(new Label("areaName", new PropertyModel<>(item.getModel(), "title")));
                }
            });

            add(new Label("noAreas", new ResourceModel("noAreas")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(getActiveResearchAreas().getObject().isEmpty());
                }
            });
        }

        private PropertyModel<Set<ResearchArea>> getActiveResearchAreas() {
            return new PropertyModel<>(getModel(), "activeResearchAreas");
        }

        @Override
        protected void onSubmit() {
            userService.save(getModelObject());
            success(getString("saved"));
        }
    }

    private IModel<User> getUserModel() {
        return new LoadableDetachableModel<User>() {
            @Override
            protected User load() {
                return userService.findOne(SciProSession.get().getUser().getId());
            }
        };
    }
}
