package se.su.dsv.scipro.supervisor.panels;

import org.apache.wicket.Page;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.activityplan.SupervisorActivityPlanPage;
import se.su.dsv.scipro.components.AbstractMenuPanel;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightFinalSeminar;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightForum;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorActivityPlan;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightSupervisorOverview;
import se.su.dsv.scipro.finalseminar.SupervisorFinalSeminarPage;
import se.su.dsv.scipro.forum.pages.SupervisorForumBasePage;
import se.su.dsv.scipro.grading.SupervisorGradingPage;
import se.su.dsv.scipro.milestones.pages.SupervisorMileStonePage;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.*;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class SupervisorSubTabMenuPanel extends AbstractMenuPanel {

    public static final String GRADING = "Grading";
    public static final String MILESTONES = "Milestones";
    public static final String OVERVIEW = "Overview";
    public static final String ACTIVITY_PLAN = "Activity plan";
    public static final String FILES = "Files";
    public static final String FORUM = "Forum";
    public static final String FINAL_SEMINAR = "Final seminar";
    public static final String ADMIN = "PhD Admin";
    public static final String INTERACT_WITH_REVIEWER = "Interact with reviewer";

    @Inject
    private MilestoneActivityTemplateService milestoneActivityTemplateService;
    @Inject
    private GradingReportService gradingReportService;

    private final PageParameters projectParams;
    private IModel<Project> projectModel;

    public SupervisorSubTabMenuPanel(String id, Class<? extends Page> containerClass, PageParameters projectParams, IModel<Project> projectModel) {
        super(id, AbstractSupervisorProjectDetailsPage.class, containerClass);
        this.projectParams = projectParams;
        this.projectModel = projectModel;
    }

    @Override
    protected List<MenuItem> getItemList() {
        List<MenuItem> items = new ArrayList<>();
        items.add(new MenuItem(OVERVIEW, SupervisorProjectDetailsPage.class, MenuHighlightSupervisorOverview.class, projectParams));
        if (mileStonesAreAvailable()) {
            items.add(new MenuItem(MILESTONES, SupervisorMileStonePage.class, projectParams));
        }
        items.add(new MenuItem(ACTIVITY_PLAN, SupervisorActivityPlanPage.class, MenuHighlightSupervisorActivityPlan.class, projectParams));
        items.add(new MenuItem(FILES, SupervisorFilePage.class, projectParams));
        items.add(new MenuItem(FORUM, SupervisorForumBasePage.class, MenuHighlightForum.class, projectParams));
        items.add(new MenuItem(FINAL_SEMINAR, SupervisorFinalSeminarPage.class, MenuHighlightFinalSeminar.class, projectParams));
        if (gradingReportTemplateIsAvailable()) {
            items.add(new MenuItem(GRADING, SupervisorGradingPage.class, projectParams));
        }
        items.add(new MenuItem(ADMIN, SupervisorAdminPage.class, projectParams));
        if (isSupervisor() && isReviewed()) {
            items.add(new MenuItem(INTERACT_WITH_REVIEWER, SupervisorInteractWithReviewerPage.class, projectParams));
        }
        return items;
    }

    private boolean isReviewed() {
        return projectModel.getObject().getProjectType().isReviewed();
    }

    private boolean isSupervisor() {
        return projectModel.getObject().isSupervisor(SciProSession.get().getUser());
    }

    @Override
    protected MenuType getMenuType() {
        return MenuType.TAB_MULTIPLE_ROWS;
    }

    private boolean gradingReportTemplateIsAvailable() {
        return gradingReportService.getTemplate(projectModel.getObject()) != null;
    }

    private boolean mileStonesAreAvailable() {
        Project project = projectModel.getObject();
        return milestoneActivityTemplateService.countActivities(project.getProjectType(), project.getDateCreated()) != 0;
    }

}
