package se.su.dsv.scipro.notifications.pages

import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.notifications.dataobject.PeerEvent
import se.su.dsv.scipro.notifications.dataobject.PeerEvent.Event
import se.su.dsv.scipro.peer.{SupervisorViewPeerReviewPage, PeerReview, ViewPeerReviewPage, PeerPortalPage}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.system.User

private[pages] trait PeerReviewRedirection {
  def peerReviewLocation(event: PeerEvent): UserBasedLocation = {
    val peerReview = event.getReview
    val peerRequest = peerReview.getPeerRequest

    val pp = new PageParameters
    pp.set(parameter[Project], peerRequest.getProject.getId)
    pp.set(parameter[PeerReview], peerReview.getId)

    def isPeerRequester(user: User) = user == peerRequest.getRequester
    def isAuthor(user: User) = peerRequest.getProject.isParticipant(user)

    event.getEvent match {
      case Event.REVIEW_COMPLETED | Event.REVIEW_COMMENT => {
        case user if isPeerRequester(user) || isAuthor(user) =>
          (classOf[ViewPeerReviewPage], pp)
        case user if peerRequest.getProject.isSupervisor(user) =>
          (classOf[SupervisorViewPeerReviewPage], pp)
        case user if peerReview.getReviewer == user =>
          val reviewerParameters = new PageParameters(pp)
          reviewerParameters.set(parameter[Project], peerReview.getProject.getId)
          (classOf[ViewPeerReviewPage], reviewerParameters)
        case user if peerReview.getProject.isSupervisor(user) =>
          val reviewerParameters = new PageParameters(pp)
          reviewerParameters.set(parameter[Project], peerReview.getProject.getId)
          (classOf[SupervisorViewPeerReviewPage], reviewerParameters)
      }
      case Event.REVIEW_ACCEPTED => {
        case user if user == peerReview.getPeerRequest.getRequester =>
          (classOf[PeerPortalPage], pp)
      }
      case _ => PartialFunction.empty
    }
  }
}
