package se.su.dsv.scipro.notifications.pages

import org.apache.wicket.Page
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.activityplan.{ProjectActivityPlanPage, SupervisorActivityPlanPage}
import se.su.dsv.scipro.finalseminar.{ProjectFinalSeminarPage, SupervisorFinalSeminarPage}
import se.su.dsv.scipro.grading.SupervisorGradingPage
import se.su.dsv.scipro.milestones.pages.{ProjectMileStonePage, SupervisorMileStonePage}
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent.Event
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.project.pages.{ProjectDetailsPage, ProjectFilePage}
import se.su.dsv.scipro.reviewer.{ReviewerStartPage, ReviewerGradingPage}
import se.su.dsv.scipro.supervisor.pages.{SupervisorFilePage, SupervisorInteractWithReviewerPage, SupervisorProjectDetailsPage}

private[pages] trait ProjectRedirection {
  def projectLocation(event: ProjectEvent): UserBasedLocation = {
    val project = event.getProject

    val pp = new PageParameters()
    pp.set(parameter[Project], project.getId)

    def simpleSplit(author: Class[_ <: Page], supervisor: Class[_ <: Page]): UserBasedLocation = {
      case user if project.isParticipant(user) => (author, pp)
      case user if project.hasSupervisingRole(user) => (supervisor, pp)
    }

    val overview = simpleSplit(classOf[ProjectDetailsPage], classOf[SupervisorProjectDetailsPage])
    val activityPlan = simpleSplit(classOf[ProjectActivityPlanPage], classOf[SupervisorActivityPlanPage])

    event.getEvent match {
      case Event.CREATED => overview
      case Event.COMPLETED ⇒ overview
      case Event.AUTHORS_CHANGED => overview
      case Event.HEAD_SUPERVISOR_CHANGED => overview
      case Event.REVIEWERS_CHANGED => overview
      case Event.CO_SUPERVISOR_CHANGED => overview
      case Event.STATE_CHANGED => overview
      case Event.ACTIVITY_ADDED => activityPlan
      case Event.ACTIVITY_REMOVED => activityPlan
      case Event.ACTIVITY_EDITED => activityPlan
      case Event.FILE_UPLOADED => simpleSplit(classOf[ProjectFilePage], classOf[SupervisorFilePage])
      case Event.CHECKLIST_ANSWERED => activityPlan
      case Event.CHECKLIST_ADDED => activityPlan
      case Event.MILESTONE_CONFIRMED => simpleSplit(classOf[ProjectMileStonePage], classOf[SupervisorMileStonePage])
      case Event.MILESTONE_REVOKED => simpleSplit(classOf[ProjectMileStonePage], classOf[SupervisorMileStonePage])
      case Event.FINAL_THESIS_APPROVED => overview
      case Event.FILE_SIZE_TOO_BIG => simpleSplit(classOf[ProjectFinalSeminarPage], classOf[SupervisorFinalSeminarPage])
      case Event.FINAL_THESIS_UPLOADED => simpleSplit(classOf[ProjectDetailsPage], classOf[SupervisorGradingPage])
      case Event.PROJECT_ACTIVATED => overview
      case Event.PROJECT_DEACTIVATED => overview
      case Event.FINAL_SEMINAR_APPROVAL_REQUESTED | Event.FINAL_SEMINAR_APPROVAL_APPROVED | Event.FINAL_SEMINAR_APPROVAL_REJECTED =>
        val specificPp = new PageParameters(pp)
        specificPp.set("anchor", "finalSeminarApproval")

        {
          case user if project.isSupervisor(user) => (classOf[SupervisorInteractWithReviewerPage], specificPp)
          case user if project.isReviewer(user) => (classOf[ReviewerStartPage], specificPp)
        }
      case Event.ROUGH_DRAFT_APPROVAL_REQUESTED | Event.ROUGH_DRAFT_APPROVAL_APPROVED | Event.ROUGH_DRAFT_APPROVAL_REJECTED => {
        case user if project.isSupervisor(user) => (classOf[SupervisorInteractWithReviewerPage], pp)
        case user if project.isReviewer(user) => (classOf[ReviewerStartPage], pp)
      }
      case Event.REVIEWER_GRADING_REPORT_SUBMITTED | Event.REVIEWER_GRADING_INITIAL_ASSESSMENT_DONE => {
        case user if project.isSupervisor(user) => (classOf[SupervisorGradingPage], pp)
      }
      case Event.ONE_YEAR_PASSED_FROM_LATEST_ANNUAL_REVIEW => overview
      case Event.SUPERVISOR_GRADING_INITIAL_ASSESSMENT_DONE => {
        case user if project.isReviewer(user) => (classOf[ReviewerGradingPage], pp)
      }
      case Event.EXPORTED_SUCCESS => overview
    }
  }
}
