package se.su.dsv.scipro.statistics

import java.util.Date
import java.util.{List ⇒ JList}
import javax.inject.Inject

import org.apache.wicket.markup.html.basic.Label
import org.apache.wicket.markup.html.list.{ListItem, ListView}
import org.apache.wicket.markup.html.panel.GenericPanel
import org.apache.wicket.model.{LoadableDetachableModel, IModel}
import se.su.dsv.scipro.`match`._
import se.su.dsv.scipro.system.{ProjectType, User}

import scala.collection.JavaConversions._

class SupervisorApplicationPeriodStatisticsPanel(id: String, supervisorModel: IModel[User]) extends GenericPanel[User](id, supervisorModel) {
  @Inject private var applicationPeriodService: ApplicationPeriodService = _
  @Inject private var targetService: TargetService = _

  val targets = new LoadableDetachableModel[JList[(ApplicationPeriod, ProjectType, Target)]] {
    override def load() = for {
      period ← upcomingPeriods
      projectType ← period.getProjectTypes
      target = targetService.findOne(period, getModelObject, projectType)
      if target.getTarget > 0
    } yield (period, projectType, target)
  }

  add(new ListView[(ApplicationPeriod, ProjectType, Target)]("targets", targets) {
    override def populateItem(item: ListItem[(ApplicationPeriod, ProjectType, Target)]): Unit = {
      val (period, projectType, target) = item.getModelObject
      item.add(new Label("period", period.getName))
      item.add(new Label("projectType", projectType.getName))
      item.add(new Label("target", target.getTarget))
    }
  })

  private def upcomingPeriods: List[ApplicationPeriod] = {
    applicationPeriodService.findAll(QApplicationPeriod.applicationPeriod.courseStartDate.after(new Date)).toList
  }
}