package se.su.dsv.scipro.activityplan;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklist.ChecklistCategory;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.text.SimpleDateFormat;
import java.util.*;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;

public class EditActivityPanelTest extends SciProTest {

    private Project project;

    @Before
    public void setUp() throws Exception {

        setModules(ProjectModule.ACTIVITY_PLAN);

        final User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.AUTHOR)));
        setLoggedInAs(user);

        final User supervisor = new UserBuilder().setFirstName("Super").setLastName("Supersson").create();

        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        final ActivityPlan activityPlan = new ActivityPlan();
        activityPlan.setId(1L);
        activityPlan.setProject(project);

        ChecklistTemplate checklistTemplate = new ChecklistTemplate();
        checklistTemplate.setId(1L);
        checklistTemplate.setName("nice template");
        checklistTemplate.setCategories(new ArrayList<ChecklistCategory>());
        checklistTemplate.setCreator(user);
        checklistTemplate.setDescription("nice description");
        checklistTemplate.setProjectTypes(new ArrayList<>(Collections.singletonList(project.getProjectType())));
        checklistTemplate.setQuestions(new ArrayList<>(Arrays.asList("who is your daddy", "and what does he do")));
        checklistTemplate.setTemplateNumber(0);

        Mockito.when(checklistTemplateService.findByProject(isA(Project.class))).thenReturn(new ArrayList<>(Collections.singletonList(checklistTemplate)));
        Mockito.when(activityPlanService.findOne(isA(Long.class))).thenReturn(activityPlan);
        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        Mockito.when(projectService.isPartOfProject(project, user)).thenReturn(true);
        Mockito.when(activityPlanFacade.saveActivity(isA(Activity.class))).thenReturn(new Activity());

        startPanel();
    }

    @Test
    public void requires() throws Exception {
        FormTester formTester = tester.newFormTester(path("panel", "form"));
        formTester.submit();
        tester.assertErrorMessages("'title' is required.", "'date' is required.", "'time' is required.");
    }

    @Test
    public void saves() throws Exception {
        FormTester formTester = tester.newFormTester(path("panel", "form"));
        formTester.setValue("title", "nice title");
        formTester.setValue("description", "nice description");

        Date now = new Date();
        String dateString = new SimpleDateFormat(dateService.getFormat(DateStyle.DATE)).format(now);
        String timeString = new SimpleDateFormat(dateService.getFormat(DateStyle.TIME)).format(now);
        formTester.setValue(path("dateTime", "date"), dateString);
        formTester.setValue(path("dateTime", "time"), timeString);

        formTester.submit();
        tester.assertNoErrorMessage();
        verify(activityPlanFacade).saveActivity(isA(Activity.class));
        verify(notificationController).notifyProject(isA(Project.class), isA(ProjectEvent.Event.class), isA(NotificationSource.class));
    }



    private void startPanel() {
        tester.startComponentInPage(new EditActivityPanel("panel", Model.of(project), Model.of(new Activity())));
    }

}