package se.su.dsv.scipro.activityplan;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Test;
import org.mockito.Matchers;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.basepages.errorpages.AccessDeniedPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorEditActivityPageTest extends SciProTest {

    @Test
    public void notParticipant() throws Exception {

        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.SUPERVISOR)));
        setLoggedInAs(user);

        setRoles(getRoles());
        setLoggedIn(true);
        setModules(ProjectModule.ACTIVITY_PLAN);

        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        when(projectService.isSupervisorOrFollower(project, user)).thenReturn(false);

        ActivityPlan activityPlan = new ActivityPlan();
        activityPlan.setProject(project);
        when(activityPlanService.findOne(isA(Long.class))).thenReturn(activityPlan);

        startWithParametersAndExpect(AccessDeniedPage.class);

    }

    @Test
    public void participant() throws Exception {

        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.SUPERVISOR)));
        setLoggedInAs(user);

        setRoles(getRoles());
        setLoggedIn(true);
        setModules(ProjectModule.ACTIVITY_PLAN);

        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(user)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        when(projectService.isSupervisorOrFollower(project, user)).thenReturn(true);

        ActivityPlan activityPlan = new ActivityPlan();
        activityPlan.setProject(project);
        when(activityPlanService.findOne(isA(Long.class))).thenReturn(activityPlan);

        Activity activity = new Activity();
        activity.setActivityPlan(activityPlan);
        when(activityService.findOne(isA(Long.class))).thenReturn(activity);

        startWithParametersAndExpect(SupervisorEditActivityPage.class);

    }

    private void startWithParametersAndExpect(Class<? extends Page> pageClass) {
        tester.startPage(SupervisorEditActivityPage.class, getPageParameters());
        tester.assertRenderedPage(pageClass);
    }

    public Roles[] getRoles() {
        return new Roles[]{Roles.SUPERVISOR};
    }

    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1);
        pageParameters.add(PageParameterKeys.MAP.get(Activity.class), 1);
        pageParameters.add(PageParameterKeys.MAP.get(ActivityPlan.class), 1);
        return pageParameters;
    }

}