package se.su.dsv.scipro.activityplan;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

public class UpcomingActivitiesPanelTest extends SciProTest {

    private UpcomingActivitiesPanel panel;
    private static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description");
    private Project project;


    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder()
                .setFirstName("Some first name")
                .setLastName("Some last name")
                .create();
        user.setId(1L);

        project = Project.builder()
                .title("Some title")
                .projectType(SOME_PROJECT_TYPE)
                .headSupervisor(user)
                .build();
        project.setId(1L);

        Mockito.when(dateService.getFormat(Matchers.isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
    }

    @Test
    public void testRenders() throws Exception {
        startPanel(true, false);
        Assert.assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testList() throws Exception {
        startPanel(true, true);
        tester.assertComponent(path(panel.getId(), UpcomingActivitiesPanel.LIST), ListView.class);
    }

    @Test
    public void testNoLabelVisibleWhenNoEvents() throws Exception {
        startPanel(true, false);
        tester.assertComponent(path(panel.getId(), UpcomingActivitiesPanel.NO), Label.class);
        tester.assertVisible(path(panel.getId(), UpcomingActivitiesPanel.NO));
    }

    @Test
    public void testNoActivitiesLabelInvisibleWhenEvents() throws Exception {
        startPanel(true, true);
        tester.assertInvisible(path(panel.getId(), UpcomingActivitiesPanel.NO));
    }

    @Test
    public void testDate() throws Exception {
        startPanel(true, true);
        panel.listView.visitChildren(ListItem.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                boolean isVisible = component.get(path(UpcomingActivitiesPanel.DATE)).isVisibleInHierarchy();
                Assert.assertTrue(isVisible);
                visit.stop();
            }
        });
    }

    @Test
    public void testEmployeeLink() throws Exception {
        startPanel(false, true);
        panel.listView.visitChildren(ListItem.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                boolean isVisible = component.get(path(UpcomingActivitiesPanel.EMPLOYEE_LINK)).isVisibleInHierarchy();
                Assert.assertTrue(isVisible);
                visit.stop();
            }
        });
    }

    @Test
    public void testEmployeeLabel() throws Exception {
        startPanel(false, true);
        panel.listView.visitChildren(ListItem.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                boolean isVisible = component.get(path(UpcomingActivitiesPanel.EMPLOYEE_LINK, UpcomingActivitiesPanel.EMPLOYEE_LABEL)).isVisibleInHierarchy();
                Assert.assertTrue(isVisible);
                visit.stop();
            }
        });
    }

    @Test
    public void testAuthorLink() throws Exception {
        startPanel(true, true);
        panel.listView.visitChildren(ListItem.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                boolean isVisible = component.get(path(UpcomingActivitiesPanel.AUTHOR_LINK)).isVisibleInHierarchy();
                Assert.assertTrue(isVisible);
                visit.stop();
            }
        });
    }

    @Test
    public void testAuthorLabel() throws Exception {
        startPanel(true, true);
        panel.listView.visitChildren(ListItem.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                boolean isVisible = component.get(path(UpcomingActivitiesPanel.AUTHOR_LINK, UpcomingActivitiesPanel.AUTHOR_LABEL)).isVisibleInHierarchy();
                Assert.assertTrue(isVisible);
                visit.stop();
            }
        });
    }

    private void startPanel(final boolean isAuthor, boolean addEvents) {
        ArrayList<Activity> events = new ArrayList<>();
        if (addEvents) {
            ActivityPlan ps = new ActivityPlan();
            ps.setProject(project);
            Activity pse = new Activity();
            pse.setActivityPlan(ps);
            events.add(pse);
        }

        Mockito.when(activityPlanFacade.findEventsByProject(Matchers.isA(Project.class), Matchers.isA(Date.class), Matchers.isA(Date.class), Matchers.isA(Pageable.class))).thenReturn(events);
        Mockito.when(activityPlanFacade.findEventsByProject(Matchers.isA(Project.class), Matchers.isA(Date.class), Matchers.isNull(Date.class), Matchers.isA(Pageable.class))).thenReturn(events);

        panel = tester.startComponentInPage(new UpcomingActivitiesPanel("panel", Model.of(project)) {
            @Override
            public boolean isAuthorView() {
                return isAuthor;
            }
        });
    }
}
