package se.su.dsv.scipro.admin.pages;

import org.apache.wicket.Page;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DataTable;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.mail.MailEvent;

import java.util.List;

import static java.util.Arrays.asList;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.admin.pages.AdminMailLogPage.MAIL_EVENTS;
import static se.su.dsv.scipro.admin.pages.AdminMailLogPage.MAIL_MODAL;

public class AdminMailLogPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        when(mailEventService.findAll(isA(Pageable.class))).thenReturn(pageContaining(newMailEvent(), newMailEvent()));
        startPage();
    }

    @Test
    public void has_data_table_of_mail_events() throws Exception {
        tester.assertComponent(MAIL_EVENTS, DataTable.class);
        DataTable<?, ?> table = (DataTable<?, ?>) tester.getComponentFromLastRenderedPage("mailEvents");
        assertTrue(table.getDataProvider() instanceof FilteredDataProvider);
    }

    @Test
    public void has_mail_content_modal_window() throws Exception {
        tester.assertComponent(MAIL_MODAL, ModalWindowPlus.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminMailLogPage.class;
    }

    private static MailEvent newMailEvent() {
        return new MailEvent();
    }

    private static List<MailEvent> pageContaining(MailEvent... events) {
        return asList(events);
    }
}
