package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.Page;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.system.Language;

import java.util.Arrays;
import java.util.Collections;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AdminLanguageSettingsPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        Language english = new Language();
        english.setName("English");
        when(languageService.findAll()).thenReturn(Collections.singletonList(english));

        startPage();
    }

    @Test
    public void lists_all_current_languages() throws Exception {
        tester.assertListView(path("wmc", "languages"), languageService.findAll());
    }

    @Test
    public void create_language() throws Exception {
        FormTester formTester = tester.newFormTester(path("wmc", "form"));
        formTester.setValue("name", "Swedish");
        tester.executeAjaxEvent(path("wmc", "form", "add"), "onclick");

        ArgumentCaptor<Language> captor = ArgumentCaptor.forClass(Language.class);
        verify(languageService).save(captor.capture());
        assertThat(captor.getValue().getName(), is("Swedish"));
    }

    @Test
    public void deletes() throws Exception {
        tester.clickLink(path("wmc", "languages", 0, "delete"), true);
        verify(languageService).delete(Matchers.isA(Language.class));
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminLanguageSettingsPage.class;
    }
}