package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Target;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class AdminEditTargetsPageTest extends SciProTest {

    public static final User SOME_EMPLOYEE = new UserBuilder().create();
    public static final String TARGET_PATH = "employeeList:body:rows:1:cells:5:cell:input";
    private ApplicationPeriod applicationPeriod;
    private Target target;

    @Before
    public void setUp() throws Exception {
        when(supervisorService.findAll(any(SupervisorService.FilterParams.class), any(Pageable.class))).thenReturn(supervisorPage());
        when(supervisorService.count(any(SupervisorService.FilterParams.class))).thenReturn(1L);
        when(supervisorService.findOne(1L)).thenReturn(SOME_EMPLOYEE);
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
        applicationPeriod = createApplicationPeriod(projectType);
        when(applicationPeriodService.getPreviousPeriods(applicationPeriod, 2)).thenReturn(Collections.singletonList(createApplicationPeriod(projectType)));
        target = new Target(SOME_EMPLOYEE, applicationPeriod, projectType);
        target.setTarget(3);
        when(targetService.findOne(any(ApplicationPeriod.class), any(User.class), any(ProjectType.class))).thenReturn(target);
        tester.startPage(new AdminEditTargetsPage(Model.of(applicationPeriod)));
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminEditTargetsPage.class);
    }

    @Test
    public void header_contains_info_about_application_period() throws Exception {
        tester.assertLabel(AdminEditTargetsPage.HEADER, tester.getLastRenderedPage().getString("header", Model.of(applicationPeriod)));
    }

    @Test
    public void target_model_value_is_correct() throws Exception {
        tester.assertModelValue(TARGET_PATH, target.getTarget());
    }

    @Test
    public void updating_feedback_when_submitting_target() throws Exception {
        tester.getRequest().getPostParameters().setParameterValue(getInputName(TARGET_PATH), String.valueOf(3));
        tester.executeAjaxEvent(TARGET_PATH, "change");
        tester.assertComponentOnAjaxResponse(AdminEditTargetsPage.FEEDBACK);
        tester.assertComponentOnAjaxResponse(AdminEditTargetsPage.EMPLOYEE_LIST);
    }

    private String getInputName(String path) {
        return ((FormComponent<?>) tester.getComponentFromLastRenderedPage(path)).getInputName();
    }

    private ApplicationPeriod createApplicationPeriod(ProjectType projectType) {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.addProjectType(projectType);
        applicationPeriod.setStartDate(new Date(0));
        applicationPeriod.setEndDate(new Date(86400));
        applicationPeriod.setCourseStartDate(new Date(86400 * 2));
        return applicationPeriod;
    }

    private List<User> supervisorPage() {
        return Collections.singletonList(SOME_EMPLOYEE);
    }
}
