package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.activityplan.ActivityPlanTemplate;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodProjectType;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import static se.su.dsv.scipro.applicationperiod.EditApplicationPeriodActivityPlanTemplatePanel.*;

public class EditApplicationPeriodActivityPlanTemplatePanelTest extends SciProTest {
    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");

    private EditApplicationPeriodActivityPlanTemplatePanel panel;
    private ApplicationPeriod applicationPeriod;

    @Before
    public void setUp() throws Exception {
        applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setId(1L);

        ApplicationPeriodProjectType appt = new ApplicationPeriodProjectType(applicationPeriod, BACHELOR);

        applicationPeriod.getApplicationPeriodProjectTypes().add(appt);

        Mockito.when(activityPlanTemplateService.findAll()).thenReturn(Collections.singletonList(new ActivityPlanTemplate()));
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        Assert.assertNotNull("Panel does not render", panel);
    }

    @Test
    public void form() throws Exception {
        startPanel();
        tester.assertComponent(path(FORM), Form.class);
    }

    @Test
    public void list() throws Exception {
        startPanel();
        tester.assertListView(path(FORM, LIST), new ArrayList<>(applicationPeriod.getApplicationPeriodProjectTypes()));
    }

    @Test
    public void project_class_label() throws Exception {
        startPanel();
        tester.assertLabel(path(FORM, LIST, 0, PC), BACHELOR.getName());
    }

    @Test
    public void drop_down() throws Exception {
        startPanel();
        tester.assertComponent(path(FORM, LIST, 0, DROPDOWN), DropDownChoice.class);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new EditApplicationPeriodActivityPlanTemplatePanel("id", Model.of(applicationPeriod)));
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel, super.path(parts));
    }
}
