package se.su.dsv.scipro.components;

import com.google.common.collect.Sets;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.*;

public class ListAdapterModelTest {

    @Test
    public void detaches_underlying_model() throws Exception {
        IModel<List<?>> model = Mockito.spy(Model.ofList(new ArrayList<>()));

        ListAdapterModel<Object> adapterModel = new ListAdapterModel<>(model);
        adapterModel.getObject();
        adapterModel.detach();

        Mockito.verify(model).detach();
    }

    @Test
    public void contains_all_objects_of_the_underlying_model() throws Exception {
        Set<String> expected = Sets.newHashSet("Foo", "Bar");
        IModel<Collection<? extends String>> model = Model.of(expected);

        ListAdapterModel<String> adapterModel = new ListAdapterModel<>(model);
        List<String> actual = adapterModel.getObject();

        Iterator<String> expectedIterator = expected.iterator();
        Iterator<String> actualIterator = actual.iterator();

        Assert.assertTrue(iteratorEquals(expectedIterator, actualIterator));
    }

    private static boolean iteratorEquals(Iterator<String> expectedIterator, Iterator<String> actualIterator) {
        while (expectedIterator.hasNext()) {
            if (!actualIterator.hasNext() || !actualIterator.next().equals(expectedIterator.next())) {
                return false;
            }
        }
        return !actualIterator.hasNext();
    }
}
