package se.su.dsv.scipro.components;

import com.google.common.base.Joiner;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;

import java.util.Arrays;
import java.util.List;

import static se.su.dsv.scipro.components.ListFieldTest.ListFieldForm.FORM;
import static se.su.dsv.scipro.components.ListFieldTest.ListFieldForm.LIST_FIELD;

public class ListFieldTest extends SciProTest {

    private ListFieldForm panel;
    private List<String> defaultList;

    @Before
    public void setUp() throws Exception {
        defaultList = Arrays.asList("foo", "bar");
        panel = tester.startComponentInPage(new ListFieldForm("id", Model.ofList(defaultList)));
    }

    @Test
    public void testRenders() throws Exception {
        tester.assertComponent(panel.getId(), ListFieldForm.class);
    }

    @Test
    public void testSubmitValues() throws Exception {
        List<String> values = Arrays.asList("kalle", "testare");
        String join = Joiner.on(ListField.SEPARATOR).join(values);

        FormTester formTester = tester.newFormTester(panel.getId() + ":form");
        formTester.setValue("listField", join);
        formTester.submit();

        tester.assertModelValue(path(panel, FORM, LIST_FIELD), values);
    }

    @Test
    public void testShowsListValues() throws Exception {
        FormComponent listField = (FormComponent) tester.getComponentFromLastRenderedPage(path(panel, FORM, LIST_FIELD));

        Assert.assertEquals("Does not show list values", Joiner.on(ListField.SEPARATOR).join(defaultList), listField.getValue());
    }

    public static class ListFieldForm extends Panel {

        public static final String FORM = "form";
        public static final String LIST_FIELD = "listField";

        public ListFieldForm(String id, IModel<List<? extends String>> listIModel) {
            super(id);

            Form<Void> form = new Form<>(FORM);
            form.add(new ListField<>(LIST_FIELD, listIModel, null));
            add(form);
        }
    }
}
