package se.su.dsv.scipro.components;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.mockito.internal.verification.VerificationModeFactory;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;

public class SortOrderPanelTest extends SciProTest {

    SortOrderPanel<MilestonePhaseTemplate> panel;
    MilestonePhaseTemplate milestonePhaseTemplate;

    @Before
    public void setUp() throws Exception {
        milestonePhaseTemplate = new MilestonePhaseTemplate("title", "desc");
        mileStonePhaseTemplateService = Mockito.mock(MilestonePhaseTemplateService.class, Mockito.withSettings().serializable());
        panel = tester.startComponentInPage(new SortOrderPanel<>("panel", Model.of(milestonePhaseTemplate), mileStonePhaseTemplateService));

    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testClickOnMoveUpCallsCorrectServiceMethod() throws Exception {
        String moveUpPath = path(panel.getId(), SortOrderPanel.MOVE_UP);
        tester.assertComponent(moveUpPath, AjaxLink.class);
        tester.executeAjaxEvent(moveUpPath, "onclick");
        Mockito.verify(mileStonePhaseTemplateService, VerificationModeFactory.times(1)).moveUp(Matchers.isA(MilestonePhaseTemplate.class));
    }

    @Test
    public void testClickOnMoveDownCallsCorrectServiceMethod() throws Exception {
        String moveDownPath = path(panel.getId(), SortOrderPanel.MOVE_DOWN);
        tester.assertComponent(moveDownPath, AjaxLink.class);
        tester.executeAjaxEvent(moveDownPath, "onclick");
        Mockito.verify(mileStonePhaseTemplateService, VerificationModeFactory.times(1)).moveDown(Matchers.isA(MilestonePhaseTemplate.class));
    }

}
