package se.su.dsv.scipro.economy;

import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static se.su.dsv.scipro.economy.EconomyPanel.*;

public class EconomyPanelTest extends SciProTest {

    private EconomyPanel panel;
    private Project project;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        User supervisor = new UserBuilder().setFirstName("Super").setLastName("Supersson").create();
        setLoggedInAs(supervisor);
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(project);
        Mockito.when(projectService.isPartOfProject(project, user)).thenReturn(true);

        Transaction transaction = new Transaction();
        transaction.setId(1L);
        transaction.setUser(user);
        transaction.setProject(project);
        transaction.setAmount(100);
        transaction.setComment("bla bla");
        transaction.setDateCreated(new Date());

        Mockito.when(transactionService.findByProject(isA(Project.class))).thenReturn(Collections.singletonList(transaction));

        startPanel();
    }

    @Test
    public void submits() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, CONTAINER, FORM));
        formTester.setValue(AMOUNT, "100");
        formTester.submit();
        tester.assertNoErrorMessage();
    }

    @Test
    public void amount_required() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, CONTAINER, FORM));
        formTester.setValue(AMOUNT, null);
        formTester.submit();
        tester.assertErrorMessages("'Amount' is required.");
    }

    @Test
    public void negative_disallowed() throws Exception {
        FormTester formTester = tester.newFormTester(path(panel, CONTAINER, FORM));
        formTester.setValue(AMOUNT, "-1500");
        formTester.submit();
        tester.assertErrorMessages("Negative balance not allowed");
    }

    @Test
    public void deletes() throws Exception {
        tester.clickLink(path(panel, CONTAINER, TRANSACTIONS, 0, DELETE));
//        tester.assertInfoMessages("Deleted transaction"); doesnt work with ajax
        verify(transactionService).delete(isA(Transaction.class));
    }

    private void startPanel(){
        panel = tester.startComponentInPage(new EconomyPanel("panel", new Model<>(project)));
    }
}
