package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.list.ListView;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Arrays;
import java.util.List;

import static se.su.dsv.scipro.finalseminar.AdminFinalSeminarSettingsPage.*;

public class AdminFinalSeminarSettingsPageTest extends SciProTest {

    AdminFinalSeminarSettingsPage page;
    List<ProjectType> levels;

    @Before
    public void setUp() throws Exception {
        setLoggedIn(true);
        setRoles(Roles.SYSADMIN);
        levels = Arrays.asList(new ProjectType(ProjectType.BACHELOR, "Bachelor", "Bachelor"));
        Mockito.when(projectTypeService.findAllActive()).thenReturn(levels);

        FinalSeminarSettings settings = new FinalSeminarSettings();
        Mockito.when(finalSeminarSettingsService.getInstance()).thenReturn(settings);

        settings.setDaysAheadToCreate(10);
        settings.setDaysAheadToUploadThesis(5);

        page = tester.startPage(AdminFinalSeminarSettingsPage.class);
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminFinalSeminarSettingsPage.class);
    }

    @Test
    public void testContainsSettingsForm() throws Exception {
        tester.assertComponent(SETTINGS_FORM, Form.class);
    }

    @Test
    public void testFormContainsListView() throws Exception {
        assertFormComponent(PROJECT_TYPE_LIST, ListView.class);
    }

    @Test
    public void testListViewContainsAllActiveLevels() throws Exception {
        tester.assertListView(path(SETTINGS_FORM, PROJECT_TYPE_LIST), levels);
    }

    @Test
    public void testLabelForEachListItem() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE), Label.class);
        }
    }

    @Test
    public void testInnerFormForEachListItem() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE_FORM), Form.class);
        }
    }

    @Test
    public void testInnerFormContainsMaxOpponentsField() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE_FORM, SEMINAR_MAX_OPPONENTS), RequiredTextField.class);
        }
    }

    @Test
    public void testInnerFormContainsMaxParticipantsField() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE_FORM, SEMINAR_MAX_PARTICIPANTS), RequiredTextField.class);
        }
    }

    @Test
    public void testInnerFormContainsMinParticipantsField() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE_FORM, SEMINAR_MIN_PARTICIPANTS), RequiredTextField.class);
        }
    }

    @Test
    public void testInnerFormContainsMinOpponentsField() throws Exception {
        for (int i = 0; i < levels.size(); i++) {
            tester.assertComponent(path(SETTINGS_FORM, PROJECT_TYPE_LIST, i, PROJECT_TYPE_FORM, SEMINAR_MIN_OPPONENTS), RequiredTextField.class);
        }
    }

    @Test
    public void testFormContainsMinDaysRegisterOpponentField() throws Exception {
        assertFormComponent(SEMINAR_MIN_DAYS_REGISTER_OPPONENT, RequiredTextField.class);
    }

    @Test
    public void testFormContainsMinDaysRegisterParticipantField() throws Exception {
        assertFormComponent(SEMINAR_MIN_DAYS_REGISTER_ACTIVE, RequiredTextField.class);
    }

    @Test
    public void testFormContainsCreateDaysAhead() throws Exception {
        assertFormComponent(SEMINAR_CREATE_DAYS_AHEAD, RequiredTextField.class);

    }

    @Test
    public void testFormContainsPartnerConfirmationCheckBox() throws Exception {
        assertFormComponent(SEMINAR_PDF, CheckBox.class);
    }

    private void assertFormComponent(String componentPath, Class<? extends Component> expectedClass) {
        tester.assertComponent(path(SETTINGS_FORM, componentPath), expectedClass);
    }
}
