package se.su.dsv.scipro.finalseminar;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class PublicFinalSeminarListingPageTest extends SciProTest {

    private List<FinalSeminar> finalSeminars;

    @Before
    public void setUp() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        User student = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        finalSeminar.getRespondents().add(new FinalSeminarRespondent(student, finalSeminar));
        finalSeminars = Collections.singletonList(finalSeminar);
        when(finalSeminarService.findAll(correctFilter(), correctLimitAndSort())).thenReturn(finalSeminars);

        tester.startPage(PublicFinalSeminarListingPage.class);
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(PublicFinalSeminarListingPage.class);
    }

    @Test
    public void public_pages_must_be_stateless() throws Exception {
        assertTrue(tester.getLastRenderedPage().isPageStateless());
    }

    @Test
    public void lists_all_final_seminars() throws Exception {
        tester.assertListView("finalSeminars", finalSeminars);
    }

    private FinalSeminarService.Filter correctFilter() {
        FinalSeminarService.Filter filter = new FinalSeminarService.Filter();
        filter.setFromDate(startOfToday());
        return filter;
    }

    private Date startOfToday() {
        return new DateTime().withTimeAtStartOfDay().toDate();
    }

    private PageRequest correctLimitAndSort() {
        return new PageRequest(0, 20, new Sort(Sort.Direction.ASC, "startDate"));
    }
}