package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.markup.html.form.AbstractTextComponent;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.*;

public class RejectFinalThesisPanelTest extends SciProTest {

    @Mock
    Project project;

    RejectFinalThesisPanel panel;

    @Before
    public void setUp() throws Exception {
        ProjectThread thread = new ForumBuilder().createThread();
        thread.getForumThread().addPost(new ForumBuilder().createPost());

        when(projectForumService.createThread(isA(Project.class), isA(User.class), isA(String.class),
                isA(String.class), anySetOf(FileUpload.class))).thenReturn(thread);
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Panel does not render", panel);
    }

    @Test
    public void testContainsContentInput() throws Exception {
        tester.assertComponent(path(panel.getId(),  RejectFinalThesisPanel.FORM,  RejectFinalThesisPanel.CONTENT), AbstractTextComponent.class);
    }

    @Test
    public void testContainsUploadField() throws Exception {
        tester.assertComponent(path(panel.getId(),  RejectFinalThesisPanel.FORM,  RejectFinalThesisPanel.ATTACHMENT), FileUploadField.class);
    }

    @Test
    public void testContainsFeedbackPanel() throws Exception {

        tester.assertComponent(path(panel.getId(),  RejectFinalThesisPanel.FORM,  RejectFinalThesisPanel.FEEDBACK), FeedbackPanel.class);
    }

    @Test
    public void testContainsCancelLink() throws Exception {

        tester.assertComponent(path(panel.getId(),  RejectFinalThesisPanel.FORM,  RejectFinalThesisPanel.CANCEL_REJECTION), AbstractLink.class);
    }

    @Test
    public void testSubmittingFormCreatesThread() throws Exception {
        // given
        User user = new UserBuilder().setFirstName("Kalle").setLastName("Testare").create();
        String content = "Content";
        String contentType = "application/pdf";
        File file = new File(RejectFinalThesisPanelTest.class.getResource("thesis.pdf").toURI());

        setLoggedInAs(user);

        // when
        FormTester formTester = tester.newFormTester(path(panel.getId(),  RejectFinalThesisPanel.FORM));
        formTester.setFile(RejectFinalThesisPanel.ATTACHMENT, file, contentType);
        formTester.setValue(RejectFinalThesisPanel.CONTENT, content);
        formTester.submit();

        // then
        verify(projectForumService, times(1)).createThread(eq(project), eq(user), anyString(), eq(content), anySetOf(FileUpload.class));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new RejectFinalThesisPanel("panel", Model.of(project)));
    }



}

