package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class ProjectViewForumThreadPageTest extends PageTest {

    Page page;
    Project project;
    ProjectThread thread;

    @Before
    public void setUp() throws Exception {
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        TreeSet<User> projectParticipants = new TreeSet<>(new User.ByNameComparator());
        projectParticipants.addAll(Collections.singletonList(student));
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .projectParticipants(projectParticipants)
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        when(projectService.isPartOfProject(isA(Project.class), isA(User.class))).thenReturn(true);

        thread = new ForumBuilder().createThread();
        thread.setProject(project);
        when(projectForumService.findOne(isA(Long.class))).thenReturn(thread);
        setLoggedInAs(student);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectViewForumThreadPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(1L));
        pp.add(PageParameterKeys.MAP.get(ForumThread.class), String.valueOf(1L));
        return pp;
    }

    @Test
    public void testEnterPageWithoutThreadIdRedirects() throws Exception {
        PageParameters parameters = new PageParameters();
        parameters.set(PageParameterKeys.MAP.get(Project.class), 1L);
        page = tester.startPage(ProjectViewForumThreadPage.class, parameters);
        tester.assertRenderedPage(NotFoundPage.class);
    }

    @Test
    public void testEnterPageWithoutExistingThreadRedirects() throws Exception {
        when(projectForumService.findOne(isA(Long.class))).thenReturn(null);
        page = tester.startPage(ProjectViewForumThreadPage.class, getPageParameters());
        tester.assertRenderedPage(NotFoundPage.class);
    }
}
