package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.forum.panels.threaded.SubmitForumThreadPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorCreateForumThreadPageTest extends PageTest {

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorCreateForumThreadPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1);
        return pageParameters;
    }

    @Before
    public void setUp() throws Exception {
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(isA(Long.class))).thenReturn(project);

        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        tester.startPage(SupervisorCreateForumThreadPage.class, getPageParameters());
    }

    @Test
    public void testPageContainsBackLink() throws Exception {
        tester.assertComponent(SupervisorCreateForumThreadPage.BACK_LINK, BookmarkablePageLink.class);
    }

    @Test
    public void testClickLinkTakesUserBackToForumPage() throws Exception {
        tester.clickLink(SupervisorCreateForumThreadPage.BACK_LINK, false);
        tester.assertRenderedPage(SupervisorThreadedForumPage.class);
    }

    @Test
    public void testPageContainsCreateThreadPanel() throws Exception {
        tester.assertComponent(SupervisorCreateForumThreadPage.CREATE_THREAD, SubmitForumThreadPanel.class);
    }
}
