package se.su.dsv.scipro.forum.pages.threaded;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.basepages.errorpages.NotFoundPage;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorViewForumThreadPageTest extends PageTest {

    Page page;
    ProjectThread thread;

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorViewForumThreadPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(ForumThread.class), String.valueOf(1L));
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(1L));
        return pp;
    }

    @Before
    public void setUp() throws Exception {

        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        thread = new ForumBuilder().createThread();
        when(projectForumService.findOne(isA(Long.class))).thenReturn(thread);

        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        page = tester.startPage(SupervisorViewForumThreadPage.class, getPageParameters());
    }

    @Test
    public void testEnterPageWithoutThreadIdRedirects() throws Exception {
        page = tester.startPage(SupervisorViewForumThreadPage.class, new PageParameters());
        tester.assertRenderedPage(NotFoundPage.class);
    }

    @Test
    public void testEnterPageWithoutExistingThreadRedirects() throws Exception {
        when(projectForumService.findOne(isA(Long.class))).thenReturn(null);
        page = tester.startPage(SupervisorViewForumThreadPage.class, getPageParameters());
        tester.assertRenderedPage(NotFoundPage.class);
    }
}
