package se.su.dsv.scipro.generalsystemsettings;

import com.google.common.base.Joiner;
import org.apache.wicket.Page;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.components.ListField;
import se.su.dsv.scipro.security.auth.roles.Roles;

import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class AdminGeneralSettingsPageTest extends PageTest {

    AdminGeneralSettingsPage page;

    @Before
    public void setUp() throws Exception {
        setLoggedIn(true);
        setRoles(Roles.SYSADMIN);

        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(new GeneralSystemSettings());

        page = tester.startPage(AdminGeneralSettingsPage.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminGeneralSettingsPage.class;
    }

    @Test
    public void testSavesExpirationMails() throws Exception {
        // given
        List<String> mails = Arrays.asList("kalle@testare.dsv.su.se", "rasmus@luffen.dsv.su.se");

        // when
        submitForm(AdminGeneralSettingsPage.ALARM_MAILS, Joiner.on(ListField.SEPARATOR).join(mails));

        // then
        GeneralSystemSettings settings = captureSaveArgument();
        List<String> alarmMails = settings.getAlarmMails();
        assertEquals("Did not save the correct number of emails", mails.size(), alarmMails.size());
    }
    
    @Test
    public void testSavesSupervisoChangeMails() throws Exception {
    	List<String> mails = Arrays.asList("test@test.dsv.su.se", "test2@dsv.su.se");
    	submitForm(AdminGeneralSettingsPage.SUPERVISOR_CHANGE_MAILS, Joiner.on(ListField.SEPARATOR).join(mails));
    	
    	GeneralSystemSettings settings = captureSaveArgument();
    	List<String> supervisorChangeMails = settings.getSupervisorChangeMails();
    	assertEquals(mails.size(), supervisorChangeMails.size());
    }

    private GeneralSystemSettings captureSaveArgument() {
        ArgumentCaptor<GeneralSystemSettings> captor = ArgumentCaptor.forClass(GeneralSystemSettings.class);
        verify(generalSystemSettingsService).save(captor.capture());
        return captor.getValue();
    }

    private void submitForm(String component, String value) {
        FormTester formTester = tester.newFormTester(AdminGeneralSettingsPage.SETTINGS_FORM);
        formTester.setValue(component, value);
        formTester.submit();
    }

}
