package se.su.dsv.scipro.grading;

import org.apache.wicket.model.Model;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import static se.su.dsv.scipro.grading.TemplatePanel.*;

public class TemplatePanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);
    public static final ProjectType MASTER = new ProjectType(DegreeLevel.SECOND_CYCLE, ProjectType.MASTER_STRING, ProjectType.MASTER_STRING);
    private TemplatePanel panel;

    @Test
    public void bachelor_container_visible_if_bachelor_project() throws Exception {
        startPanel(BACHELOR, 15);
        tester.assertVisible(path(panel, BACHELOR_CONTAINER));
    }

    @Test
    public void bachelor_container_invisible_if_not_bachelor_project() throws Exception {
        startPanel(MASTER, 15);
        tester.assertInvisible(path(panel, BACHELOR_CONTAINER));
    }

    @Test
    public void master_15_container_visible_if_master_15_project() throws Exception {
        startPanel(MASTER, 15);
        tester.assertVisible(path(panel, MASTER_15_CONTAINER));
    }

    @Test
    public void master_15_container_invisible_if_not_master() throws Exception {
        startPanel(BACHELOR, 15);
        tester.assertInvisible(path(panel, MASTER_15_CONTAINER));
    }

    @Test
    public void master_15_container_invisible_if_wrong_credits() throws Exception {
        startPanel(MASTER, 30);
        tester.assertInvisible(path(panel, MASTER_15_CONTAINER));
    }

    @Test
    public void master_30_container_visible_if_master_30_project() throws Exception {
        startPanel(MASTER, 30);
        tester.assertVisible(path(panel, MASTER_30_CONTAINER));
    }

    @Test
    public void master_30_container_invisible_if_not_master() throws Exception {
        startPanel(BACHELOR, 30);
        tester.assertInvisible(path(panel, MASTER_30_CONTAINER));
    }

    @Test
    public void master_30_container_invisible_if_wrong_credits() throws Exception {
        startPanel(MASTER, 15);
        tester.assertInvisible(path(panel, MASTER_30_CONTAINER));
    }

    @Test
    public void master_30_container_visible_if_master_and_0_credits() throws Exception {
        startPanel(MASTER, 0);
        tester.assertVisible(path(panel, MASTER_30_CONTAINER));
    }

    private void startPanel(ProjectType projectType, int credits) {
        Project project = Project.builder().title("some title").projectType(projectType).build();
        project.setCredits(credits);
        panel = tester.startComponentInPage(new TemplatePanel("id", Model.of(project)));
    }
}
