package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.Password;
import se.su.dsv.scipro.system.PasswordHandler;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.Username;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

public class FirstTimeRegistrationPageTest extends SciProTest {

    @Test
    public void renders() throws Exception {
        startPage();
    }

    @Test
    public void if_there_are_users_should_redirect_to_login_page() throws Exception {
        when(userService.count()).thenReturn(1L);
        startPage();
        tester.assertRenderedPage(getHomePage());
    }

    @Test
    public void stateless() throws Exception {
        startPage();
        tester.assertRenderedPage(FirstTimeRegistrationPage.class);
        assertTrue(tester.getLastRenderedPage().isPageStateless());
    }

    @Test
    public void creates_admin() throws Exception {
        String expectedName = "admin";
        String password = "password";

        User reloadedUser = User.builder().firstName("").lastName("").emailAddress("").build();
        when(userService.save(isA(User.class))).thenReturn(reloadedUser);

        startPage();

        FormTester formTester = tester.newFormTester(path("form"));
        formTester.setValue("firstName", expectedName);
        formTester.setValue("lastName", expectedName);
        formTester.setValue("email", expectedName);
        formTester.setValue("password", password);
        formTester.submit();

        ArgumentCaptor<User> userCaptor = ArgumentCaptor.forClass(User.class);
        verify(userService, times(2)).save(userCaptor.capture());
        verify(userNameService).save(isA(Username.class));
        ArgumentCaptor<Password> passwordCaptor = ArgumentCaptor.forClass(Password.class);
        verify(passwordRepo).save(passwordCaptor.capture());

        User user = userCaptor.getAllValues().get(0);

        assertFalse(user.getUsernames().isEmpty());
        for (Username username : user.getUsernames()) {
            assertEquals(expectedName, username.getUsername());
        }

        assertTrue(PasswordHandler.isMatch(password.toCharArray(), passwordCaptor.getValue()));
    }

    @Test
    public void submitting_empty_does_not_save_user() throws Exception {
        startPage();

        FormTester formTester = tester.newFormTester(path("form"));
        formTester.submit();

        verify(userService, never()).save(isA(User.class));
    }

    @Test
    public void has_feedback_panel() throws Exception {
        startPage();
        tester.assertComponent(path("feedback"), FeedbackPanel.class);
    }

    @SuppressWarnings("unchecked")
    @Test
    public void has_link_to_login_page() throws Exception {
        startPage();
        tester.assertBookmarkablePageLink("back", (Class<? extends WebPage>) tester.getApplication().getHomePage(), new PageParameters());
    }

    private void startPage() {
        tester.startPage(FirstTimeRegistrationPage.class);
    }
}
