package se.su.dsv.scipro.loginlogout.panels;

import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.security.auth.Credentials;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

public class LoginPanelTest extends SciProTest {

    private LoginPanel panel;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = User.builder().firstName("Kalle").lastName("Tester").emailAddress("kalle@dsv.su.se").build();
        DomainObjects.injectId(user, 17L);
        startPanel();
    }

    @Test
    public void when_authentication_is_successful_sets_user_in_session() throws Exception {
        logInSuccessfully();

        assertEquals(user, SciProSession.get().getUser());
    }

    @Test
    public void when_successfully_logging_in_show_home_page() throws Exception {
        logInSuccessfully();

        tester.assertRenderedPage(getHomePage());
    }

    @Test
    public void when_failing_to_log_in_show_feedback() throws Exception {
        logInFailed();

        tester.assertErrorMessages(panel.getString("loginFailed"));
    }

    private void logInFailed() {
        when(authenticationService.authenticate(any(Credentials.class))).thenReturn(false);
        logIn();
    }

    private void logInSuccessfully() {
        when(userService.findByUsername(anyString())).thenReturn(user);
        when(authenticationService.authenticate(any(Credentials.class))).thenReturn(true);
        logIn();
    }

    private void logIn() {
        FormTester formTester = tester.newFormTester(path("signInForm"));
        formTester.setValue("username", "kalle");
        formTester.setValue("password", "secret");
        formTester.submit();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(LoginPanel.class);
    }
}
