package se.su.dsv.scipro.match;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

public class StudentIdeaStatusColumnPanelTest extends SciProTest {

    private Idea idea;
    private StudentIdeaStatusColumnPanel panel;
    private Match match;

    @Before
    public void setUp() {
        match = new Match();
        match.setStatus(Idea.Status.COMPLETED);
        idea = new Idea();
        idea.setMatch(match);
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void shows_project_created_when_idea_has_project() throws Exception {
        idea.setProject(new Project());
        startPanel();
        tester.assertLabel(path(panel, "label"), "Matched, project started");
    }

    @Test
    public void idea_with_supervisor_and_confirmed_students_after_course_start_awaits_project_creation() throws Exception {
        match.setSupervisor(new User());
        idea.addIdeaParticipation(new IdeaParticipation(new User(), idea));
        idea.setApplicationPeriod(new ApplicationPeriod());
        Mockito.when(applicationPeriodService.courseStartHasPassed(Matchers.any(ApplicationPeriod.class))).thenReturn(true);
        startPanel();
        tester.assertLabel(path(panel, "label"), "Matched, course started, awaiting project creation");
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new StudentIdeaStatusColumnPanel("id", Model.of(idea)));
    }
}
