package se.su.dsv.scipro.match;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class SupervisorAllStudentIdeasPanelTest extends SciProTest {

    SupervisorAllStudentIdeasPanel panel;

    @Before
    public void setUp() throws Exception {
        when(applicationPeriodService.isOpen(any(ApplicationPeriod.class))).thenReturn(true);
    }

    @Test
    public void testRenders() {
        panel = tester.startComponentInPage(new SupervisorAllStudentIdeasPanel("panel", Model.of(createCurrentPeriod())));
        tester.assertNoErrorMessage();
    }

    @Test
    public void is_keyword_filtering_panel_invisible() throws Exception {
        panel = tester.startComponentInPage(new SupervisorAllStudentIdeasPanel("panel", Model.of(createCurrentPeriod())));
        tester.assertInvisible(path(panel, "keywordFilter"));
    }

    @Test
    public void renders_with_a_matched_student_idea() throws Exception {
        Match match = new Match();
        match.setSupervisor(User.builder().firstName("Bob").lastName("Supervisor").emailAddress("bob@example.com").build());
        final Idea matchedIdea = new Idea();
        matchedIdea.setType(Idea.Type.STUDENT);
        matchedIdea.setMatch(match);

        when(ideaService.count(any(IdeaService.Filter.class)))
                .thenReturn(1L);
        when(ideaService.findAll(any(IdeaService.Filter.class), any(Pageable.class)))
                .thenReturn(Collections.singletonList(matchedIdea));

        panel = tester.startComponentInPage(new SupervisorAllStudentIdeasPanel("panel", Model.of(createCurrentPeriod())));
    }

    private ApplicationPeriod createCurrentPeriod() {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setStartDate(new Date(0));
        applicationPeriod.setEndDate(new Date(System.currentTimeMillis() + 239874293784L));
        return applicationPeriod;
    }
}
