package se.su.dsv.scipro.match;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.when;

public class SupervisorMyIdeasPanelTest extends SciProTest {

    private static final String TITLE_LINK = "id:dataTable:body:rows:1:cells:3:cell:link";
    private static final String FIRST_MEETING_LINK = "id:dataTable:body:rows:1:cells:7:cell:label:link";
    private static final String EDIT_LINK = "id:dataTable:body:rows:1:cells:8:cell:link";
    private static final String DELETE_LINK = "id:dataTable:body:rows:1:cells:10:cell:link";
    private SupervisorMyIdeasPanel panel;
    private User supervisorUser;
    private Idea idea;

    @Before
    public void setUp() throws Exception {
        supervisorUser = new UserBuilder().create();
        setLoggedInAs(supervisorUser);
        idea = createIdea();
        List<Idea> ideas = Collections.singletonList(idea);
        when(ideaService.findOne(Matchers.anyLong())).thenReturn(idea);
        when(ideaService.findAll(any(IdeaService.Filter.class), any(Pageable.class))).thenReturn(ideas);
        when(ideaService.count(any(IdeaService.Filter.class))).thenReturn((long) ideas.size());
        when(ideaService.getSelectionState(any(Idea.class))).thenReturn(SelectionState.AVAILABLE);
        startPanel();
    }

    @Test
    public void renders() throws Exception {
        assert panel != null;
    }

    @Test
    public void click_on_title_opens_dialog() throws Exception {
        tester.clickLink(TITLE_LINK);
        tester.assertRenderedPage(SupervisorIdeaDetailsPage.class);
    }

    @Test
    public void click_on_first_meeting_opens_dialog() throws Exception {
        tester.clickLink(FIRST_MEETING_LINK);
        tester.assertComponentOnAjaxResponse(path(panel, "dialog"));
    }

    @Test
    public void click_edit_takes_supervisor_to_edit_idea_page() throws Exception {
        unmatchIdea();
        Mockito.when(userService.findOne(Matchers.anyLong())).thenReturn(supervisorUser);
        startPanel();
        tester.clickLink(EDIT_LINK);
        tester.assertRenderedPage(SupervisorIdeaSubmissionPage.class);
    }

    @Test
    public void edit_link_not_visible_if_matched() throws Exception {
        tester.assertInvisible(EDIT_LINK);
    }

    @Test
    public void click_delete_deletes_idea() throws Exception {
        unmatchIdea();
        startPanel();
        tester.clickLink(DELETE_LINK);
        Mockito.verify(ideaService).deleteUnmatchedIdea(any(Idea.class));
    }

    @Test
    public void delete_link_not_visible_if_matched() throws Exception {
        tester.assertInvisible(DELETE_LINK);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SupervisorMyIdeasPanel("id", new UserBuilder().create(), Model.of(idea)));
    }

    private Idea createIdea() {
        Idea idea = new Idea();
        DomainObjects.injectId(idea, 5L);
        idea.setType(Idea.Type.SUPERVISOR);
        idea.setProjectType(ObjectMother.BACHELOR);
        idea.setApplicationPeriod(new ApplicationPeriod());
        idea.setResearchArea(new ResearchArea());
        idea.addIdeaParticipation(new IdeaParticipation(new UserBuilder().create(), idea));
        Match match = new Match();
        match.setSupervisor(supervisorUser);
        idea.setMatch(match);
        return idea;
    }

    private void unmatchIdea() {
        idea.getIdeaParticipations().clear();
    }
}
