package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListView;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.DisplayMultiplesPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.system.EventService;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class MilestoneActivityAdminPanelTest extends SciProTest {

    @Mock
    private EventService eventService;

    MileStoneActivityAdminPanel panel;
    MilestonePhaseTemplate phase;
    List<MilestoneActivityTemplate> activityList;
    List<MilestonePhaseTemplate> phaseList;

    @Before
    public void setUp() throws Exception {

        activityList = Collections.singletonList(new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT, "Create final seminar", "Description"));
        phase = new MilestonePhaseTemplate("This is a title", "This is a description");
        phaseList = Collections.singletonList(phase);

        Mockito.when(mileStonePhaseTemplateService.findAll(Matchers.isA(Sort.class))).thenReturn(phaseList);
        Mockito.when(milestoneActivityTemplateService.findByMileStonePhase(Matchers.isA(MilestonePhaseTemplate.class))).thenReturn(activityList);
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testPanelContainsCreateNewLink() throws Exception {
        tester.assertComponent(path(panel.getId(), MileStoneActivityAdminPanel.CREATE_LINK), AjaxLink.class);
    }

    @Test
    public void testClickCreateNewOpensDialog() throws Exception {
        tester.executeAjaxEvent(path(panel.getId(), MileStoneActivityAdminPanel.CREATE_LINK), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStoneActivityAdminPanel.DIALOG));
    }

    @Test
    public void testPanelContainsPhaseList() throws Exception {
        String listPath = path(panel.getId(), MileStoneActivityAdminPanel.LIST_CONTAINER, MileStoneActivityAdminPanel.PHASE_LIST);
        tester.assertComponent(listPath, ListView.class);
        tester.assertListView(listPath, phaseList);
    }

    @Test
    public void testEachPhaseContainsPhaseTitleLabel() throws Exception {
        String titlePath = phasePath(MileStoneActivityAdminPanel.TITLE);
        tester.assertComponent(titlePath, Label.class);
        tester.assertLabel(titlePath, phase.getTitle());
    }

    @Test
    public void testPhaseContainsListOfActivities() throws Exception {
        String activityListPath = phasePath(MileStoneActivityAdminPanel.MILESTONE_LIST);
        tester.assertComponent(activityListPath, ListView.class);
        tester.assertListView(activityListPath, activityList);
    }

    @Test
    public void testActivityContainsTitle() throws Exception {
        tester.assertLabel(activityPath(MileStoneActivityAdminPanel.MILESTONE_TITLE), activityList.get(0).getTitle());
    }

    @Test
    public void testActivityContainsType() throws Exception {
        tester.assertLabel(activityPath(MileStoneActivityAdminPanel.TYPE), activityList.get(0).getType().toString());
    }

    @Test
    public void testActivityContainsProjectTypes() throws Exception {
        tester.assertComponent(activityPath(MileStoneActivityAdminPanel.PROJECT_TYPES), DisplayMultiplesPanel.class);
    }

    @Test
    public void testClickEditOpensDialog() throws Exception {
        tester.executeAjaxEvent(activityPath(MileStoneActivityAdminPanel.EDIT), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStoneActivityAdminPanel.DIALOG));
    }

    @Test
    public void testClickSwitchCallsService() throws Exception {
        MilestoneActivityTemplate activity = activityList.get(0);
        activity.setMilestonePhaseTemplate(new MilestonePhaseTemplate("title", "desc"));
        boolean deleted = activity.isDeleted();
        tester.executeAjaxEvent(activityPath(MileStoneActivityAdminPanel.STATE_SWITCH), "onclick");
        Mockito.verify(milestoneActivityTemplateService, Mockito.times(1)).save(Matchers.eq(activity), Matchers.isA(MilestonePhaseTemplate.class));
        Assert.assertNotSame(deleted, activity.isDeleted());
    }

    @Test
    public void testClickSwitchRendersContainer() throws Exception {
        tester.executeAjaxEvent(activityPath(MileStoneActivityAdminPanel.STATE_SWITCH), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStoneActivityAdminPanel.LIST_CONTAINER));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new MileStoneActivityAdminPanel("panel"));
    }

    private String phasePath(String... paths) {
        return path(panel.getId(), MileStoneActivityAdminPanel.LIST_CONTAINER, MileStoneActivityAdminPanel.PHASE_LIST, 0, path((Object[]) paths));
    }

    private String activityPath(String... paths) {
        return path(panel.getId(), MileStoneActivityAdminPanel.LIST_CONTAINER, MileStoneActivityAdminPanel.PHASE_LIST, 0, MileStoneActivityAdminPanel.MILESTONE_LIST, 0, path((Object[]) paths));
    }
}
