package se.su.dsv.scipro.milestones;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListView;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.SortOrderPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class MilestonePhaseAdminPanelTest extends SciProTest {

    MileStonePhaseAdminPanel panel;
    List<MilestonePhaseTemplate> phaseList;

    @Before
    public void setUp() throws Exception {
        MilestonePhaseTemplate phase = new MilestonePhaseTemplate("title", "description");
        phaseList = Collections.singletonList(phase);
        Mockito.when(mileStonePhaseTemplateService.findAll(Matchers.isA(Sort.class))).thenReturn(phaseList);
        panel = tester.startComponentInPage(MileStonePhaseAdminPanel.class);
    }

    @Test
    public void testRenders() throws Exception {
        Assert.assertNotNull(panel);
    }

    @Test
    public void testPanelContainsCreateNewLink() throws Exception {
        tester.assertComponent(path(panel.getId(), MileStonePhaseAdminPanel.CREATE_LINK), AjaxLink.class);
    }

    @Test
    public void testPanelContainsPhaseList() throws Exception {
        String listPath = path(panel.getId(), MileStonePhaseAdminPanel.LIST_CONTAINER, MileStonePhaseAdminPanel.PHASE_LIST);
        tester.assertComponent(listPath, ListView.class);
        tester.assertListView(listPath, phaseList);
    }

    @Test
    public void testPhaseContainsMovePanel() throws Exception {
        tester.assertComponent(phasePath(MileStonePhaseAdminPanel.MOVE), SortOrderPanel.class);
    }

    @Test
    public void testPhaseContainsTitle() throws Exception {
        tester.assertComponent(phasePath(MileStonePhaseAdminPanel.TITLE), Label.class);
        tester.assertLabel(phasePath(MileStonePhaseAdminPanel.TITLE), phaseList.get(0).getTitle());
    }

    @Test
    public void testClickCreateNewOpensDialog() throws Exception {
        tester.executeAjaxEvent(path(panel.getId(), MileStonePhaseAdminPanel.CREATE_LINK), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStonePhaseAdminPanel.DIALOG));
    }

    @Test
    public void testClickEditOpensDialog() throws Exception {
        tester.executeAjaxEvent(phasePath(MileStonePhaseAdminPanel.EDIT), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStonePhaseAdminPanel.DIALOG));
    }

    @Test
    public void testClickSwitchCallsService() throws Exception {
        MilestonePhaseTemplate phase = phaseList.get(0);
        boolean deleted = phase.isDeleted();
        tester.executeAjaxEvent(phasePath(MileStonePhaseAdminPanel.STATE_SWITCH), "onclick");
        Mockito.verify(mileStonePhaseTemplateService, Mockito.times(1)).save(Matchers.eq(phase));
        Assert.assertNotSame(deleted, phase.isDeleted());
    }

    @Test
    public void testClickSwitchRendersContainer() throws Exception {
        tester.executeAjaxEvent(phasePath(MileStonePhaseAdminPanel.STATE_SWITCH), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel.getId(), MileStonePhaseAdminPanel.LIST_CONTAINER));
    }

    private String phasePath(String... paths) {
        return path(panel.getId(), MileStonePhaseAdminPanel.LIST_CONTAINER, MileStonePhaseAdminPanel.PHASE_LIST, 0, path((Object[]) paths));
    }
}
