package se.su.dsv.scipro.milestones.pages;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.milestones.panels.MileStoneOverviewPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.TreeSet;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class SupervisorMilestonePageTest extends PageTest {

    SupervisorMileStonePage page;
    Project project;

    @Before
    public void setUp() throws Exception {
        User headSupervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User student2 = new UserBuilder().setFirstName("author").setLastName("2").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(headSupervisor)
                .build();
        DomainObjects.injectId(project, 1L);
        when(projectService.findOne(isA(Long.class))).thenReturn(project);
        when(projectService.isSupervisorOrFollower(isA(Project.class), isA(User.class))).thenReturn(true);
        page = tester.startPage(SupervisorMileStonePage.class, getPageParameters());
    }

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorMileStonePage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), String.valueOf(1L));
        return pp;
    }


    @Test
    public void testPageContainsOverviewPanel() throws Exception {
        tester.assertComponent(SupervisorMileStonePage.OVERVIEW_PANEL, MileStoneOverviewPanel.class);
    }
}
