package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import static org.mockito.Matchers.eq;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;

public class MilestoneCounterPanelTest extends SciProTest {

    MileStoneCounterPanel panel;

    private Project project;

    @Before
    public void setUp() throws Exception {
        project = new Project();
        project.setProjectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        project.setDateCreated(new DateTime().minusDays(1).toDate());
        when(milestoneActivityTemplateService.countActivities(isA(MilestoneActivityTemplate.Type.class), eq(project.getProjectType()), eq(project.getDateCreated()))).thenReturn(TOTAL);
        startPanel();
    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testIncludeLabelWithCompletedMileStones() throws Exception {
        tester.assertComponent(path(panel.getId(), MileStoneCounterPanel.COMPLETED), Label.class);
        tester.assertLabel(path(panel.getId(), MileStoneCounterPanel.COMPLETED), String.valueOf(COMPLETED));
    }

    @Test
    public void testIncludeLabelWithTotalMileStones() throws Exception {
        tester.assertComponent(path(panel.getId(), MileStoneCounterPanel.TOTAL), Label.class);
        tester.assertLabel(path(panel.getId(), MileStoneCounterPanel.TOTAL), String.valueOf(TOTAL));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new MileStoneCounterPanel("panel", Model.of(COMPLETED), MilestoneActivityTemplate.Type.PROJECT, Model.of(project)));
    }

    static final long TOTAL = 3L;
    static final long COMPLETED = 1L;
}
