package se.su.dsv.scipro.notifications.panels;

import com.google.common.base.Joiner;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.notifications.NotificationService;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class NotificationDataPanelTest extends SciProTest {

    NotificationDataPanel panel;

    @Before
    public void setUp() throws Exception {
        startPanel(ObjectMother.SOME_PROJECT, ObjectMother.SOME_USER);
    }

    @Test
    public void test_renders() throws Exception {
        tester.assertNoErrorMessage();
    }

    @Test
    public void test_mark_read() throws Exception {
        // given
        User user = new UserBuilder().setId(72L).setFirstName("Kalle").setLastName("Testare").create();
        setLoggedInAs(user);

        // when
        tester.clickLink(path("markRead"));

        // then
        verify(notificationService).markRead(eq(user));
        tester.assertComponentOnAjaxResponse(panel);
    }

    @Test
    public void renders_without_project_attached_to_notification_event() throws Exception {
        startPanel(null, ObjectMother.SOME_USER);
        tester.assertNoErrorMessage();
    }

    @Test
    public void renders_without_caused_by_user_attached_to_notification_event() throws Exception {
        startPanel(null, null);
        tester.assertNoErrorMessage();
    }

    private void startPanel(Project project, User causedBy) {
        when(notificationService.findAll(any(NotificationService.Filter.class), any(Pageable.class))).thenReturn(Collections.singletonList(createNotification(project, causedBy)));
        when(notificationService.count(any(NotificationService.Filter.class))).thenReturn(1L);
        when(projectService.getMemberType(project, causedBy)).thenReturn(Member.Type.SUPERVISOR);
        panel = tester.startComponentInPage(NotificationDataPanel.class);
    }

    private Notification createNotification(Project project, User causedBy) {
        Notification notification = new Notification();
        ProjectEvent notificationEvent = new ProjectEvent();
        notificationEvent.setCausedBy(causedBy);
        notificationEvent.setProject(project);
        notificationEvent.setType(Notification.Type.PROJECT);
        notificationEvent.setEvent(ProjectEvent.Event.CREATED);
        notification.setNotificationEvent(notificationEvent);
        return notification;
    }

    private String path(String... components) {
        return panel.getId() + ":" + Joiner.on(":").join(components);
    }
}
