package se.su.dsv.scipro.notifications.settings.panels;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.notifications.dataobject.Notification;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.notifications.settings.entities.DeliveryMethod;
import se.su.dsv.scipro.notifications.settings.service.DeliveryConfigurationService;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;

import static org.junit.Assert.assertNotNull;
import static org.mockito.Mockito.*;

public class TypeDeliveryConfigurationPanelTest extends SciProTest {

    @Mock
    private DeliveryConfigurationService configuration;

    private TypeDeliveryConfigurationPanel panel;

    @Before
    public void setUp() throws Exception {
        panel = tester.startComponentInPage(new TypeDeliveryConfigurationPanel("panel", Notification.Type.PROJECT, ProjectEvent.Event.values()));
    }

    @Test
    public void testRenders() throws Exception {
        assertNotNull("Does not render", panel);
    }

    @Test
    public void testContainsListOfDeliveryMethods() throws Exception {
        tester.assertListView(path(panel, TypeDeliveryConfigurationPanel.METHODS), Arrays.asList(DeliveryMethod.values()));
    }

    @Test
    public void testContainsRowForEvents() throws Exception {
        tester.assertListView(path(panel, TypeDeliveryConfigurationPanel.EVENTS), Arrays.asList(ProjectEvent.Event.values()));
    }

    @Test
    public void testEventRowContainsListOfDeliveryMethods() throws Exception {
        tester.assertListView(path(panel, TypeDeliveryConfigurationPanel.EVENTS, 0, TypeDeliveryConfigurationPanel.METHODS), Arrays.asList(DeliveryMethod.values()));
    }

    @Test
    public void testClickingCheckboxUpdatesMethod() throws Exception {
        tester.executeAjaxEvent(path(panel.getId(),
                TypeDeliveryConfigurationPanel.EVENTS, "0",
                TypeDeliveryConfigurationPanel.METHODS, "0",
                TypeDeliveryConfigurationPanel.METHOD), "onclick");

        verify(configuration, times(1)).setDelivery(isA(Notification.Type.class), isA(Enum.class), isA(DeliveryMethod.class), isA(Boolean.class), any(User.class));
    }

    @Test
    public void testCheckboxStartsAtCorrectState() throws Exception {
        // given
        boolean setting = true;

        // when
        when(configuration.isDelivery(isA(Notification.Type.class), isA(Enum.class), isA(DeliveryMethod.class), (User) eq(null))).thenReturn(setting, !setting);
        setUp(); // Re-render the panel

        // then
        tester.assertModelValue(path(panel.getId(),
                TypeDeliveryConfigurationPanel.EVENTS, "0",
                TypeDeliveryConfigurationPanel.METHODS, "0",
                TypeDeliveryConfigurationPanel.METHOD), setting); // First checkbox should be checked
        tester.assertModelValue(path(panel.getId(),
                TypeDeliveryConfigurationPanel.EVENTS, "0",
                TypeDeliveryConfigurationPanel.METHODS, "1",
                TypeDeliveryConfigurationPanel.METHOD), !setting); // All after should not be checked

    }
}
