package se.su.dsv.scipro.peer;

import org.apache.wicket.Page;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class PeerPortalPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("author").setLastName("1").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(user)
                .build();
        DomainObjects.injectId(project, 1L);

        setLoggedInAs(user);
        when(projectService.findOne(anyLong())).thenReturn(project);
        when(projectService.isPartOfProject(project, user)).thenReturn(true);
        when(peerRequestService.findByRequesterAndProject(SciProSession.get().getUser(), project)).thenReturn(Collections.singletonList(new PeerRequest()));
        when(performReviewService.getInProgressReview(any(User.class), eq(project))).thenReturn(Option.<PeerReview>empty());
    }

    @Override
    protected Class<? extends Page> getPage() {
        return PeerPortalPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.set(PageParameterKeys.MAP.get(Project.class), "1");
        return pageParameters;
    }

    @Test
    public void assert_components() throws Exception {
        startPage();
        tester.assertComponent(path(AbstractPeerPortalPage.PEER_REVIEW_GUIDE_PANEL), PeerReviewGuidePanel.class);
        tester.assertComponent(path(AbstractPeerPortalPage.MY_REQUEST_AND_REVIEWS_PANEL), MyPeerPanel.class);
        tester.assertComponent(path(AbstractPeerPortalPage.LATEST_REVIEW_PANEL), LatestReviewPanel.class);
    }
}
