package se.su.dsv.scipro.project.panels;

import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.pages.AbstractProjectDetailsPage;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.hamcrest.core.IsNot.not;
import static org.hamcrest.core.StringContains.containsString;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.when;

public class ProjectTabMenuPanelTest extends SciProTest {

    private static final User LOGGED_IN_USER = new UserBuilder().create();
    public static final Project SOME_PROJECT = Project.builder()
            .title("some project")
            .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor"))
            .build();
    private static final Project ICT4D_PROJECT = Project.builder()
            .title("ICT4D")
            .projectType(new ProjectType(DegreeLevel.NONE, "ICT4D", "ICT4D"))
            .build();

    @Test
    public void renders() throws Exception {
        setLoggedInAs(LOGGED_IN_USER);
        when(projectService.getProjectsByUser(LOGGED_IN_USER)).thenReturn(Collections.singletonList(SOME_PROJECT));
        assertNotNull(startPanel());
    }

    @Test
    public void all_final_seminars_tab_is_visible_for_student_with_normal_projects() throws Exception {
        setLoggedInAs(LOGGED_IN_USER);
        when(projectService.getProjectsByUser(LOGGED_IN_USER)).thenReturn(Collections.singletonList(SOME_PROJECT));
        assertVisible(ProjectTabMenuPanel.ALL_FINAL_SEMINARS);
    }

    @Test
    public void all_final_seminars_tab_is_not_visible_for_student_with_ICT4D_project() throws Exception {
        setLoggedInAs(LOGGED_IN_USER);
        when(projectService.getProjectsByUser(LOGGED_IN_USER)).thenReturn(Collections.singletonList(ICT4D_PROJECT));
        assertNotVisible(ProjectTabMenuPanel.ALL_FINAL_SEMINARS);
    }

    @Test
    public void ideas_tab_visible_if_any_project_type_has_module_turned_on() throws Exception {
        when(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH)).thenReturn(true);
        assertVisible(ProjectTabMenuPanel.IDEAS);
    }

    @Test
    public void ideas_tab_not_visible_if_not_turned_on() throws Exception {
        when(projectTypeService.projectTypeWithModuleExists(ProjectModule.MATCH)).thenReturn(false);
        assertNotVisible(ProjectTabMenuPanel.IDEAS);
    }

    private String renderedPanel() {
        startPanel();
        return tester.getLastResponseAsString();
    }

    private void assertVisible(String menuChoice) {
        assertThat(renderedPanel(), containsString(menuChoice));
    }

    private void assertNotVisible(String menuChoice) {
        assertThat(renderedPanel(), not(containsString(menuChoice)));
    }

    private ProjectTabMenuPanel startPanel() {
        return tester.startComponentInPage(new ProjectTabMenuPanel("id", AbstractProjectDetailsPage.class));
    }
}
