package se.su.dsv.scipro.reviewer;

import org.apache.wicket.Page;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.reviewing.ReviewerDeadlineSettings;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class ReviewerStartPageTest extends PageTest {
	@Before
    public void setUp() throws Exception {
        Project project = new Project();
        project.setId(234L);
        FinalThesis finalThesis = new FinalThesis();
        finalThesis.setProject(project);
        when(finalThesisService.findOne(anyLong())).thenReturn(finalThesis);
        when(finalThesisRepo.reviewers(any(User.class), isA(Boolean.class), any(Pageable.class))).thenReturn(Collections.singletonList(finalThesis));
        when(finalThesisRepo.countReviewers(any(User.class), isA(Boolean.class))).thenReturn(1L);
        when(reviewerDeadlineSettingsService.getInstance()).thenReturn(new ReviewerDeadlineSettings());
    }

	@Override
	protected Class<? extends Page> getPage() {
		return ReviewerStartPage.class;
	}

    @Override
    public Roles[] getRoles() {
        return new Roles[]{Roles.REVIEWER};
	}

    @Test
    public void shows_undecided_approvals_on_the_tab() throws Exception {
        long undecided = 42L;
        when(myReviewService.countUndecidedRoughDraft(any(User.class))).thenReturn(undecided);
        startPage();
        tester.assertModelValue("undecidedRoughDraftApprovals", undecided);
    }

    @Test
    public void shows_unread_supervisor_interactions_on_the_tab() throws Exception {
        Project project = new Project();
        List<Project> projects = Collections.singletonList(project);

        ForumPost post = new ForumPost();
        List<ForumPost> posts = Arrays.asList(post, post);

        when(projectService.findAll(any(ProjectService.Filter.class))).thenReturn(projects);
        when(reviewerInteractionService.getPosts(project)).thenReturn(posts);
        when(basicForumService.isRead(any(User.class), any(ForumPost.class))).thenReturn(true, false, true, false, true);
        startPage();
        tester.assertModelValue("unreadSupervisorsPosts", 1);
    }
}
