package se.su.dsv.scipro.security.auth;

import org.apache.wicket.Component;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.security.auth.roles.Roles;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class RoleBasedAuthorizationStrategyTest extends SciProTest {

    @Mock
    private Authorization authorization;
    @Mock
    private Component component;

    private RoleBasedAuthorizationStrategy authorizationStrategy;

    @Before
    public void setUp() throws Exception {
        authorizationStrategy = new RoleBasedAuthorizationStrategy() {
            @Override
            protected Authorization internalGetAnnotation(Class<? extends Component> authRequired) {
                return authorization;
            }
        };
    }

    @Test
    public void sysadmin_with_employee_role_can_render_components_requiring_employee_role() throws Exception {
        setRoles(Roles.SYSADMIN, Roles.SUPERVISOR);

        when(authorization.requiresLoggedInUser()).thenReturn(true);
        when(authorization.authorizedRoles()).thenReturn(new Roles[]{Roles.SYSADMIN});

        assertRenderAllowed();
    }

    @Test
    public void public_component() throws Exception {
        setLoggedIn(false);

        when(authorization.requiresLoggedInUser()).thenReturn(false);

        assertRenderAllowed();
    }

    @Test
    public void component_without_annotation_is_public() throws Exception {
        setLoggedIn(false);

        authorization = null;

        assertRenderAllowed();
    }

    @Test
    public void not_logged_in_denies_access() throws Exception {
        setLoggedIn(false);

        when(authorization.requiresLoggedInUser()).thenReturn(true);

        assertRenderDisallowed();
    }

    private void assertRenderAllowed() {
        assertTrue(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

    private void assertRenderDisallowed() {
        assertFalse(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }
}
