package se.su.dsv.scipro.security.auth;

import org.apache.wicket.Component;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.SystemModule;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class SystemModuleBasedAuthorizationStrategyTest extends SciProTest {

    @Mock
    private SystemModuleComponent authorization;
    @Mock
    private Component component;

    private SystemModuleBasedAuthorizationStrategy authorizationStrategy;

    @Before
    public void setUp() throws Exception {
        authorizationStrategy = new SystemModuleBasedAuthorizationStrategy() {
            @Override
            protected SystemModuleComponent internalGetAnnotation(Class<? extends Component> authRequired) {
                return authorization;
            }
        };
    }

    @Test
    public void instantiation_always_authorized() throws Exception {
        assertTrue(authorizationStrategy.isInstantiationAuthorized(component.getClass()));
    }

    @Test
    public void authorized_if_module_is_undefined() throws Exception {
        authorization = null;
        assertTrue(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

    @Test
    public void authorized_if_module_attached_to_session() throws Exception {
        setSystemModules(SystemModule.GROUP);
        when(authorization.value()).thenReturn(SystemModule.GROUP);
        assertTrue(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

    @Test
    public void not_authorized_if_module_not_present() throws Exception {
        setSystemModules();
        when(authorization.value()).thenReturn(SystemModule.GROUP);
        assertFalse(authorizationStrategy.isActionAuthorized(component, Component.RENDER));
    }

}