package se.su.dsv.scipro.settings.pages;

import org.apache.wicket.Page;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.User;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.settings.pages.BasicProfilePage.*;

@RunWith(MockitoJUnitRunner.class)
public class BasicProfilePageTest extends PageTest {
    @Before
    public void setUp() throws Exception {
        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        setLoggedInAs(user);

        when(userProfileService.findByUser(user)).thenReturn(new UserProfile());
        when(hashService.createForUser(isA(User.class))).thenReturn(Hash.builder().hash("anicehash").user(user).build());

        tester.startPage(BasicProfilePage.class);
    }

    @Override
    protected Class<? extends Page> getPage() {
        return BasicProfilePage.class;
    }

    @Test
    public void shows_feedback_after_submitting_form() throws Exception {
    	tester.newFormTester(FORM).submit();

        tester.assertFeedback(FEEDBACK, tester.getLastRenderedPage().getString("profile.saved"));
    }

    @Test
    public void submitting_form_saves_new_info() throws Exception {
        String skypeId = "testing skype";
        String phone = "testing phone";
        String other = "testing other info";

        FormTester formTester = tester.newFormTester(FORM);
        formTester.setValue(SKYPE, skypeId);
        formTester.setValue(PHONE, phone);
        formTester.setValue(OTHER, other);
        formTester.submit();

        ArgumentCaptor<UserProfile> captor = ArgumentCaptor.forClass(UserProfile.class);
        verify(userProfileService).save(captor.capture());

        assertEquals("Does not save correct skype ID", skypeId, captor.getValue().getSkypeId());
        assertEquals("Does not save correct phone info", phone, captor.getValue().getPhoneNumber());
        assertEquals("Does not save correct other info", other, captor.getValue().getOtherInfo());
    }

    @Test
    public void change_password_generates_notification() throws Exception {
        tester.clickLink(path("form", "changePassword"));
        verify(hashService).createForUser(isA(User.class));
    }
}
