package se.su.dsv.scipro.statistics;

import org.apache.wicket.Page;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.reviewing.Decision;
import se.su.dsv.scipro.reviewing.ReviewerApproval;
import se.su.dsv.scipro.reviewing.ReviewerDeadlineSettings;
import se.su.dsv.scipro.util.Pair;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class AdminReviewerDeadlinesPageTest extends PageTest {

    @Before
    public void setUp() throws Exception {
        final ReviewerApproval reviewerApproval = new ReviewerApproval() {
            @Override
            public Step getStep() {
                return Step.ROUGH_DRAFT_APPROVAL;
            }
        };

        Decision decision = mock(Decision.class);
        List<Pair<ReviewerApproval, Decision>> lateReviewerApprovals = Collections.singletonList(new Pair<>(reviewerApproval, decision));
        List<Pair<? extends ReviewerApproval, Decision>> list = new ArrayList<>();
        for (Pair<ReviewerApproval, Decision> ra: lateReviewerApprovals) {
            list.add(0, ra);
        }
        when(reviewerDeadlineFollowupService.getLateReviewerApprovals(eq(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL), any(Date.class), any(Date.class), any(Date.class))).thenReturn(list);
        when(reviewerDeadlineSettingsService.getInstance()).thenReturn(new ReviewerDeadlineSettings());

        startPage();
    }

    @Test
    public void lists_all_rough_draft_deadlines_that_are_missed() throws Exception {
        tester.assertListView(path("container", "reviewerApprovals"), reviewerDeadlineFollowupService.getLateReviewerApprovals(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL, new Date(), new Date(), new Date()));
    }

    @Override
    protected Class<? extends Page> getPage() {
        return AdminReviewerDeadlinesPage.class;
    }
}