package se.su.dsv.scipro.statistics;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class ProjectMilestoneDetailsPanelTest extends SciProTest {
    public static final String PROJECT_PLAN_APPROVED = "Project plan approved";
    public static final User HEAD_SUPERVISOR = new UserBuilder().create();
    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
    public static final Project SOME_PROJECT = Project.builder().title("project").projectType(SOME_PROJECT_TYPE).headSupervisor(HEAD_SUPERVISOR).build();

    private MilestoneActivityTemplate activity;

    @Before
    public void setUp() throws Exception {
        createActivity();
    }

    @Test
    public void shows_milestone_activity_title() throws Exception {
        startPanel();
        tester.assertLabel(path("id", "title"), activity.getTitle());
    }

    private void mockList(MileStoneService.Filter filter) {
        Mockito.when(mileStoneService.findCompletedMilestones(Matchers.eq(activity), Matchers.eq(filter), Matchers.any(Pageable.class))).thenReturn(getMileStoneList());
        Mockito.when(mileStoneService.countCompletedMileStones(activity, filter)).thenReturn(1L);
    }

    private List<Milestone> getMileStoneList() {
        return Collections.singletonList(new Milestone(SOME_PROJECT, activity));
    }

    private void startPanel() {
        startPage(new MileStoneService.Filter());
    }

    private void startPage(MileStoneService.Filter filter) {
        mockList(filter);
        tester.startComponentInPage(new ProjectMilestoneDetailsPanel("id", Model.of(activity), filter));
    }

    private void createActivity() {
        activity = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT, PROJECT_PLAN_APPROVED, "description");
    }

}
