package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.junit.Before;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.util.PageParameterKeys;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

public abstract class SupervisorPageTest extends PageTest {

	protected Project project;

	@Before
    public void setUpProject() throws Exception {
        Project project = createProject();
		setLoggedInAs(project.getHeadSupervisor());

        when(projectService.findOne(anyLong())).thenReturn(project);
        when(projectService.isSupervisorOrFollower(any(Project.class), any(User.class))).thenReturn(true);
    }

    protected Project createProject() {
	    User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
	    DomainObjects.injectId(user, 6814684L);

	    project = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
	    DomainObjects.injectId(project, 1L);
        project.setHeadSupervisor(user);
        return project;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), 1L);
        return pageParameters;
    }
}
