package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class SupervisorProfilePageTest extends SciProTest {

    @Before
    public void setUp() throws Exception {
        when(userService.findOne(anyLong())).thenReturn(ObjectMother.SOME_USER);
    }

    @Test
    public void test_render_page() throws Exception {
        startPage();
        tester.assertRenderedPage(SupervisorProfilePage.class);
    }

    @Test
    public void lists_users_research_areas() throws Exception {
        User user = mockUserWithResearchAreaAndUnit();
        startPage();
        tester.assertListView(path("form", "researchAreas"), new ArrayList<>(user.getActiveResearchAreas()));
    }

    @Test
    public void no_area_label_is_visible_if_no_research_areas_exist() throws Exception {
        startPage();
        tester.assertVisible(path("form", "noAreas"));
    }

    @Test
    public void no_unit_label_is_visible_if_no_unit_exists() throws Exception {
        startPage();
        tester.assertVisible(path("form", "noUnit"));
    }

    @Test
    public void no_unit_label_not_visible_if_unit_exists() throws Exception {
        mockUserWithResearchAreaAndUnit();
        startPage();
        tester.assertInvisible(path("form", "noUnit"));
    }

    @Test
    public void submit_saves_user() throws Exception {
        startPage();
        submit();
        verify(userService).save(ObjectMother.SOME_USER);
    }

    private void submit() {
        FormTester form = tester.newFormTester("form");
        form.submit();
    }

    @Test
    public void feedback_when_saving_languages() throws Exception {
        startPage();
        submit();
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        assertThat(messages, hasItem(tester.getLastRenderedPage().getString("saved")));
    }

    private User mockUserWithResearchAreaAndUnit() {
        User user = User.builder().firstName("first").lastName("last").emailAddress("first@last.com").build();
        user.addResearchArea(new ResearchArea());
        user.setUnit(new Unit());
        when(userService.findOne(anyLong())).thenReturn(user);
        return user;
    }

    private SupervisorProfilePage startPage() {
        return tester.startPage(SupervisorProfilePage.class);
    }
}
