package se.su.dsv.scipro.user;

import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.FormComponent;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.UserService;

import static se.su.dsv.scipro.user.AdminRolePage.INTERNAL_USER_FILTER;
import static se.su.dsv.scipro.user.AdminRolePage.USER_DATA_PANEL;

public class AdminRolePageTest extends SciProTest {

    @Test
    public void renders() throws Exception {
        startPage();
        tester.assertRenderedPage(AdminRolePage.class);
    }

    @Test
    public void has_checkbox_for_internal_users_only() throws Exception {
        startPage();
        tester.assertComponent(path(INTERNAL_USER_FILTER), CheckBox.class);
    }

    @Test
    public void checking_internal_users_should_refresh_table() throws Exception {
        startPage();
        clickInternalUsersCheckbox();
        tester.assertComponentOnAjaxResponse(path(USER_DATA_PANEL));
    }

    @Test
    public void checking_internal_users_should_set_filter() throws Exception {
        startPage();
        selectInternalUsersCheckbox();
        assertInternalUserFlagIsSetOnFilter();
    }

    private void assertInternalUserFlagIsSetOnFilter() {
        ArgumentCaptor<UserService.Filter> captor = ArgumentCaptor.forClass(UserService.Filter.class);
        Mockito.verify(userService, Mockito.atLeastOnce()).count(captor.capture());
        Assert.assertTrue(captor.getValue().isInternalUsersOnly());
    }

    private void selectInternalUsersCheckbox() {
        FormComponent<?> internalUsersOnly = (FormComponent<?>) tester.getComponentFromLastRenderedPage(path(INTERNAL_USER_FILTER));
        tester.getRequest().setParameter(internalUsersOnly.getInputName(), Boolean.toString(true));
        clickInternalUsersCheckbox();
    }

    private void clickInternalUsersCheckbox() {
        tester.executeAjaxEvent(path(INTERNAL_USER_FILTER), "click");
    }

    private void startPage() {
        tester.startPage(AdminRolePage.class);
    }
}
